;----------------------------------------------------
; Disassembly of the file Bega's Battle ROMs 
;        by Jeff Kulczycki, July 1, 2002
;----------------------------------------------------
;
;----------------------------------------------------
;    RAM VARIABLES
;----------------------------------------------------
;    Address      Description
;----------------------------------------------------
;   0081          Player1 Lives
;   0082          Player2 Lives
;   0083          Unknown
;   0084          Unknown
;   0093          joy
;   0094          joy
;   0219          Credits
;   021A          Coin Count
;   0251          unknown 25
;   0252          unknown 25
;   04BD          Joystick direction
;   1000          Input Bank
;        .0       Down  joystick
;        .1       Up    joystick
;        .2       Left  joystick
;        .3       Right joystick
;        .4       S3   Service Sw?
;        .5       S2   "Barrier"
;        .6       S1   "Teleport"?
;        .7       Tile
;   1001 .4       Player2 button
;        .5       Player1 button
;        .6       Coin slot1
;        .7       Coin slot2
;   1002          DIP Switch 2
;        .0-.1    Sw1-2 Number of Begas
;        .2       Sw3   0=Continuous,1=Not
;        .3-.4    Sw4-5 Bonus Threshold
;        .5       Sw6   0=Hard, 1=Normal
;        .6       Sw7   0=Upright,1=Cockpit // Factory Diagnosis
;        .7       Sw8   0=Diagnostics,1=Normal
;   1004          Sound Board
;   1005          Sound Board
;   1007          unknown
; ---------------------------------------------------
;           Address 1006/1007
; ---------------------------------------------------
;    .......1  6850 UART Read:  Receive data reg full
;    ......1.  6850 UART Read:  Xmit data reg empty
;    .....1..  6850 UART Read:  Data carrier detect (or has been down)
;    ....1...  6850 UART Read:  Clear to send
;    ...1....  6850 UART Read:  Framing error
;    ..1.....  6850 UART Read:  Receiver overrun
;    .1......  6850 UART Read:  Parity error
;    1.......  6850 UART Read:  Interrupt request
; ---------------------------------------------------
;   0000-0FFF     ZeroPage RAM
;   1000-1FFF     Hardware Inputs/Outputs
;   1800-1837     Video Color RAM
;   2000-201F     unknown
;   2020-23FF     Screen1 Text 32x30 (Tiles)
;   2400-27FF     Screen1 Character Bank register
;   2800-281F     Sprite Bank1 (8 sprites)
;   2820-2BDF     Screen2 Text 32x30 (Tiles)
;   2BE0-2BFF     Sprite Bank2 (8 sprites)
;   2C00-2FFF     Screen2 Character Bank register
;   3000-301F     unknown
;   3020-33FF     Screen3 Text 32x30 (Tiles)
;   3400-37FF     Screen3 Character Bank register
;   3800-381F     Sprite Bank3 (8 sprites)
;   3820-3BDF     Screen4 Text 32x30 (Tiles)
;   3BE0-3BFF     Sprite Bank4 (8 sprites)
;   3C00-3FFF     Screen4 Character Bank register
;   4000-DFFF     Program ROM
; ---------------------------------------------------
; Note:  BPL Branch if bit7 is clear
; Start of code
;
         ORG $4000
; ---------------------------------------------------
;                Interrupt Routines
; ---------------------------------------------------
L4000:   JMP L4D64              ; Check for Coins
L4003:   JMP L4009              ; Start Program
L4006:   JMP L4FD2              ; Check for LDPlayer Errors

; ---------------------------------------------------
;         Start program
; ---------------------------------------------------
L4009:   SEI                    ; Enable Interrupts
         CLD                    ; Clear Decimal Mode
         LDX #$FF
         TXS
         LDA #$00
         STA L1000
         STA VDSLST
         STA L1001
         JSR L43C2
         JSR L43B4
         LDA #$01
         STA VDSLST
         LDA #$00
         STA VTIMR4
         JSR L4501              ; Determine number of Begas
;----------------------------------------------------
;               Delay Time for Startup
;----------------------------------------------------
         LDY #$0F               ; Delay Loop Y = 15
L402E:   LDX #$3F               ; Delay X = 63
         JSR L4BD2              ; Run Programmable Delay
         DEY                    ; Continue delay loop
         BNE L402E              ; Loop until Y = 0
;----------------------------------------------------
;                   
;----------------------------------------------------
         LDA #$03               ;
         STA L1000              ;
         CLI                    ; Disable interrupts
;----------------------------------------------------   `
;                   Attract Mode
;----------------------------------------------------
L403C:   JSR L4098              ; Show instructions if credit awarded
L403F:   JSR L459A              ; Erase Video Screen
         JSR L4098              ; Show instructions if credit awarded
         JSR L719E              ; Screen Transition, Fill from outside in
         JSR L4098              ; Show instructions if credit awarded
         LDA #$2D               ; Video Sequence = Attract Mode (Frame 43901)
         JSR L54A7              ; Play LaserDisc video sequence
;----------------------------------------------------
;                 STICK1 = $00
;----------------------------------------------------
L4050:   LDA STICK1             ;
         CMP #$01               ;
         BEQ L4066              ;
         CMP #$02               ;
         BEQ L4066              ;
         LDX #$01               ; Delay X = 1
         JSR L4BD2              ; Run Programmable Delay
         JSR L4098              ; Show instructions if credit awarded
         JMP L4050              ; Wait for SEARCH complete
;----------------------------------------------------
;                 STICK1 = $01
;----------------------------------------------------
L4066:   JSR L71B8              ; Screen Transition, Clear from inside out
L4069:   LDA STICK1             ;
         CMP #$02               ;
         BEQ L407B              ; Yes, so show High Scores
         LDX #$01               ; Delay = 1
         JSR L4BD2              ; Run Programmable Delay
         JSR L4098              ; Show instructions if credit awarded
         JMP L4069              ; Loop back and wait
;----------------------------------------------------
;         High Score Screen during Attract Mode
;----------------------------------------------------
L407B:   JSR L4496              ; Print High Scores
         LDA #$2E               ; Video Sequence = Attract Mode (Frame 46343)
         JSR L54A7              ; Play LaserDisc video sequence
;----------------------------------------------------
;                 STICK1 = $02
;----------------------------------------------------
L4083:   LDA STICK1             ;
         CMP #$02               ;
         BEQ L4095              ; Loop back to Attract Mode
         LDX #$01               ; Delay = 1
         JSR L4BD2              ; Run Programmable Delay
         JSR L4098              ; Show instructions if credit awarded
         JMP L4083              ;
L4095:   JMP L403C              ; Go back to Attract Mode



; ---------------------------------------------------
;          Show Instructions if Credit Awarded  
; ---------------------------------------------------
L4098:   LDA VTIMR4             ; Check for Instruction Mode/Credit Awarded
         BEQ L40A0              ; No credits inserted so skip ahead
         JMP L4E3F              ; Give instructions wait for START BUTTON
L40A0:   RTS                    ; Return


;----------------------------------------------------
;                  Start Game
;----------------------------------------------------
L40A1:   JSR L4501              ; Determine number of Begas
         JSR L452D              ; Clear variables, Clear "swap variables"
         LDA #$01               ;
         STA VIMIRQ             ;
         LDA #$00               ;
         STA PADDL1             ;
;----------------------------------------------------
;                Main Game Loop
;----------------------------------------------------
L40B1:   LDA #$00               ;
         JSR L51CE              ; Play Sound
         JSR L6992              ; level setup?
;----------------------------------------------------
;          Check if game has been tilted
;----------------------------------------------------
L40B9:   LDA L1000              ; Get Joystick input
         AND L1000              ; Was TILT alarmed?
         BMI L40C4              ; No, so skip ahead
         JMP L583D              ; Machine has tilted, print error
;----------------------------------------------------
;              check process list
;----------------------------------------------------
L40C4:   LDA L1003              ;
         BPL L40B9              ;
         LDY #$05               ; 5 Processes to check
L40CB:   LDA L009C,Y            ;
         BPL L40D6              ;
         CMP #$FF               ; Is process finished?
         BEQ L40D6              ; No, check next process
         BNE L40E7
L40D6:   DEY
         BPL L40CB
         LDA VNTP+1
         BMI L40E7
         LDA STICK1             ; Software State
         CMP #$02               ; Time to update sprites?
         BNE L40E7              ; Yes, so do refresh of sprite data
         JMP L40B1              ; Else, return to main game loop

;----------------------------------------------------
;           Update Sprite and System Clock  
;----------------------------------------------------
L40E7:   JSR L47D0              ; Update Sprites
         INC SysClock           ; Increment System Clock low byte (0-64)
         LDA SysClock           ; Check System Clock low byte
         AND #$3F               ; Does System Clock = 64
         BNE L40FA              ; No, so skip ahead
         INC SysClock+1         ; Increment System Clock High byte
         INC L04BB
;----------------------------------------------------
;          Pseudo-random Number Generator
;----------------------------------------------------
; Reads a byte from a certain memory range ($4000-$47FF)
; then adds the previous random number and $3BFB (raster
; line Y-position) to it, returning the new random number
; in the accumulator. The next byte in the memory range
; is read on the next execution of this routine.
;----------------------------------------------------
L40FA:   INC RandSeed           ; Increment seed ROM pointer
         BCC L4100              ; Check if carry is set
         INC RandSeed+1         ; Increment seed ROM pointer+1
L4100:   LDA RandSeed+1         ; Get seed ROM pointer
         AND #$07               ; Set bounds ($4000-$4700)
L4104:   ORA #$40               ; Set bounds ($4000-$4700)
         STA RandSeed+1         ; Limit seed ROM pointer+1
         INC Random             ; Increment the random number
         LDA L1000              ; Mix the number with any joystick presses
         ADC Random             ; Add previous random number
         LDY RandSeed           ; Use seed ROM pointer
         ADC (RandSeed),Y       ; Add ROM data byte to random number
         ADC L3BFB              ; Add Y-raster position(?)
         STA Random             ; Save random number
;----------------------------------------------------
;          Check if game has been TILTed
;----------------------------------------------------
L411B:   LDA L1000              ; Get Joystick input
         AND L1000              ; Was TILT alarmed?
         BMI L4126              ; No, so skip ahead
         JMP L583D              ; Machine has tilted, print error
L4126:   LDA L1003              ;
         BMI L411B              ;
;----------------------------------------------------
;          
;----------------------------------------------------
         JSR L45D7              ; Print "1UP" or "2UP"
         JSR L58BA              ; Do Bega's Left/Right Movement
         JSR L5E23              ; LUNA/JOE movements
         JSR L6325              ; Girder Updates
         JSR L4891              ; Handle FIRE button
         JSR L690F
         JSR L6D6E
         LDA CDTMA1+1
         BEQ L4153
L4145:   LDA VIMIRQ+1
         BNE L4156
         LDA #$41               ; Return address = $4152
         PHA                    ; Put return address on stack
         LDA #$52               ; Return address = $4152
         PHA                    ; Put return address on stack
         JMP (L00CF)            ; Jump to routine
;----------------------------------------------------
;           Return here after Jump Routine
;----------------------------------------------------
L4153:   JMP L40B9              ; Loop back and check for tilt
;----------------------------------------------------
;           
;----------------------------------------------------
L4156:   LDA #$FF               ;
         JSR L720B              ;
         LDA #$00
         STA VIMIRQ+1
         LDA VTIMR4+1
         BNE L4168
         JMP L403C              ; Go back to Attract Mode
L4168:   LDA NumPlayers         ; How many players?
         BNE L417C              ; Two players so skip ahead
         LDA Player1Lives       ; Get Player1 lives
         BMI L41E2              ;
         LDA #$01               ;
         STA CDTMV3             ;
         JSR L459A              ; Erase Video Screen             
         JMP L40B1              ;
L417C:   LDX PlayerNumber       ; Get Player Number
         BNE L41AE              ; Player2 so skip ahead
;----------------------------------------------------
;                Handle Player 1
;----------------------------------------------------
         LDA Player1Lives       ; Check Player 1 playing
         AND Player2Lives       ; Check Player 2 playing 
         BMI L41E2              ; Nobody playing so skip ahead
         LDA Player2Lives       ;
         BPL L4196              ;
         LDA #$01               ;
         STA CDTMV3             ;
         JSR L459A              ; Erase Video Screen
         JMP L40B1              ;
L4196:   LDA Player1Lives       ;
         BPL L419D              ;
         JSR L425A              ;
L419D:   LDA #$01               ; Set for Player2
         STA PlayerNumber       ; Switch to Player2
         STA CDTMV3             ;
         JSR L470B              ; Swap Player1/2 Data
         JSR L459A              ; Erase Video Screen
         JMP L40B1              ; Return to main game loop
;----------------------------------------------------
;                Handle Player 2
;----------------------------------------------------
L41AE:   LDA Player2Lives       ; Check Player 1 playing
         LDA Player2Lives       ; Check Player 1 playing
         AND Player1Lives       ; Check Player 2 playing 
         BMI L41E2              ; Nobody playing so skip ahead
         LDA Player1Lives       ;
         BPL L41C5              ;
         LDA #$01               ;
         STA CDTMV3             ;
         JSR L459A              ; Erase Video Screen
         JMP L40B1
L41C5:   LDA Player2Lives       ; Get Player2 lives
         BPL L41CF              ; Game not over, so skip ahead
         JSR L42C3              ; Player2 GAME OVER
         JSR L459A              ; Erase Video Screen
L41CF:   LDA #$00               ; Set for Player1
         STA PlayerNumber       ; Switch to Player1
         LDA #$01
         STA CDTMV3
         JSR L470B              ; Swap Player1/2 Data
         JSR L459A              ; Erase Video Screen
         JMP L40B1              ; Return to main game loop
;----------------------------------------------------
;             Player 1 & 2 Game Over
;----------------------------------------------------
L41E2:   JSR L4329              ; Player1 GAME OVER
         LDA #$00
         STA VTIMR4+1
;----------------------------------------------------
;               Copy Player's Score
;----------------------------------------------------
         LDA Score              ; Get Score byte 0
         STA ZRAM_01            ; Save copy of score byte 0
         LDA Score1             ; Get Score byte 1
         STA ZRAM_02            ; Save copy of score byte 0
         LDA Score2             ; Get Score byte 2
         STA ZRAM_02+1          ; Save copy of score byte 0
         JSR L5211              ;
         LDA SDLSTL             ;
         STA ZRAM_01            ;
         LDA SDLSTH             ;
         STA ZRAM_02            ;
         LDA SSKCTL             ;
         STA ZRAM_02+1          ;
         JSR L5211              ;
         LDA Credits            ; Get number of credits
         NOP                    ; pause
         NOP                    ; pause
         BEQ L421D              ; No credits so erase screen and loop back to Attract Mode
         LDA #$00
         STA VTIMR4+1
         JMP L4E3F              ; Give instructions wait for START BUTTON
L421D:   JSR L459A              ; Erase Video Screen
L4220:   JMP L403C              ; Go back to Attract Mode



;----------------------------------------------------
;             Print "GAME START"
;----------------------------------------------------
         LDX #$9B               ; Message = "GAME START" ($439B)
         LDY #$43               ; Message = "GAME START" ($439B)
         JSR L4831              ; Print Text Message
L422A:   LDA #$03               ;
         JSR L51CE              ; Play Sound
L422F:   LDX #$1F               ; Delay = 31
         JSR L4BD2              ; Run Programmable Delay
         JSR L71EC              ;
         RTS                    ; Return



;----------------------------------------------------
;           Print "PLAYER 1 GAME START"
;----------------------------------------------------
         LDX #$85               ; Message = "PLAYER 1" ($4385)
         LDY #$43               ; Message = "PLAYER 1" ($4385)
         JSR L4831              ; Print Text Message
         LDX #$9B               ; Message = "GAME START" ($439B)
         LDY #$43               ; Message = "GAME START" ($439B)
         JSR L4831              ; Print Text Message
         JMP L422A              ;



;----------------------------------------------------
;           Print "PLAYER 2 GAME START"
;----------------------------------------------------
         LDX #$90               ; Message = "PLAYER 2" ($4390)
         LDY #$43               ; Message = "PLAYER 2" ($4390)
         JSR L4831              ; Print Text Message
         LDX #$9B               ; Message = "GAME START" ($439B)
         LDY #$43               ; Message = "GAME START" ($439B) 
         JSR L4831              ; Print Text Message
         JMP L422A              ;



;----------------------------------------------------
;                   unknown
;----------------------------------------------------
L425A:   JSR L55E1              ; Check for Continue Feature
         JSR L719E              ; Screen Transition, Fill from outside in
         JSR L71D0
         JSR L71DE
         LDX #$00
         TXA
L4269:   STA L3A00,X
         STA L3B00,X
         STA L3E00,X
         STA L3F00,X
         DEX
         BNE L4269
         LDA #$14               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
         INC VTIMR2+1
         LDX #$85               ; Message = "PLAYER 1" ($4385)
         LDY #$43               ; Message = "PLAYER 1" ($4385)
         JSR L4831              ; Print Text Message
         INC VTIMR2+1
         LDX #$A8               ; Message = "GAME OVER" ($43A8)
         LDY #$43               ; Message = "GAME OVER" ($43A8)
         JSR L4831              ; Print Text Message
         LDA #$00
         JSR L51CE              ; Play Sound
L4296:   LDA STICK1
         CMP #$01
         BEQ L42A9
         CMP #$02
         BEQ L42A9
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L4296
L42A9:   JSR L71B8              ; Screen Transition, Clear from inside out
;----------------------------------------------------
;             Wait for SEARCH complete
;----------------------------------------------------
L42AC:   LDA STICK1
         CMP #$02
         BEQ L42BB
         LDX #$01               ; Delay = 1
         JSR L4BD2              ; Run Programmable Delay
         JMP L42AC
L42BB:   LDX #$80               ; Delay = 128
         JSR L4BD2              ; Run Programmable Delay
         JMP L422F



;----------------------------------------------------
;              Player 2 GAME OVER
;----------------------------------------------------
L42C3:   JSR L55E1              ; Check for Continue Feature
         JSR L719E              ; Screen Transition, Fill from outside in
         JSR L71D0
         LDX #$00
         TXA
L42CF:   STA L3A00,X
         STA L3B00,X
         STA L3E00,X
         STA L3F00,X
         DEX
         BNE L42CF
         LDA #$14               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
         INC VTIMR2+1
         LDX #$90               ; Message = "PLAYER 2" ($4390)
         LDY #$43               ; Message = "PLAYER 2" ($4390)
         JSR L4831              ; Print Text Message
         INC VTIMR2+1
         LDX #$A8               ; Message = "GAME OVER" ($43A8)
         LDY #$43               ; Message = "GAME OVER" ($43A8)
         JSR L4831              ; Print Text Message
         LDA #$00
         JSR L51CE              ; Play Sound
L42FC:   LDA STICK1
         CMP #$01
         BEQ L430F
         CMP #$02
         BEQ L430F
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L42FC
L430F:   JSR L71B8              ; Screen Transition, Clear from inside out
L4312:   LDA STICK1
         CMP #$02
         BEQ L4321
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L4312
L4321:   LDX #$80
         JSR L4BD2              ; Run Programmable Delay
         JMP L422F


;----------------------------------------------------
;              Player 1 GAME OVER
;----------------------------------------------------
L4329:   JSR L55E1              ; Check for Continue Feature
         JSR L719E              ; Screen Transition, Fill from outside in
         JSR L71D0
         LDX #$00
         TXA
L4335:   STA L3A00,X
         STA L3B00,X
         STA L3E00,X
         STA L3F00,X
         DEX
         BNE L4335
         LDA #$14               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
         INC VTIMR2+1
         LDX #$A8               ; Message = "GAME OVER" ($43A8)
         LDY #$43               ; Message = "GAME OVER" ($43A8)
         JSR L4831              ; Print Text Message
         LDA #$00
         JSR L51CE              ; Play Sound
L4358:   LDA STICK1
         CMP #$01
         BEQ L436B
         CMP #$02
         BEQ L436B
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L4358
L436B:   JSR L71B8              ; Screen Transition, Clear from inside out
L436E:   LDA STICK1
         CMP #$02
         BEQ L437D
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L436E
L437D:   LDX #$80
         JSR L4BD2              ; Run Programmable Delay
         JMP L422F


;----------------------------------------------------
;                Text Messages
;----------------------------------------------------
L4385:   .WORD $39CD  ; location on screen 
         .BYTE $1A,$16,$0B,$23,$0F,$1C,$00,$02,$FF  ; "PLAYER 1"

L4390:   .WORD $39CD  ; location on screen
         .BYTE $1A,$16,$0B,$23,$0F,$1C,$00,$03,$FF  ; "PLAYER 2"

L439B:   .WORD $3A0C  ; location on screen
         .BYTE $11,$0B,$17,$0F,$00,$1D,$1E,$0B,$1C,$1E,$FF  ; "GAME START"

L43A8:   .WORD $3A0C  ; location on screen
         .BYTE $11,$0B,$17,$0F,$00,$19,$20,$0F,$1C,$FF      ; "GAME OVER"


;----------------------------------------------------
;             unknown
;----------------------------------------------------
L43B4:   LDA #$00               ; A = No Sound
         STA L1004              ; Write to Sound Chip
L43B9:   LDA #$80
         STA VBREAK
         STA L1001
         RTS
L43C2:   JSR L459A              ; Erase Video Screen
;----------------------------------------------------
;                clear ???
;----------------------------------------------------
         LDA #$00
         TAX
L43C8:   STA ZRAM_01,X
         INX
         CPX #$E1
         BNE L43C8
;----------------------------------------------------
;                clear ???
;----------------------------------------------------
         TAX                    ; A = 0, X = 0
L43D0:   STA VDSLST,X           ;
         STA DDEVIC,X           ;
         STA L0400,X            ;
         INX                    ;
         BNE L43D0              ;
;----------------------------------------------------
;            Initialize Serial Comm Port
;----------------------------------------------------
         LDA #$03               ; UART Master Reset
         STA L1006              ; Send UART Master Reset
         LDA #$96               ; 8 bit,NO parity,1 stopbit, Clock 32x, Enable Rx Interrupt
         STA L1006              ; Set UART Control Register
         LDA L1007              ; Read UART Status Register
;----------------------------------------------------
;               setup pointers?
;----------------------------------------------------
         LDA #$40               ; Mini = $4000
         STA Mini_HI            ;
         STA FLPTR+1            ;
         STA FPTR2+1            ;
         LDA #$00               ;
         STA Mini_LO            ;
         LDA #$03
         STA FLPTR
         LDA #$06
         STA FPTR2
         LDA #$01
         STA LPENV
         LDA #$39
         STA LPENH              ;
         LDA #$50               ;
         STA SPARE              ;
;----------------------------------------------------
;   Initialize Dummy Info for High Score Table
;----------------------------------------------------
         LDX #$36               ; 54 bytes to copy (27 score,27 initials)
L440E:   LDA L4460,X            ; Get Dummy High Score info from ROM
         STA BRKKY,X            ; Save Dummy High Score info
         DEX                    ; Go to next byte
         BPL L440E              ; Loop until entire table is written
; ---------------------------------------------------
;     Copy 4428-? to 1800-1838 Video Color RAM
; ---------------------------------------------------
         LDY #$00               ; Loop = 0
L4419:   LDA L4428,Y            ; Get data
         STA L1800,Y            ; Copy data here
         INY                    ; Next byte
         CPY #$38               ; All bytes copied?
         BNE L4419              ; Loop until all bytes copied
         INC Level              ;
         RTS                    ; Return

;----------------------------------------------------
;            Video Color RAM
;----------------------------------------------------
L4428:   .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00

;----------------------------------------------------
;               Dummy High Scores
;----------------------------------------------------
L4460:   .BYTE $50,$39,$01   ; 13950 (1)
         .BYTE $00,$14,$01   ; 11400 (2)
         .BYTE $50,$83,$00   ;  8350 (3)
         .BYTE $00,$62,$00   ;  6200 (4)
         .BYTE $00,$50,$00   ;  5000 (5)
         .BYTE $50,$41,$00   ;  4150 (6)
         .BYTE $00,$36,$00   ;  3600 (7)
         .BYTE $00,$28,$00   ;  2800 (8)
         .BYTE $50,$98,$00   ;  9850 (-)

;----------------------------------------------------
;            Dummy High Score Initials
;----------------------------------------------------
         .BYTE $19,$0B,$1E   ; OAT   (1)
         .BYTE $12,$0D,$19   ; HEO   (2)
         .BYTE $0C,$0F,$15   ; BEK   (3)
         .BYTE $0C,$13,$19   ; BIO   (4)
         .BYTE $0B,$0F,$18   ; AEN   (5)
         .BYTE $0E,$0B,$0C   ; DAB   (6)
         .BYTE $1C,$1D,$0F   ; RSE   (7)
         .BYTE $12,$0F,$18   ; HEN   (8)
         .BYTE $00,$00,$00   ; ---   (-)



;----------------------------------------------------
;               Print High Scores
;----------------------------------------------------
L4496:   JSR L459A              ; Erase Video Screen
         LDX #$6D               ; Message = "(C) DATA EAST USA INC 1983" ($4C6D)
L449B:   LDY #$4C               ; Message = "(C) DATA EAST USA INC 1983" ($4C6D)
         JSR L4831              ; Print Text Message
L44A0:   LDX #$B9               ; Message = "TODAYS GREATEST" ($4CB9)
         LDY #$4C               ; Message = "TODAYS GREATEST" ($4CB9)
         JSR L4831              ; Print Text Message
         LDA #$C7               ; Setup screen position ($38C7)
         STA ZRAM_01            ; Save screen position ($38C7)
         LDA #$38               ; Setup screen position ($38C7)
         STA ZRAM_02            ; Save screen position ($38C7)
         LDX #$00               ; Loop for initials
L44B1:   LDY #$00               ; Loop used for indexing
         LDA L0251,X            ; Get first initial
         STA (ZRAM_01),Y        ; Print first initial
         INY                    ; Go to next initial
         LDA L0252,X            ; Get middle initial
         STA (ZRAM_01),Y        ; Print middle initial
         INY                    ; Go to next initial
         LDA L0253,X            ; Get middle initial
         STA (ZRAM_01),Y        ; Print middle initial
         CLC                    ; Clear carry
         LDA ZRAM_01            ; Get text position
         ADC #$41               ; Skip 1 line (65 characters)
         STA ZRAM_01            ; Save text position
         LDA ZRAM_02            ; Get text position
         ADC #$00               ; Add carry
         STA ZRAM_02            ; Save text position
         INX                    ; Get next players initials
         INX                    ; Get next players initials
         INX                    ; Get next players initials
         CPX #$18               ; Check for 8 players (8*3=24)
         BCC L44B1              ; Loop back until all 8 players printed
;----------------------------------------------------
;            Print Scores 1 thru 8
;----------------------------------------------------
         LDX #$03               ; Score 1
         JSR L463A              ; Print score
         LDX #$04               ; Score 2
         JSR L463A              ; Print score
         LDX #$05               ; Score 3
         JSR L463A              ; Print score
         LDX #$06               ; Score 4
         JSR L463A              ; Print score
         LDX #$07               ; Score 5
         JSR L463A              ; Print score
         LDX #$08               ; Score 6
         JSR L463A              ; Print score
         LDX #$09               ; Score 7
         JSR L463A              ; Print score
         LDX #$0A               ; Score 8
         JSR L463A              ; Print score
         RTS                    ; Return



;----------------------------------------------------
;       Assign Initial Number of Begas (lives)
;----------------------------------------------------
L4501:   LDA L1002              ; Get DIP Bank 2
L4504:   EOR #$FF               ; Toggle all bits
         TAY                    ; Y = DIP Bank 2
         AND #$03               ; Get DIP Sw1-2
         TAX                    ; X = DIP Sw1-2
         LDA L4529,X            ; Get Number of Lives from table
         STA Player1Lives       ; Save Player1 lives
         STA Player2Lives       ; Save Player2 lives
;----------------------------------------------------
;         Setup Bonus Begas score threshold
;----------------------------------------------------
         TYA                    ; Get DIP Bank 2
         LSR A                  ; Isolate sw4,5
         LSR A                  ; Isolate sw4,5
         LSR A                  ; Isolate sw4,5
         AND #$03               ; Isolate sw4,5
         TAX                    ; X = sw4,5
         LDA L4525,X            ; Get Bonus Threshold from table
         STA Bonus              ; Save Bonus Threshold
         STA Bonus+1            ; Save Bonus Threshold 
         STA Bonus+2            ; Save Bonus Threshold
         RTS                    ; Return

;----------------------------------------------------
;        Initial Begas (lives) and Bonus Begas
;----------------------------------------------------
L4525:   .BYTE  2, 5, 3, 0   ; 20K,30K,40K,50K Points
L4529:   .BYTE  2, 4, 3, 5   ; 2,3,4,5 Initial Lives
         


;----------------------------------------------------
;       Clear variables, Clear "swap variables"
;----------------------------------------------------
L452D:   LDA #$FF               ;
         STA Level              ;
         STA Level _swap        ;
         STA Lunas              ;
         STA Lunas_swap         ;
         STA STARP_swap         ;
         STA STARP              ;
         STA STMCUR_swap        ;
         STA STMCUR             ;
         STA VVTP+1             ;
         STA STMTAB             ;
         STA RUNSTK+1           ;
         STA TOPSTK             ;
         STA TOPSTK+1           ;
         STA MEOLFLG
         STA STARP+1
         STA RUNSTK
         STA L0093
         STA L0094
         STA POKADR
         STA POKADR+1
         STA L0097
         STA L0098
         STA L0099
         STA L009A
         STA L00A7
         STA L00A8
         STA L00A9
         STA L00AA
         STA L00AB
         STA L00AC
         LDA #$00               ; Set for Player1
         STA PlayerNumber       ; Switch to Player1
         STA CDTMV3
         STA CDTMV3_swap
         STA VVBLKD
         STA VVBLKD_swap
         STA CDTMV5_swap
         STA CDTMV5
         STA CDTMA1+1
         STA CDTMA2
         STA CDTMA2+1
         STA CDTMF3
         STA L04D8
         STA L04D9
         STA L04DA              ; Reset Continue Option
         RTS                    ; Return




;----------------------------------------------------
;          Erase Video Screen (2000-3FFF)
;----------------------------------------------------
L459A:   LDA #$00               ; Address low  $2000
         STA ZRAM_01            ; Setup address to erase
         LDA #$20               ; Address high $2000
         STA ZRAM_02            ; Setup address to erase
         LDX #$20               ; 32 blocks of 256 to erase (2K)
L45A4:   LDY #$00               ; Loop = 0
         LDA #$00               ; A = $00
L45A8:   STA (ZRAM_01),Y        ; Clear address location
         INY                    ; Next location
         BNE L45A8              ; Loop for 256 bytes to clear
         INC ZRAM_02            ; Increment address pointer
         DEX                    ; More blocks to erase
         BNE L45A4              ; Loop until all is erased
; ---------------------------------------------------
;             Erase Bank1 Sprites (7 sprites)
; ---------------------------------------------------
         LDA #$00               ; Loop = 28 (4 x 7 sprites)
         LDX #$1B               ; Clear with $00
L45B6:   STA Sprite1_Prop,X     ; Erase sprite properties
         DEX                    ; Erase sprite character
         BPL L45B6              ; Erase sprite Y coordinate
         LDA #$08               ; Erase sprite X coordinate
         STA L1003              ; Go to next sprite
         RTS                    ; Loop until all 7 sprites erased




;----------------------------------------------------
;         Print "HI 1UP/2UP" overlay screen
;----------------------------------------------------
L45C1:   LDA #$66               ; Char ="H" (RED)
         STA L3843              ; Print "H"
         LDA #$67               ; Char ="I" (RED)
         STA L3844              ; Print "I"
         JSR L4609              ; Print "1UP"
         LDA NumPlayers         ; Get number of players
         BEQ L45D6              ; Only 1 player so skip ahead
         JSR L4619              ; Print "2UP"
L45D6:   RTS                    ; Return




;----------------------------------------------------
;            Print "1UP" or "2UP"
;----------------------------------------------------
L45D7:   LDA NumPlayers         ; Get number of players
         BEQ L4628              ; Only 1 player so skip altogether
         LDA SysClock           ; Check for System Clock
         AND #$3F               ; Do we need to blink 1UP/2UP message?
         BNE L4600              ; Time to print it, skip ahead
         LDA PlayerNumber       ; Get player number
         BNE L45F4              ; Player 2 so skip ahead
;----------------------------------------------------
;               Erase "1UP"
;----------------------------------------------------
         LDA #$00               ; A = blank
         STA L3881              ; Erase character
         STA L3882              ; Erase character
         STA L3883              ; Erase character
         RTS                    ; Return
;----------------------------------------------------
;               Erase "2UP"
;----------------------------------------------------
L45F4:   LDA #$00               ; A = blank
         STA L38C1              ; Erase character
         STA L38C2              ; Erase character
         STA L38C3              ; Erase character
         RTS                    ; ReturnReturn
;----------------------------------------------------
;              Check Player 1 or 2 
;----------------------------------------------------
L4600:   CMP #$0F               ; Check lives
         BNE L4628              ; No lives left so leave
         LDA PlayerNumber       ; Which player is playing?
         BNE L4619              ; Player 2, so print "2UP"
;----------------------------------------------------
;                Print "1UP"
;----------------------------------------------------
L4609:   LDA #$02               ; Char = '1'
         STA L3881              ; Print character
         LDA #$1F               ; Char = 'U'
         STA L3882              ; Print character
         LDA #$1A               ; Char = 'P'
         STA L3883              ; Print character
         RTS                    ; Return
;----------------------------------------------------
;                Print "2UP"
;----------------------------------------------------
L4619:   LDA #$03               ; Char = '2'
         STA L38C1              ; Print character
         LDA #$1F               ; Char = 'U'
         STA L38C2              ; Print character
         LDA #$1A               ; Char = 'P'
         STA L38C3              ; Print character
L4628:   RTS                    ; Return



;----------------------------------------------------
;              unknown data
;----------------------------------------------------
L4629:   EOR #$38
         .BYTE $12,$13
         AND ABUFPT+1
         ORA L1C19
         .BYTE $0F
         INC L3859,X
         AND L2F2E
         .BYTE $FF



;----------------------------------------------------
;             Print Score to display
;----------------------------------------------------
L463A:   JMP L4693              ; Print Score to display


;----------------------------------------------------
;                    unused
;----------------------------------------------------
         LDY L528F,X
         TYA
         LDY L46FF,X
         TAX
         LDA L46E9,Y
         STA ZRAM_01
         LDA L46EA,Y
         STA ZRAM_02
         LDY #$00
L4651:   LDA Score,X
         LSR A
         LSR A
         LSR A
         LSR A
         CLC
         ADC #$01
         CMP #$01
         BNE L467F
         INY
         LDA Score,X
         AND #$0F
         CLC
         ADC #$01
         CMP #$01
         BNE L468A
         DEX
         INY
         CPY #$06
         BNE L4651
         DEY
         BNE L468A
L4675:   LDA Score,X
         LSR A
         LSR A
         LSR A
         LSR A
         CLC
         ADC #$01
L467F:   STA (ZRAM_01),Y
         INY
         LDA Score,X
         AND #$0F
         CLC
         ADC #$01
L468A:   STA (ZRAM_01),Y
         DEX
         INY
         CPY #$06               ;
         BNE L4675              ;
         RTS                    ; Return



;----------------------------------------------------
;             Print Score to display
;----------------------------------------------------
;             X = Score to print
;                 0      Player1 score
;                 1      Player2 score
;                 2      HI score
;                 3-10   8 High Scores
;----------------------------------------------------
L4693:   LDY L528F,X            ; Y = (2,5,8,11,...)
         TYA                    ; A = (2,5,8,11,...) 
         LDY L46FF,X            ; Y = (0,2,4,...)
         TAX                    ; X = (2,5,8,11,...)
;----------------------------------------------------
;         X =          Y =
;----------------------------------------------------
L469B:   LDA L46E9,Y            ;
         STA ZRAM_01            ;
         LDA L46EA,Y            ;
         STA ZRAM_02            ;
         LDY #$00               ;
L46A7:   LDA Score,X            ;
         LSR A
         LSR A
         LSR A
         LSR A
         CLC
         ADC #$01
         CMP #$01
         BNE L46D5
         INY
         LDA Score,X
         AND #$0F
         CLC
         ADC #$01               ;
         CMP #$01               ; Check for leading zeros
         BNE L46E0              ; No more zeros, so print number
         DEX                    ;
         INY                    ;
         CPY #$06               ; Have we check all 6 digits?
         BNE L46A7              ; No, loop back until all digits checked for zero
         DEY
         BNE L46E0
L46CB:   LDA Score,X
         LSR A
         LSR A
         LSR A
         LSR A
         CLC
         ADC #$01
L46D5:   STA (ZRAM_01),Y        ; Print score?
         INY
         LDA Score,X
         AND #$0F
         CLC
         ADC #$01
L46E0:   STA (ZRAM_01),Y        ; Print score digit to screen
         DEX                    ;
         INY                    ;
         CPY #$06               ; Have we printed all 6 digits?
         BNE L46CB              ; No, loop back until all digits printed
         RTS                    




; 3800 = properties
; 3801 = character number
; 3802 = y coord
; 3803 = x coord
;----------------------------------------------------
;              Sprite data location
;----------------------------------------------------
L46E9:   .WORD $38A1,$38E1,$3861,$38CB,$390C,$394D
         .WORD $398E,$39CF,$3A10,$3A51,$3A92
;----------------------------------------------------
;                data
;----------------------------------------------------
L46FF:   .BYTE $00,$02,$04,$06,$08,$0A
         .BYTE $0C,$0E,$10,$12,$14,$16



; ---------------------------------------------------
;      Swap Player 1 and Player 2 Data pointers
; ---------------------------------------------------
L470B:   LDA Level              ; Swap Player Data
         LDX Level_swap         ; Swap Player Data
         STA Level_swap         ; Swap Player Data
         STX Level              ; Swap Player Data

         LDA VVBLKD             ; Swap Player Data
         LDX VVBLKD_swap        ; Swap Player Data
         STA VVBLKD_swap        ; Swap Player Data
         STX VVBLKD             ; Swap Player Data

         LDA CDTMV3             ; Swap Player Data
         LDX CDTMV3_swap        ; Swap Player Data
         STA CDTMV3_swap        ; Swap Player Data
         STX CDTMV3             ; Swap Player Data

         LDA CDTMV5_swap        ; Swap Player Data
         LDX CDTMV5             ; Swap Player Data
         STA CDTMV5             ; Swap Player Data
         STX CDTMV5_swap        ; Swap Player Data

         LDA Lunas              ; Swap Player Data
         LDX Lunas_swap         ; Swap Player Data
         STA Lunas_swap         ; Swap Player Data
         STX Lunas              ; Swap Player Data

         LDA STARP_swap         ; Swap Player Data
         LDX STARP              ; Swap Player Data
         STA STARP              ; Swap Player Data
         STX STARP_swap         ; Swap Player Data

         LDA STMCUR_swap        ; Swap Player Data
         LDX STMCUR             ; Swap Player Data
         STA STMCUR             ; Swap Player Data
         STX STMCUR_swap        ; Swap Player Data

         LDA VVTP+1             ; Swap Player Data
         LDX STMTAB             ; Swap Player Data
         STA STMTAB             ; Swap Player Data
         STX VVTP+1             ; Swap Player Data

         LDA RUNSTK+1           ; Swap Player Data
         LDX TOPSTK             ; Swap Player Data
         STA TOPSTK             ; Swap Player Data
         STX RUNSTK+1           ; Swap Player Data

         LDA TOPSTK+1           ; Swap Player Data
         LDX MEOLFLG            ; Swap Player Data
         STA MEOLFLG            ; Swap Player Data
         STX TOPSTK+1           ; Swap Player Data

         LDA STARP+1            ; Swap Player Data
         LDX RUNSTK             ; Swap Player Data
         STA RUNSTK             ; Swap Player Data
         STX STARP+1            ; Swap Player Data

         LDA L0093              ; Swap Player Data
         LDX L0097              ; Swap Player Data
         STA L0097              ; Swap Player Data
         STX L0093              ; Swap Player Data

         LDA L0094              ; Swap Player Data
         LDX L0098              ; Swap Player Data
         STA L0098              ; Swap Player Data
         STX L0094              ; Swap Player Data

         LDA POKADR             ; Swap Player Data
         LDX L0099              ; Swap Player Data
         STA L0099              ; Swap Player Data
         STX POKADR             ; Swap Player Data

         LDA POKADR+1           ; Swap Player Data
         LDX L009A              ; Swap Player Data
         STA L009A              ; Swap Player Data
         STX POKADR+1           ; Swap Player Data

         LDA L00A7              ; Swap Player Data
         LDX L00A8              ; Swap Player Data
         STA L00A8              ; Swap Player Data
         STX L00A7              ; Swap Player Data

         LDA L00A9              ; Swap Player Data
         LDX L00AA              ; Swap Player Data
         STA L00AA              ; Swap Player Data
         STX L00A9              ; Swap Player Data

         LDA L00AB              ; Swap Player Data
         LDX L00AC              ; Swap Player Data
         STA L00AC              ; Swap Player Data
         STX L00AB              ; Swap Player Data

         LDA CDTMA1+1           ; Swap Player Data
         LDX CDTMA2             ; Swap Player Data
         STA CDTMA2             ; Swap Player Data
         STX CDTMA1+1           ; Swap Player Data

         LDA CDTMA2+1           ; Swap Player Data
         LDX CDTMF3             ; Swap Player Data
         STA CDTMF3             ; Swap Player Data
         STX CDTMA2+1           ; Swap Player Data

         LDA L04D8              ; Swap Player Data
         LDX L04D9              ; Swap Player Data
         STA L04D9              ; Swap Player Data
         STX L04D8              ; Swap Player Data

         RTS                    ; Return




; ---------------------------------------------------
;                Refresh All Sprites
; ---------------------------------------------------
L47D0:   LDX #$06               ; 7 sprites per bank (8 sprites available)
L47D2:   LDY L4829,X            ; Get Sprite Index
; ---------------------------------------------------
;               Update Sprites Bank 1
; ---------------------------------------------------
         LDA Sprite1_Prop,X     ; Get Properties byte
         STA L2800,Y            ; Save sprite properties
         LDA Sprite1_Char,X     ; Get Character number
         STA L2801,Y            ; Save sprite character number
         LDA Sprite1_Y,X        ; Get Y Coordinate
         STA L2802,Y            ; Save sprite Y coordinate
         LDA Sprite1_X,X        ; Get X Coordinate
         STA L2803,Y            ; Save sprite X coordinate
; ---------------------------------------------------
;               Update Sprites Bank 2
; ---------------------------------------------------
         LDA Sprite2_Prop,X     ; Get Properties byte
         STA L2BE0,Y            ; Save sprite properties
         LDA Sprite2_Char,X     ; Get Character number
         STA L2BE1,Y            ; Save sprite character number
         LDA Sprite2_Y,X        ; Get Y Coordinate
         STA L2BE2,Y            ; Save sprite Y coordinate
         LDA Sprite2_X,X        ; Get X Coordinate
         STA L2BE3,Y            ; Save sprite X coordinate
; ---------------------------------------------------
;               Update Sprites Bank 3
; ---------------------------------------------------
         LDA Sprite3_Prop,X     ; Get Properties byte
         STA L3800,Y            ; Save sprite properties
         LDA Sprite3_Char,X     ; Get Character number
         STA L3801,Y            ; Save sprite character number
         LDA Sprite3_Y,X        ; Get Y Coordinate
         STA L3802,Y            ; Save sprite Y coordinate
         LDA Sprite3_X,X        ; Get X Coordinate
         STA L3803,Y            ; Save sprite X coordinate
; ---------------------------------------------------
;               Update Sprites Bank 4
; ---------------------------------------------------
         LDA Sprite4_Prop,X     ; Get Properties byte
         STA L3BE0,Y            ; Save sprite properties
         LDA Sprite4_Char,X     ; Get Character number
         STA L3BE1,Y            ; Save sprite character number
         LDA Sprite4_Y,X        ; Get Y Coordinate
         STA L3BE2,Y            ; Save sprite Y coordinate
         LDA Sprite4_X,X        ; Get X Coordinate
         STA L3BE3,Y            ; Save sprite X coordinate
         DEX                    ; Go to next sprite
         BPL L47D2              ; Loop back until all 7 sprites per bank are updated
         RTS                    ; Return

;----------------------------------------------------
;                Sprite Indexes
;----------------------------------------------------
L4829:   .BYTE $00,$04,$08,$0C,$10,$14,$18,$1C




;----------------------------------------------------
;             Print Text Message
;----------------------------------------------------
;           X,Y = Message Address
;----------------------------------------------------
L4831:   STX ZRAM_01            ;
         STY ZRAM_02            ;
L4835:   LDY #$00               ;
         STY TRAMSZ             ;
         LDA (ZRAM_01),Y         ;
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
L4842:   INY
L4843:   LDA (ZRAM_01),Y
         CMP #$FF
         BEQ L488B
         CMP #$FE
         BEQ L4868
         CMP #$FD
         BEQ L4878
         STY RAMLO+1
         LDY TRAMSZ
         STA (ZRAM_02+1),Y
         INC TRAMSZ
         LDY RAMLO+1
         LDA VTIMR2+1
         BEQ L4842
         LDX #$0A
         JSR L4BD2              ; Run Programmable Delay
         JMP L4842
L4868:   INY
         CLC
         TYA
         ADC ZRAM_01
         STA ZRAM_01
         LDA ZRAM_02
         ADC #$00
         STA ZRAM_02
         JMP L4835
L4878:   CLC
         LDA ZRAM_02+1
         ADC #$20
         STA ZRAM_02+1
         LDA RAMLO
         ADC #$00
         STA RAMLO
         LDA #$00
         STA TRAMSZ
         BEQ L4842
L488B:   LDA #$00
         STA VTIMR2+1
         RTS                    ; Return




;----------------------------------------------------
;               Handle FIRE Button
;----------------------------------------------------
L4891:   LDA DMASK              ; Check fire function
         CMP #$01               ; Is fire allowed on this level?
         BNE L4899              ; Yes, so go check fire button
         RTS                    ; Not allowed so just leave

L4899:   LDA L00D2              ; Is another shot ready?
         BEQ L48A0              ; Yes, so allow fire button
         JMP L4948              ; No shot available, so skip ahead
L48A0:   LDA VTIMR4+1
         BEQ L48CC
         LDA L1002              ; Get DIP bank 2
         AND #$18               ; Check Bonus Threshold
         BNE L48B4              ; Bonus available so skip ahead
;----------------------------------------------------
;           No Bonus, do Continuous Fire 
;----------------------------------------------------          
         LDA L1000              ; Get Joystick input
         EOR #$40               ; Automatically press S1 for continuous fire
         JMP L48B9              ; Skip ahead
L48B4:   LDA L1000              ; Get Joystick input
         EOR L00D3              ; Compare to Joystick previous state
L48B9:   AND #$40               ; Check if S1 switch has changed state
         BEQ L48C4              ; No change in state of switch so skip ahead
;----------------------------------------------------
;          Fire Button has changed state
;----------------------------------------------------
         LDA L1000              ; Get Joystick input
         AND #$40               ; Check S1 switch
         BEQ L48CC              ; Button has been pressed so skip ahead
L48C4:   LDA L1000              ; Get Joystick input
         STA L00D3              ; Move current Joystick state to previous state
         JMP L4948              ; Skip fire button
;----------------------------------------------------
;         Fire Button has been pressed
;----------------------------------------------------          
L48CC:   LDA L1000              ; Get Joystick input
         STA L00D3              ; Move current Joystick state to previous state
         LDA VNTP+1
         AND #$F0
         BNE L4948
         LDA L1002              ; Get DIP bank 2
         AND #$18               ; Check Bonus Threshold
         BNE L48E5              ; Bonus available so skip ahead
         LDA #$18               ; Fire Repeat Time = 24 (slow)
         STA L00D1              ; Save Fire Repeat Time
         JMP L48E9              ; Skip ahead
L48E5:   LDA #$0A               ; Fire Repeat Time = 10 (fast)
         STA L00D1              ; Save Fire Repeat Time
L48E9:   LDY #$00               ;
         LDX #$05               ;
L48ED:   LDA L009C,X            ;
         AND #$F0               ;
         BNE L48F4              ;
         INY                    ;
L48F4:   DEX
         BPL L48ED
         LDX L4B9B,Y
         LDA #$00
         STA POKMSK
L48FE:   LDA DAUX1,X
         CMP #$FF
         BNE L490E
         LDA DDEVIC,X
         CMP #$FF
         BNE L490E
         INC POKMSK
L490E:   DEX
         BPL L48FE
         CPY POKMSK
         BCS L4948
         JSR L4A98
         LDA #$18
         STA DDEVIC,X
         STA DAUX1,X
         LDA #$02
         STA HATABS+4,X
         CLC                    ; Clear carry
         LDA BegaXPos1L         ; Get Bega's X position
         ADC #$0C               ; Add 12 to that
         STA TEMP2,X            ; Save Joe's position
         LDA #$01
         STA L00D2
         LDY #$05
L4933:   LDA L009C,Y
         AND #$F0
         BNE L493D
         JSR L4A5F              ; add joe and luna?
L493D:   DEY
         BPL L4933
         JSR L4AB0              ;
         LDA #$01               ;
         JSR L51CE              ; Play Sound

;----------------------------------------------------
;             Check Fire Repeat Timer
;----------------------------------------------------
L4948:   LDA L00D1              ; Get fire repeat timer
         BEQ L4954              ; Timer not running so skip ahead
         DEC L00D1              ; Decrement Fire Repeat Timer
         BNE L4954              ; Fire Repeat Timer not expired yet, so skip ahead
         LDA #$00               ; Timer expired, allow shot
         STA L00D2              ; Allow new shot
L4954:   LDA #$09
         STA POKMSK
L4958:   LDX POKMSK
         LDA DAUX1,X
         CMP #$FF
         BNE L4964
         JMP L4A57
L4964:   LDA DDEVIC,X
         BPL L496C
         JMP L49D8
L496C:   ASL A
         TAY
         LDA L4AC8,Y
         STA ZRAM_01
         LDA L4AC9,Y
         STA ZRAM_02
         LDA HATABS+4,X
         ASL A
         TAY
         LDA L4AFE,Y
         STA ZRAM_02+1
         LDA L4AFF,Y
         STA RAMLO
         LDY DDEVIC,X
         CLC
         LDA (ZRAM_02+1),Y
         ADC TEMP2,X
         STA APPMHI+1
         AND #$F8
         LSR A
         LSR A
         LSR A
         CLC
         ADC ZRAM_01
         STA ZRAM_01
         LDA #$00
         ADC ZRAM_02
         STA ZRAM_02
         LDA DDEVIC,X
         CMP #$18
         BNE L49AD
         LDA #$02
         BNE L49AF
L49AD:   AND #$01
L49AF:   ASL A
         TAY
         LDA L4B55,Y
         STA ZRAM_02+1
         LDA L4B56,Y
         STA RAMLO
         DEC DDEVIC,X
         LDA APPMHI+1
         AND #$07
         ASL A
         TAY
         TAX
         LDA (ZRAM_02+1),Y
         LDY #$00
         STA (ZRAM_01),Y
         TXA
         TAY
         INY
         LDA (ZRAM_02+1),Y
         CMP #$FF
         BEQ L49D8
         LDY #$01
         STA (ZRAM_01),Y
L49D8:   LDX POKMSK
         LDA DDEVIC,X
         BPL L49F6
         CMP #$FF
         BEQ L4A04
         AND #$7F
         SEC
         SBC DAUX1,X
         BNE L4A04
         LDA #$FF
         STA DAUX1,X
         STA DDEVIC,X
         JMP L4A57
L49F6:   SEC
         LDA DAUX1,X
         SBC DDEVIC,X
         CMP #$0B
         BCS L4A04
         JMP L4A57
L4A04:   LDA DAUX1,X
         ASL A
         TAY
         LDA L4AC8,Y
         STA ZRAM_01
         LDA L4AC9,Y
         STA ZRAM_02
         LDA HATABS+4,X
         ASL A
         TAY
         LDA L4AFE,Y
         STA ZRAM_02+1
         LDA L4AFF,Y
         STA RAMLO
         LDY DAUX1,X
         CLC
         LDA (ZRAM_02+1),Y
         ADC TEMP2,X
         STA APPMHI+1
         AND #$F8
         LSR A
         LSR A
         LSR A
         CLC
         ADC ZRAM_01
         STA ZRAM_01
         LDA #$00
         ADC ZRAM_02
         STA ZRAM_02
         DEC DAUX1,X
         LDA APPMHI+1
         AND #$07
         ASL A
         TAX
         LDA L4B8B,X
         LDY #$00
         STA (ZRAM_01),Y
         LDA L4B8C,X
         CMP #$FF
         BEQ L4A57
         INY
         STA (ZRAM_01),Y
L4A57:   DEC POKMSK
         BMI L4A5E
         JMP L4958
L4A5E:   RTS                    ; Return




;----------------------------------------------------
;          add joe and luna?
;----------------------------------------------------
L4A5F:   TYA
         PHA
         JSR L4A98
         LDA #$18
         STA DDEVIC,X
         STA DAUX1,X
         LDA L009C,Y
         AND #$02
         LSR A
         STA HATABS+4,X
         TXA
         PHA
         LDX #$00
         CPY L0093
         BEQ L4A88
         INX
         CPY L0094
         BEQ L4A88
         INX
         CPY POKADR
         BEQ L4A88
         INX
L4A88:   CLC
         LDA L4BA0,X
         ADC BegaXPos1L
         TAY
         PLA
         TAX
         TYA
         STA TEMP2,X
         PLA
         TAY
         RTS
L4A98:   LDX #$00
L4A9A:   LDA DAUX1,X
         CMP #$FF
         BNE L4AAA
         LDA DDEVIC,X
         CMP #$FF
         BNE L4AAA
         BEQ L4AAF
L4AAA:   INX
         CPX #$0A
         BNE L4A9A
L4AAF:   RTS
L4AB0:   LDA #$13
         ORA VNTP+1
         STA VNTP+1
         LDX #$05
L4AB8:   LDA L009C,X
         AND #$F0
         BNE L4AC4
         LDA #$10
         ORA L009C,X
         STA L009C,X
L4AC4:   DEX
         BPL L4AB8
         RTS
L4AC8:   JSR L4028
         PLP
         RTS
         PLP
         .BYTE $80
         PLP
         LDY #$28
         CPY #$28
         CPX #$28
         BRK
         AND #$20
         AND #$40
         AND #$60
         AND #$80
         AND #$A0
         AND #$C0
         AND #$E0
         AND #$00
         ROL A
         JSR L402A
         ROL A
         RTS
         ROL A
         .BYTE $80
         ROL A
         LDY #$2A
         CPY #$2A
         CPX #$2A
         BRK
         .BYTE $2B
         JSR L402B
         .BYTE $2B
         RTS
         .BYTE $2B
L4AFE:   .BYTE $04
L4AFF:   .BYTE $4B,$1F,$4B
L4B02:   .BYTE $3A,$4B
         PHP
         PHP
         PHP
         .BYTE $07,$07
         ASL TRAMSZ
         ORA RAMLO
         .BYTE $04,$03,$02
         ORA (L0000,X)
         BRK
         .BYTE $FF,$FF
         INC LFDFE,X
         SBC LFEFE,X
         .BYTE $FF
         BRK
         BRK
         BRK
         SED
         SED
         SED
         SBC LFAF9,Y
         .BYTE $FA,$FB,$FC,$FC
         SBC LFFFE,X
         BRK
         BRK
         ORA (ZRAM_01,X)
         .BYTE $02,$02,$03,$03
         .BYTE $02,$02
         ORA (L0000,X)
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
L4B55:   .BYTE $5B
L4B56:   .BYTE $4B,$6B,$4B,$7B
         .BYTE $4B
         ORA (FPTR2+1),Y
         .BYTE $13,$14
         ORA BUFADR+1,X
         .BYTE $17
         CLC
         ORA L1B1A,Y
         .BYTE $1C
         ORA L2F1E,X
         JSR LFF21
         .BYTE $23
         BIT ICBAHZ
         ROL ICPTHZ
         PLP
         AND #$2A
         .BYTE $2B
         BIT L2E2D
         .BYTE $3F
         BMI L4B7D
         .BYTE $FF
L4B7D:   .BYTE $03,$04
         ORA TRAMSZ
         .BYTE $07
         PHP
         ORA #$0A
         .BYTE $0B,$0C
         ORA L0F0E
         BPL L4B8C
L4B8C:   .BYTE $FF
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK

;----------------------------------------------------
;              unknown data
;----------------------------------------------------
L4B9B:   .BYTE $01,$03,$05,$07,$09
         .BYTE $F4,$24,$E4,$34


;----------------------------------------------------
;              erase screen?   
;----------------------------------------------------
L4BA4:   LDX #$06               ;
         LDA #$00               ;
L4BA8:   STA L3820,X            ;
         STA L3840,X            ;
         STA L3860,X            ;
         STA L3880,X            ;
         STA L38A0,X            ;
         STA L3839,X            ;
         STA L3859,X            ;
         STA L3879,X            ;
         STA L3899,X            ;
         LDY NumPlayers         ;
         BEQ L4BCE              ;
         STA L38C0,X            ;
         STA L38E0,X            ;
L4BCE:   DEX                    ;
         BPL L4BA8              ;
         RTS                    ;


;----------------------------------------------------
;              Programmable Delay
;----------------------------------------------------
;      X = Delay Count  Y = Y location
;----------------------------------------------------
L4BD2:   LDA L1003              ; Check Verticle Blanking
         BPL L4BD2              ; Wait here for the Verticle Blanking
;----------------------------------------------------
;          Check if game has been tilted
;----------------------------------------------------
         LDA L1000              ; Get Joystick input
         AND L1000              ; Was TILT alarmed?
         BMI L4BE2              ; No, so skip ahead
         JMP L583D              ; Machine has tilted, print error
L4BE2:   LDA VTIMR4
         BEQ L4BEB
         TXA                    ; A = X location
         PHA                    ; Push A
         PLA                    ; Pull A
         TAX                    ; A = X
L4BEB:   LDA L1003              ; Wait for serial comm
         BMI L4BEB
         JSR L51A4
         DEX
         BNE L4BD2
         RTS                    ; Return



;----------------------------------------------------
;            Print BEGA and LUNA lives
;----------------------------------------------------
;              Print "BEGA"
;----------------------------------------------------
L4BF7:   LDX #$0C               ; Char = 'B'
         STX L3859              ; Print character
         LDX #$0F               ; Char = 'E'
         STX L385A              ; Print character
         LDX #$11               ; Char = 'G'
         STX L385B              ; Print character
         LDX #$0B               ; Char = 'A'
         STX L385C              ; Print character
;----------------------------------------------------
;              Print "LUNA"
;----------------------------------------------------
         LDX #$16               ; Char = 'L'
         STX L3879              ; Print character
         LDX #$1F               ; Char = 'U'
         STX L387A              ; Print character
         LDX #$18               ; Char = 'N'
         STX L387B              ; Print character
         LDX #$0B               ; Char = 'A'
         STX L387C              ; Print character
;----------------------------------------------------
;              Print  "STAGE"
;----------------------------------------------------
         LDX #$6C               ; Char = STAGE
         STX L3899              ; Print character
         INX                    ; Char = STAGE
         STX L389A              ; Print character
         INX                    ; Char = STAGE
         STX L389B              ; Print character
         INX                    ; Char = STAGE
         STX L389C              ; Print character
;----------------------------------------------------
;                   Print Lives
;----------------------------------------------------
         LDY PlayerNumber       ; Which player is playing
         LDA Player1Lives,Y     ; Get that player's lives
         CLC                    ; Clear carry
         ADC #$02               ; Create number character
         STA L385E              ; Display number of lives
;----------------------------------------------------
;                    Print Lunas
;---------------------------------------------------
         LDA Lunas              ; Get number of Lunas
         CLC                    ; Clear carry
         ADC #$02               ; Create number character
         STA L387E              ; Display number of Lunas
;----------------------------------------------------
;            Print STAGE Number to screen
;---------------------------------------------------
         LDA Level              ; Get Level Number (0-49)
         ASL A                  ; Multiply by 2
         TAX                    ; Index to level
         LDA LB200,X            ; Get Level address
         STA ZRAM_01            ; Save address
         LDA LB201,X            ; Get Level address
         STA ZRAM_02            ; Save address
         LDY VVBLKD             ; Y = 0,1,2
         LDA (ZRAM_01),Y        ; Get STAGE Number
         TAY                    ; Temporarily save STAGE number
         AND #$0F               ; Get one's place digit
         CLC                    ; Clear carry
         ADC #$01               ; Make printable character
         STA L389E              ; Print STAGE Number (one's place)
         TYA                    ; Retrieve STAGE number
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         CLC                    ; Clear carry
         ADC #$01               ; Make printable character
         STA L389D              ; Print STAGE Number (tens place)
         RTS                    ; Return
 

;----------------------------------------------------
;                Text Messages
;----------------------------------------------------
L4C6D:   .WORD $3B84  ; location on screen     
         .BYTE $68,$00,$0E,$0B,$1E,$0B,$00,$0F,$0B,$1D,$1E,$00,$1F,$1D
         .BYTE $0B,$00,$13,$18,$0D,$00,$02,$0A,$09,$04,$FF ; "(C) DATA EAST USA INC 1983"

L4C88:   .WORD $C2C3  ; location on screen  
         .BYTE $0C,$19,$18,$1F,$1D,$00,$00,$00,$10,$19,$1C  ; "BONUS   FOR EVER      PTS"
         .BYTE $00,$0F,$20,$0F,$1C,$23,$00,$00,$00,$00,$00,$00
         .BYTE $1A,$1E,$1D,$FE

         .WORD $C2A8  ; location on screen
         .BYTE $1C,$1D,$FE ; "RS"

         .WORD $C2C8  ; location on screen
         .BYTE $1E,$1F,$FE  ; "TU"

         .WORD $C6A8  ; location on screen
         .BYTE $01,$01,$FE  ; "00"

         .WORD $C6C8  ; location on screen
         .BYTE $01,$01,$FF  ; "00"

L4CB9:   .WORD $3888  ; location on screen
         .BYTE $1E,$19,$0E,$0B,$23,$1D,$00,$11,$1C,$0F,$0B,$1E,$0F,$1D,$1E,$FE ; "TODAYS GREATEST"

         .BYTE $C5,$38,$02
         .BYTE $FE,$06,$39,$03
         .BYTE $FE,$47,$39,$04
         .BYTE $FE,$88,$39,$05
         .BYTE $FE,$C9,$39,$06
         .BYTE $FE,$0A,$3A,$07,$FE

         .WORD $3A4B
         .BYTE $08,$FE

         .WORD $3A8C
         .BYTE $09,$FF

L4CEB:   .WORD $3AC2  ; location on screen
         .BYTE $11,$1C,$0F,$0B,$1E,$00,$1A,$16,$0B,$23,$0F,$1C,$65,$FE  ; "GREAT PLAYER!"

L4CFB:   .WORD $3B02  ; location on screen
         .BYTE $1C,$0F,$11,$13,$1D,$1E,$0F,$1C,$00,$23,$19,$1F,$1C,$00,$13  ; "REGISTER YOUR INITIAL BELOW"
         .BYTE $18,$13,$1E,$13,$0B,$16,$00,$0C,$0F,$16,$19,$21,$FE

L4D19:   .WORD $3B4A  ; location on screen
         .BYTE $25,$00,$0B,$25,$0B,$25,$0B,$FF  ; "- A-A-A"

L4D23:   .WORD $398C  ; location on screen
         .BYTE $1D,$1E,$0B,$11,$0F,$FE  ; "STAGE"

L4D2B:   .WORD $3A0B  ; location on screen
         .BYTE $11,$0B,$17,$0F,$00,$1C,$0F,$0B,$0E,$23,$FF  ; "GAME READY"

L4D38:   .WORD $398C  ; location on screen
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$FE  ; "        "

L4D43:   .WORD $3A0B  ; location on screen
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FF  ; "          "

L4D50:   .WORD $390C  ; location on screen
         .BYTE $1A,$16,$0B,$23,$0F,$1C,$FF  ; "PLAYER"

L4D59:   .WORD $390C  ; location on screen
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$FF  ; "        "
        


; ---------------------------------------------------
;    Interrupt routine 1 - Check for Coins
; ---------------------------------------------------
L4D64:   PHA                    ; Save A Register
         TXA                    ; Move X
         PHA                    ; Save X Register
         TYA                    ; Move Y
         PHA                    ; Save Y Register
         CLD                    ; Clear Decimal Mode
         LDA VDSLST             ; In Diagnostic Mode?
         BEQ L4DE3              ; Yes, so leave interrupt
         JSR L4E18              ; Delay for Xms
         LDA L1001              ; Get inputs
         AND #$C0               ; Check coin input
         BEQ L4DE3              ; No coins so leave interrupt
; ---------------------------------------------------
;             Debounce Coin Slot Inputs
; ---------------------------------------------------
         STA VINTER+1           ; Mark which slot has coin
         JSR L4E18              ; Delay for Xms
         LDA L1001              ; Get Coin Input
         AND VINTER+1           ; Is Coin Input still active?
         BEQ L4DE3              ; No, so leave
         JSR L4E18              ; Delay for Xms
         LDA L1001              ; Get Coin Input
         AND VINTER+1           ; Is Coin Input still active?
         BEQ L4DE3              ; No, so leave
         JSR L4E18              ; Delay for Xms
         LDA L1001              ; Get Coin Input
         AND VINTER+1           ; Is Coin Input still active?
         BEQ L4DE3              ; No, so leave
; ---------------------------------------------------
;  Coin registered successfully, Check Coin(s)/Credit
; ---------------------------------------------------
         INC CoinCount          ; Increment coin count
         LDA #$19               ; Coin Count sound
         STA L1004              ; Send to sound chip
         LDY #$00               ; Y = 0
         LDX #$00               ; X = 0
         LDA L1001              ; Get Button inputs
         EOR #$FF               ; Toggle data
         AND #$0F               ; Isolate DIP Switch 1
         ASL VINTER+1           ; Which chute has coin?
         BCC L4DB7              ; Right coin chute, so skip ahead
         LSR A                  ; Isolate DIP1 sw3-4, left coin chute
         LSR A                  ; Isolate DIP1 sw3-4
L4DB7:   AND #$03               ; 1 Coin/1 Credit?
         BEQ L4DEC              ; Yes, so skip ahead
         INX                    ; X = 1
         CMP #$02               ; 1 Coins/2 Credits?
         BEQ L4DEC              ; Yes, so skip ahead
         INX                    ; X = 2
         CMP #$01               ; 3 Coins/1 Credit?
         BEQ L4DD3              ; Yes, so check for 3 coins
; ---------------------------------------------------
;                  2 Coins/1 Credit
; ---------------------------------------------------
         INX                    ; X = 3
         LDA CoinCount          ; Get number of coins
         CMP #$02               ; Do we have at least 2 coin?
         BCC L4DE3              ; No, so leave
         DEC CoinCount          ; Subtract 1 Coin
         JMP L4DEC              ; Award Credit
; ---------------------------------------------------
;                  3 Coins/1 Credit
; ---------------------------------------------------
L4DD3:   LDA CoinCount          ; Get number of coins?
         CMP #$03               ; Do we have at least 3 coins?
         BCC L4DE3              ; No, so leave
         DEC CoinCount          ; Subtract 1 Coin
         DEC CoinCount          ; Subtract 1 Coin
         JMP L4DEC              ; Award Credit
; ---------------------------------------------------
;            Leave Coin Debounce Interrupt
; ---------------------------------------------------
L4DE3:   STA L1001              ; Clear coin slot input
         PLA                    ; Restore Y Register
         TAY                    ; Move Y
         PLA                    ; Restore X Register
         TAX                    ; Move X
         PLA                    ; Restore A Register
         RTI                    ; Return from interrupt

; ---------------------------------------------------
;                  Award Credit
; ---------------------------------------------------
;     X = 0,1,2,3  Coin scenario
; ---------------------------------------------------
L4DEC:   LDA Credits            ; Get Credits
         SED                    ; Set Decimal Mode
         CLC                    ; Clear Carry
         ADC L4E2F,X            ; Award proper credits
         CMP L4E37,Y            ; More than 9 credits?
         BCC L4E01              ; Yes, so 9 is the max
         LDA #$00               ; A = 0 (coin blocker)
         STA L1000              ; Clear coin inputs
         LDA L4E37,Y            ; Get max number of credits
L4E01:   STA Credits            ; Save credits
         CLD                    ; Clear Decimal Mode
         DEC CoinCount          ; Subtract 1 coin
         LDA VTIMR4+1           ; Get Timer4
         ORA VTIMR4             ; Has Timer4 expired?
         BNE L4DE3              ; No, just leave 
         LDA #$01               ; 1 = Credit has been awarded
         STA VTIMR4             ; Notify Main Loop that credit has been awarded
         JMP L4DE3              ; Leave Interrupt


;----------------------------------------------------
;                Delay for Xms
;----------------------------------------------------
L4E18:   LDX #$80               ; Delay = 128
L4E1A:   LDA (L0000,X)          ; Waste time
         LDA (L0000,X)          ; Waste time
         LDA (L0000,X)          ; Waste time
         DEX                    ; Decrement delay
         BNE L4E1A              ; Delay not finished so wait
         RTS                    ; Return when done

;----------------------------------------------------
;               Delay -- unused
;----------------------------------------------------
         LDX #$E7               ; Delay = 231
L4E26:   LDA FPTR2+1            ; Waste time
         LDA FPTR2+1            ; Waste time
         NOP                    ; Waste time
         DEX                    ; Decrement delay
         BNE L4E26              ; Delay not finished so wait
         RTS                    ; Return when done

;----------------------------------------------------
;                 Coin Data
;----------------------------------------------------
L4E2F:   .BYTE 1, 2, 1, 1      ; credits awarded
         .BYTE 6, 8, 3, 1

L4E37:   .BYTE 9, 9, 9, 9      ; Max credits
         .BYTE 9, 9, 9, 9                
                
                
;----------------------------------------------------
;    Give instructions wait for START BUTTON
;----------------------------------------------------                                
L4E3F:   JSR L4EF2              ; Display user instructions/credits
         JSR L4F78              ; Display number of credits on the screen
         LDA STICK1             ;
         CMP #$02               ;
         BNE L4E6D              ; Check for P1 or P2 selection
L4E4C:   JSR L719E              ; Screen Transition, Fill from outside in
         LDA #$2E               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
L4E54:   LDA STICK1
         CMP #$01
         BEQ L4E67
         CMP #$02
         BEQ L4E67
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L4E54
L4E67:   JSR L71D0
         JSR L71B8              ; Screen Transition, Clear from inside out
;----------------------------------------------------
;         Wait for Player1 or Player2 button            
;----------------------------------------------------
L4E6D:   LDA L1001              ; Get switch inputs
         AND #$20               ; Check Player1 button
         BEQ L4E8D              ; Player1 button pressed, skip ahead
         LDA L1001              ; Get switch inputs
         AND #$10               ; Check Player2 button
         BEQ L4EAB              ; Player2 button pressed, skip ahead
         JSR L4EF2              ; Display user instructions/credits
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         LDA STICK1             ;
         CMP #$02               ;
         BEQ L4E4C              ;
L4E8A:   JMP L4E6D              ; Loop back and wait for P1 or P2

;----------------------------------------------------
;                 Player1 button pressed                      
;----------------------------------------------------
L4E8D:   JSR L4FC1              ; Deduct 1 Credit
         LDA #$00               ; 1 player
         STA NumPlayers         ; Save number of players
         STA CDTMV3             ;
         STA VIMIRQ             ;
         LDA #$01               ;
         STA VTIMR4+1           ;
         LDA #$00               ;
         STA VTIMR4             ;
         JSR L459A              ; Erase Video Screen
         JMP L4ED8              ; Jump ahead and continue
;----------------------------------------------------
;                 Player2 button pressed                      
;----------------------------------------------------
L4EAB:   LDA Credits            ; Get Credits
         CMP #$02               ; Are there at least 2 credits?
         BCS L4EB5              ; Yes, so skip ahead
         JMP L4E8A              ; Not enough credits, go back
L4EB5:   JSR L4FC1              ; Deduct 1 Credit           
         JSR L4FC1              ; Deduct 1 Credit
         LDA #$01               ; 2 players
         STA NumPlayers         ; Save number of players
         LDA #$00
         STA CDTMV3
         STA CDTMV3_swap
         STA VIMIRQ
         LDA #$01
         STA VTIMR4+1
         LDA #$00
         STA VTIMR4
         JSR L459A              ; Erase Video Screen
;----------------------------------------------------
;              
;----------------------------------------------------
L4ED8:   LDX #$05               ;
         LDA #$00               ;
L4EDC:   STA Score,X           ;
         DEX                    ;
         BPL L4EDC              ;
         LDA #$03
         STA L1000
         LDX #$FF
         TXS
         LDA #$2E               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
         JMP L40A1              ; Go Start Game




;----------------------------------------------------
;        Update display with user instructions
;----------------------------------------------------
L4EF2:   LDA L3927              ; Check for message "PUSH START BUTTON"
         CMP L4F8F              ; Has message already been displayed?
         BEQ L4F2E              ; Yes, so skip ahead
         JSR L459A              ; Erase Video Screen
         JSR L71D0              ;
         JSR L71EC              ;
         JSR L719E              ; Screen Transition, Fill from outside in
         LDA #$2E               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
L4F0B:   LDA STICK1             ;
         CMP #$01               ;
         BEQ L4F1E              ;
         CMP #$02               ;
         BEQ L4F1E              ;
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         JMP L4F0B              ;
L4F1E:   JSR L71B8              ; Screen Transition, Clear from inside out
;----------------------------------------------------
;        Display "PUSH START BUTTON" on screen
;----------------------------------------------------
         LDY #$00               ; Loop = 0
L4F23:   LDA L4F8F,Y            ; Get Message character 
         STA L3927,Y            ; Write character to display
         INY                    ; Next character
         CPY #$11               ; Have we printed all characters?
         BNE L4F23              ; Loop until whole message is displayed
;----------------------------------------------------
;       Check credits for 1 or 2 player game
;----------------------------------------------------
L4F2E:   LDA Credits            ; Get number of credits
         CMP #$02               ; Are there at least 2 credits?
         BCS L4F4C              ; Yes, so skip ahead
;----------------------------------------------------
;        Display "ONLY 1 PLAYER" on screen
;----------------------------------------------------
L4F35:   LDA L3987              ; Check for message "ONLY 1 PLAYER"
         CMP L4FA0              ; Has message already been displayed?
         BEQ L4F6B              ; Yes, so skip ahead
         LDY #$00               ; Loop = 0
L4F3F:   LDA L4FA0,Y            ; Get Message character
         STA L3987,Y            ; Write character to display
         INY                    ; Next character
         CPY #$0D               ; Have we printed all characters?
         BNE L4F3F              ; Loop until whole message is displayed
         BEQ L4F6B              ; Jump ahead and display credits
L4F4C:   LDA L1002              ; Get DIP bank 2
         AND #$40               ; Check Sw7 Upright or Cockpit
         BEQ L4F56              ; Upright version, so allow 2 players
         JMP L4F35              ; Cockpit version, 1 player allowed
;----------------------------------------------------
;        Display "1 OR 2 PLAYERS" on screen
;----------------------------------------------------
L4F56:   LDA L3987              ; Check for message "1 OR 2 PLAYERS"
         CMP L4FAD              ; Has message already been displayed?
         BEQ L4F6B              ; Yes, so skip ahead
         LDY #$00               ; Loop = 0
L4F60:   LDA L4FAD,Y            ; Get Message character
         STA L3987,Y            ; Write character to display
         INY                    ; Next character
         CPY #$0E               ; Have we printed all characters?
         BNE L4F60              ; Loop until whole message is displayed
;----------------------------------------------------
;            Display "CREDIT" on screen
;----------------------------------------------------
L4F6B:   LDY #$00               ; Loop = 0
L4F6D:   LDA L4FBB,Y            ; Get Message character
         STA L39EB,Y            ; Write character to display
         INY                    ; Next character
         CPY #$06               ; Have we printed all characters?
         BNE L4F6D              ; Loop until whole message is displayed
;----------------------------------------------------
;       Display number of credits on screen
;----------------------------------------------------
L4F78:   LDA Credits            ; Get number of credits
         LSR A                  ; Go to ten's spot
         LSR A                  ; Go to ten's spot
         LSR A                  ; Go to ten's spot
         LSR A                  ; Go to ten's spot
         TAX                    ; A = Ten's credits
         INX                    ; Create character
         STX L39F2              ; Write to credit ten's spot

         LDA Credits            ; Get number of credits
         AND #$0F               ; Isolate one's spot
         TAX                    ; A = One's credits
         INX                    ; Create character
         STX L39F3              ; Write to credit one's spot
         RTS                    ; Return



;----------------------------------------------------
;     Text strings to screen  ASC(x + 54)
;----------------------------------------------------
L4F8F:   .BYTE $1A,$1F,$1D,$12,$00,$1D,$1E,$0B     ; "PUSH START BUTTON"  
         .BYTE $1C,$1E,$00,$0C,$1F,$1E,$1E,$19,$18

L4FA0:   .BYTE $19,$18,$16,$23,$00,$02,$00,$1A  ; "ONLY 1 PLAYER"
         .BYTE $16,$0B,$23,$0F,$1C              

L4FAD:   .BYTE $02,$00,$19,$1C,$00,$03,$00  ; "1 OR 2 PLAYERS"
         .BYTE $1A,$16,$0B,$23,$0F,$1C,$1D

L4FBB:   .BYTE $0D,$1C,$0F,$0E,$13,$1E   ; "CREDIT"




;----------------------------------------------------
;                  Deduct 1 Credit
;----------------------------------------------------
L4FC1:   SED                    ; Set Decimal Mode
         SEC                    ; Set Carry
         LDA Credits            ; Get Credits
         SBC #$01               ; Subtract 1 credit
         STA Credits            ; Save Credits
         CLD                    ; Clear Decimal Mode
         LDA #$03               ; reset
         STA L1000              ; reset
         RTS                    ; Return




; ---------------------------------------------------
;        Check for LDPlayer Errors (IRQ_Interrupt)
; ---------------------------------------------------
L4FD2:   PHA                    ; Save A Register
         TXA                    ; Move X
         PHA                    ; Save X Register
         TYA                    ; Move Y
         PHA                    ; Save Y Register
         CLD                    ; Clear Decimal Mode
         LDA PADDL7             ;
         BNE L4FE0              ;
         JMP L5102              ;
; ---------------------------------------------------
;      Setup timed loop to wait for receive data
; ---------------------------------------------------
L4FE0:   LDA #$00               ; Timer = 0
         STA RecTimeout         ; Reset Timeout 
         STA RecTimeout+1       ; Reset Timeout                      
L4FE8:   INC RecTimeout         ; Increment timer while waiting
         BNE L4FF5              ; Timer not rolled, skip ahead
         INC RecTimeout+1       ; Increment timer while waiting
         LDA RecTimeout+1       ; Check timer
         BEQ L4FFC              ; Timer timed out, get received byte
L4FF5:   LDA L1006              ; Get UART Status Register
         AND #$01               ; Check if Receive Data Reg full
         BEQ L4FE8              ; No data received, loop back 
; ---------------------------------------------------
;      Check if command was accepted (ERROR HEX)
; ---------------------------------------------------
L4FFC:   LDA L1007              ; Get serial data byte (LDP Status Byte)
         CMP #$02               ; Was response ERROR HEX ($02)?
         BNE L502F              ; No, so skip ahead and check other codes
         INC ErrorHEX           ; Count this as an error
         LDA ErrorHEX           ; Check how many errors we have had
         CMP #$04               ; Has command been rejected 4 times?
         BCC L501A              ; Not yet so skip ahead
; ---------------------------------------------------
;             Print LDP Error Message
; ---------------------------------------------------
L500D:   JSR L459A              ; Erase Video Screen
         LDX #$A7               ; Message = "LDP ERROR" ($58A7)
         LDY #$58               ; Message = "LDP ERROR" ($58A7)
         JSR L4831              ; Print Text Message
L5017:   JMP L5017              ; Stop here
; ---------------------------------------------------
;            
; ---------------------------------------------------
L501A:   LDA #$00
         STA HOLD3
         LDA #$01
         STA STICK0
         STA PADDL7
         LDA #$41
         STA L1007
         JMP L5102
; ---------------------------------------------------
;          Check for Not Acknowledged (NAK)
; ---------------------------------------------------
L502F:   CMP #$0B               ; Was response NOT ACKNOWLEDGE ($0B)?
         BNE L505D              ; No, so skip ahead
         INC ErrorNAK           ; Count number of times we get NAK
         LDA ErrorNAK           ; Check how many times we get error
         CMP #$04               ; Have we got 4 errors?
         BCC L5040              ; Not yet so skip ahead
         JMP L500D              ; Print "LDP ERROR"
; ---------------------------------------------------
;            
; ---------------------------------------------------
L5040:   LDA #$00
         STA HOLD3
         LDA STICK0
         BNE L504D
         JMP L5102
L504D:   LDA #$00
L504F:   STA STICK0
         LDA #$01
         STA PADDL7
         JSR L510E
         JMP L5102
; ---------------------------------------------------
;          Check for Acknowledged (ACK)            
; ---------------------------------------------------
L505D:   CMP #$0A               ; Was response ACKNOWLEDGE ($0A)?
         BNE L507E              ; No, so skip ahead
         LDA #$00
         STA HOLD3
         LDA STICK0
         BNE L506E
         JMP L5102
L506E:   LDA PADDL7
         CMP #$02
         BNE L5078
         JMP L5102
L5078:   JSR L510E
         JMP L5102
; ---------------------------------------------------
;          Check for SEARCH COMPLETE
; ---------------------------------------------------
L507E:   CMP #$01               ; Was response COMPLETION ($01)?
         BNE L5097              ; No, so skip ahead
         LDA PADDL7
         CMP #$02
         BNE L5102
         INC STICK1             ; Mark SEACH COMPLETE
         LDA #$01
         STA PADDL7
         JSR L510E
         JMP L5102
L5097:   CMP #$05
         BNE L50F2
         LDA PADDL7
         CMP #$02
         BNE L5102
         INC STICK1             ;
         LDA STICK1             ; kiwi
         CMP #$02
         BNE L50B7
         LDA #$01
         STA PADDL7
         JSR L510E
         JMP L5102
L50B7:   INC VKEYBD+1           ; Count number of times we get error
         LDA VKEYBD+1           ; Check how many times we get error
         CMP #$04               ; Have we got 4 errors?
         BCC L50CE              ; Not yet so skip ahead
; ---------------------------------------------------
;           Print LaserDisc Error Message
; ---------------------------------------------------
         JSR L459A              ; Erase Video Screen
         LDX #$9A               ; Message = "DISC ERROR" ($589A)
         LDY #$58               ; Message = "DISC ERROR" ($589A)
         JSR L4831              ; Print Text Message
L50CB:   JMP L50CB              ; Stop here
; ---------------------------------------------------
;            
; ---------------------------------------------------
L50CE:   LDX #$05
L50D0:   INC PTRIG2,X
         LDA PTRIG2,X
         CMP #$3A
         BNE L50E2
         LDA #$30
         STA PTRIG2,X
         DEX
         BPL L50D0
L50E2:   LDA #$01
         STA STICK0
         STA PADDL7
         LDA #$41
         STA L1007
         JMP L5102
L50F2:   INC STICK4
         LDA STICK4
         CMP #$04
         BCC L50FF
         JMP L500D              ; Print "LDP ERROR"
L50FF:   JMP L504D              ;
L5102:   LDA L1007              ;
         STA L1002              ;
         PLA                    ; Restore Y Register
         TAY                    ; Move Y
         PLA                    ; Restore X Register
         TAX                    ; Move X
         PLA                    ; Restore A Register
         RTI                    ; Return from Interrupt



;----------------------------------------------------
;       Send SEARCH and REPEAT data to LDPlayer
;----------------------------------------------------
L510E:   LDA HOLD3              ;
         BEQ L5134              ;
         LDY #$00               ;
L5115:   LDA HOLD3              ;
         BEQ L5134
         LDA L1003
         BPL L5115
L511F:   LDA L1003
         BMI L511F
         DEY
         BNE L5115
         LDX #$00               ; Loop = 0 bytes
         STX STICK0
         LDX #$01
         STX PADDL7
         JMP L5134
L5134:   LDX STICK0
         BNE L513E
         LDA #$00
         STA VKEYBD+1
L513E:   LDA PTRIG1,X           ; Get byte from BufferOut
         CMP #$FF               ; Is this the last byte?
         BNE L514D              ; No, so skip ahead
         LDA #$00
         STA PADDL7
         JMP L51A3              ; Last byte so leave
L514D:   INC STICK0
         INC HOLD3
         LDA #$00
         STA HATABS+24
         STA HATABS+25
L515B:   INC HATABS+24
         BNE L5168
         INC HATABS+25
         LDA HATABS+25
         BEQ L516F
L5168:   LDA L1006              ; Get UART Status Register
         AND #$02               ; Check if Xmit Data Reg Empty
         BEQ L515B              ; Register not empty, loop back
;----------------------------------------------------
;             Send out serial bytes
;----------------------------------------------------
L516F:   LDA PTRIG1,X           ;
         STA L1007              ;
         LDX #$03               ;
         CMP #$43               ;
         BEQ L51A0              ;
         INX
         CMP #$44
         BEQ L51A0
         CMP #$40
         BNE L51A3
         LDA PADDL7
         CMP #$01
L5189:   BEQ L51A3
         DEC PADDL7
         LDA PADDL7
         CMP #$02
         BNE L51A3
         LDA #$00
         STA SUBTMP
         STA SUBTMP+1
         JMP L51A3
L51A0:   STX PADDL7
L51A3:   RTS                    ; Finished so return



; ---------------------------------------------------
;            
; ---------------------------------------------------
L51A4:   LDA PADDL7             ;
         CMP #$02               ;
         BNE L51CD              ;
         CLC                    ;
         LDA SUBTMP             ;
         ADC #$01
         STA SUBTMP
         LDA SUBTMP+1
         ADC #$00               ;
         STA SUBTMP+1           ;
         CMP #$2F               ;
         BCC L51CD              ;
         LDA #$00               ;
         STA STICK0
         LDA #$01
         STA PADDL7
         JSR L510E
L51CD:   RTS



; ---------------------------------------------------
;                   Sound Loop
; ---------------------------------------------------
;        A = Sound Number     X = Duration?
; ---------------------------------------------------
L51CE:   STX VTIMR2             ;
         LDX VTIMR4+1           ;
         BEQ L51DD              ;
         TAX                    ;
         LDA L51E1,X            ;
         STA L1004              ;
L51DD:   LDX VTIMR2             ;
         RTS                    ; Return


; ---------------------------------------------------
;              Sound Note Data (LOW to HIGH)
; ---------------------------------------------------
L51E1:   .BYTE $00,$1A,$02,$03,$04,$05,$06,$07,$08,$FF,$0A,$0B
         .BYTE $0C,$0D,$0E,$0F,$10,$11,$12,$13,$14,$15,$16,$17
         .BYTE $18,$19,$1A,$1B,$1C,$1D,$1E,$1F,$20,$21,$22,$23
         .BYTE $24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F




; ---------------------------------------------------
;        Check if Player is "GREATEST PLAYER"
; ---------------------------------------------------
L5211:   JSR L5270              ;
         LDA Ranking            ; Get ranking
         CMP #$08               ; Is score in the top 8?
         BCS L526F              ; No, so leave
         JSR L529B              ; Insert Initials
         JSR L44A0              ;
         LDX #$EB               ; Message = "GREAT PLAYER!" ($4CEB)
         LDY #$4C               ; Message = "GREAT PLAYER!" ($4CEB) 
         JSR L4831              ; Print Text Message
         LDX Ranking
         LDA L528F,X
         STA Ranking
         LDA #$00
         STA RAMLO+1
L5235:   LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         LDA L1000              ; Read Inputs
         STA WARMST             ; Save Inputs to buffer
         AND #$0F               ; Isolate Joystick input
         STA L04BD              ; Save Joystick direction to buffer
         LDA WARMST             ; Check button inputs
         AND #$60               ; Isolate TELEPORT and BARRIER buttons
         LSR A                  ; Shift bits 
         STA WARMST             ; Save TELEPORT and BARRIER buttons
         LDA L1001              ; Get Button inputs
         AND #$30               ; Check Player1/2 Buttons
         ASL A                  ; Shift bits into place
         ASL A                  ; Shift bits into place
         ORA WARMST             ; Combine all buttons
         ORA L04BD              ; Combine button data with joystick direction data
         STA WARMST             ; Save combined Joystick data byte
         JSR L52F0              ; Debounce Joystick?
         LDA RAMLO+1            ; Get Joystick (debounced)
         AND #$7F               ; Isolate joystick data
         CMP #$03               ; Check if UP/DOWN pressed
         BCC L5235
         JSR L459A              ; Erase Video Screen
         JSR L4496              ; Print High Scores
         LDX #$7F               ;
         JSR L4BD2              ;
L526F:   RTS                    ; Return


; ---------------------------------------------------
;     Get Players Ranking in the "Greatest Players"
; ---------------------------------------------------
L5270:   LDX #$07               ;
L5272:   LDY L528F,X            ;
         SEC                    ;
         LDA ZRAM_01            ; Get player's score 0
         SBC LPENH,Y            ; Compare player's score to high scores
         LDA ZRAM_02            ; Get player's score 1
         SBC LPENV,Y            ; Compare player's score to high scores
         LDA ZRAM_02+1          ; Get player's score 2
         SBC BRKKY,Y            ; Compare player's score to high scores
         BCC L528A              ; Score is less than so put ranking here
         DEX
         BPL L5272
L528A:   INX                    ; Add 1 to ranking
         STX Ranking            ; Save Player ranking
         RTS                    ; Return

; ---------------------------------------------------
;              sprite indexes?
; ---------------------------------------------------
L528F:   .BYTE  2,  5,  8, 11, 14, 17
         .BYTE 20, 23, 26, 29, 32, 35


; ---------------------------------------------------
;     Insert New Player's Initials into High Scores
; ---------------------------------------------------
L529B:   LDX #$07               ;
L529D:   LDY L528F,X            ;
         CPX Ranking            ; 
         BCC L52D1              ;
         CPX #$FF               ;
         BEQ L52D1              ;
         LDA L0251,Y            ;
         STA L0254,Y
         LDA L0250,Y
         STA L0253,Y
         LDA L024F,Y
         STA L0252,Y
         LDA BRKKY,Y
         STA VPIRQ+1,Y
         LDA LPENV,Y
         STA VPIRQ,Y
         LDA LPENH,Y
         STA BRKKY+1,Y
         DEX
         JMP L529D
; ---------------------------------------------------
;         Copy Initials to correct spot
; ---------------------------------------------------
L52D1:   INX                    ;
         LDY L528F,X            ;
         LDA ZRAM_02+1          ;
         STA BRKKY,Y
         LDA ZRAM_02
         STA LPENV,Y
         LDA ZRAM_01
         STA LPENH,Y
         LDA #$25                ; Char = "-"
         STA L0251,Y             ; Print dash for initials
         STA L0250,Y             ; Print dash for initials
         STA L024F,Y             ; Print dash for initials
         RTS                     ; Return


;----------------------------------------------------
;               debounce?
;----------------------------------------------------
L52F0:   LDA RAMLO+1
         BMI L5308
         LDA #$10
         STA CMCMD
         LDA #$00
         STA TRAMSZ
         STA POKMSK
         LDA RAMLO+1
         ORA #$80
         STA RAMLO+1
         LDA #$FF
         STA DOSVEC
L5308:   ASL A
         TAX
         LDA L53A1,X
         STA DOSVEC+1
         LDA L53A2,X
         STA DOSINI
         LDA Ranking
         ASL A
         STA DOSINI+1
         ASL A
         CLC
         ADC DOSINI+1
         STX DOSINI+1
         CLC
         ADC DOSINI+1
         TAX
         LDA L53A7,X
         STA DOSINI+1
         LDA L53A8,X
         STA APPMHI
         INC POKMSK
         LDA POKMSK
         AND #$3F
         BNE L533A
         DEC CMCMD
         BEQ L5386
L533A:   LDA WARMST
         AND #$03
         EOR BOOT
         BEQ L5374
         LDA WARMST
         AND #$03
         STA BOOT
         CMP #$03
         BEQ L5374
         CMP #$01
         BEQ L535B
         CMP #$02
         BEQ L5357
         JMP L5374
L5357:   DEC TRAMSZ
         DEC TRAMSZ
L535B:   INC TRAMSZ
         LDX #$1A
         LDA TRAMSZ
         BMI L5369
         CMP #$1B
         BCC L536B
         LDX #$00
L5369:   STX TRAMSZ
L536B:   LDA TRAMSZ
         CLC
         ADC #$0B
         LDY #$00
         STA (DOSVEC+1),Y
L5374:   LDA WARMST
         AND #$20
         STA DOSINI
         EOR DOSVEC
         BEQ L539C
         LDA DOSVEC
         BNE L539C
         LDA DOSINI
         STA DOSVEC
L5386:   INC RAMLO+1
         LDY Ranking
         LDA TRAMSZ
         CLC
         ADC #$0B
         STA L024F,Y
         INC Ranking
         LDA RAMLO+1
         AND #$7F
         STA RAMLO+1
L539C:   LDA DOSINI
         STA DOSVEC
         RTS



;----------------------------------------------------
;           Screen Pointers for High Scores
;----------------------------------------------------
L53A1:   .WORD $3B4C,$3B4E,$3B50,$40C7,$40C8,$40C9,$4108
         .WORD $4109,$410A,$4149,$414A,$414B,$418A,$418B
         .WORD $418C,$41CB,$41CC,$41CD,$420C,$420D,$420E
         .WORD $424D,$424E,$424F,$428E,$428F,$4290



;----------------------------------------------------
;               Have we reached Bonus?
;----------------------------------------------------
L53D7:   STA ZRAM_01            ;
         TXA                    ;
         PHA                    ;
         LDA VTIMR4+1           ;
         BNE L53E3              ;
         JMP L5475              ;
L53E3:   LDX PlayerNumber
         LDY L54A5,X
         LDX ZRAM_01
         CLC
         SED
         LDA Score,Y
         ADC L5478,X
         STA Score,Y
         LDA Score1,Y
         ADC L5487,X
         STA Score1,Y
         LDA Score2,Y
         ADC L5496,X
         STA Score2,Y
         CLD                    ; Clear Decimal Mode
         LDX PlayerNumber       ; Get Player Number
         JSR L463A              ; Print Player's Score
         LDX PlayerNumber
         LDA L54A5,X
         TAX
         LDA LPENV
L5419:   CMP Score2,X
         BCC L5432
         BNE L5449
         LDA LPENH
         CMP Score1,X
         BCC L5432
         BNE L5449
         LDA SPARE
         CMP Score,X
         BCS L5449
L5432:   LDA Score,X
         STA SPARE
         LDA Score1,X
         STA LPENH
         LDA Score2,X
         STA LPENV
         LDX #$02
         JSR L463A              ; Print Score 
L5449:   LDA Bonus
         BEQ L5475
         LDX PlayerNumber
         LDY L54A5,X
         LDA Score2,Y
         CMP Bonus+1,X
         BCC L5475
;----------------------------------------------------
;               Have we reached Bonus?
;----------------------------------------------------
         CLC                    
         SED
         LDA Bonus+1,X
         ADC Bonus
         STA Bonus+1,X
         CLD                    ; Clear Decimal Mode
         LDA #$02
         JSR L51CE              ; Play Sound
         LDX PlayerNumber       ; Check which player is playing
         INC Player1Lives,X     ; Award bonus life
         JSR L4BF7              ; Print BEGA and LUNA lives
L5475:   PLA                    ; Get X Register
         TAX                    ; Restore X Register
         RTS                    ; Return



;----------------------------------------------------
;                 unknown data
;----------------------------------------------------
L5478:   BVC L547A
L547A:   BVC L547C
L547C:   BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
L5487:   BRK
         ORA (ZRAM_01,X)
         .BYTE $02,$04
         PHP
         ASL POKMSK,X
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
L5496:   BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
L54A5:   BRK
         .BYTE $03



;----------------------------------------------------
;                 Frame Number Setup
;----------------------------------------------------
;         A = Frame Number Index (0-46)
;----------------------------------------------------
L54A7:   STA ZRAM_02+1          ; Save Series Number (0-94)
         LDA #$00               ; Clear data
         STA STICK1             ; Clear ?
         STA STICK0             ; Clear ?
         STA ErrorHEX           ; Clear LDP Error Count
         STA ErrorNAK           ; Clear LDP Error Count
         STA STICK4             ; Clear ?
;----------------------------------------------------
;               Setup SEARCH Data
;----------------------------------------------------
L54BA:   LDA #$56               ; Command = CLEAR_ALL ($56)
         STA PTRIG1             ; Save command in BufferOut
         LDA #$43               ; Command = SEARCH ($43)    
         STA PTRIG2             ; Save command in BufferOut
         ASL ZRAM_02+1          ; Multiply index by 2
         LDX ZRAM_02+1          ; X = index
         LDA LCD12,X            ; Get Frame Number pointer
         STA ZRAM_01            ; Setup pointer
         LDA LCD13,X            ; Get Frame Number pointer
         STA ZRAM_02            ; Setup pointer
;----------------------------------------------------
;       Get Frame Number from Table ($027F-$0283)
;----------------------------------------------------
         LDY #$04               ; 5 digits to copy from table
L54D4:   LDA (ZRAM_01),Y        ; Get digit from table
         STA PTRIG3,Y           ; Save digit into $027F-$0283
         DEY                    ; Go to next digit in number
         BPL L54D4              ; Loop until all 5 digits copied
;----------------------------------------------------
;               Setup REPEAT Data
;----------------------------------------------------
         LDA #$40               ; Command = ENTER ($40)
         STA STRIG0             ; Save command in BufferOut
         LDA #$44               ; Command = REPEAT ($44)
         STA STRIG1             ; Save command in BufferOut
         LDX ZRAM_02+1          ; X = index
         LDA LCD70,X            ; Get Frame Number pointer
         STA ZRAM_01            ; Setup pointer
         LDA LCD71,X            ; Get Frame Number pointer
         STA ZRAM_02            ; Setup pointer
;----------------------------------------------------
;       Get Frame Number from Table ($0286-$028A)
;----------------------------------------------------
         LDY #$04               ; 5 digits to copy from table
L54F4:   LDA (ZRAM_01),Y        ; Get digit from table
         STA STRIG2,Y           ; Save digit into $0286-$028A
         DEY                    ; Go to next digit in number
         BPL L54F4              ; Loop until all 5 digits copied
;----------------------------------------------------
;                
;----------------------------------------------------
         LDA #$40               ; Command = ENTER ($40)
         STA STRIG3+4           ; Save command in BufferOut
         STA STRIG3+5           ; Save command in BufferOut
         LDA #$FF               ; FF = End Buffer Marker
         STA STRIG3+6           ; Mark End of BufferOut
         JSR L510E              ;
         LDA #$01               ;
         STA PADDL7             ;
         RTS                    ; Return



;----------------------------------------------------
;                  unused?
;----------------------------------------------------
         LDA #$00
         STA ZRAM_02
         STA ZRAM_02+1
         STA RAMLO
         LDY #$08
L551C:   LSR RAMLO+1
         BCC L552D
         CLC
         LDA ZRAM_02+1
         ADC ZRAM_01
         STA ZRAM_02+1
         LDA RAMLO
         ADC ZRAM_02
         STA RAMLO
L552D:   ASL ZRAM_01
         ROL ZRAM_02
         DEY
         BNE L551C
         RTS


;----------------------------------------------------
;              Update Scoring Overlay
;----------------------------------------------------
L5535:   JSR L4BA4              ;
         JSR L45C1              ; Print "HI 1UP/2UP" overlay screen
         LDA NumPlayers         ; How many players?
         BEQ L5545              ; One player so skip ahead
         LDX #$01               ; Setup printing of Player2 score
         JSR L463A              ; Print Player2 score
L5545:   LDX #$00               ; Setup printing of Player1 score
         JSR L463A              ; Print Player1 score
         LDX #$02               ; Setup printing of HI Score
         JSR L463A              ; Print HI Score
         JSR L4BF7              ; Print BEGA and LUNA lives
         RTS                    ; Return



;----------------------------------------------------
;         Print Number of Points scored (unused)
;----------------------------------------------------
         STX ZRAM_01            ;
         LDA L55C9,Y            ;
         TAY                    ;
         LDA PADDL4
         BNE L5565
         LDA #$06
         CLC
         ADC ZRAM_01
         BNE L5575
L5565:   CMP #$01
         BNE L5570
         LDA #$03
         CLC
         ADC ZRAM_01
         BNE L5575
L5570:   LDA #$00
         CLC
         ADC ZRAM_01
L5575:   CPX #$06
         BNE L55A1
         TAX
         LDA L55D2,X
         STA L29A3,Y
         INX
         LDA L55D2,X
         STA L29A4,Y
         INX
         LDA L55D2,X
         STA L29A5,Y
;----------------------------------------------------
;               Print "PTS" for points
;----------------------------------------------------
         LDA L55CF              ; Char = 'P'
         STA L29A6,Y            ; Print 'P'
         LDA L55D0              ; Char = 'T'
         STA L29A7,Y            ; Print 'T'
         LDA L55D1              ; Char = 'S'
         STA L29A8,Y            ; Print 'S'
         RTS                    ; Return
;----------------------------------------------------
;                
;----------------------------------------------------
L55A1:   TAX
         LDA L55D2,X
         STA L28A3,Y
         INX
         LDA L55D2,X
         STA L28A4,Y
         INX
         LDA L55D2,X
         STA L28A5,Y
         LDA L55CF
         STA L28A6,Y
         LDA L55D0
         STA L28A7,Y
         LDA L55D1
         STA L28A8,Y
         RTS                    ; Return



;----------------------------------------------------
;              unknown data
;----------------------------------------------------
L55C9:   .BYTE $02,$0B,$14,$94,$8B,$82
L55CF:   .BYTE $1A  ; 'P'
L55D0:   .BYTE $1E  ; 'T'
L55D1:   .BYTE $1D  ; 'S'
L55D2:   .BYTE $00,$06,$01,$02
         .BYTE $01,$01,$02,$06
         .BYTE $01,$03,$01,$01
         .BYTE $03,$06,$01



;----------------------------------------------------
;            Play Continue Play Feature
;----------------------------------------------------
L55E1:   LDA L04DA              ; Have we continued once yet?
         BEQ L55EC              ; No, so check for Continuous Play
         LDA #$00               ;
         STA L04DA              ;
         RTS                    ; Return

;----------------------------------------------------
;             Check for Continuous Play
;----------------------------------------------------
L55EC:   LDA L1002              ; Get DIP Bank 2
         AND #$04               ; Check Sw3, Continuous Play
         BEQ L55F4              ; Jump ahead for continuous play
         RTS                    ; No continue so leave
;----------------------------------------------------
;             Continuous Play Selected
;----------------------------------------------------
L55F4:   JSR L719E              ; Screen Transition, Fill from outside in
         LDA #$58               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
L55FC:   JSR L71B8              ; Screen Transition, Clear from inside out
         LDA STICK1             ;
         CMP #$01               ;
         BEQ L5612              ;
         CMP #$02               ;
         BEQ L5612              ;
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         JMP L55FC
L5612:   JSR L71D0
         JSR L71DE
         LDX #$00
         TXA
L561B:   STA L3A00,X
         STA L3B00,X
         STA L3E00,X
         STA L3F00,X
         DEX
         BNE L561B
         LDA #$08
         STA L04BD
         LDA #$3F
         STA SRTIMR
         LDA Credits            ; Get number of Credits
         BEQ L563C              ; No credits left so skip ahead
         JMP L569A
;----------------------------------------------------
;            Prompt to insert more coins
;----------------------------------------------------
L563C:   LDX SRTIMR
         LDA L04BD
         STA L3B10
         LDA #$00
         STA L3F10
L564A:   LDA L1003
         BPL L564A
         TXA
         PHA
         LDX #$E0               ; Message = "        " ($57E0)
         LDY #$57               ; Message = "        " ($57E0)
         AND #$30
         BEQ L565D
;----------------------------------------------------
;         "TO CONTINUE THE GAME AT
;          THE STAGE YOU LEFT OFF
;          INSERT COINS FOR MORE"
;----------------------------------------------------
         LDX #$4E               ; Message = "TO CONTINUE THE GAME AT" ($574E)
         LDY #$57               ; Message = "TO CONTINUE THE GAME AT" ($574E)
L565D:   JSR L4831              ; Print Text Message
         LDA Credits
         BEQ L5687
         JSR L56FD
         LDA L1002              ; Get DIP bank 2
         EOR #$FF               ; Toggle all bits
         AND #$03               ; Get DIP Sw1-2
         TAY                    ; Y = DIP Sw1-2
         LDA L4529,Y            ; Get initial Number of Lives from table
         LDY PlayerNumber       ; Find out which player is continuing
         STA Player1Lives,Y     ; Restore that players total lives
         LDA #$01
         STA CDTMV3
         JSR L4FC1              ; Deduct 1 Credit
         LDX #$FF
         TXS
         JMP L40B1
L5687:   PLA
         TAX
         DEX
         BPL L564A
         DEC L04BD
         LDA L04BD
         CMP #$01
         BNE L563C
         JSR L459A              ; Erase Video Screen
         RTS                    ; Return
;----------------------------------------------------
;         Prompt to press button to continue
;----------------------------------------------------
L569A:   JSR L4F2E              ;
L569D:   LDX SRTIMR             ;
         LDA L04BD              ;
         STA L3B10
         LDA #$00
         STA L3F10
L56AB:   LDA L1003
         BPL L56AB
         TXA
         PHA
         LDX #$E0               ; Message = "        " ($57E0)
         LDY #$57               ; Message = "        " ($57E0)
         AND #$30
         BEQ L56BE
;----------------------------------------------------
;         "TO CONTINUE THE GAME AT
;          THE STAGE YOU LEFT OFF
;          PUSH START BUTTON"
;----------------------------------------------------
         LDX #$99               ; Message = "TO CONTINUE THE GAME AT" ($5799)
         LDY #$57               ; Message = "TO CONTINUE THE GAME AT" ($5799)
L56BE:   JSR L4831              ; Print Text Message
         LDA L1001              ; Get Switch Inputs
         AND #$20               ; Check Player1 button
         BNE L56EA              ; Player1 button not pressed, skip ahead
         JSR L56FD
         LDA L1002              ; Get DIP bank 2
         EOR #$FF               ; Toggle all bits
         AND #$03               ; Get DIP Sw1-2
         TAY                    ; Y = DIP Sw1-2
         LDA L4529,Y            ; Get initial Number of Lives from table
         LDY PlayerNumber       ; Find out which player is continuing
         STA Player1Lives,Y     ; Restore that players total lives
         LDA #$01
         STA CDTMV3
         JSR L4FC1              ; Deduct 1 Credit
         LDX #$FF
         TXS
         JMP L40B1
L56EA:   PLA
         TAX
         DEX
         BPL L56AB
         DEC L04BD
         LDA L04BD
         CMP #$01
         BNE L569D
         JSR L459A              ; Erase Video Screen
         RTS
L56FD:   LDA PlayerNumber
         BEQ L5705
         JMP L5726
L5705:   JSR L459A              ; Erase Video Screen
;----------------------------------------------------
;              Copy Player's Score
;----------------------------------------------------
         LDA Score
         STA ZRAM_01
         LDA Score1
         STA ZRAM_02
         LDA Score2
         STA ZRAM_02+1
         JSR L5211
;----------------------------------------------------
;          Reset Player's Score to 00000
;----------------------------------------------------
         LDA #$00               ; Score = 0
         STA Score              ; Reset score
         STA Score1             ; Reset score
         STA Score2             ; Reset score
         RTS                    ; Return
;----------------------------------------------------
;              
;----------------------------------------------------
L5726:   JSR L459A              ; Erase Video Screen
         LDA SDLSTL
         STA ZRAM_01
         LDA SDLSTH
         STA ZRAM_02
         LDA SSKCTL
         STA ZRAM_02+1
         JSR L5211
         LDX #$00
         STX L1000
         NOP
         NOP
         LDA #$00
         STA SDLSTL
         STA SDLSTH
         STA SSKCTL
         RTS                    ; Return



;----------------------------------------------------
;               Continue Text Messages
;----------------------------------------------------
L574E:   .WORD $3A44
         .BYTE $1E,$19,$00,$0D,$19,$18,$1E,$13,$18,$1F,$0F,$00 ; "TO CONTINUE THE GAME AT"
         .BYTE $1E,$12,$0F,$00,$11,$0B,$17,$0F,$00,$0B,$1E,$FE
         
L5768:   .WORD $3A84
         .BYTE $1E,$12,$0F,$00,$1D,$1E,$0B,$11,$0F,$00,$23,$19 ; "THE STAGE YOU LEFT OFF"
         .BYTE $1F,$00,$16,$0F,$10,$1E,$00,$19,$10,$10,$FE
         
L5781:   .WORD $3AC5
         .BYTE $13,$18,$1D,$0F,$1C,$1E,$00,$0D,$19,$13,$18,$1D ; "INSERT COINS FOR MORE"
         .BYTE $00,$10,$19,$1C,$00,$17,$19,$1C,$0F,$FF
         
L5799:   .WORD $3A44
         .BYTE $1E,$19,$00,$0D,$19,$18,$1E,$13,$18,$1F,$0F,$00 ; "TO CONTINUE THE GAME AT"
         .BYTE $1E,$12,$0F,$00,$11,$0B,$17,$0F,$00,$0B,$1E,$FE
         
L57B3:   .WORD $3A84
         .BYTE $1E,$12,$0F,$00,$1D,$1E,$0B,$11,$0F,$00,$23,$19 ; "THE STAGE YOU LEFT OFF"
         .BYTE $1F,$00,$16,$0F,$10,$1E,$00,$19,$10,$10,$FE
         
L57CC:   .WORD $3AC5
         .BYTE $1A,$1F,$1D,$12,$00,$1D,$1E,$0B,$1C,$1E,$00,$0C ; "PUSH START BUTTON"
         .BYTE $1F,$1E,$1E,$19,$18,$FF
         
L57E0:   .WORD $3A44
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ; "        "
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
         .BYTE $00,$00,$00,$00,$00,$00,$FE
         
L5801:   .WORD $3A84
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ; "        "
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
         .BYTE $00,$00,$00,$FE
         
L581F:   .WORD $3AC5
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00 ; "        "
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
         .BYTE $00,$00,$00,$FF



;----------------------------------------------------
;              Machine has tilted
;----------------------------------------------------
L583D:   JSR L459A              ; Erase Video Screen
         LDA #$09               ;
         STA L1003              ;
         LDX #$B3               ; Message = "TILT" ($58B3)
         LDY #$58               ; Message = "TILT" ($58B3) 
         JSR L4831              ; Print Text Message
         LDX #$FF               ;
         JSR L588C              ;
         LDX #$80               ;
         JSR L588C              ;
         LDA Credits            ; Get number of Credits
         BEQ L5877              ; No credits left so skip ahead
         JSR L4FC1              ; Deduct 1 Credit
         LDA Credits            ; Get number of credits
         BEQ L5877              ; That was the last credit so skip ahead
         LDX #$01
         STX VTIMR4
         DEX
         STX VTIMR4+1
         STX L1004
         LDA #$08
         STA L1003
         JMP L4E3F              ; Give instructions wait for START BUTTON
;----------------------------------------------------
;      No credits left, return to Attract Mode
;----------------------------------------------------
L5877:   LDX #$00               ;
         STX VTIMR4             ;
         STX VTIMR4+1           ;
         STX L1004              ;
         DEX                    ;
         TXS                    ;
         LDA #$08               ;
         STA L1003              ;
         JMP L403C              ; Go back to Attract Mode


;----------------------------------------------------
;                unknown
;----------------------------------------------------
L588C:   LDA L1003              ;
         BPL L588C              ;
L5891:   LDA L1003              ;
         BMI L5891              ;
         DEX                    ;
         BPL L588C              ;
         RTS                    ; Return


;----------------------------------------------------
;              Error Text Messages
;----------------------------------------------------
L589A:   .WORD $398A  ; location on screen
         .BYTE $0E,$13,$1D,$0D,$00,$0F,$1C,$1C,$19,$1C,$FF  ; "DISC ERROR"

L58A7:   .WORD $396A  ; location on screen
         .BYTE $16,$0E,$1A,$00,$0F,$1C,$1C,$19,$1C,$FF  ; "LDP ERROR"

L58B3:   .WORD $39AD  ; location on screen
         .BYTE $1E,$13,$16,$1E,$FF  ; "TILT"



;----------------------------------------------------
;           Do Bega's Horizontal Movement
;----------------------------------------------------
L58BA:   LDA VNTP+1
         BPL L58C1
         JMP L5BA5              ; Do bega glow?
L58C1:   ASL A
         BPL L58C7
         JMP L59D1              ; Print STAGE Information Screen
L58C7:   LDA L1000              ; Get Joystick data
         EOR #$FF               ; Toggle all bits
         AND #$0C               ; Test only Left and Right
         BNE L58D3              ; If pressed then go check them
         JMP L5973              ; Neither are pressed so skip ahead
L58D3:   AND #$08               ; Test for Right joystick press
         BNE L592B              ; Right pressed so jump ahead
;----------------------------------------------------
;              Joystick LEFT pressed
;----------------------------------------------------
         LDA BegaXPos1L         ; Get Bega's X Position
         CMP #$10               ; Is it at far left?
         BCS L58E0              ; No, so continue
         JMP L5973              ; No left movement so leave
L58E0:   LDA L0093              ; Check attachments
         CMP #$FF               ; Is a person attached to Bega?
         BEQ L58FB              ; No so skip ahead
         LDA BegaXPos1L         ; Get Bega's X Position
         CMP #$18               ; Is it 24?
         BCS L58EF              ; Yes, so continue
         JMP L5973              ; Just leave
L58EF:   LDA POKADR
         CMP #$FF
         BEQ L58FB
         LDA BegaXPos1L
         CMP #$28
         BCC L5973
;----------------------------------------------------
;    Move 6 sprites that make up Bega to the LEFT
;----------------------------------------------------
L58FB:   LDX #$05               ; 6 sprites make up BEGA
L58FD:   LDA BegaXPos1L,X       ; Get sprite x position
         SEC                    ; Clear carry
         SBC L5C51              ; Subtract left movement
         STA BegaXPos1L,X       ; Save sprite x position
         DEX                    ; Next sprite in BEGA
         BPL L58FD              ; Loop until all sprites moved
         LDX #$05
L590A:   LDA L009C,X
         AND #$F0
         BNE L5916
         LDA #$FE
         AND L009C,X
         STA L009C,X
L5916:   DEX
         BPL L590A
         LDA VNTP+1
         AND #$30
         BEQ L5922
         JMP L5973
L5922:   LDA #$FD
         AND VNTP+1
         STA VNTP+1
         JMP L5973
;----------------------------------------------------
;            Joystick RIGHT Pressed
;----------------------------------------------------
L592B:   LDA BegaXPos1L         ; Get Bega's X Position
         CMP #$D1               ; Is it at far right?
         BCS L5962              ; Yes, so skip ahead
         LDA L0094              
         CMP #$FF
         BEQ L5949
         LDA BegaXPos1L
         CMP #$C9
         BCS L5962
         LDA POKADR+1           ;
         CMP #$FF               ;
         BEQ L5949              ;
         LDA BegaXPos1L         ;
         CMP #$B9               ;
         BCS L5962
;----------------------------------------------------
;   Move 6 sprites that make up Bega to the right
;----------------------------------------------------
L5949:   LDX #$05               ; 6 sprites make up BEGA
L594B:   LDA BegaXPos1L,X       ; Get sprite x position
         CLC                    ; Clear carry
         ADC L5C51              ; Add right movement
         STA BegaXPos1L,X       ; Save sprite x position
         DEX                    ; Next sprite in BEGA
         BPL L594B              ; Loop until all sprites moved
         LDA VNTP+1
         AND #$30
         BNE L5962
         LDA #$02
         ORA VNTP+1
         STA VNTP+1
L5962:   LDX #$05
L5964:   LDA L009C,X
         AND #$F0
L5968:   BNE L5970
         LDA #$01
         ORA L009C,X
         STA L009C,X
L5970:   DEX
         BPL L5964
;----------------------------------------------------
;            
;----------------------------------------------------
L5973:   LDA VNTP+1
         AND #$20
         BEQ L597C
         JMP L5B66
L597C:   LDA VNTP+1
         AND #$10
         BEQ L5985
         JMP L5B3C
L5985:   LDA VNTP+1
         AND #$02
         ASL A
         ORA #$01
         LDX #$05
L598E:   STA ICPTHZ,X
         DEX
         BPL L598E
         LDA VNTD
         BNE L59AD
         LDA #$02
         STA VNTD               ;
         LDA VNTP+1             ;
         AND #$01               ;
         EOR #$01               ;
         TAX                    ;
         LDA #$FE
         AND VNTP+1
         STA VNTP+1
         TXA
         ORA VNTP+1
         STA VNTP+1
L59AD:   LDA SysClock           ; Check System Clock
         AND #$03               ; Has it been 4ms?
         BNE L59B6              ; No, so skip ahead
         DEC VNTD
L59B6:   LDA VNTP+1             ; Do Glow Bega
         AND #$03
         ASL A
         TAY
         LDA L5C52,Y            ; Glow Bega
         STA ZRAM_01
         LDA L5C53,Y
         STA ZRAM_02
         LDY #$05               ; 6 sprites to update
L59C8:   LDA (ZRAM_01),Y        ; Get picture
         STA FMSZPG,Y           ; Set Bega sprite picture
         DEY                    ; Next picture
         BPL L59C8              ; Loop until all 6 sprites updated
         RTS                    ; Return



;----------------------------------------------------
;           Print STAGE Information Screen
;----------------------------------------------------
L59D1:   LDA VNTD               ;
         BNE L59D9              ;
         LDA #$01               ;
         STA VNTD               ;
L59D9:   LDA SysClock             ;
         AND #$01
         BNE L59E2
         DEC VNTD
L59E2:   BEQ L59E5
         RTS
L59E5:   LDA VNTP+1
         AND #$3F
         BNE L5A17
;----------------------------------------------------
;                 Add Y step increment
;----------------------------------------------------
         LDX #$05               ; 6 sprites to update
L59ED:   CLC                    ;
         LDA OLDADR+1,X         ;
         ADC L5C50
         STA OLDADR+1,X
         DEX
         BPL L59ED

         LDA OLDADR+1
         CMP #$B8
         BCS L59FF
         RTS
L59FF:   LDA #$06
         JSR L51CE              ; Play Sound
         LDX #$05
         LDY #$75
L5A08:   STY FMSZPG,X
         DEY
         DEX
         BPL L5A08
         LDA #$41
         STA VNTP+1
         LDA #$10
         STA VNTD
         RTS
L5A17:   LDX #$05               ; Loop = 6 sprites update
L5A19:   LDA L5C88,X            ; Get picture
         STA FMSZPG,X           ; Set Bega sprite picture
         DEX                    ; Next picture
         BPL L5A19              ; Loop until all 6 sprites updated
         LDA #$00
         STA VNTP+1
;----------------------------------------------------
;             Print STAGE Number to screen
;----------------------------------------------------
         LDX #$23               ; Message = "STAGE" ($4D23)
         LDY #$4D               ; Message = "STAGE" ($4D23)
         JSR L4831              ; Print Text Message
         LDA Level 
         ASL A
         TAX
         LDA LB200,X            ; Get Level address
         STA ZRAM_01            ; Save address
         LDA LB201,X            ; Get Level address
         STA ZRAM_02            ; Save address
         LDY VVBLKD             ; Y = 0,1,2
         LDA (ZRAM_01),Y        ; Get STAGE Number
         TAY                    ; Temporarily save STAGE number
         AND #$0F               ; Get one's place digit
         CLC                    ; Clear carry
         ADC #$01               ; Make printable character
         STA L3993              ; Print STAGE Number (one's place)
         TYA                    ; Retrieve STAGE number
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         CLC                    ; Clear carry
         ADC #$01               ; Make printable character
         STA L3992              ; Print STAGE Number (tens place)
         LDA NumPlayers
         BEQ L5A69
         LDX #$50               ; Message = "PLAYER" ($4D50)
         LDY #$4D               ; Message = "PLAYER" ($4D50)
         JSR L4831              ; Print Text Message
         CLC
         LDA PlayerNumber
         ADC #$02
         STA L3913
L5A69:   LDA CDTMA1+1
         BNE L5AC7
         LDX #$00
         STX SRTIMR
;----------------------------------------------------
;        Flash game play instructions for stage
;----------------------------------------------------
L5A73:   LDA STICK1             ; Get FlashText status
         CMP #$02
         BEQ L5A97
         LDX #$CB               ; Message = "" ($5DCB)
         LDY #$5D               ; Message = "" ($5DCB)
         INC SRTIMR
         LDA SRTIMR
         AND #$30
         BEQ L5A8C
         LDX #$F4               ; Message = "TOUCH TO RESCUE LUNA!" ($5CF4)
         LDY #$5C               ; Message = "TOUCH TO RESCUE LUNA!" ($5CF4)
L5A8C:   JSR L4831              ; Print Text Message
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         JMP L5A73

L5A97:   LDX #$CB               ; Message = "        " ($5DCB)
         LDY #$5D               ; Message = "        " ($5DCB)
         JSR L4831              ; Print Text Message
         JSR L719E              ; Screen Transition, Fill from outside in
         LDA #$15               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
         LDA #$01
         STA CDTMA1+1
L5AAB:   LDA STICK1
         CMP #$01
         BEQ L5ABE
         CMP #$02
         BEQ L5ABE
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         JMP L5AAB
L5ABE:   JSR L71D0
         JSR L71B8              ; Screen Transition, Clear from inside out
         JMP L5B26
L5AC7:   LDX #$00
         STX SRTIMR
L5ACC:   LDX #$CB               ; Message = "         " ($5DCB)
         LDY #$5D               ; Message = "         " ($5DCB)
         LDA SRTIMR
         AND #$30
         BEQ L5B19
         LDA Level              ; Check Level
         BEQ L5AF5              ; Go print LUNA instructions (STAGE1)
         LDA CDTMA2+1
         BNE L5B19
         LDA Lunas              ; Get number of Lunas
         BMI L5B19
         LDA Level              ; Get level
         CMP #$01               ; Are we at the JOE Level (STAGE2)?
         BEQ L5B06              ; Yes, so print JOE instructions
         LDA Level              ; Get level
         CMP #$04               ; Are we at the SONY level (STAGE5)?
         BEQ L5B11              ; Yes, so print SONY instructions
         BNE L5B19
L5AF5:   LDA Lunas              ; Get number of Lunas
         CMP #$FF
         BEQ L5AFF
         CMP #$08               ; Do we already have 8 Lunas?
         BCS L5B26              ; Yes, so instructions
L5AFF:   LDX #$F4               ; Message = "TOUCH TO RESCUE LUNA!" ($5CF4)
         LDY #$5C               ; Message = "TOUCH TO RESCUE LUNA!" ($5CF4)
         JMP L5B19
L5B06:   LDA VVTP+1
         BEQ L5B26
         LDX #$3F               ; Message = "TOUCH TO RESCUE JOE !" ($5D3F)
         LDY #$5D               ; Message = "TOUCH TO RESCUE JOE !" ($5D3F)
         JMP L5B19
L5B11:   LDA STMCUR_swap
         BEQ L5B26
         LDX #$84               ; Message = "TOUCH TO RESCUE SONY !" ($5D84)
         LDY #$5D               ; Message = "TOUCH TO RESCUE SONY !" ($5D84)
L5B19:   JSR L4831              ; Print Text Message
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         INC SRTIMR
         BNE L5ACC
L5B26:   LDX #$CB               ; Message = "        " ($5DCB)
         LDY #$5D               ; Message = "        " ($5DCB)
         JSR L4831              ; Print Text Message
         LDX #$38               ; Message = "        " ($4D38)
         LDY #$4D               ; Message = "        " ($4D38)
         JSR L4831              ; Print Text Message
         LDX #$59               ; Message = "        " ($4D59)
         LDY #$4D               ; Message = "        " ($4D59)
         JSR L4831              ; Print Text Message
         RTS
L5B3C:   LDA VNTP+1
         AND #$03
         BEQ L5B61
         CMP #$03
         BEQ L5B50
         LDA SysClock
         AND #$07
         BNE L5B4F
         DEC VNTP+1
L5B4F:   RTS
L5B50:   LDX #$05
         LDY #$01
L5B54:   LDA L5CA0,X            ;
         STA FMSZPG,X           ;
         STY ICPTHZ,X           ;
         DEX                    ;
         BPL L5B54
         DEC VNTP+1
         RTS
L5B61:   LDA #$00
         STA VNTP+1
         RTS
L5B66:   LDA VNTP+1
         AND #$30
         CMP #$30
         BEQ L5B71
         JMP L5C43
L5B71:   LDA SysClock             ; Check fire status
         AND #$03               ; Are we shooting?
         BEQ L5B79              ; Yes
         RTS
L5B79:   LDA BarrierPic
         ASL A
         TAY
         LDA L5C5A,Y            ; Barrier Bega
         STA ZRAM_01
         LDA L5C5B,Y
         STA ZRAM_02
         AND ZRAM_01
         CMP #$FF
         BEQ L5BA0
         INC BarrierPic         ; Go to the next barrier picture
         LDY #$05               ; Loop = 6 sprites update
         LDX #$01               
L5B95:   LDA (ZRAM_01),Y        ; Get picture
         STA FMSZPG,Y           ; Set Bega sprite picture
         STX ICPTHZ,X           
         DEY                    ; Next picture
         BPL L5B95              ; Loop until all 6 sprites updated
         RTS
L5BA0:   LDA #$00
         STA VNTP+1
         RTS

;----------------------------------------------------
;         
;----------------------------------------------------
L5BA5:   LDA VNTP+1
         AND #$7F               ; Have we finished exploding?
         BNE L5BBE              ; No, so continue exploding
         LDX PlayerNumber       ; Which player is playing?
         DEC Player1Lives,X     ; Player loses a life
         LDA #$01
         STA VNTD
         INC VNTP+1             ; Erase Bega explosion
         LDA #$03
         JSR L51CE              ; Play Sound
         JSR L4BF7              ; Print BEGA and LUNA lives
L5BBE:   LDA SysClock
         AND #$03
         BEQ L5BC6
         RTS
L5BC6:   DEC VNTD
         BEQ L5BCB
         RTS
L5BCB:   LDA #$02
         STA VNTD
;----------------------------------------------------
;         Update Bega's sprite pictures
;----------------------------------------------------
         LDA VNTP+1             ; Get Explode sequence
         AND #$7F               ; Isolate Explode sprite sequence
         ASL A                  ; Turn it into a pointer
         TAX                    ; Index to correct Explode sequence
         LDA L5C72,X            ; Get Bega Explode picture pointer
         STA ZRAM_01
         LDA L5C73,X
         STA ZRAM_02
         LDY #$05
L5BE1:   LDA (ZRAM_01),Y
         STA FMSZPG,Y
         LDA #$01
         STA ICPTHZ,Y
         DEY
         BPL L5BE1
         LDY #$05
L5BF0:   CLC
         LDA L5C44,Y
         ADC BegaXPos1L,Y
         CMP #$10
         BCS L5BFD
         LDA #$10
L5BFD:   CMP #$F0
         BCC L5C03
         LDA #$F0
L5C03:   STA BegaXPos1L,Y
         CLC
         LDA L5C4A,Y
         ADC OLDADR+1,Y
         CMP #$20
         BCS L5C13
         LDA #$20
L5C13:   STA OLDADR+1,Y
         DEY
         BPL L5BF0
         INC VNTP+1             ; Go to next explode picture
         LDA VNTP+1             ; Get explode picture
         AND #$7F               ; Isolate Explode sprite sequence
         CMP #$0B               ; Have we completed the explosion sequence?
         BEQ L5C24              ; Yes, so skip ahead
         RTS                    ; Return
;----------------------------------------------------
;           Bega Explosion Complete
;----------------------------------------------------
L5C24:   LDA #$00               ;
         STA VNTP+1             ;
         LDX #$05               ;
         LDA #$00               ;
L5C2C:   STA ICPTHZ,X
         STA FMSZPG,X
         DEX
         BPL L5C2C
         LDA #$01
         STA VIMIRQ+1
         JSR L47D0              ; Update Sprites
         JSR L71DE              ;
         LDX #$25               ;
         JSR L4BD2              ; Run Programmable Delay
L5C43:   RTS                    ; Return


;----------------------------------------------------
;                    Data
;----------------------------------------------------
L5C44:   .BYTE $FC,$04,$FA,$06
         .BYTE $FC,$04
L5C4A:   .BYTE $F8,$F8,$FC,$FC
         .BYTE $00,$00
;----------------------------------------------------
;                 Y Step Increment?
;----------------------------------------------------
L5C50:   .BYTE $02
;----------------------------------------------------
;                 X Step Increment
;----------------------------------------------------
L5C51:   .BYTE $02



;----------------------------------------------------
;            Bega Glow Sprite Pointers
;----------------------------------------------------
L5C52:   .WORD $5C88,$5C8E,$5C94,$5C9A
;----------------------------------------------------
;          Bega Barrier Sprite Pointers
;----------------------------------------------------
L5C5A:   .WORD $5CA6,$5CAC,$5CB2,$5CAC,$5CB2,$5CAC
         .WORD $5CB2,$5CAC,$5CB2,$5CAC,$5CA6,$FFFF
;----------------------------------------------------
;           Bega Explode Sprite Pointers
;----------------------------------------------------
L5C72:   .WORD $5CB8,$5CB8,$5CBE,$5CC4,$5CCA,$5CD0
         .WORD $5CD6,$5CDC,$5CE2,$5CE8,$5CEE
;----------------------------------------------------
;  Bega Sprite Assignments (6 picture for Bega Glow)    
;----------------------------------------------------
L5C88:   .BYTE $28,$29,$2A,$2B,$2C,$2D    ; Bega Glow 1
L5C8E:   .BYTE $2E,$2F,$30,$31,$32,$33    ; Bega Glow 2
L5C94:   .BYTE $29,$28,$2B,$2A,$2D,$2C    ; Bega Glow 3
L5C9A:   .BYTE $2F,$2E,$31,$30,$33,$32    ; Bega Glow 4
;----------------------------------------------------
;         Bega Sprite Assignments (Bega)    
;----------------------------------------------------
L5CA0:   .BYTE $34,$35,$36,$37,$38,$39    ; Bega
;----------------------------------------------------
;          Bega Sprite Assignments (Bega Barrier)    
;----------------------------------------------------
L5CA6:   .BYTE $46,$47,$48,$49,$4A,$4B    ; Bega Barrier 1
         .BYTE $4C,$4D,$4E,$4F,$50,$51    ; Bega Barrier 2
         .BYTE $52,$53,$54,$55,$56,$57    ; Bega Barrier 3
;----------------------------------------------------
;          Bega Sprite Assignments (Bega Explode)    
;----------------------------------------------------
L5CB8:   .BYTE $58,$29,$2A,$2B,$2C,$2D    ; Bega Explode 1
         .BYTE $58,$59,$2A,$2B,$2C,$2D    ; Bega Explode 2
         .BYTE $5E,$59,$5A,$2B,$2C,$2D    ; Bega Explode 3
         .BYTE $5E,$5F,$5A,$5B,$5C,$2D    ; Bega Explode 4
         .BYTE $64,$5F,$60,$5B,$5C,$2D    ; Bega Explode 5
         .BYTE $64,$65,$60,$61,$5C,$5D    ; Bega Explode 6
         .BYTE $64,$65,$66,$61,$62,$5D    ; Bega Explode 7
         .BYTE $64,$65,$66,$67,$62,$63    ; Bega Explode 8
         .BYTE $64,$65,$66,$67,$68,$63    ; Bega Explode 9
         .BYTE $64,$65,$66,$67,$68,$69    ; Bega Explode 10
                                          
                                          
;----------------------------------------------------
;          Level Instructions for Gameplay
;----------------------------------------------------
L5CF4:   .WORD $3A44               ; "TOUCH TO RESCUE LUNA !"
         .BYTE $1E,$19,$1F,$0D,$12,$00,$1E,$19,$00,$1C
         .BYTE $0F,$1D,$0D,$1F,$0F,$00,$16,$1F,$18,$0B,$00,$65
         .BYTE $FE
         
L5D0D:   .WORD $3A84                    ; "ONLY SHE CAN CALL OTHER"
         .BYTE $19,$18,$16,$23,$00,$1D,$12,$0F,$00,$0D,$0B,$18,$00 
         .BYTE $0D,$0B,$16,$16,$00,$19,$1E,$12,$0F,$1C,$FE

L5D27:   .WORD $3AC4                    ; "FIGHERS TO YOUR SIDE"
         .BYTE $10,$13,$11,$12,$1E,$0F,$1C,$1D,$00,$1E,$19,$00,$23,$19,$1F
         .BYTE $1C,$00,$1D,$13,$0E,$0F,$FF

L5D3F:   .WORD $3A44                    ; "TOUCH TO RESCUE JOE !"
         .BYTE $1E,$19,$1F,$0D,$12,$00,$1E,$19,$00,$1C,$0F
         .BYTE $1D,$0D,$1F,$0F,$00,$14,$19,$0F,$00,$65,$FE

L5D57:   .WORD $3A84
         .BYTE $14,$19,$0F,$00,$13,$1D,$00,$1E,$12,$0F,$00 ; "JOE IS THE GUARDIAN OF"
         .BYTE $11,$1F,$0B,$1C,$0E,$13,$0B,$18,$00,$19,$10
         .BYTE $FE
         
L5D7X:   .WORD $3AC5                                        
         .BYTE $1E,$12,$0F,$00,$80,$81,$82,$83,$84        ; "THE PKBARRIER BUTTON"
         .BYTE $85,$00,$0C,$1F,$1E,$1E,$19,$18,$FF
         
L5D84:   .WORD $3A44                                       ; "TOUCH TO RESCUE SONY !"
         .BYTE $1E,$19,$1F,$0D,$12,$00,$1E,$19,$00,$1C,$0F,$1D,$0D
         .BYTE $1F,$0F,$00,$1D,$19,$18,$23,$00,$65,$FE

         .WORD $3A84
         .BYTE $1D,$19,$18,$23,$00,$13,$1D,$00,$1E,$12,$0F,$00 ; "SONY IS THE GUARDIAN OF"
         .BYTE $11,$1F,$0B,$1C,$0E,$13,$0B,$18,$00,$19,$10,$FE

         
         .WORD $3AC5
         .BYTE $1E,$12,$0F,$00,$86,$87,$88,$89,$8A,$8B,$00,$0C,$1F,$1E ; "THE TELEPORT BUTTON"
         .BYTE $1E,$19,$18,$FF

         .WORD $3A44
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
         .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FE

         .WORD $84,$3A,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$FE,$C4
         .BYTE $3A,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$FF



;----------------------------------------------------
;               LUNA Movement Updates
;----------------------------------------------------
;       A = 
;----------------------------------------------------
L5E23:   CMP #$40               ;
         BNE L5E2A              ;
         JMP L5F43              ;
L5E2A:   LDA L009B              ;
         CMP #$FF               ;
         BEQ L5E3B
         BPL L5E35
         JMP L6099              ; LUNA hit bottom?
L5E35:   ASL A
         BPL L5E3B
         JMP L5F43
L5E3B:   LDA Lunas
         CMP #$FF
         BNE L5E42
         RTS
L5E42:   LDA Level 
         BNE L5E48
         RTS
L5E48:   LDA L009B
         AND #$30
         CMP #$20
         BEQ L5E8A
         LDA L00A5
         AND #$01
         TAX
         CLC
         LDA L614C,X
         ADC L00A6
         STA L00A6
         CMP #$E0
         BCS L5E86
         CMP #$0A
         BCC L5E80
         CMP #$D0
         BCS L5E8A
         CLC
         ADC #$30
         CMP BegaXPos1R
         BCC L5E80
         LDA L00A6
         CMP #$18
         BCC L5E8A
         SEC
         SBC #$18
         CMP BegaXPos1R
         BCS L5E86
         JMP L5E8A
L5E80:   LDA #$01
         STA L00A5
         BNE L5E8A
L5E86:   LDA #$00
         STA L00A5
L5E8A:   LDA L00A6
         AND #$F8
         LSR A
         LSR A
         LSR A
         STA POKMSK
         DEC POKMSK
         CLC
         LDA #$40
         ADC POKMSK
         STA ZRAM_01
         LDA #$00
         ADC #$3B
         STA ZRAM_02
         CLC
         LDA #$60
         ADC POKMSK
         STA ZRAM_02+1
         LDA #$00
         ADC #$3B
         STA RAMLO
         CLC
         LDA #$80
         ADC POKMSK
         STA RAMLO+1
         LDA #$00
L5EB8:   ADC #$3B
         STA TRAMSZ
         CLC
         LDA #$A0
         ADC POKMSK
         STA CMCMD
         LDA #$00
         ADC #$3B
         STA WARMST
L5EC9:   LDA L00A6
         AND #$06
         LSR A
         STA POKMSK
         ASL A
         ASL A
         CLC
         ADC POKMSK
         TAX
         LDA L009B
         BEQ L5EE2
         CMP #$FF
         BEQ L5EE2
         AND #$20
         BNE L5F01
L5EE2:   LDY #$00
         STY L009B
L5EE6:   LDA L614E,X
         STA (ZRAM_01),Y
         LDA L6162,X
         STA (ZRAM_02+1),Y
         LDA L6176,X
         STA (RAMLO+1),Y
         LDA L618A,X
         STA (CMCMD),Y
         INX
         INY
         CPY #$05
         BNE L5EE6
         RTS
L5F01:   LDA L009B
         AND #$10
         BNE L5F08
         RTS
L5F08:   LDA SysClock
         AND #$03
         BEQ L5F10
         RTS
L5F10:   LDA HATABS+15
         ASL A
         ASL A
         CLC
         ADC HATABS+15
         TAX
         LDY #$00
         INC HATABS+15
L5F1F:   LDA L619E,X
         BEQ L5F3C
         STA (ZRAM_01),Y
         LDA L61DB,X
         STA (ZRAM_02+1),Y
         LDA L6217,X
         STA (RAMLO+1),Y
         LDA L6253,X
         STA (CMCMD),Y
         INX
         INY
         CPY #$05
         BNE L5F1F
         RTS
L5F3C:   LDA #$00
         STA L009B
         JMP L5EC9
L5F43:   LDA L009B
         AND #$30
         BNE L5F5E
         LDA Lunas
         CMP #$FF
         BEQ L5F5E
         CMP #$08
         BCC L5F54
         RTS
L5F54:   LDA Level 
         BEQ L5F5E
         LDA #$FF
         STA L009B
         RTS
L5F5E:   LDA L009B
         AND #$20
         BNE L5FB7
L5F64:   LDA VNTP+1
         AND #$F0
         BEQ L5F6B
         RTS
;----------------------------------------------------
;        Reset for new Random LUNA movement 
;----------------------------------------------------
L5F6B:   LDA Random             ; Get Random Number
         AND #$0F               ; Random number between 0-15
         STA L00A2              ; Save as Movement Pattern (0-15)
         LDA #$00               ; A = 0
         STA L00A3              ; Clear movement pointer
         LDA L00A2              ; Clear movement pointer
         ASL A
         TAX
         LDA LBF42,X
         STA ZRAM_01
         LDA LBF43,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y        ; Get 8
         STA L00A4              ; Save 8 in counter step
         LDA #$01
         STA SpritePic1
         STA SpritePic2
         LDA #$76
         STA Sprite3_Cha  = $003B
         LDA #$77
         STA Buff5
         LDX L00A2
;----------------------------------------------------
;           Start LUNA at top of screen
;----------------------------------------------------
         LDA Random             ; Get Random Number between 48-79
         AND #$1F               ; Start with number between 0-31
         CLC                    ; Clear carry
         ADC #$30               ; Adjust number between 48-79
         STA Sprite3_X          ; Start LUNA at X location between 48-79
         STA Sprite3a_X         ; Set X for LUNA sprite(2)
         LDA #$00               ; Y location = 0
         STA Sprite3_Y          ; Start LUNA at the top of the screen
         LDA #$10               ; Y locatoin = 32
         STA Sprite3a_Y         ; Put sprite(2) right below sprite(1)
         LDA #$60
         STA L009B
         LDA #$09
         JSR L51CE              ; Play Sound
L5FB7:   LDA L009B
         AND #$10
         BEQ L5FC0
         JMP L6064              ; Skip all LUNA movements
L5FC0:   LDA L00A2              ; Get which movement pattern we're using
         ASL A                  ; Create an index
         TAX                    ; X = index to movement pattern
;----------------------------------------------------
;       Setup Pointer to LUNA Movement Pattern
;----------------------------------------------------
         LDA LBF42,X            ;
         STA ZRAM_01            ;
         LDA LBF43,X            ;
         STA ZRAM_02            ;
         LDY L00A3              ;
         LDA (ZRAM_01),Y        ; Get LUNA pattern (21,8,f
         TAX                    ; X = pattern position
         LDA LB100,X            ; Get pattern movement
         BPL L5FE0              ; LUNA Moving left, skip ahead
         LDA #$01               ; LUNA Picture facing LEFT
         STA SpritePic1         ; Update LUNA picture
         STA SpritePic2         ; Update LUNA picture
         BNE L5FE6              ; Go to movement
L5FE0:   LDA #$05               ; LUNA Picture facing RIGHT
         STA SpritePic1         ; Update LUNA picture
         STA SpritePic2         ; Update LUNA picture
;----------------------------------------------------
;            Update LUNA's X Coordinate
;----------------------------------------------------
L5FE6:   CLC                    ; Clear carry for addition
         LDA Sprite3_X          ; Get sprite(1) x coordinate
         ADC LB100,X            ; Adjust x movement (add/subtract)
         STA Sprite3_X          ; Save new sprite x coordinate
         CLC                    ; Clear carry for addition
         LDA Sprite3a_X              ; Get sprite(2) x coordinate
         ADC LB100,X            ; Adjust x movement (add/subtract)
         STA Sprite3a_X              ; Save new sprite x coordinate
;----------------------------------------------------
;            Update LUNA's Y Coordinate
;----------------------------------------------------
         CLC                    ; Clear carry for addition
         LDA Sprite3_Y          ; Get sprite(1) y coordinate
         ADC LB000,X            ; Add y position
         STA Sprite3_Y          ; Save sprite y coordinate
         CLC                    ; Clear carry for addition
         LDA Sprite3a_Y         ; Get sprite(2) y coordinate
         ADC LB000,X            ; Add y position
         STA Sprite3a_Y         ; Save sprite y coordinate
;----------------------------------------------------
;        Move a set distance in current direction
;----------------------------------------------------
         DEC L00A4              ; Count down number of moves in that direction
         BNE L6015              ; Not done moving in that direction so skip ahead
         INC L00A3              ; Increment movement pointer
         INC L00A3              ; Increment movement pointer
         LDY L00A3              ;
         INY                    ;
         LDA (ZRAM_01),Y        ;
L6013:   STA L00A4              ;
;----------------------------------------------------
;         Check X position for edge of screen
;----------------------------------------------------
L6015:   LDA Sprite3_X          ; Get X position
         CMP #$04               ; Are we past far left of screen?
         BCC L604D              ; No, so skip ahead
         CMP #$EC               ; Are we past far right of screen?
         BCS L604D              ; No, so skip ahead
;----------------------------------------------------
;            Check if LUNA was caught
;----------------------------------------------------
         LDA Sprite3_Y          ; Check LUNA Y Position
         CMP #$E0               ; Has LUNA passed bottom of screen?
         BCS L604D              ; Yes, so skip ahead
         LDA VNTP+1
         AND #$F0
         BNE L604C
         LDA Sprite3_X          ; Get LUNA X position
         SEC                    ; Set Carry
         SBC BegaXPos2R         ; Compare to Bega's X position
         CLC                    ; Clear carry
         ADC #$10               ; Add 16
         CMP #$20               ; Are we close enough to her?
         BCS L604C              ; No catch, so leave 
         LDA Sprite3_Y          ; Get LUNA Y position
         SEC                    ; Set Carry
         SBC FKDEF              ; Compare to Bega's Y position
         CLC                    ; Clear carry
         ADC #$10               ; Add 16
         CMP #$20               ; Are we close enough to her?
         BCS L604C              ; No catch, so leave
;----------------------------------------------------
;               LUNA was caught
;----------------------------------------------------
         LDA #$70               ; Erase LUNA
         STA L009B              ; Save LUNA status
         INC Lunas              ; Increment number of Lunas caught
         JSR L4BF7              ; Print BEGA and LUNA lives
L604C:   RTS                    ; Return
;----------------------------------------------------
;              Erase LUNA from screen
;----------------------------------------------------
L604D:   LDA #$00               ; A = 0
         STA SpritePic1         ; Clear LUNA sprite(1) pic
         STA Sprite3_Cha              ; Clear LUNA sprite(1) prop
         STA Sprite3_X          ; Clear LUNA sprite(1) X
         STA Sprite3_Y          ; Clear LUNA sprite(1) Y
         STA SpritePic2         ; Clear LUNA sprite(2) pic
         STA Buff5              ; Clear LUNA sprite(2) prop
         STA Sprite3a_X         ; Clear LUNA sprite(2) X
         STA Sprite3a_Y         ; Clear LUNA sprite(2) Y
         LDA #$40               ;
         STA L009B              ; Save LUNA status
         RTS                    ; Return
;----------------------------------------------------
;             
;----------------------------------------------------
L6064:   LDA L009B              ;
         AND #$0F               ;
         BNE L6085              ;
         LDA #$02               ;
         JSR L51CE              ; Play Sound
         LDA #$72               ;
         STA L009B              ;
         LDA #$01               ;
         STA SpritePic1         ;
         LDA #$23               ;
         STA Sprite3_Cha              ;
         LDA #$00               ;
         STA SpritePic2         ;
         LDA #$03               ;
         JSR L53D7              ; check bonus?
         RTS                    ; Return


;----------------------------------------------------
;             
;----------------------------------------------------
L6085:   LDA SysClock             ;
         AND #$0F               ;
         BEQ L608D              ;
         RTS                    ;
L608D:   DEC L009B              ;
         LDA L009B              ;
         AND #$0F
         BNE L6098
         JMP L604D
L6098:   RTS
L6099:   LDA L009B
         AND #$7F
         BNE L60B2
         LDA #$04
         JSR L51CE              ; Play Sound
         LDA #$81
         STA L009B
         DEC Lunas              ; Decrement number of Lunas 
         JSR L4BF7              ; Print BEGA and LUNA lives
         LDA #$01
         STA HATABS+15
L60B2:   LDA SysClock
         AND #$07
         BEQ L60BA
         RTS
L60BA:   DEC HATABS+15
         BEQ L60C0
         RTS
L60C0:   LDA #$04
         STA HATABS+15
         LDA L00A6
L60C7:   LSR A
         LSR A
         LSR A
         STA POKMSK
         DEC POKMSK
         CLC
         LDA #$40
         ADC POKMSK
         STA ZRAM_01
         LDA #$00
         ADC #$3B
         STA ZRAM_02
         CLC
         LDA #$60
         ADC POKMSK
         STA ZRAM_02+1
         LDA #$00
         ADC #$3B
         STA RAMLO
         CLC
         LDA #$80
         ADC POKMSK
         STA RAMLO+1
         LDA #$00
         ADC #$3B
         STA TRAMSZ
         CLC
         LDA #$A0
         ADC POKMSK
         STA CMCMD
         LDA #$00
         ADC #$3B
         STA WARMST
         CLC
         LDA #$A0
         ADC POKMSK
         STA BOOT
         LDA #$00
         ADC #$3B
         STA DOSVEC
         LDA L009B
         AND #$7F
         STA POKMSK
         ASL A
         ASL A
         CLC
         ADC POKMSK
         TAX
         LDY #$00
L611D:   LDA L628F,X
         STA (ZRAM_01),Y
         LDA L62AD,X
         STA (ZRAM_02+1),Y
         LDA L62CB,X
         STA (RAMLO+1),Y
         LDA L62E9,X
         STA (CMCMD),Y
         LDA L6307,X
         STA (BOOT),Y
         INX
         INY
         CPY #$05
         BNE L611D
         INC L009B
         LDA L009B
         AND #$7F
         CMP #$06
         BEQ L6147
         RTS
L6147:   LDA #$FF
         STA L009B
         RTS
L614C:   INC LFF02,X
         JSR L2221
         .BYTE $FF,$FF
         BIT L2E2D
         .BYTE $FF,$FF
         SEC
         AND LFF3A,Y
         .BYTE $FF,$44
         EOR FMSZPG+3
         .BYTE $FF
L6162:   .BYTE $FF,$23
         BIT ICBAHZ
         .BYTE $FF,$FF,$2F
         BMI L619C
         .BYTE $FF,$FF,$3B,$3C
         AND LFFFF,X
         .BYTE $47
         PHA
         EOR #$FF
L6176:   .BYTE $FF
         ROL ICPTHZ
         PLP
         .BYTE $FF,$FF,$32,$33
         .BYTE $34,$FF,$FF
         ROL L403F,X
         .BYTE $FF,$FF
         LSR A
         .BYTE $4B
         JMP LFFFF
         AND #$2A
         .BYTE $2B,$FF,$FF
         AND LTEMP,X
         .BYTE $37,$FF,$FF
         EOR (CRITIC,X)
         .BYTE $43,$FF,$FF
         EOR L4F4E
         .BYTE $FF
L619E:   .BYTE $FF
         BVC L61F2
         .BYTE $FF,$FF,$FF
         CLI
         EOR LFFFF,Y
         .BYTE $FF
         RTS
         ADC (FPTR2+1,X)
         .BYTE $FF,$FF
         CLI
         EOR LFFFF,Y
         .BYTE $FF
         RTS
         ADC (FPTR2+1,X)
         .BYTE $FF,$FF
         CLI
         EOR LFFFF,Y
         .BYTE $FF
         RTS
         ADC (FPTR2+1,X)
         .BYTE $FF,$FF
         CLI
         EOR LFFFF,Y
         .BYTE $FF
         RTS
         ADC (FPTR2+1,X)
         .BYTE $FF,$FF
         CLI
         EOR LFFFF,Y
         .BYTE $FF
         RTS
         ADC (FPTR2+1,X)
         .BYTE $FF,$FF
         BVC L6229
         .BYTE $FF,$FF
         BRK
L61DB:   .BYTE $FF,$52,$53,$FF
         .BYTE $FF,$FF,$5A,$5B
         .BYTE $FF,$FF,$FF,$62
         .BYTE $63,$FF,$FF,$FF
         .BYTE $5A,$5B,$FF,$FF
         .BYTE $FF,$62,$63
L61F2:   .BYTE $FF,$FF,$FF,$5A
         .BYTE $5B,$FF,$FF,$FF
         .BYTE $62,$63,$FF,$FF
         .BYTE $FF,$5A,$5B,$FF
         .BYTE $FF,$FF,$62,$63
         .BYTE $FF,$FF,$FF,$5A
         .BYTE $5B,$FF,$FF,$FF
         .BYTE $62,$63,$FF,$FF
         .BYTE $FF,$52,$53,$FF
         .BYTE $FF
L6217:   .BYTE $FF,$54
         EOR FPTR2+1,X
         .BYTE $FF,$FF,$5C
         EOR LFFFF,X
         .BYTE $FF,$64
         ADC FPTR2+1
         .BYTE $FF,$FF,$5C
         EOR LFFFF,X
         .BYTE $FF,$64
         ADC FPTR2+1
         .BYTE $FF,$FF,$5C
         EOR LFFFF,X
         .BYTE $FF,$64
         ADC FPTR2+1
         .BYTE $FF,$FF,$5C
         EOR LFFFF,X
         .BYTE $FF,$64
         ADC FPTR2+1
         .BYTE $FF,$FF,$5C
         EOR LFFFF,X
         .BYTE $FF,$64
         ADC FPTR2+1
         .BYTE $FF,$FF,$54
         EOR FPTR2+1,X
         .BYTE $FF
L6253:   .BYTE $FF
         LSR Sprite3_Y,X
         .BYTE $FF,$FF,$FF
         LSR LFF5F,X
         .BYTE $FF,$FF
         ROR MLTTMP+1
         .BYTE $FF,$FF,$FF
         LSR LFF5F,X
         .BYTE $FF,$FF
         ROR MLTTMP+1
         .BYTE $FF,$FF,$FF
         LSR LFF5F,X
         .BYTE $FF,$FF
         ROR MLTTMP+1
         .BYTE $FF,$FF,$FF
         LSR LFF5F,X
         .BYTE $FF,$FF
         ROR MLTTMP+1
         .BYTE $FF,$FF,$FF
         LSR LFF5F,X
         .BYTE $FF,$FF
         ROR MLTTMP+1
         .BYTE $FF,$FF,$FF
         LSR Sprite3_Y,X
         .BYTE $FF,$FF
L628F:   .BYTE $FF
         PLA
         ADC #$FF
         .BYTE $FF,$FF
         PLA
         ADC #$FF
         .BYTE $FF,$FF,$72,$73
         .BYTE $FF,$FF,$FF,$7C
         ADC LFFFF,X
         .BYTE $FF
         STX VVTP+1
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF
L62AD:   .BYTE $FF
         ROR A
         .BYTE $6B,$FF,$FF,$FF
         ROR A
         .BYTE $6B,$FF,$FF,$FF
         .BYTE $74
         ADC FPTR2+1,X
         .BYTE $FF,$FF
         ROR LFF7F,X
         .BYTE $FF,$FF
         DEY
         .BYTE $89,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
L62CB:   .BYTE $FF
         JMP (LFF6D)
         .BYTE $FF,$FF
         JMP (LFF6D)
         .BYTE $FF,$FF
         ROR DELTAC,X
         .BYTE $FF,$FF,$FF,$80
         STA (FPTR2+1,X)
         .BYTE $FF,$FF
         TXA
         .BYTE $8B,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
L62E9:   .BYTE $FF
         ROR LFF6F
         .BYTE $FF,$FF
         ROR LFF6F
         .BYTE $FF,$FF
         SEI
         ADC LFFFF,Y
         .BYTE $FF,$82,$83,$FF
         .BYTE $FF,$FF
         STY LFF8D
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
L6307:   .BYTE $FF
         BVS L637B
         .BYTE $FF,$FF,$FF
         BVS L6380
         .BYTE $FF,$FF,$FF,$7A
         .BYTE $7B,$FF,$FF,$FF
         STY Lunas
         .BYTE $FF,$FF,$FF
         DEY
         .BYTE $8F,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF



;----------------------------------------------------
;         Set Bega's starting position x,y
;----------------------------------------------------
L6325:   LDX #$05               ;
L6327:   TXA                    ;
         PHA                    ;
         LDA L009C,X            ;
         CMP #$FF               ;
         BNE L6332              ;
         JMP L6738              ;
L6332:   LDA L009C,X
         BPL L6339
         JMP L6652
L6339:   ASL A
         BPL L633F
         JMP L63A4
L633F:   STX POKMSK
         LDY #$03
L6343:   LDA L0093,Y
         CMP POKMSK
         BEQ L6350
         DEY
         BPL L6343
         JMP L6738
L6350:   LDA L6741,Y
         STA APPMHI+1
         LDA L009C,X
         AND #$0C
         LSR A
         TAY
         LDA L009C,X
         AND #$01
         ASL A
         ASL A
         ORA #$01
         STA SpritePic1,Y
         STA SpritePic2,Y
         LDA #$C8
         STA Sprite3_Y,Y
         LDA #$D8
         STA Sprite3a_Y,Y
         CLC
         LDA BegaXPos1L
         ADC APPMHI+1
         STA Sprite3_X,Y
L637B:   CLC
         LDA BegaXPos1L
         ADC APPMHI+1
L6380:   STA Sprite3a_X,Y
         LDA L009C,X
         AND #$20
         BEQ L638C
         JMP L65F7
L638C:   LDA L009C,X
         AND #$10
         BEQ L6395
         JMP L65D1
L6395:   LDA L67C9,X
         STA Sprite3_Cha ,Y
         LDA L67CF,X
         STA Buff5,Y
         JMP L6738
L63A4:   LDA L009B
         AND #$F0
         BEQ L63AD
         JMP L6738
L63AD:   LDA L009C,X
         AND #$30
         BNE L63CA
         JSR L68E9
         CPY #$FF
         BNE L63BD
         JMP L6738
L63BD:   TXA
         ASL A
         TAY
         LDA VVTP+1,Y
         CMP #$FF
         BEQ L63CA
         JMP L6738
L63CA:   LDA L009C,X
         AND #$20
         BEQ L63D3
         JMP L643C
L63D3:   LDA Random             ; Get Random Number
         AND #$0F               ; Random number between 0-15
         STA L00A2
         LDA #$00
         STA L00A3
         LDA L00A2
         ASL A
         TAY
         LDA LBF42,Y
         STA ZRAM_01
         LDA LBF43,Y
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         STA L00A4
         JSR L68E9
         TYA
         ASL A
         ASL A
         ORA L009C,X
         STA L009C,X
         TYA
         ASL A
         TAY
         LDA #$01
         STA SpritePic1,Y
         STA SpritePic2,Y
         LDA L67D5,X
         STA Sprite3_Cha ,Y
         LDA L67DB,X
         STA Buff5,Y
         STY POKMSK
         LDY L00A2
         LDA Random             ; Get Random Number between 48-79
         AND #$1F               ; Start with number between 0-31
         CLC                    ; Clear carry
         ADC #$30               ; Adjust number between 48-79
         LDY POKMSK
         STA Sprite3_X,Y
         STA Sprite3a_X,Y
         LDA #$00
         STA Sprite3_Y,Y
         LDA #$10
         STA Sprite3a_Y,Y
         LDA #$60
         ORA L009C,X
         STA L009C,X
         LDA #$09
         JSR L51CE              ; Play Sound
L643C:   LDA L009C,X
         AND #$10
         BEQ L6445
         JMP L652A
L6445:   LDA L00A2
         ASL A
         TAY
         LDA LBF42,Y
         STA ZRAM_01
         LDA LBF43,Y
         STA ZRAM_02
         LDY L00A3
         LDA (ZRAM_01),Y
         TAY
         LDA LB100,Y
         BPL L646D
         LDA L009C,X
         AND #$0C
         LSR A
         TAY
         LDA #$01
         STA SpritePic1,Y
         STA SpritePic2,Y
         BNE L647B
L646D:   LDA L009C,X
         AND #$0C
         LSR A
         TAY
         LDA #$05
         STA SpritePic1,Y
         STA SpritePic2,Y
L647B:   STY POKMSK
         LDY L00A3
         LDA (ZRAM_01),Y
         STA APPMHI+1
         TAY
         CLC
         LDA LB100,Y
         LDY POKMSK
         ADC Sprite3_X,Y
         STA Sprite3_X,Y
         LDY APPMHI+1
         CLC
         LDA LB100,Y
         LDY POKMSK
         ADC Sprite3a_X,Y
         STA Sprite3a_X,Y
         LDY APPMHI+1
         CLC
         LDA LB000,Y
         LDY POKMSK
         ADC Sprite3_Y,Y
         STA Sprite3_Y,Y
         LDY APPMHI+1
         CLC
         LDA LB000,Y
         LDY POKMSK
         ADC Sprite3a_Y,Y
         STA Sprite3a_Y,Y
         DEC L00A4
         BNE L64C9
         INC L00A3
         INC L00A3
         LDY L00A3
         INY
         LDA (ZRAM_01),Y
         STA L00A4
L64C9:   LDA L009C,X
         AND #$0C
         LSR A
         TAY                    ; Get girder number
;----------------------------------------------------
;    Check JOE's X Coordinate for edge of screen
;----------------------------------------------------
         LDA Sprite3_X,Y        ; Get X position
         CMP #$04               ; Are we past far left of screen?
         BCC L6509              ; No, so skip ahead
         CMP #$EC               ; Are we past far right of screen?
         BCS L6509              ; No, so skip ahead
;----------------------------------------------------
;            Check if JOE was caught
;----------------------------------------------------
         LDA Sprite3_Y,Y        ; Check JOE Y Position
         CMP #$E0               ; Has JOE passed bottom of screen?
         BCS L6509              ; Yes, so skip ahead
         LDA Sprite3_X,Y        ; Get JOE X position
         SEC                    ; Set Carry
         SBC BegaXPos2R         ; Compare to Bega's X position
         CLC                    ; Clear carry
         ADC #$10               ; Add 16
         CMP #$20               ; Are we close enough to him?
         BCS L6506              ; No catch, so skip ahead
         LDA Sprite3_Y,Y        ; Get JOE Y position
         SEC                    ; Set Carry
         SBC FKDEF              ; Compare to Bega's Y position
         CLC                    ; Clear carry
         ADC #$10               ; Add 16
         CMP #$20               ; Are we close enough to him?
         BCS L6506              ; No catch, so skip ahead
;----------------------------------------------------
;              girder WAS HIT (or JOE was caught)
;----------------------------------------------------
         LDA #$70               ;
         ORA L009C,X            ;
         STA L009C,X            ;
         TXA                    ;
         ASL A                  ;
         TAX                    ;
         INC VVTP+1,X           ;
L6506:   JMP L6738              ;
;----------------------------------------------------
;              
;----------------------------------------------------
L6509:   LDA #$00
         STA SpritePic1,Y
         STA Sprite3_Cha ,Y
         STA Sprite3_X,Y
         STA Sprite3_Y,Y
         STA SpritePic2,Y
         STA Buff5,Y
         STA Sprite3a_X,Y
         STA Sprite3a_Y,Y
         LDA #$40
         STA L009C,X
         JMP L6738
L652A:   LDA L009C,X
         AND #$03
         BNE L6558
         LDA #$02
         JSR L51CE              ; Play Sound
         LDA #$03
         ORA L009C,X
         STA L009C,X
         AND #$0C
         LSR A
         TAY
         LDA #$01
         STA SpritePic1,Y
         LDA L68DD,X
         STA Sprite3_Cha ,Y
         LDA #$00
         STA SpritePic2,Y
         LDA L68E3,X
         JSR L53D7
         JMP L6738
L6558:   LDA SysClock
         AND #$1F
L655D:   BEQ L6562
         JMP L6738
L6562:   LDA L009C,X
L6564:   AND #$0C
         LSR A
         TAY
         DEC L009C,X
         LDA L009C,X
         AND #$03
         BEQ L6573
         JMP L6738
L6573:   LDA #$00
         STA SpritePic1,Y
         STA Sprite3_Cha ,Y
         STA Sprite3_X,Y
         STA Sprite3_Y,Y
         STA SpritePic2,Y
         STA Buff5,Y
         STA Sprite3a_X,Y
         STA Sprite3a_Y,Y
         LDX #$05
         LDA #$FF
L6591:   STA L009C,X
         DEX
         BPL L6591
         STA L0093
         STA L0094
         STA POKADR
         STA POKADR+1
         LDY #$00
         STY POKMSK
L65A2:   LDA L009C,Y
         CMP #$FF
         BNE L65C8
         TYA
         ASL A
         TAX
         LDA VVTP+1,X
         CMP #$FF
         BEQ L65C8
         JSR L6CEF
         TXA
         AND #$01
         ASL A
         STA ZRAM_01
         LDA POKMSK
         AND #$03
         INC POKMSK
         ASL A
         ASL A
         ORA ZRAM_01
         STA L009C,Y
L65C8:   INY
         CPY #$06
         BNE L65A2
         PLA
         JMP L6325
L65D1:   LDA VNTP+1
         AND #$F0
         BEQ L65EE
         LDA #$01
         STA SpritePic1,Y
         STA SpritePic2,Y
         LDA L67E1,X
         STA Sprite3_Cha ,Y
         LDA L67E7,X
         STA Buff5,Y
         JMP L6738
L65EE:   LDA #$0E
         AND L009C,X
         STA L009C,X
         JMP L6738
L65F7:   LDA L009C,X
         AND #$30
         CMP #$30
         BEQ L6602
         JMP L6738
L6602:   LDA SysClock
         AND #$03
         BEQ L660C
         JMP L6738
L660C:   LDA HATABS+16,X
         ASL A
         TAX
         LDA L6745,X
         STA ZRAM_01
         LDA L6746,X
         STA ZRAM_02
         LDA L6761,X
         STA ZRAM_02+1
         LDA L6762,X
         STA RAMLO
         LDA ZRAM_01
         AND ZRAM_02
         CMP #$FF
         BEQ L6647
         LDX POKMSK
         INC HATABS+16,X
         TYA
         TAX
         LDY POKMSK
         LDA (ZRAM_01),Y
         STA Sprite3_Cha ,X
         LDA (ZRAM_02+1),Y
         STA Buff5,X
         LDA #$01
         STA SpritePic1,X
         STA SpritePic2,X
         JMP L6738
L6647:   LDX POKMSK
         LDA #$0E
         AND L009C,X
         STA L009C,X
         JMP L6738
L6652:   LDA L009C,X
         AND #$C0
         CMP #$C0
         BNE L665D
         JMP L6738
L665D:   LDA HATABS+16,X
         BNE L6674
         LDA #$04
         JSR L51CE              ; Play Sound
         TXA
         TAY
         ASL A
         TAX
         DEC VVTP+1,X
         TYA
         TAX
         LDA #$11
         STA HATABS+16,X
L6674:   LDA SysClock
         AND #$03
         BEQ L667E
         JMP L6738
L667E:   DEC HATABS+16,X
         AND #$0F
         BEQ L6688
         JMP L6738
L6688:   LDA #$02
         ORA HATABS+16,X
         STA HATABS+16,X
         AND #$F0
         LSR A
         LSR A
         LSR A
         TAY
         LDA L677B,Y
         STA ZRAM_01
         LDA L677C,Y
         STA ZRAM_02
         LDA L6795,Y
         STA ZRAM_02+1
         LDA L6796,Y
         STA RAMLO
         LDA L67AF,Y
         STA RAMLO+1
         LDA L67B0,Y
         STA TRAMSZ
         TXA
         TAY
         LDA L009C,X
         AND #$0C
         LSR A
         TAX
         LDA (ZRAM_01),Y
         STA Sprite3_Cha ,X
         LDA (ZRAM_02+1),Y
         STA Buff5,X
         LDA (RAMLO+1),Y
         STA SpritePic1,X
         STA SpritePic2,X
         INC Sprite3_Y,X
         INC Sprite3a_Y,X
         CLC
         LDA HATABS+16,Y
         ADC #$10
         STA HATABS+16,Y
         AND #$F0
         CMP #$D0
         BEQ L66E0
         JMP L6738
L66E0:   LDA #$FF
         STA L009C,Y
         LDA #$00
         STA SpritePic1,X
         STA SpritePic2,X
         STA Sprite3_Cha ,X
         STA Buff5,X
         STA Sprite3_X,X
         STA Sprite3a_X,X
         STA Sprite3_Y,X
         STA Sprite3a_Y,X
         TYA
         TAX
         LDY #$00
         CPX L0093
         BEQ L670A
         INY
         CPX L0094
         BEQ L670A
         INY
         CPX POKADR
         BEQ L670A
         INY
L670A:   CPY #$02
         BCC L6716
         LDA #$FF
         STA L0093,Y
         JMP L6738
L6716:   CPY #$00
         BNE L6729
         LDA POKADR
         CMP #$FF
         BEQ L6735
         STA L0093
         LDA #$FF
         STA POKADR             ; get luna?
         JMP L6738
L6729:   LDA POKADR+1
         CMP #$FF
         BEQ L6735
         STA L0094
         LDA #$FF
         STA POKADR+1
L6735:   JMP L6738
L6738:   PLA
         TAX
         DEX
         BMI L6740
         JMP L6327
L6740:   RTS
L6741:   BEQ L6763
         CPX #$30
L6745:   SBC LF967
         .BYTE $67
         ORA SAVADR
         SBC L0567,Y
         PLA
         SBC L0567,Y
         PLA
         SBC L0567,Y
         PLA
         SBC L0567,Y
         PLA
         SBC LC967,Y
         .BYTE $67,$FF,$FF
L6761:   .BYTE $F3
L6762:   .BYTE $67
L6763:   .BYTE $FF,$67,$0B
         PLA
         .BYTE $FF,$67,$0B
         PLA
         .BYTE $FF,$67,$0B
         PLA
         .BYTE $FF,$67,$0B
         PLA
         .BYTE $FF,$67,$0B
         PLA
         .BYTE $FF,$67,$CF,$67
L677B:   CMP #$67
         ORA (SAVADR),Y
         ORA L2968,X
         PLA
         AND SAVADR,X
         EOR (SAVADR,X)
         EOR L5968
         PLA
         ADC SAVADR
         ADC (SAVADR),Y
         ADC L8968,X
         PLA
         STA SAVADR,X
L6795:   .BYTE $CF
L6796:   .BYTE $67,$17
         PLA
         .BYTE $23
         PLA
         .BYTE $2F
         PLA
         .BYTE $3B
         PLA
         .BYTE $47
         PLA
         .BYTE $53
         PLA
         .BYTE $5F
         PLA
         .BYTE $6B
         PLA
         .BYTE $77
         PLA
         .BYTE $83
         PLA
         .BYTE $8F
         PLA
         .BYTE $9B
         PLA
L67AF:   LDA (SAVADR,X)
         LDA (SAVADR,X)
         LDX SAVADR
         .BYTE $AB
         PLA
         BCS L6821
         LDA SAVADR,X
         TSX
         PLA
         .BYTE $BF
         PLA
         CPY SAVADR
         CMP #$68
         DEC LD368
         PLA
         CLD
         PLA
L67C9:   .BYTE $82
         STX LBEA6
         TXS
         .BYTE $B2
L67CF:   .BYTE $83,$8F,$A7,$BF
         .BYTE $9B,$B3
L67D5:   .BYTE $80
         STY LBCA4
         TYA
         BCS L675D
         STA LBDA5
         STA L84B1,Y
         BCC L678C
         CPY #$9C
         LDY Lunas,X
         STA (L00A9),Y
         CMP (L009D,X)
         LDA STMCUR,X
         STX L00AE,X
         .BYTE $C7
         LDX #$BA
         .BYTE $8B,$97,$AF
         INY
         .BYTE $A3,$BB,$7C,$7C
         .BYTE $7C,$7C,$7C,$7C
         ADC L7D7D,X
         ADC L7D7D,X
         ROR L7E7E,X
         ROR L7E7E,X
         .BYTE $7F,$7F,$7F,$7F
         .BYTE $7F,$7F,$83,$8F
         .BYTE $A7,$BF,$9B,$B3
         .BYTE $82
         STX LBEA6
         TXS
         .BYTE $B2,$83,$8F,$A7
         .BYTE $BF
L6821:   .BYTE $9B,$B3,$82
         STX LBEA6
         TXS
         .BYTE $B2,$83,$8F,$A7
         .BYTE $BF,$9B,$B3,$82
         STX LBEA6
         TXS
         .BYTE $B2,$83,$8F,$A7
         .BYTE $BF,$9B,$B3,$82
         STX LBEA6
         TXS
         .BYTE $B2
         SEI
         SEI
         SEI
         SEI
         SEI
         SEI
         ADC L7979,Y
         ADC L7979,Y
         SEI
         SEI
         SEI
         SEI
         SEI
         SEI
         ADC L7979,Y
         ADC L7979,Y
         SEI
         SEI
         SEI
         SEI
         SEI
         SEI
         ADC L7979,Y
         ADC L7979,Y
         SEI
         SEI
         SEI
         SEI
         SEI
         SEI
         ADC L7979,Y
         ADC L7979,Y
         .BYTE $7A,$7A,$7A,$7A
         .BYTE $7A,$7A,$7B,$7B
         .BYTE $7B,$7B,$7B,$7B
         .BYTE $7A,$7A,$7A,$7A
         .BYTE $7A,$7A,$7B,$7B
         .BYTE $7B,$7B,$7B,$7B
         .BYTE $7A,$7A,$7A,$7A
         .BYTE $7A,$7A,$7B,$7B
         .BYTE $7B,$7B,$7B,$7B
         .BYTE $7A,$7A,$7A,$7A
         .BYTE $7A,$7A,$7B,$7B
         .BYTE $7B,$7B,$7B,$7B
         .BYTE $03,$03,$03,$03
         .BYTE $03,$07,$07,$07
         .BYTE $07,$07,$03,$03
         .BYTE $03,$03,$03,$07
         .BYTE $07,$07,$07,$07
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (RAMLO+1,X)
         ORA RAMLO+1
         ORA RAMLO+1
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (RAMLO+1,X)
         ORA RAMLO+1
         ORA RAMLO+1
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (RAMLO+1,X)
         ORA RAMLO+1
         ORA RAMLO+1
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (RAMLO+1,X)
         ORA RAMLO+1
         ORA RAMLO+1
L68DD:   BIT DiagTestNo
         AND ICBAHZ
         .BYTE $27,$27
L68E3:   .BYTE $04,$04
         ORA RAMLO+1
         ASL TRAMSZ
L68E9:   LDY #$03
         STY POKMSK
L68ED:   LDY #$05
L68EF:   LDA POKMSK
         ASL A
         ASL A
         STA APPMHI+1
         LDA L009C,Y
         CMP #$FF
         BEQ L6902
         AND #$0C
         CMP APPMHI+1
         BEQ L6908
L6902:   DEY
         BPL L68EF
         LDY POKMSK
         RTS
L6908:   DEC POKMSK
         BPL L68ED
         LDY #$FF
         RTS


;----------------------------------------------------
;          Shot fired (called from main loop)
;----------------------------------------------------
L690F:   LDA L009C              ;
         AND #$F0               ;
         BEQ L6917              ;
         BNE L697D              ;
L6917:   LDA VNTP+1             ;
         AND #$F0
         BEQ L691F
         BNE L697D
L691F:   LDA VTIMR4+1
         BNE L692D
         LDA SysClock+1
         AND #$07
         BNE L6936
         BEQ L6957
L692D:   LDA L1000              ; Get Joystick inputs
         EOR #$FF               ; Invert all bits
         AND #$20               ; Check Barrier button (S2)
         BNE L6957
L6936:   LDA SysClock
         AND #$20
         BNE L697D
         LDX #$80
         STX L38B9
         INX
         STX L38BA
         INX
         STX L38BB
         INX
         STX L38BC
         INX
         STX L38BD
         INX
         STX L38BE
         RTS
L6957:   LDA #$07
         JSR L51CE              ; Play Sound
         LDA #$30
         ORA VNTP+1
         STA VNTP+1
         LDA #$00               ; Start at first sprite picture
         STA BarrierPic         ; Reset the barrier picture sequence for Bega
         LDX #$06
L6969:   LDA L009B,X
         AND #$F0
         BNE L697A
         LDA #$30
         ORA L009B,X
         STA L009B,X
         LDA #$00
         STA HATABS+15,X
L697A:   DEX
         BPL L6969
L697D:   LDA #$00
         STA L38B9
         STA L38BA
         STA L38BB
         STA L38BC
         STA L38BD
         STA L38BE
         RTS



; ---------------------------------------------------
;                   Level Setup
; ---------------------------------------------------
L6992:   LDA #$00               ;
         STA VVBLKD             ;
         LDA CDTMV3             ; Have we completed level?
         BEQ L699F              ; Yes so go to next level
         JMP L69FE              ;
L699F:   INC Level              ; Next level
         JSR L719E              ; Screen Transition, Fill from outside in
; ---------------------------------------------------
;                Get data address 1 (kiwi)
; ---------------------------------------------------
L69A5:   LDA Level              ; Get level
         ASL A                  ; Multiply by 2
         TAX                    ; X = index into address table
         LDA LB3EA,X            ; Get data address (LOW)
         STA ZRAM_01            ; Save address in pointer
         LDA LB3EB,X            ; Get data address (HIGH)
         STA ZRAM_02            ; Save address in pointer + 1
         LDY VVBLKD             ; Get byte index (0-2)
         LDA (ZRAM_01),Y        ; Get Video Sequence byte 
         CMP #$FE               ; Are we at the next level?
         BEQ L6A01              ; Yes jump ahead for data address 2
         CMP #$FF               ; Is there a video sequence for this level?
         BNE L69C4              ; Yes, so skip ahead and play sequence
         JMP L6C1C              ; No video sequence so go do default
L69C4:   JSR L54A7              ; Play LaserDisc video sequence
L69C7:   LDA STICK1
         CMP #$01
         BEQ L69DA
         CMP #$02
         BEQ L69DA
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L69C7
L69DA:   JSR L71D0
         JSR L71EC
         JSR L71B8              ; Screen Transition, Clear from inside out
L69E3:   LDA STICK1
         CMP #$02
         BEQ L69F2
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         JMP L69E3
L69F2:   JSR L719E              ; Screen Transition, Fill from outside in
         LDA CDTMA1+1
         BNE L69FE
         LDA #$01
         BNE L6A24
L69FE:   JSR L719E              ; Screen Transition, Fill from outside in
; ---------------------------------------------------
;                Get data address 2
; ---------------------------------------------------
L6A01:   LDA #$00               ;
         STA CDTMV3             ;
         LDA Level              ; Get level
         ASL A                  ; Multiply by 2
         TAX                    ; X = index into address table
         LDA LB4DF,X            ; Get data address (LOW)
         STA ZRAM_01            ; Save address in pointer
         LDA LB4E0,X            ; Get data address (HIGH)
         STA ZRAM_02            ; Save address in pointer + 1
         LDY VVBLKD             ; Get byte index (0-2)
         LDA (ZRAM_01),Y        ; Get data byte 
         CMP #$FE               ; Are we at the next level?
         BNE L6A24              ; No so use this level
         INC Level              ; Go to next level
         JMP L69A5              ; Jump back and load address 3
L6A24:   JSR L54A7              ; Play LaserDisc video sequence
L6A27:   LDA STICK1
         CMP #$01
         BEQ L6A3A
         CMP #$02
         BEQ L6A3A
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         JMP L6A27
L6A3A:   JSR L71D0
         JSR L71EC
         JSR L71B8              ; Screen Transition, Clear from inside out
         LDX #$06
         LDA #$FF
L6A47:   STA L009B,X
         DEX
         BPL L6A47
         LDY #$40
         LDX Level 
         CPX #$0C
         BNE L6A5B
         LDA Lunas
         BPL L6A5B
         STY L009B
L6A5B:   LDA #$00
         STA L04BD
         LDX #$05
L6A62:   TXA
         ASL A
         TAY
         LDA VVTP+1,Y
         BMI L6A77
         INC L04BD
         LDA L04BD
         CMP #$02
         BNE L6A77
         JMP L6ACD
L6A77:   DEX
         BPL L6A62
; ---------------------------------------------------
;         Assign Processes based on Level
; ---------------------------------------------------
         LDY #$40               ; Y = Process Start Code
         LDX Level              ; Get Level
         BNE L6A85              ; Past Level 0, so skip ahead
; ---------------------------------------------------
;                 Level 0 Process
; ---------------------------------------------------
         LDA #$40               ; A = Process Start Code
         STA L009B              ; Start Process9B
; ---------------------------------------------------
;                 Level 1 Process
; ---------------------------------------------------
L6A85:   CPX #$01
         BNE L6A8F
         LDA VVTP+1
         BPL L6A8F
         STY L009C
; ---------------------------------------------------
;                 Level 13 Process
; ---------------------------------------------------
L6A8F:   CPX #$0D
         BNE L6A99
         LDA VVTP+1
         BPL L6A99
         STY L009C
; ---------------------------------------------------
;                 Level 4 Process
; ---------------------------------------------------
L6A99:   CPX #$04
         BNE L6AA3
         LDA STMCUR_swap
         BPL L6AA3
         STY L009D
; ---------------------------------------------------
;                 Level 14 Process
; ---------------------------------------------------
L6AA3:   CPX #$0E
         BNE L6AAF
         LDA STMCUR_swap
         BPL L6AAF
         STY L009D
         BNE L6AAF
; ---------------------------------------------------
;                 Level 5 Process
; ---------------------------------------------------
L6AAF:   CPX #$05
         BNE L6AB9
         LDA STARP+1
         BPL L6AB9
         STY L009F
; ---------------------------------------------------
;                 Level 6 Process
; ---------------------------------------------------
L6AB9:   CPX #$06
         BNE L6AC3
         LDA STARP+1
         BPL L6AC3
         STY L009F
; ---------------------------------------------------
;                Level 9 Process
; ---------------------------------------------------
L6AC3:   CPX #$09
         BNE L6ACD
         LDA RUNSTK+1
         BPL L6ACD
         STY L00A0
; ---------------------------------------------------
;                
; ---------------------------------------------------
L6ACD:   LDX #$FF               ; X = Cancel Process Code (FF)
         STX L0093              ; Cancel Process93
         STX L0094              ; Cancel Process94
         STX POKADR
         STX POKADR+1
         LDY #$00
         STY POKMSK
L6ADB:   LDA L009C,Y
         CMP #$FF
         BNE L6B01
         TYA
         ASL A
         TAX
         LDA VVTP+1,X
         CMP #$FF
         BEQ L6B01
         JSR L6CEF
         TXA
         AND #$01
         ASL A
         STA ZRAM_01
         LDA POKMSK
         AND #$03
         INC POKMSK
         ASL A
         ASL A
         ORA ZRAM_01
         STA L009C,Y
L6B01:   INY
         CPY #$06
         BNE L6ADB
         JSR L5535              ; Update Scoring Overlay
; ---------------------------------------------------
;             Erase Bank1 Sprites (7 sprites)
; ---------------------------------------------------
         LDX #$1B               ; Loop = 28 (4 x 7 sprites)
         LDA #$00               ; Clear with $00
L6B0D:   STA Sprite1_Prop,X     ; Erase sprite properties
         STA Sprite1_Char,X     ; Erase sprite character
         STA Sprite1_Y,X        ; Erase sprite Y coordinate
         STA Sprite1_X,X        ; Erase sprite X coordinate
         DEX                    ; Go to next sprite
         BPL L6B0D              ; Loop until all 7 sprites erased
; ---------------------------------------------------
;                
; ---------------------------------------------------
         LDA Level              ; Get Level number
         ASL A                  ; Mulitply by 2
         TAY                    ; Index based on Level
         LDA LB2F5,Y            ; Get address where level data is stored
         STA ZRAM_01            ; Save address
         LDA LB2F6,Y            ; Get address where level data is stored
         STA ZRAM_02            ; Save address
         LDY VVBLKD             ; Point to routine number
         LDA (ZRAM_01),Y        ; Get routine number
         STA CDTMA1             ; Save routine number
         ASL A                  ; Multiply by 2
         TAY                    ; Create index to routine address
         LDA L6CA7,Y            ; Get routine address LO
         STA L00CF              ; Setup routine address
         LDA L6CA8,Y            ; Get routine address HI
         STA L00D0              ; Setup routine address
         LDY CDTMA1             ; Y = Routine number
         LDA L6CD7,Y            ; Get Sound Code
         JSR L51CE              ; Go play sound
         LDA CDTMA1             ; Get routine number
         JSR L720B              ; set color ram?
;----------------------------------------------------
;               
;---------------------------------------------------- 
         LDX #$0D               ;
L6B4C:   LDA #$00               ;
         STA L041C,X            ;
         STA L042A,X            ;
         STA L0446,X            ;
         STA L0454,X            ;
         STA L0462,X
         STA L0470,X
         STA L047E,X
         STA L040E,X
         LDA #$FF
         STA L0400,X
         STA L0438,X
         DEX
         BPL L6B4C
;----------------------------------------------------
;               
;----------------------------------------------------
         LDX #$0A               ;
         LDA #$00               ;
L6B75:   STA L048C,X            ;
         STA L0497,X
         STA L04A2,X
         STA L04AD,X
         DEX
         BPL L6B75
;----------------------------------------------------
;               
;----------------------------------------------------
         LDX #$11               ;
         LDA #$00               ;
L6B88:   STA L04B8,X            ;
         DEX
         BPL L6B88
;----------------------------------------------------
;               
;----------------------------------------------------
         LDA #$00               ;
         STA L00A7              ;
         STA L00A9              ;
         LDA #$7F               ;
         STA L00AB              ;
;----------------------------------------------------
;               
;----------------------------------------------------
         LDX #$0D               ;
         LDA #$00               ;
L6B9C:   STA L04CA,X            ;
         DEX                    ;
         BPL L6B9C              ;
;----------------------------------------------------
;               Reset Bega and Shot   
;----------------------------------------------------
         LDA #$00               ;
         STA CDTMV5+1           ;
         JSR L6D10              ; Set Bega's starting position
         LDA #$00               ;
         STA L00D2              ;
         STA L00D1              ;
;----------------------------------------------------
;               
;----------------------------------------------------
         LDX #$09               ;
         LDA #$FF               ;
L6BB4:   STA DDEVIC,X           ;
         STA DAUX1,X            ;
         STA TEMP2,X            ;
         STA HATABS+4,X         ;
         DEX                    ;
         BPL L6BB4              ;
;----------------------------------------------------
;               
;----------------------------------------------------
         LDX #$00               ;
L6BC5:   LDA #$03               ;
         STA L3EE0,X            ;
         LDA #$FF               ;
         STA L3AE0,X            ;
         INX                    ;
         BNE L6BC5              ;
;----------------------------------------------------
;               
;----------------------------------------------------
         LDA #$74               ;
         STA L00A6              ;
         LDA #$00               ;
         STA DMASK              ;
         LDA Level              ; Get Level
         CMP #$0C               ; Are we at Level 12?
         BCC L6BF6              ; Not yet so skip ahead
         CMP #$15
         BCC L6BEE
         LDA #$02
         STA CDTMA2+1
         JMP L6BFB
L6BEE:   LDA #$01
         STA CDTMA2+1
         JMP L6BFB
L6BF6:   LDA #$00
         STA CDTMA2+1
;----------------------------------------------------
;              Do Difficulty Check
;----------------------------------------------------
L6BFB:   LDA L1002              ; Get DIP bank 2
         AND #$20               ; Check difficulty
         BEQ L6C08              ; Hard difficulty so jump ahead
         LDA CDTMA2+1           ;
         JMP L6C0E              ;
L6C08:   CLC
         LDA CDTMA2+1
         ADC #$04
L6C0E:   STA L04BA
         CLC
         LDA L04BA
         ADC L04D8
         STA L04BA
         RTS                    ;

L6C1C:   LDA #$13               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
L6C21:   LDA STICK1
         CMP #$01
         BEQ L6C34
         CMP #$02
         BEQ L6C34
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         JMP L6C21
L6C34:   JSR L71D0
         JSR L71EC
         JSR L71B8              ; Screen Transition, Clear from inside out
         JSR L5535              ; Update Scoring Overlay
         LDX #$72               ; Message = "" ($6C72)
         LDY #$6C               ; Message = "" ($6C72)
         JSR L4831              ; Print Text Message
L6C47:   LDA STICK1
         CMP #$02
         BEQ L6C56
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L6C47
L6C56:   JSR L719E              ; Screen Transition, Fill from outside in
         LDA #$01
         STA VIMIRQ+1
         JSR L459A              ; Erase Video Screen
         LDY PlayerNumber
         LDA #$FF
         STA Player1Lives,Y
         LDX #$FF
         STX L04DA
         TXS
         JMP L4145



;----------------------------------------------------
;               unknown data
;----------------------------------------------------
         ASL A
         .BYTE $3A
         ORA L1A1F,X
         .BYTE $0F,$1C
         BRK
         .BYTE $1A
         ASL DOSVEC+1,X
         .BYTE $23,$0F,$1C
         INC L3A67,X
         .BYTE $23
         ORA L561F,Y
         JSR APPMHI+1
         .BYTE $12,$0F
         ASL DiagMessage_HI,X
         .BYTE $0F
         ASL L0C00
         .BYTE $0F
         ORA (DOSVEC+1),Y
         INC L3AC9,X
         ORA L200B,X
         .BYTE $0F
         BRK
         ASL L0F12,X
         BRK
         AND (DiagMessage_LO,X)
         .BYTE $1C
         ASL APPMHI,X
         .BYTE $FF

; ---------------------------------------------------
;             Jump Routine Address
; ---------------------------------------------------
L6CA7:   .WORD L8A51,L84BF
         .WORD L84F4,L8486
         .WORD L8A86,L86AF
         .WORD L893B,L87E5
         .WORD L886C,L88F8
         .WORD L8974,L89AC
         .WORD L84BF,L8A51
         .WORD L8533,L8A86
         .WORD L86AF,L893B
         .WORD L8533,L886C
         .WORD L89AC,L8A51
         .WORD L844B,L8A86

; ---------------------------------------------------
;              Byte (0-23)
; ---------------------------------------------------
L6CD7:   .BYTE $09,$0D,$11,$10
         .BYTE $0F,$0E,$12,$0F
         .BYTE $0E,$14,$13,$13
         .BYTE $0D,$09,$0C,$0F
         .BYTE $0E,$12,$0C,$0F
         .BYTE $13,$09,$0C,$0F


; ---------------------------------------------------
;              
; ---------------------------------------------------
L6CEF:   LDX #$03               ;
L6CF1:   LDA L0093,X            ;
         CMP #$FF               ;
         BEQ L6CFD
         STA ZRAM_01
         CPY ZRAM_01
         BEQ L6D0D
L6CFD:   DEX
         BPL L6CF1
         INX
L6D01:   LDA L0093,X
         CMP #$FF
         BEQ L6D0D
         INX
         CPX #$04
         BNE L6D01
         BRK                    ;
L6D0D:   STY L0093,X            ;
         RTS                    ;



;----------------------------------------------------
;         Setup Bega's starting position 
;----------------------------------------------------
L6D10:   LDA #$40               ;
         STA VNTP+1             ;
         LDA #$00               ;
         STA VNTD               ;
         LDA #$01               ;
         LDX #$05               ;
L6D1C:   STA ICPTHZ,X           ;
         DEX
         BPL L6D1C
;----------------------------------------------------
;         Setup Bega's sprite pictures
;----------------------------------------------------
         LDX #$05               ; Loop = 6 sprites to setup
         LDY #$6F               ; Pictures = $6A - $6F 
L6D25:   STY FMSZPG,X           ; Set Bega sprite picture
         DEY                    ; Next picture
         DEX                    ; Next sprite
         BPL L6D25              ; Loop until all 6 sprites initialized
;----------------------------------------------------
;         Set Bega's starting position x,y
;----------------------------------------------------
         LDA #$70               ; XPos = 112
         STA BegaXPos1L         ; Set Bega's x position (left half)
         STA BegaXPos2L         ; Set Bega's x position (left half)
         STA BegaXPos3L         ; Set Bega's x position (left half)
         LDA #$80               ; XPos = 128
         STA BegaXPos1R         ; Set Bega's x position (right half)
         STA BegaXPos2R         ; Set Bega's x position (right half)
         STA BegaXPos3R         ; Set Bega's x position (right half)
         LDA CDTMA1+1
         BNE L6D5B
         JMP L6D5B
         LDA #$05
         JSR L51CE              ; Play Sound
         LDA #$10
         STA OLDADR+1
         STA FKDEF
         LDA #$20
         STA FKDEF+1
         STA PALNTS
         LDA #$30
         STA LOGCOL
         STA ADRESS
         RTS
L6D5B:   LDA #$B0
         STA OLDADR+1
         STA FKDEF
         LDA #$C0
         STA FKDEF+1
         STA PALNTS
         LDA #$D0
         STA LOGCOL
         STA ADRESS
         RTS




;----------------------------------------------------
;         (called from Main Loop)
;----------------------------------------------------
L6D6E:   LDA DMASK              ;
         BEQ L6D76              ;
         JMP L6DFE
L6D76:   LDA L009D
         AND #$F0
         BEQ L6D7E
         BNE L6DA4
L6D7E:   LDA VNTP+1
         AND #$F0
         BEQ L6D86
         BNE L6DA4
L6D86:   LDA VTIMR4+1
         BNE L6D94
         LDA SysClock+1
         CMP #$10
         BCC L6D9D
         BCS L6DD3
L6D94:   LDA L1000              ; Get Switch Input
         EOR #$FF               ; Toggle all bits
         AND #$10               ; Check S3, Service switch
         BNE L6DD3              ;
L6D9D:   LDA SysClock             ;
         AND #$20
         BNE L6DB9
L6DA4:   LDA #$00
         STA L38D9
         STA L38DA
         STA L38DB
         STA L38DC
         STA L38DD
         STA L38DE
         RTS
L6DB9:   LDX #$86
         STX L38D9
         INX
         STX L38DA
         INX
         STX L38DB
         INX
         STX L38DC
         INX
         STX L38DD
         INX
         STX L38DE
         RTS
L6DD3:   LDA #$08
         JSR L51CE              ; Play Sound
         LDA #$20
         ORA VNTP+1
         STA VNTP+1
         LDX #$06
L6DE0:   LDA L009B,X
         AND #$F0
         BNE L6DEC
         LDA #$20
         ORA L009B,X
         STA L009B,X
L6DEC:   DEX
         BPL L6DE0
         LDA #$00
         STA TMPLBT
         LDA #$1D
         STA ESCFLG
         LDA #$01
         STA DMASK
L6DFE:   LDA DMASK
         CMP #$01
         BEQ L6E08
         JMP L6F18
L6E08:   JSR L700E
         LDA TMPLBT
         CMP #$1E
         BEQ L6E13
         RTS
L6E13:   JSR L71D0
         LDX #$0D
L6E18:   LDA #$00
         STA L041C,X
         STA L042A,X
         STA L0446,X
         STA L0454,X
         STA L0462,X
         STA L0470,X
         STA L047E,X
         STA L040E,X
         LDA #$FF
         STA L0400,X
         STA L0438,X
         DEX
         BPL L6E18
         LDX #$0A
         LDA #$00
L6E41:   STA L048C,X
         STA L0497,X
         STA L04A2,X
         STA L04AD,X
         DEX
         BPL L6E41
         LDX #$11
         LDA #$00
L6E54:   STA L04B8,X
         DEX
         BPL L6E54
         LDA #$00
         STA L00A7
         STA L00A9
         LDA #$7F
         STA L00AB
         LDX #$0D
         LDA #$00
L6E68:   STA L04CA,X
         DEX
         BPL L6E68
         LDA #$00
         STA L00D2
         STA L00D1
         LDX #$09
         LDA #$FF
L6E78:   STA DDEVIC,X
         STA DAUX1,X
         STA TEMP2,X
         STA HATABS+4,X
         DEX
         BPL L6E78
         LDX #$0D
         LDA #$00
L6E8B:   STA Sprite1_Prop,X
         DEX
         BPL L6E8B
         LDA CDTMA2+1
         AND #$03
         ASL A
         ASL A
         ASL A
         ASL A
         ASL A
         STA ZRAM_01
         LDA Random             ; Get Random Number
         AND #$1F
         CLC
         ADC ZRAM_01
         TAY
         LDA L6F4E,Y
         STA Level              ; 
         LDA L6FAE,Y
         STA VVBLKD
         LDA Level 
         ASL A
         TAY
         LDA LB2F5,Y
         STA ZRAM_01
         LDA LB2F6,Y
         STA ZRAM_02
         LDY VVBLKD
         LDA (ZRAM_01),Y
         STA CDTMA1
         ASL A
         TAY
         LDA L6CA7,Y            ; Get routine address LO
         STA L00CF              ; Setup routine address
         LDA L6CA8,Y            ; Get routine address HI
         STA L00D0              ; Setup routine address
         LDA CDTMA1
         JSR L720B
         LDA #$05
         STA L04BB
         LDA Level 
         ASL A
         TAX
         LDA LB4DF,X
         STA ZRAM_01
         LDA LB4E0,X
         STA ZRAM_02
         LDY VVBLKD
         LDA (ZRAM_01),Y        ; Get Video Sequence
         JSR L54A7              ; Play LaserDisc video sequence
L6EF6:   LDA STICK1
         CMP #$01
         BEQ L6F09
         CMP #$02
         BEQ L6F09
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L6EF6
L6F09:   LDA #$1C
         STA TMPLBT
         LDA #$01
         STA ESCFLG
         LDA #$02
         STA DMASK
L6F18:   JSR L70A8
         LDA TMPLBT
         BMI L6F21
         RTS                    ; Return
L6F21:   JSR L5535              ; Update Scoring Overlay
         LDA VNTP+1
         AND #$F0
         CMP #$20
         BNE L6F32
         LDA #$0F
         AND VNTP+1
         STA VNTP+1
L6F32:   LDX #$06
L6F34:   LDA L009B,X
         AND #$F0
         CMP #$20
         BNE L6F42
         LDA #$0F
         AND L009B,X
         STA L009B,X
L6F42:   DEX
         BPL L6F34
         JSR L7BB5
         LDA #$00
         STA DMASK
         RTS
L6F4E:   ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
L6FAE:   BRK
         BRK
         BRK
         BRK
         BRK
L6FB3:   BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK


; ---------------------------------------------------
;                   Draw Box
; ---------------------------------------------------
L700E:   LDX TMPLBT             ; Get index (0-14)
; ---------------------------------------------------
;               Setup Boxes - layer 1
; ---------------------------------------------------
         LDA L7144,X            ; Get Left edge screen pointer (21 apart)
         STA ZRAM_01            ; Save EDGE_1
         STA RAMLO+1            ; Save EDGE_2
         LDA L7145,X            ; Get Left edge screen pointer (21 apart)
         STA ZRAM_02            ; Save EDGE_1
         STA TRAMSZ             ; Save EDGE_2
; ---------------------------------------------------
;            Setup Box Corners and Width
; ---------------------------------------------------
         LDA L7180,X            ; Get Upper Left pointer
         STA ZRAM_02+1          ; Save LOWER_RIGHT
         LDA L7181,X            ; Get Upper Left pointer
         STA RAMLO              ; Save LOWER_RIGHT
         LDA L7162,X            ; Get Upper Left pointer
         STA CMCMD              ; Save UPPER_LEFT
         LDA L7163,X            ; Get Upper Left pointer
         STA WARMST             ; Save UPPER_LEFT
         LDA ESCFLG             ; Get box width
         STA APPMHI             ; Save box width
; ---------------------------------------------------
;                  Draw Line
; ---------------------------------------------------
L7038:   LDA #$42               ;
         LDY APPMHI             ;
         STA (ZRAM_01),Y        ;
         STA (ZRAM_02+1),Y      ; Draw LOWER_RIGHT
         LDY #$00               ;
         STA (RAMLO+1),Y        ;
         STA (CMCMD),Y          ; Draw UPPER_LEFT
         CLC                    ;
         LDA #$20               ;
         ADC RAMLO+1
         STA RAMLO+1
         LDA #$00
         ADC TRAMSZ
         STA TRAMSZ
         CLC
         LDA #$20
         ADC CMCMD
         STA CMCMD
         LDA #$00
         ADC WARMST
         STA WARMST
         DEC APPMHI
         BPL L7038
; ---------------------------------------------------
;              Setup Screen Pointers
; ---------------------------------------------------
         LDA #$20
         STA ZRAM_01
         LDA #$28
         STA ZRAM_02
         LDA #$3F
         STA ZRAM_02+1
         LDA #$28               ;
         STA RAMLO              ;
         LDX #$1D               ;
         LDY #$00               ;
; ---------------------------------------------------
;                   Draw Line
; ---------------------------------------------------
L7078:   LDA #$42               ; Char = |
         STA (ZRAM_01),Y        ; Clear screen left edge
         STA (ZRAM_02+1),Y      ; Clear screen right edge
         CLC                    ; Clear carry for addition
         LDA #$20               ; Move down one row (add 32)
         ADC ZRAM_01            ; Add to LEFT edge pointer
         STA ZRAM_01            ; Save LEFT edge pointer
         LDA #$00               ; Add carry to high byte
         ADC ZRAM_02            ; Add to LEFT edge pointer
         STA ZRAM_02            ; Save LEFT edge pointer
         CLC                    ; Clear carry for addition
         LDA #$20               ; Move down one row (add 32)
         ADC ZRAM_02+1          ; Add to RIGHT edge pointer
         STA ZRAM_02+1          ; Save RIGHT edge pointer
         LDA #$00               ; Add carry to high byte
         ADC RAMLO              ; Add to RIGHT edge pointer
         STA RAMLO              ; Save RIGHT edge pointer
         DEX
         BPL L7078
         DEC ESCFLG
         DEC ESCFLG
         INC TMPLBT
         INC TMPLBT
         RTS                    ; Return



; ---------------------------------------------------
;                   Erase Box
; ---------------------------------------------------
L70A8:   LDX TMPLBT             ;
         LDA L7144,X            ;
         STA ZRAM_01            ;
         STA RAMLO+1
         LDA L7145,X
         STA ZRAM_02
         STA TRAMSZ
         LDA L7180,X
         STA ZRAM_02+1
         LDA L7181,X
         STA RAMLO
         LDA L7162,X
         STA CMCMD
         LDA L7163,X
         STA WARMST
         LDA ESCFLG
         STA APPMHI
L70D2:   LDA #$00
         LDY APPMHI
         STA (ZRAM_01),Y
         STA (ZRAM_02+1),Y
         LDY #$00
         STA (RAMLO+1),Y
         STA (CMCMD),Y
         CLC
         LDA #$20
         ADC RAMLO+1
         STA RAMLO+1
         LDA #$00
         ADC TRAMSZ
         STA TRAMSZ
         CLC
         LDA #$20
         ADC CMCMD
         STA CMCMD
         LDA #$00
         ADC WARMST
         STA WARMST
         DEC APPMHI
         BPL L70D2
         INC ESCFLG
         INC ESCFLG
         DEC TMPLBT
         DEC TMPLBT
         BPL L7143              ; Yes, so leave
; ---------------------------------------------------
;             Clear Edges of Text Screen
; ---------------------------------------------------
         LDA #$20               ; Address = $2820
         STA ZRAM_01            ; Set left edge
         LDA #$28               ; Address = $2820
         STA ZRAM_02            ; Set left edge
         LDA #$3F               ; Address = $283F
         STA ZRAM_02+1          ; Set right edge
         LDA #$28               ; Address = $283F
         STA RAMLO              ; Set right edge
         LDX #$1D               ; Loop = 30 Rows
         LDY #$00               ; Index = 0
L7120:   LDA #$00               ; Char = SPACE
         STA (ZRAM_01),Y        ; Clear screen left edge
         STA (ZRAM_02+1),Y      ; Clear screen right edge
         CLC                    ; Clear carry for addition
         LDA #$20               ; Move down one row (add 32)
         ADC ZRAM_01            ; Add to LEFT edge pointer
         STA ZRAM_01            ; Save LEFT edge pointer
         LDA #$00               ; Add carry to high byte
         ADC ZRAM_02            ; Add to LEFT edge pointer
         STA ZRAM_02            ; Save LEFT edge pointer
         CLC                    ; Clear carry for addition
         LDA #$20               ; Move down one row (add 32)
         ADC ZRAM_02+1          ; Add to RIGHT edge pointer
         STA ZRAM_02+1          ; Save RIGHT edge pointer
         LDA #$00               ; Add carry to high byte
         ADC RAMLO              ; Add to RIGHT edge pointer
         STA RAMLO              ; Save RIGHT edge pointer
         DEX                    ; Have we reached to bottom?
         BPL L7120              ; No, loop back and continue to erase
L7143:   RTS                    ; Return



; ---------------------------------------------------
;            Text Screen Locations (15 each)
; ---------------------------------------------------   
L7144:   .WORD $2821,$2842,$2863,$2884,$28A5   ; $21 apart
         .WORD $28C6,$28E7,$2908,$2929,$294A
         .WORD $296B,$298C,$29AD,$29CE,$29EF

L7162:   .WORD $283E,$285D,$287C,$289B,$28BA   ; $1F apart
         .WORD $28D9,$28F8,$2917,$2936,$2955   ; Upper Left Corner
         .WORD $2974,$2993,$29B2,$29D1,$29F0

L7180:   .WORD $2BC1,$2BA2,$2B83,$2B64,$2B45   ; $1F backward
         .WORD $2B26,$2B07,$2AE8,$2AC9,$2AAA   ; Lower Right Corner
         .WORD $2A8B,$2A6C,$2A4D,$2A2E,$2A0F



; ---------------------------------------------------
;    Transitional Effect - Fill from outside in
; ---------------------------------------------------
L719E:   LDA #$00               ; Loop = 0
         STA TMPLBT             ; Set Loop (0-30)
         LDA #$1D               ; Height = 30 rows
         STA ESCFLG             ; Set Box Width
L71A8:   JSR L700E              ; Draw box
         LDX #$01               ; Delay = 1
         JSR L4BD2              ; Run Programmable Delay
         LDA TMPLBT             ; Check Loop
         CMP #$1E               ; Have we finished 30 rows?
         BNE L71A8              ; No, so loop back and continue
         RTS                    ; Return


;----------------------------------------------------
;     Transitional Effect - Clear from inside out
;----------------------------------------------------
L71B8:   LDA #$1C               ;
         STA TMPLBT             ;
         LDA #$01               ;
         STA ESCFLG             ;
L71C2:   JSR L70A8              ; Erase box
         LDX #$01               ; Delay = 1
         JSR L4BD2              ; Run Programmable Delay
         LDA TMPLBT             ; Have we finished all rows?
         BPL L71C2              ; No, so loop back and continue
         RTS                    ; Return


;----------------------------------------------------
;       Clear Video RAM (2800-281F,2BE0-2BFF)
;----------------------------------------------------
L71D0:   LDA #$00               ; Clear RAM
         LDX #$1F               ; Loop = 1F
L71D4:   STA L2800,X            ; Erase 2800-281F
         STA L2BE0,X            ; Erase 2BE0-2BFF
         DEX                    ; Decrement loop
         BPL L71D4              ; Loop until RAM cleared
         RTS                    ; Return


;----------------------------------------------------
;       Clear Video RAM (3800-381F,3BE0-3BFF)
;----------------------------------------------------
L71DE:   LDA #$00               ; Clear RAM
         LDX #$1F               ; Loop = 1F
L71E2:   STA L3800,X            ; Erase 3800-381F
         STA L3BE0,X            ; Erase 3BE0-3BFF
         DEX                    ; Decrement loop
         BPL L71E2              ; Loop until RAM cleared
         RTS                    ; Return


;----------------------------------------------------
;             Erase Sprite RAM (3800-3FFF)
;----------------------------------------------------
L71EC:   LDX #$00               ;
         TXA                    ;
L71EF:   STA L3800,X            ;
         STA L3900,X
         STA L3A00,X            ;
         STA L3B00,X            ;
         STA L3C00,X
         STA L3D00,X
         STA L3E00,X
         STA L3F00,X
         DEX
         BNE L71EF
         RTS                    ; Return



;----------------------------------------------------
;               unknown
;----------------------------------------------------
;               A = 0 - 147
;----------------------------------------------------
L720B:   BPL L721E              ;
L720D:   LDA L1003              ;
         BMI L720D              ;
         LDY #$37               ;
L7214:   LDA L4428,Y            ;
         STA L1800,Y            ; Write Video Color RAM
         DEY
         BPL L7214
         RTS
L721E:   ASL A
         TAY
         LDA L723A,Y
         STA ZRAM_01
         LDA L723B,Y
         STA ZRAM_02
L722A:   LDA L1003
         BMI L722A
         LDY #$37
L7231:   LDA (ZRAM_01),Y
         STA L1800,Y            ; Copy data here
         DEY                    ;
         BPL L7231              ;
         RTS                    ; Return



;----------------------------------------------------
;           unknown data pointers (0-147)
;----------------------------------------------------
L723A:   .WORD $735A,$7392,$73CA,$7402,$743A,$7472,$74AA,$74E2
         .WORD $751A,$7552,$758A,$75C2,$75FA,$7632,$766A,$76A2
         .WORD $76DA,$7712,$774A,$7782,$77BA,$77F2,$782A,$7862
         .WORD $789A,$78D2,$790A,$7942,$797A,$79B2,$79EA,$7A22
         .WORD $7A5A,$7A92,$7ACA,$7B02,$735A,$7392,$73CA,$7402
         .WORD $743A,$7472,$74AA,$74E2,$751A,$7552,$758A,$75C2
         .WORD $75FA,$7632,$766A,$76A2,$76DA,$7712,$774A,$7782
         .WORD $77BA,$77F2,$782A,$7862,$789A,$78D2,$790A,$7942
         .WORD $797A,$79B2,$79EA,$7A22,$7A5A,$7A92,$7ACA,$7B02
         .WORD $735A,$7392,$73CA,$7402,$743A,$7472,$74AA,$74E2
         .WORD $751A,$7552,$758A,$75C2,$75FA,$7632,$766A,$76A2
         .WORD $76DA,$7712,$774A,$7782,$77BA,$77F2,$782A,$7862
         .WORD $789A,$78D2,$790A,$7942,$797A,$79B2,$79EA,$7A22
         .WORD $7A5A,$7A92,$7ACA,$7B02,$735A,$7392,$73CA,$7402
         .WORD $743A,$7472,$74AA,$74E2,$751A,$7552,$758A,$75C2
         .WORD $75FA,$7632,$766A,$76A2,$76DA,$7712,$774A,$7782
         .WORD $77BA,$77F2,$782A,$7862,$789A,$78D2,$790A,$7942
         .WORD $797A,$79B2,$79EA,$7A22,$7A5A,$7A92,$7ACA,$7B02               

; ---------------------------------------------------
;             unknown color RAM data
; ---------------------------------------------------
L735A:   .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00

L7392:   .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00

L73CA:   .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00

L782A:   .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00 
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
L797A:   .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
L7A78:   .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00 
L7ACA:   .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F1,$C7,$C0,$3F,$38,$07,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$D5,$9B,$90,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         .BYTE $FF,$F8,$FF,$C0,$3F,$9B,$49,$00
         
          


; ---------------------------------------------------
;                     unknown
; ---------------------------------------------------
L7B3A:   LDA Lunas
         BMI L7B41
         JMP L7B68
L7B41:   LDA Level 
         CMP #$05
         BEQ L7B5F
         CMP #$10
         BEQ L7B5F
         CMP #$16
         BEQ L7B5F
         CMP #$1B
         BEQ L7B5F
         CMP #$21
         BEQ L7B5F
         CMP #$09
         BEQ L7B64
         JMP L7B68
L7B5F:   LDA VVBLKD
         BNE L7B68
L7B64:   LDA #$40
         STA L009B
L7B68:   RTS
         LDA VVTP+1
         BMI L7B70
         JMP L7B8E
L7B70:   LDA Level 
         CMP #$12
         BEQ L7B85
         CMP #$18
         BEQ L7B85
         CMP #$1F
         BEQ L7B85
         CMP #$23
         BEQ L7B85
         BNE L7B8E
L7B85:   LDA VVBLKD
         BNE L7B8E
         LDA #$40
         STA L009C
L7B8E:   RTS
         LDA STMCUR_swap
         BMI L7B96
         JMP L7BB4
L7B96:   LDA Level 
         CMP #$11
         BEQ L7BAB
         CMP #$17
         BEQ L7BAB
         CMP #$1C
         BEQ L7BAB
         CMP #$22
         BEQ L7BAB
         BNE L7BB4
L7BAB:   LDA VVBLKD
         BNE L7BB4
         LDA #$40
         STA L009D
L7BB4:   RTS



; ---------------------------------------------------
;                     unknown
; ---------------------------------------------------
L7BB5:   LDA L009D
         AND #$0C
         LSR A
         TAX
         LDA #$00
         STA SpritePic1,X
         STA SpritePic2,X
         LDA #$FF
         STA STMCUR_swap
         STA L009D
         LDX #$01
         LDY #$00
         CPX L0093
         BEQ L7BDA
         INY
         CPX L0094
         BEQ L7BDA
         INY
         CPX POKADR
         BEQ L7BDA
         INY
L7BDA:   CPY #$02
         BCC L7BE6
         LDA #$FF
         STA L0093,Y
         JMP L7C05
L7BE6:   CPY #$00
         BNE L7BF9
         LDA POKADR
         CMP #$FF
         BEQ L7C05
         STA L0093
         LDA #$FF
         STA POKADR
         JMP L7C05
L7BF9:   LDA POKADR+1
         CMP #$FF
         BEQ L7C05
         STA L0094
         LDA #$FF
         STA POKADR+1
L7C05:   RTS                    ; Return



;----------------------------------------------------
;----------------------------------------------------
;              UNUSED (duplicate of $40A1)
;----------------------------------------------------
;----------------------------------------------------
L7C06:   JSR L4501              ; Determine number of Begas
         JSR L452D              ; Clear variables, Clear "swap variables"
         LDA #$01
         STA VIMIRQ
         STA PADDL1
         LDA #$00               ; Clock = 0
         STA SysClock           ; Reset System Clock
         STA SysClock+1         ; Reset System Clock
         STA NumPlayers         ; Clear number of players
L7C1F:   LDA #$00
         JSR L51CE              ; Play Sound
         JSR L6992
         LDA #$00
         STA PADDL0
         JSR L80E8
L7C2F:   LDA L1003
         BPL L7C2F
         LDY #$05
L7C36:   LDA L009C,Y
         BPL L7C41
         CMP #$FF
         BEQ L7C41
         BNE L7C4E
L7C41:   DEY
         BPL L7C36
         LDA STICK1
         CMP #$02
         BNE L7C4E
         JMP L7CB2
L7C4E:   JSR L47D0              ; Update Sprites
         INC SysClock
         LDA SysClock
         AND #$3F
         BNE L7C61
         INC SysClock+1
         INC L04BB
;----------------------------------------------------
;          Pseudo-random Number Generator
;----------------------------------------------------
L7C61:   INC RandSeed           ; Increment seed ROM pointer
         BCC L7C67              ; Check if carry is set
         INC RandSeed+1         ; Increment seed ROM pointer+1
L7C67:   LDA RandSeed+1         ; Get seed ROM pointer
         AND #$07               ; Set bounds ($4000-$4700)
         ORA #$40               ; Set bounds ($4000-$4700)
         STA RandSeed+1         ; Limit seed ROM pointer+1
         INC Random             ; Increment the random number
         LDA L1000              ; Mix the number with any joystick presses
         ADC Random             ; Add previous random number
         LDY RandSeed           ; Use seed ROM pointer
         ADC (RandSeed),Y       ; Add ROM data byte to random number
         ADC L3BFB              ; Add Y-raster position(?)
         STA Random             ; Save random number
;----------------------------------------------------
;         
;----------------------------------------------------
L7C82:   LDA L1003              ;
         BMI L7C82              ;
         JSR L45D7              ; Print "1UP" or "2UP"
         JSR L7D9B              ; lost a life?
         JSR L5E23
         JSR L6325
         JSR L4891
         JSR L690F
         JSR L6D6E
         LDA CDTMA1+1
         BEQ L7CAA
         LDA #$7C
         PHA
         LDA #$A9
         PHA
         JMP (L00CF)
L7CAA:   LDA VIMIRQ+1
         BNE L7CB2
         JMP L7C2F
L7CB2:   LDA #$00
         STA VIMIRQ+1
         STA VVBLKD
         LDA Level 
         BNE L7CC9
         INC Level 
         LDA #$08
         STA Lunas
         JMP L7C1F
L7CC9:   CMP #$03
         BCS L7CD7
         INC Level 
         LDA #$00
         STA VVTP+1
         JMP L7C1F
L7CD7:   RTS
         LDA #$00
         JSR L51CE              ; Play Sound
         LDA #$05
         STA Level 
         LDA #$00
         STA VVBLKD
         STA VVTP+1
         LDA #$08
         STA Lunas
L7CED:   LDA #$00
         STA SysClock
         STA SysClock+1
         JSR L6992
         LDA #$00
         STA PADDL0
         JSR L80E8
L7D00:   LDA L1003
         BPL L7D00
         LDY #$05
L7D07:   LDA L009C,Y
         BPL L7D12
         CMP #$FF
         BEQ L7D12
         BNE L7D1F
L7D12:   DEY
         BPL L7D07
         LDA STICK1
         CMP #$02
         BNE L7D1F
         JMP L7D83
L7D1F:   JSR L47D0              ; Update Sprites
         INC SysClock
         LDA SysClock
         AND #$3F
         BNE L7D32
         INC SysClock+1
         INC L04BB
;----------------------------------------------------
;          Pseudo-random Number Generator
;----------------------------------------------------
L7D32:   INC RandSeed           ; Increment seed ROM pointer
         BCC L7D38              ; Check if carry is set
         INC RandSeed+1         ; Increment seed ROM pointer+1
L7D38:   LDA RandSeed+1         ; Get seed ROM pointer
         AND #$07               ; Set bounds ($4000-$4700)
         ORA #$40               ; Set bounds ($4000-$4700)
         STA RandSeed+1         ; Limit seed ROM pointer+1
         INC Random             ; Increment the random number
         LDA L1000              ; Mix the number with any joystick presses
         ADC Random             ; Add previous random number
         LDY RandSeed           ; Use seed ROM pointer
         ADC (RandSeed),Y       ; Add ROM data byte to random number
         ADC L3BFB              ; Add Y-raster position(?)
         STA Random             ; Save random number
;----------------------------------------------------
;          
;----------------------------------------------------
L7D53:   LDA L1003
         BMI L7D53
         JSR L45D7              ; Print "1UP" or "2UP"
         JSR L7D9B
         JSR L5E23
         JSR L6325
         JSR L4891
         JSR L690F
         JSR L6D6E
         LDA CDTMA1+1
         BEQ L7D7B
         LDA #$7D
         PHA
         LDA #$7A
         PHA
         JMP (L00CF)
L7D7B:   LDA VIMIRQ+1
         BNE L7D83
         JMP L7D00
L7D83:   LDA #$00
         STA VIMIRQ+1
         STA VVBLKD
         LDA Level 
         CMP #$07
         BCS L7D9A
         LDA #$00
         STA PADDL1
         JMP L7CED
L7D9A:   RTS
L7D9B:   LDA VNTP+1
         BPL L7DA2
         JMP L807D
L7DA2:   ASL A
         BPL L7DA8
         JMP L7EBB
L7DA8:   DEC PADDL2
         LDA PADDL2
         BNE L7DB3
         JSR L80E8
L7DB3:   LDA GPRIOR
         BNE L7DBB
         JMP L7E5D
L7DBB:   LSR A
         BCS L7DC1
         JMP L7E15
L7DC1:   LDA BegaXPos1L
         CMP #$08
         BCS L7DCA
         JMP L7E5D
L7DCA:   LDA L0093
         CMP #$FF
         BEQ L7DE5
         LDA BegaXPos1L
         CMP #$18
         BCS L7DD9
         JMP L7E5D
L7DD9:   LDA POKADR
         CMP #$FF
         BEQ L7DE5
         LDA BegaXPos1L
         CMP #$28
         BCC L7E5D
L7DE5:   LDX #$05
L7DE7:   LDA BegaXPos1L,X
         SEC
         SBC L5C51
         STA BegaXPos1L,X
         DEX
         BPL L7DE7
         LDX #$05
L7DF4:   LDA L009C,X
         AND #$F0
         BNE L7E00
         LDA #$FE
         AND L009C,X
         STA L009C,X
L7E00:   DEX
         BPL L7DF4
         LDA VNTP+1
         AND #$30
         BEQ L7E0C
         JMP L7E5D
L7E0C:   LDA #$FD
         AND VNTP+1
         STA VNTP+1
         JMP L7E5D
L7E15:   LDA BegaXPos1L
         CMP #$D9
         BCS L7E4C
         LDA L0094
         CMP #$FF
         BEQ L7E33
         LDA BegaXPos1L
         CMP #$C9
         BCS L7E4C
         LDA POKADR+1
         CMP #$FF
         BEQ L7E33
         LDA BegaXPos1L
         CMP #$B9
         BCS L7E4C
L7E33:   LDX #$05
L7E35:   LDA BegaXPos1L,X
         CLC
         ADC L5C51
         STA BegaXPos1L,X
         DEX
         BPL L7E35
         LDA VNTP+1
         AND #$30
         BNE L7E4C
         LDA #$02
         ORA VNTP+1
         STA VNTP+1
L7E4C:   LDX #$05
L7E4E:   LDA L009C,X
         AND #$F0
         BNE L7E5A
         LDA #$01
         ORA L009C,X
         STA L009C,X
L7E5A:   DEX
         BPL L7E4E
L7E5D:   LDA VNTP+1             ; Check which Bega to show
         AND #$20               ; Is it Barrier Bega?
         BEQ L7E66              ; No, so skip ahead
         JMP L803E              ; Show Barrier Bega
L7E66:   LDA VNTP+1             ; Check which Bega to show
         AND #$10               ; Is it Normal Bega?
         BEQ L7E6F              ; No, so skip ahead
         JMP L8014              ; Show Normal Bega
L7E6F:   LDA VNTP+1
         AND #$02
         ASL A
         ORA #$01
         LDX #$05
L7E78:   STA ICPTHZ,X
L7E7A:   DEX
         BPL L7E78
         LDA VNTD
         BNE L7E97
         LDA #$02
         STA VNTD
         LDA VNTP+1
         AND #$01               ;
         EOR #$01               ;
         TAX                    ;
         LDA #$FE               ;
         AND VNTP+1             ;
         STA VNTP+1             ;
         TXA                    ;
         ORA VNTP+1
         STA VNTP+1
L7E97:   LDA SysClock
         AND #$03
         BNE L7EA0
         DEC VNTD
L7EA0:   LDA VNTP+1
         AND #$03
         ASL A
         TAY
         LDA L5C52,Y
         STA ZRAM_01
         LDA L5C53,Y
         STA ZRAM_02
         LDY #$05               ; 6 sprites to update
L7EB2:   LDA (ZRAM_01),Y        ; Get picture
         STA FMSZPG,Y           ; Set Bega sprite picture
         DEY                    ; Next picture
         BPL L7EB2              ; Loop until all 6 sprites updated
         RTS
L7EBB:   LDA VNTD
         BNE L7EC3
         LDA #$01
         STA VNTD
L7EC3:   LDA SysClock
         AND #$01
         BNE L7ECC
         DEC VNTD
L7ECC:   BEQ L7ECF
         RTS
L7ECF:   LDA VNTP+1
         AND #$3F
         BNE L7F01
         LDX #$05
L7ED7:   CLC
         LDA OLDADR+1,X
         ADC L5C50
         STA OLDADR+1,X
         DEX
         BPL L7ED7
         LDA OLDADR+1
         CMP #$B8
         BCS L7EE9
         RTS
L7EE9:   LDA #$06
         JSR L51CE              ; Play Sound
         LDX #$05
         LDY #$75
L7EF2:   STY FMSZPG,X
         DEY
         DEX
         BPL L7EF2
         LDA #$41
         STA VNTP+1
         LDA #$10
         STA VNTD
         RTS
L7F01:   LDX #$05               ; 6 sprites to update
L7F03:   LDA L5C88,X            ; Get picture
         STA FMSZPG,X           ; Set Bega sprite picture
         DEX                    ; Next picture
         BPL L7F03              ; Loop until all 6 sprites updated
         LDA #$00
         STA VNTP+1
;----------------------------------------------------
;           Print STAGE Number to screen
;----------------------------------------------------
         LDX #$23               ; Message = "STAGE" ($4D23)
         LDY #$4D               ; Message = "STAGE" ($4D23)
         JSR L4831              ; Print Text Message
         LDA Level 
         ASL A
         TAX
         LDA LB200,X            ; Get Level address
         STA ZRAM_01            ; Save address
         LDA LB201,X            ; Get Level address
         STA ZRAM_02            ; Save address
         LDY VVBLKD             ; Y = 0,1,2
         LDA (ZRAM_01),Y        ; Get STAGE Number
         TAY                    ; Temporarily save STAGE number
         AND #$0F               ; Get one's place digit
         CLC                    ; Clear carry
         ADC #$01               ; Make printable character
         STA L3993              ; Print STAGE Number (one's place)
         TYA                    ; Retrieve STAGE number
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         LSR A                  ; Move digits over
         CLC                    ; Clear carry
         ADC #$01               ; Make printable character
         STA L3992              ; Print STAGE Number (tens place)
         LDA NumPlayers
         BEQ L7F53
         LDX #$50               ; Message = "PLAYER" ($4D50)
         LDY #$4D               ; Message = "PLAYER" ($4D50)
         JSR L4831              ; Print Text Message
         CLC
         LDA PlayerNumber
         ADC #$02
         STA L3913
L7F53:   LDA CDTMA1+1
         BNE L7FB1
         LDX #$00
         STX SRTIMR
L7F5D:   LDA STICK1
         CMP #$02
         BEQ L7F81
         LDX #$CB
         LDY #$5D
         INC SRTIMR
         LDA SRTIMR
         AND #$30
         BEQ L7F76
         LDX #$F4               ; Message = "xxxx" ($5CF4)
         LDY #$5C               ; Message = "xxxx" ($5CF4)
L7F76:   JSR L4831              ; Print Text Message
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L7F5D
L7F81:   LDX #$CB               ; Message = "xxxx" ($5DCB)  
         LDY #$5D               ; Message = "xxxx" ($5DCB)
         JSR L4831              ; Print Text Message
         JSR L719E              ; Screen Transition, Fill from outside in
         LDA #$18               ; Video Sequence = ? (Frame xxxxx)
         JSR L54A7              ; Play LaserDisc video sequence
         LDA #$01
         STA CDTMA1+1
L7F95:   LDA STICK1
         CMP #$01
         BEQ L7FA8
         CMP #$02
         BEQ L7FA8
         LDX #$01
         JSR L4BD2              ; Run Programmable Delay
         JMP L7F95
L7FA8:   JSR L71D0
         JSR L71B8              ; Screen Transition, Clear from inside out
         JMP L7FFE
L7FB1:   LDX #$00
         STA SRTIMR
L7FB6:   LDX #$CB
         LDY #$5D
         LDA SRTIMR
         AND #$30
         BEQ L7FF1
         LDA Level 
         BEQ L7FDF
         LDA CDTMA2+1
         BNE L7FF1
         LDA Lunas
         BMI L7FF1
         LDA Level 
         CMP #$02
         BEQ L7FE6
         LDA Level 
         CMP #$06
         BEQ L7FED
         BNE L7FF1
L7FDF:   LDX #$F4               ; Message = "TOUCH TO RESCUE LUNA!" ($5CF4)
         LDY #$5C               ; Message = "TOUCH TO RESCUE LUNA!" ($5CF4)
         JMP L7FF1
L7FE6:   LDX #$3F
         LDY #$5D
         JMP L7FF1
L7FED:   LDX #$84               ; Message = "xxxx" ($5D84) 
         LDY #$5D               ; Message = "xxxx" ($5D84)
L7FF1:   JSR L4831              ; Print Text Message
         LDX #$01               ; Delay Count = 1
         JSR L4BD2              ; Run programmable Delay
         INC SRTIMR
         BPL L7FB6
L7FFE:   LDX #$CB               ; Message = "xxxx" ($5DCB)
         LDY #$5D               ; Message = "xxxx" ($5DCB)
         JSR L4831              ; Print Text Message
         LDX #$38               ; Message = "" ($4D38)
         LDY #$4D               ; Message = "" ($4D38)
         JSR L4831              ; Print Text Message
         LDX #$59               ; Message = "        " ($4D59)
         LDY #$4D               ; Message = "        " ($4D59)
         JSR L4831              ; Print Text Message
         RTS                    ; Return



;----------------------------------------------------
;                    unknown
;----------------------------------------------------
L8014:   LDA VNTP+1             ;
         AND #$03               ;
         BEQ L8039              ;
         CMP #$03               ;
         BEQ L8028
         LDA SysClock
         AND #$07
         BNE L8027
         DEC VNTP+1
L8027:   RTS
L8028:   LDX #$05
         LDY #$01
L802C:   LDA L5CA0,X
         STA FMSZPG,X
         STY ICPTHZ,X
         DEX
         BPL L802C
         DEC VNTP+1
         RTS
L8039:   LDA #$00
         STA VNTP+1
         RTS




;----------------------------------------------------
;             Show Barrier Bega!   
;----------------------------------------------------
L803E:   LDA VNTP+1             ;
         AND #$30               ;
         CMP #$30               ;
         BEQ L8049              ;
         JMP L80E7              ; Just leave
L8049:   LDA SysClock
         AND #$03
         BEQ L8051
         RTS                    ; Return
;----------------------------------------------------
;     Find next sequence for Bega Barrier picture
;----------------------------------------------------
L8051:   LDA BarrierPic         ; Get Bega Barrier picture sequence
         ASL A                  ; Multiply by 2
         TAY                    ; Create an index
         LDA L5C5A,Y            ; Get Barrier picture pointer
         STA ZRAM_01            ; Save pointer
         LDA L5C5B,Y            ; Get Barrier picture pointer
         STA ZRAM_02            ; Save pointer
         AND ZRAM_01            ; Check if this is the last in the sequence
         CMP #$FF               ; Is this the last picture?
         BEQ L8078              ; Yes, so just skip ahead
         INC BarrierPic         ; Go to the next picture in the sequence
;----------------------------------------------------
;            Update Bega Barrier picture
;----------------------------------------------------
         LDY #$05               ; 6 sprites to update
         LDX #$01               ; 
L806D:   LDA (ZRAM_01),Y        ; Get picture
         STA FMSZPG,Y           ; Set Bega sprite picture
         STX ICPTHZ,X          
         DEY                    ; Next picture
         BPL L806D              ; Loop until all 6 sprites updated
         RTS                    ; Return
;----------------------------------------------------
;            Restart barrier sequence
;----------------------------------------------------
L8078:   LDA #$00               ; Restart barrier sequence
         STA VNTP+1             ; Save barrier sequence
         RTS                    ; Return


;----------------------------------------------------
;         
;----------------------------------------------------
L807D:   LDA VNTP+1
         AND #$7F
         BNE L8093
         LDX PlayerNumber
         DEC Player1Lives,X
         LDA #$01
         STA VNTD
         INC VNTP+1
         LDA #$03
         JSR L51CE              ; Play Sound
L8093:   LDA SysClock
         AND #$03               ;
         BEQ L809B              ;
         RTS                    ;
L809B:   DEC VNTD               ;
         BEQ L80A0              ;
         RTS
L80A0:   LDA #$02
         STA VNTD
         LDA VNTP+1
         AND #$7F
         ASL A
         TAX
         LDA L5C72,X
         STA ZRAM_01
         LDA L5C73,X
L80B2:   STA ZRAM_02
         LDY #$05
L80B6:   LDA (ZRAM_01),Y
         STA FMSZPG,Y
         LDA #$01
         STA ICPTHZ,Y
         DEY
         BPL L80B6
         INC VNTP+1
         LDA VNTP+1
         AND #$7F
         CMP #$0B
         BEQ L80CE
         RTS
L80CE:   LDA #$00
         STA VNTP+1
         LDX #$05
         LDA #$00
L80D6:   STA ICPTHZ,X
         STA FMSZPG,X
         DEX
         BPL L80D6
         LDA #$01
         STA VIMIRQ+1
         LDX #$25
         JSR L4BD2              ; Run Programmable Delay
L80E7:   RTS


L80E8:   LDA Level 
         AND #$07
         TAX
         LDY L811B,X
         LDA L8123,Y
         STA ZRAM_01
         LDA L8124,Y
         STA ZRAM_02
         LDA PADDL0
         ASL A
         CLC
         ADC ZRAM_01
         STA ZRAM_01
         LDA ZRAM_02
         ADC #$00
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA GPRIOR
         INY
         LDA (ZRAM_01),Y
         STA PADDL2
         INC PADDL0
         RTS



;----------------------------------------------------
;                unknown data
;----------------------------------------------------
L811B:   BRK
         BRK
         .BYTE $02,$04
         ASL L0000
         PHP
         ASL A
L8123:   .BYTE $2F
L8124:   STA (L00A7,X)
         STA (SpritePic1,X)
         .BYTE $82,$97,$82,$0F
         .BYTE $83,$87,$83
         ORA (SpritePic2,X)
         .BYTE $02
         RTI
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L813D
         BMI L8140
         BPL L8140
L8140:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8146
L8146:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8155
         BMI L8158
         BPL L8158
L8158:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L815E
L815E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L816D
         BMI L8170
         BPL L8170
L8170:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8176
L8176:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8185
         BMI L8188
         BPL L8188
L8188:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L818E
L818E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L819D
         BMI L81A0
         BPL L81A0
L81A0:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L81A6
L81A6:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L81B5
         BMI L81B8
         BPL L81B8
L81B8:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L81BE
L81BE:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L81CD
         BMI L81D0
         BPL L81D0
L81D0:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L81D6
L81D6:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L81E5
         BMI L81E8
         BPL L81E8
L81E8:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L81EE
L81EE:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L81FD
         BMI L8200
         BPL L8200
L8200:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8206
L8206:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8215
         BMI L8218
         BPL L8218
L8218:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L821E
L821E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L822D
         BMI L8230
         BPL L8230
L8230:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8236
L8236:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8245
         BMI L8248
         BPL L8248
L8248:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L824E
L824E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L825D
         BMI L8260
         BPL L8260
L8260:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8266
L8266:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8275
         BMI L8278
         BPL L8278
L8278:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L827E
L827E:   ASL ZRAM_01
L8280:   JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L828D
         BMI L8290
         BPL L8290
L8290:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8296
L8296:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L82A5
         BMI L82A8
         BPL L82A8
L82A8:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L82AE
L82AE:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L82BD
         BMI L82C0
         BPL L82C0
L82C0:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L82C6
L82C6:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L82D5
         BMI L82D8
         BPL L82D8
L82D8:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L82DE
L82DE:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L82ED
         BMI L82F0
         BPL L82F0
L82F0:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L82F6
L82F6:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8305
         BMI L8308
         BPL L8308
L8308:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L830E
L830E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L831D
         BMI L8320
         BPL L8320
L8320:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8326
L8326:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8335
         BMI L8338
         BPL L8338
L8338:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L833E
L833E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L834D
         BMI L8350
         BPL L8350
L8350:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8356
L8356:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8365
         BMI L8368
         BPL L8368
L8368:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L836E
L836E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L837D
         BMI L8380
         BPL L8380
L8380:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8386
L8386:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L8395
         BMI L8398
         BPL L8398
L8398:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L839E
L839E:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L83AD
         BMI L83B0
         BPL L83B0
L83B0:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L83B6
L83B6:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L83C5
         BMI L83C8
         BPL L83C8
L83C8:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L83CE
L83CE:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (FKDEF,X)
         BRK
         .BYTE $04,$02
         BVC L83DD
         BMI L83E0
         BPL L83E0
L83E0:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L83E6
L83E6:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (STATUS,X)
         .BYTE $02
         BPL L83F2
L83F2:   .BYTE $04
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L83F8
L83F8:   ASL ZRAM_01
         JSR L4002
         BRK
         .BYTE $0F
         ORA (SpritePic2,X)
         .BYTE $02
         BMI L8404
L8404:   ASL L00A2
         ORA L2B8E
         .BYTE $02


; ---------------------------------------------------
;              Jump Routine #?
; ---------------------------------------------------
L840A:   LDX SRTIMR
         LDA L0400,X
         BEQ L8427
         BMI L8420
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8427
L8420:   CMP #$FF
         BEQ L8427
         JSR L8F61
L8427:   JSR LA33E
         JSR LA389
         DEC SRTIMR
         BPL L840A
         LDA L04BB
         CMP #$02
         BCC L8440
         CMP #$30
         BCS L8440
         JSR L969D
L8440:   LDA L04BB
         CMP #$02
         BCC L844A
         JSR L9730
L844A:   RTS



; ---------------------------------------------------
;              Jump Routine #?
; ---------------------------------------------------
L844B:   LDX #$0D
         STX SRTIMR
L8450:   LDX SRTIMR
         LDA L0400,X
         BEQ L846D
         BMI L8466
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L846D
L8466:   CMP #$FF
         BEQ L846D
         JSR L8F61
L846D:   JSR LA63A
         JSR LA389
         DEC SRTIMR
         BPL L8450
         LDA L04BB
         CMP #$02
         BCC L8482
         JSR L9730
L8482:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L848B:   LDX SRTIMR
         LDA L0400,X
         BEQ L84A8
         BMI L84A1
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L84A8
L84A1:   CMP #$FF
         BEQ L84A8
         JSR L8F61
L84A8:   JSR LA75C
         DEC SRTIMR
         BPL L848B
         LDA L04BB
         CMP #$02
         BCC L84BE
         CMP #$30
         BCS L84BE
         JSR LA2BB
L84BE:   RTS



; ---------------------------------------------------
;              Jump Routine #3
; ---------------------------------------------------
L84BF:   LDX #$0D               ;
         STX SRTIMR             ;
L84C4:   LDX SRTIMR             ;
         LDA L0400,X            ;
         BEQ L84E1              ;
         BMI L84DA
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L84E1
L84DA:   CMP #$FF
         BEQ L84E1
         JSR L8F61
L84E1:   JSR LA3F1
         DEC SRTIMR
         BPL L84C4
         LDA L04BB
         CMP #$02
         BCC L84F3
         JSR L97F2
L84F3:   RTS
         LDX #$0D
         STX SRTIMR
L84F9:   LDX SRTIMR
         LDA L0400,X
         BEQ L8516
         BMI L850F
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8516
L850F:   CMP #$FF
         BEQ L8516
         JSR L8F61
L8516:   JSR LA593
         JSR LA63A
         DEC SRTIMR
         BPL L84F9
         LDA L04BB
         CMP #$02
         BCC L852F
         CMP #$30
         BCS L852F
         JSR L9A70
L852F:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L8538:   LDX SRTIMR
         LDA L0400,X
         BEQ L8555
         BMI L854E
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8555
L854E:   CMP #$FF
         BEQ L8555
         JSR L8F61
L8555:   JSR LA389
         JSR LA63A
         DEC SRTIMR
         BPL L8538
         LDA L04BB
         CMP #$02
         BCC L856A
         JSR L9EF3
L856A:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L8573:   LDX SRTIMR
         LDA L0400,X
         BEQ L8590
         BMI L8589
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8590
L8589:   CMP #$FF
         BEQ L8590
         JSR L8F61
L8590:   JSR LA593
         JSR LA63A
         DEC SRTIMR
         BPL L8573
         LDA L04BB
         CMP #$02
         BCC L85A5
         JSR L9FF0
L85A5:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L85AE:   LDX SRTIMR
         LDA L0400,X
         BEQ L85CB
         BMI L85C4
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L85CB
L85C4:   CMP #$FF
         BEQ L85CB
         JSR L8F61
L85CB:   JSR LA33E
         JSR LA389
         JSR LA63A
         DEC SRTIMR
         BPL L85AE
         LDA L04BB
         CMP #$02
         BCC L85E7
         CMP #$30
         BCS L85E7
         JSR L9730
L85E7:   LDA L04BB
         CMP #$02
         BCC L85F1
         JSR L969D
L85F1:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L85FA:   LDX SRTIMR
         LDA L0400,X
         BEQ L8617
         BMI L8610
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8617
L8610:   CMP #$FF
         BEQ L8617
         JSR L8F61
L8617:   JSR LA33E
         JSR LA389
         JSR LA3F1
         JSR LA63A
         DEC SRTIMR
         BPL L85FA
         LDA L04BB
         CMP #$02
         BCC L8636
         CMP #$30
         BCS L8636
         JSR L969D
L8636:   LDA L04BB
         CMP #$02
         BCC L864E
         CMP #$30
         BCS L8644
         JSR L9730
L8644:   LDA L04BB
         CMP #$02
         BCC L864E
         JSR L97F2
L864E:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L8657:   LDX SRTIMR
         LDA L0400,X
         BEQ L8674
         BMI L866D
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8674
L866D:   CMP #$FF
         BEQ L8674
         JSR L8F61
L8674:   JSR LA33E
         JSR LA389
         JSR LA3F1
         JSR LA63A
         DEC SRTIMR
         BPL L8657
         LDA L04BB
         CMP #$02
         BCC L8693
         CMP #$30
         BCS L8693
         JSR L969D
L8693:   LDA L04BB
         CMP #$02
         BCC L86AB
         CMP #$30
         BCS L86A1
         JSR L9730
L86A1:   LDA L04BB
         CMP #$02
         BCC L86AB
         JSR L97F2
L86AB:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L86B4:   LDX SRTIMR
         LDA L0400,X
         BEQ L86D1
         BMI L86CA
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L86D1
L86CA:   CMP #$FF
         BEQ L86D1
         JSR L8F61
L86D1:   JSR LA44D
         DEC SRTIMR
         BPL L86B4
         LDA L04BB
         CMP #$02
         BCC L86E7
         CMP #$30
         BCS L86E7
         JSR L98A8
L86E7:   RTS
         LDX #$0D
         STX SRTIMR
L86ED:   LDX SRTIMR
         LDA L0400,X
         BEQ L870A
         BMI L8703
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L870A
L8703:   CMP #$FF
         BEQ L870A
         JSR L8F61
L870A:   JSR LA44D
         JSR LA4E2
         JSR LA54E
         DEC SRTIMR
         BPL L86ED
         LDA L04BB
         CMP #$02
         BCC L8726
         CMP #$30
         BCS L8726
         JSR L98A8
L8726:   LDA L04BB
         CMP #$02
         BCC L8734
         CMP #$30
         BCS L8734
         JSR L9943
L8734:   LDA L04BB
         CMP #$02
         BCC L873E
         JSR L99F5
L873E:   JSR L912E
         RTS
         LDX #$0D
         STX SRTIMR
L8747:   LDX SRTIMR
         LDA L0400,X
         BEQ L8764
         BMI L875D
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8764
L875D:   CMP #$FF
         BEQ L8764
         JSR L8F61
L8764:   JSR LA44D
         JSR LA4E2
         DEC SRTIMR
         BPL L8747
         LDA L04BB
         CMP #$02
         BCC L877D
         CMP #$30
         BCS L877D
         JSR L98A8
L877D:   LDA L04BB
         CMP #$02
         BCC L8787
         JSR L9943
L8787:   JSR L912E
         RTS
         LDX #$0D
         STX SRTIMR
L8790:   LDX SRTIMR
         LDA L0400,X
         BEQ L87AD
         BMI L87A6
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L87AD
L87A6:   CMP #$FF
         BEQ L87AD
         JSR L8F61
L87AD:   JSR LA44D
         JSR LA4E2
         JSR LA54E
         DEC SRTIMR
         BPL L8790
         LDA L04BB
         CMP #$02
         BCC L87C9
         CMP #$30
         BCS L87C9
         JSR L98A8
L87C9:   LDA L04BB
         CMP #$02
         BCC L87E1
         CMP #$30
         BCS L87D7
         JSR L9943
L87D7:   LDA L04BB
         CMP #$30
         BCC L87E1
         JSR L99F5
L87E1:   JSR L912E
         RTS
         LDX #$0D
         STX SRTIMR
L87EA:   LDX SRTIMR
         LDA L0400,X
         BEQ L8807
         BMI L8800
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8807
L8800:   CMP #$FF
         BEQ L8807
         JSR L8F61
L8807:   JSR LA4E2
         JSR LA63A
         DEC SRTIMR
         BPL L87EA
         LDA L04BB
         CMP #$02
         BCC L881C
         JSR L9943
L881C:   JSR L9BD1
         RTS
         LDX #$0D
         STX SRTIMR
L8825:   LDX SRTIMR
         LDA L0400,X
         BEQ L8842
         BMI L883B
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8842
L883B:   CMP #$FF
         BEQ L8842
         JSR L8F61
L8842:   JSR LA389
         JSR LA593
         JSR LA63A
         DEC SRTIMR
         BPL L8825
         LDA L04BB
         CMP #$02
         BCC L885E
         CMP #$30
         BCS L885E
         JSR L9730
L885E:   LDA L04BB
         CMP #$02
         BCC L8868
         JSR L9A70
L8868:   JSR L9BD1
         RTS
         LDA L04BB
         CMP #$03
         BCC L8876
         JSR L9351
L8876:   LDX #$0D
         STX SRTIMR
L887B:   LDX SRTIMR
         LDA L0400,X
         BEQ L8898
         BMI L8891
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8898
L8891:   CMP #$FF
         BEQ L8898
         JSR L8F61
L8898:   JSR LA5DD
         JSR LA63A
         DEC SRTIMR
         BPL L887B
         LDA L04BB
         CMP #$02
         BCC L88B1
         CMP #$30
         BCS L88B1
         JSR L9B1F
L88B1:   RTS
         LDA L04BB
         CMP #$03
         BCC L88BC
         JSR L9351
L88BC:   LDX #$0D
         STX SRTIMR
L88C1:   LDX SRTIMR
         LDA L0400,X
         BEQ L88DE
         BMI L88D7
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L88DE
L88D7:   CMP #$FF
         BEQ L88DE
         JSR L8F61
L88DE:   JSR LA4E2
         JSR LA63A
         DEC SRTIMR
         BPL L88C1
         LDA L04BB
         CMP #$02
         BCC L88F7
         CMP #$30
         BCS L88F7
         JSR LA19E
L88F7:   RTS
         LDA L04BB
         CMP #$03
         BCC L8902
         JSR L916B
L8902:   LDX #$0D
         STX SRTIMR
L8907:   LDX SRTIMR
         LDA L0400,X
         BEQ L8924
         BMI L891D
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8924
L891D:   CMP #$FF
         BEQ L8924
         JSR L8F61
L8924:   JSR LA44D
         DEC SRTIMR
         BPL L8907
         LDA L04BB
         CMP #$02
         BCC L893A
         CMP #$30
         BCS L893A
         JSR LA238
L893A:   RTS
         LDX #$0D
         STX SRTIMR
L8940:   LDX SRTIMR
         LDA L0400,X
         BEQ L895D
         BMI L8956
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L895D
L8956:   CMP #$FF
         BEQ L895D
         JSR L8F61
L895D:   JSR LA44D
         DEC SRTIMR
         BPL L8940
         LDA L04BB
L8968:   CMP #$02
         BCC L8973
         CMP #$30
         BCS L8973
         JSR L98A8
L8973:   RTS
         LDX #$0D
         STX SRTIMR
L8979:   LDX SRTIMR
         LDA L0400,X
         BEQ L8996
         BMI L898F
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8996
L898F:   CMP #$FF
         BEQ L8996
         JSR L8F61
L8996:   JSR LA677
         DEC SRTIMR
         BPL L8979
         LDA L04BB
         CMP #$03
         BCC L89A8
         JSR L9CC7
L89A8:   JSR L912E
         RTS
         LDX #$0D
         STX SRTIMR
L89B1:   LDX SRTIMR
         LDA L0400,X
         BEQ L89CE
         BMI L89C7
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L89CE
L89C7:   CMP #$FF
         BEQ L89CE
         JSR L8F61
L89CE:   JSR LA677
         DEC SRTIMR
         BPL L89B1
         LDA L04BB
         CMP #$03
         BCC L89E0
         JSR LA087
L89E0:   JSR L912E
         RTS
         LDX #$0D
         STX SRTIMR
L89E9:   LDX SRTIMR
         LDA L0400,X
         BEQ L8A06
         BMI L89FF
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8A06
L89FF:   CMP #$FF
         BEQ L8A06
         JSR L8F61
L8A06:   JSR LA677
         DEC SRTIMR
         BPL L89E9
         LDA L04BB
         CMP #$03
         BCC L8A18
         JSR L9CC7
L8A18:   JSR L912E
         RTS
         LDX #$0D
         STX SRTIMR
L8A21:   LDX SRTIMR
         LDA L0400,X
         BEQ L8A3E
         BMI L8A37
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8A3E
L8A37:   CMP #$FF
         BEQ L8A3E
         JSR L8F61
L8A3E:   JSR LA6DF
         DEC SRTIMR
         BPL L8A21
         LDA L04BB
         CMP #$03
         BCC L8A50
         JSR L9DDE
L8A50:   RTS

; ---------------------------------------------------
;              Jump Routine #1
; ---------------------------------------------------
L8A51:   LDX #$0D               ;
         STX SRTIMR             ;
L8A56:   LDX SRTIMR             ;
         LDA L0400,X            ;
         BEQ L8A73
         BMI L8A6C
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8A73
L8A6C:   CMP #$FF
         BEQ L8A73
         JSR L8F61
L8A73:   JSR LA712
         DEC SRTIMR
         BPL L8A56
         LDA L04BB
         CMP #$03
         BCC L8A85
         JSR L9E5B
L8A85:   RTS

; ---------------------------------------------------
;              Jump Routine #2
; ---------------------------------------------------
L8A86:   LDX #$0D
         STX SRTIMR
L8A8B:   LDX SRTIMR
         LDA L0400,X
         BEQ L8AA8
         BMI L8AA1
         JSR L8ABB
         JSR L90E5
         JSR L8C60
         JMP L8AA8
L8AA1:   CMP #$FF
         BEQ L8AA8
         JSR L8F61
L8AA8:   JSR LA4E2
         DEC SRTIMR
         BPL L8A8B
         LDA L04BB
         CMP #$02
         BCC L8ABA
         JSR LA19E
L8ABA:   RTS


; ---------------------------------------------------
;             Update Sprite Position
; ---------------------------------------------------
L8ABB:   LDX SRTIMR             ;
         TXA                    ;
         ASL A                  ;
         TAX                    ;
         LDA (L00B3,X)          ;
         TAY
         LDX SRTIMR
         CLC
         LDA LB100,Y
         ADC Sprite1_X,X
         STA Sprite1_X,X
         CLC
         LDA LB000,Y
         ADC Sprite1_Y,X
         STA Sprite1_Y,X
         LDA L0400,X
         ASL A
         TAY
         LDA LB5D4,Y
         STA ZRAM_01
         LDA LB5D5,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
         STA Sprite1_Char,X
         LDA L0400,X
         ASL A
         TAY
         LDA LB5F2,Y
         STA ZRAM_01
         LDA LB5F3,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
         STA Sprite1_Prop,X
         LDA L0400,X
         ASL A
         TAY
         LDA LB610,Y
         STA ZRAM_01
         LDA LB611,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         CLC
         LDA (ZRAM_02+1),Y
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDA L0400,X
         ASL A
         TAY
         LDA LB62E,Y
         STA ZRAM_01
         LDA LB62F,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
L8B55:   INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         CLC
         LDA (ZRAM_02+1),Y
         ADC Sprite1_Y,X
         STA Sprite1_Y,X
         LDA L0454,X
         BEQ L8B70
         DEC L0454,X
         JMP L8B8F
L8B70:   TXA
         ASL A
         TAY
         CLC
         LDA L00B3,Y
         ADC #$02
         STA L00B3,Y
         STA ZRAM_01
         LDA #$00
         ADC L00B4,Y
         STA L00B4,Y
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         STA L0454,X
L8B8F:   LDA #$00
         STA L04C8
L8B94:   LDA L0438,X
         CMP #$FF
         BNE L8B9C
         RTS
L8B9C:   INC L04C8
         STX L04BF
         STA L04BD
         LDX SRTIMR
         LDA L0400,X
         ASL A
         TAY
         LDA LB5D4,Y
         STA ZRAM_01
         LDA LB5D5,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
         LDX L04BD
         STA Sprite1_Char,X
         LDX SRTIMR
         LDA L0400,X
         ASL A
         TAY
         LDA LB5F2,Y
         STA ZRAM_01
         LDA LB5F3,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
         LDX L04BD
         STA Sprite1_Prop,X
         LDX SRTIMR
         LDA L0400,X
         ASL A
         TAY
         LDA LB610,Y
         STA ZRAM_01
         LDA LB611,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDX L04BF
         LDA (ZRAM_02+1),Y
         CLC
         ADC Sprite1_X,X
         LDX L04BD
         STA Sprite1_X,X
         LDX SRTIMR
         LDA L0400,X
         ASL A
         TAY
         LDA LB62E,Y
         STA ZRAM_01
         LDA LB62F,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDX L04BF
         LDA (ZRAM_02+1),Y
         CLC
         ADC Sprite1_Y,X
         LDX L04BD
         STA Sprite1_Y,X
         JMP L8B94
L8C60:   LDX SRTIMR
         LDY #$09
         STY L04BE
L8C68:   LDX SRTIMR
         LDY L04BE
         LDA DDEVIC,Y
         BPL L8C76
         JMP L8D98
L8C76:   ASL A
         TAY
         LDA L4AC8,Y
         STA ZRAM_01
         LDA L4AC9,Y
         STA ZRAM_02
         LSR ZRAM_02
         ROR ZRAM_01
         LSR ZRAM_02
         ROR ZRAM_01
         LDA Sprite1_Y,X
L8C8C:   SEC
         SBC ZRAM_01
         CLC
         ADC #$10
         CMP #$18
         BCC L8C99
         JMP L8D51
L8C99:   LDY L04BE
         LDA HATABS+4,Y
         ASL A
         TAY
         LDA L4AFE,Y
         STA ZRAM_02+1
         LDA L4AFF,Y
         STA RAMLO
         LDY L04BE
         LDA DDEVIC,Y
         TAY
         LDA (ZRAM_02+1),Y
         LDY L04BE
         CLC
         ADC TEMP2,Y
         SEC
         SBC Sprite1_X,X
         SEC
         SBC #$FC
         CMP #$10
         BCC L8CC8
         JMP L8D51
L8CC8:   LDX SRTIMR
         LDY L04BE
         LDA #$80
         ORA DDEVIC,Y
         STA DDEVIC,Y
         LDA L0400,X
         CMP #$0A
         BEQ L8CEE
         CMP #$0B
         BNE L8CE4
         JMP L8D98
L8CE4:   CMP #$0C
         BNE L8CEB
         JMP L8D98
L8CEB:   JMP L8D1C
L8CEE:   TAY
         LDA L8F51,Y
         JSR L53D7
         DEC L00AB
         LDA #$8A
         STA L040D
         LDA #$8B
         LDX #$0B
L8D00:   STA L0401,X
         DEX
         BPL L8D00
         LDA #$8C
         STA L0400
         LDX #$0D
         LDA #$00
L8D0F:   STA L040E,X
         STA L041C,X
         STA L042A,X
         DEX
         BPL L8D0F
         RTS
L8D1C:   LDX SRTIMR
         LDA #$80
         ORA L0400,X
         STA L0400,X
         TXA
         PHA
         LDA L0400,X
         AND #$7F
         TAX
         LDA L8F41,X
         JSR L51CE              ; Play Sound
         PLA
         TAX
         LDA #$00
         STA L042A,X
         STA L041C,X
         STA L040E,X
         LDA L0400,X
         AND #$7F
         TAY
         LDA L8F51,Y
         JSR L53D7
         JMP L8D98
L8D51:   LDA L0438,X
         CMP #$FF
         JMP L8D98
         TAX
         LDA Sprite1_Y,X
         SEC
         SBC ZRAM_01
         CLC
         ADC #$10
         CMP #$18
         BCC L8D69
         JMP L8D51
L8D69:   LDY L04BE
         LDA HATABS+4,Y
         ASL A
         TAY
         LDA L4AFE,Y
         STA ZRAM_02+1
         LDA L4AFF,Y
         STA RAMLO
         LDA DDEVIC,Y
         TAY
         LDA (ZRAM_02+1),Y
         LDY L04BE
         CLC
         ADC TEMP2,Y
         SEC
         SBC Sprite1_X,X
         SEC
         SBC #$FC
         CMP #$10
         BCC L8D95
         JMP L8D98
L8D95:   JMP L8CC8
L8D98:   LDX SRTIMR
         DEC L04BE
         BMI L8DA3
         JMP L8C68
L8DA3:   LDA PADDL1
         BEQ L8DA9
         RTS
L8DA9:   LDX SRTIMR
         LDA VNTP+1
         AND #$E0
         BNE L8DDD
         SEC
         LDA Sprite1_X,X
         SBC BegaXPos1L
         CLC
         ADC #$08
         CMP #$20
         BCS L8DDD
         LDA Sprite1_Y,X
         CMP #$B8
         BCC L8DDD
         CMP #$E0
         BCS L8DDD
         LDA #$80
         STA VNTP+1
         ORA L0400,X
         STA L0400,X
         LDA #$00
         STA L042A,X
         STA L040E,X
         STA L041C,X
L8DDD:   LDA L009B
         AND #$E0
         BNE L8E0E
         SEC
         LDA Sprite1_X,X
         SBC L00A6
         CLC
         ADC #$08
         CMP #$10
         BCS L8E0E
         LDA Sprite1_Y,X
         CMP #$D8
         BCC L8E0E
         CMP #$F0
         BCS L8E0E
         LDA #$80
         STA L009B
         ORA L0400,X
         STA L0400,X
         LDA #$00
         STA L042A,X
         STA L040E,X
         STA L041C,X
L8E0E:   LDY #$05
         STY L04BE
L8E13:   LDY L04BE
         LDA L009C,Y
         AND #$E0
         BNE L8E62
         LDA L009C,Y
         AND #$0C
         LSR A
         TAY
         SEC
         LDA Sprite1_X,X
         SBC Sprite3_X,Y
         CLC
         ADC #$08
         CMP #$10
         BCS L8E62
         LDA Sprite1_Y,X
         CMP #$C8
         BCC L8E62
         CMP #$E0
         BCS L8E62
         LDA #$0F
         LDY L04BE
         LDA L009C,Y
         AND #$0F
         ORA #$80
         STA L009C,Y
         LDA #$00
         STA HATABS+16,Y
         LDA #$80
         ORA L0400,X
         STA L0400,X
         LDA #$00
         STA L042A,X
         STA L040E,X
         STA L041C,X
L8E62:   DEC L04BE
         BPL L8E13
L8E67:   LDA L0438,X
         CMP #$FF
         BNE L8E6F
         RTS
L8E6F:   TAX
         STX L04BF
         LDA VNTP+1
         AND #$E0
         BNE L8EAA
         SEC
         LDA Sprite1_X,X
         SBC BegaXPos1L
         CLC
         ADC #$08
         CMP #$20
         BCS L8EAA
         LDA Sprite1_Y,X
         CMP #$B8
         BCC L8EAA
         CMP #$E0
         BCS L8EAA
         LDA #$80
         STA VNTP+1
         LDX SRTIMR
         ORA L0400,X
         STA L0400,X
         LDA #$00
         STA L042A,X
         STA L040E,X
         STA L041C,X
         LDX L04BF
L8EAA:   LDA L009B
         AND #$E0
         BNE L8EE1
         SEC
         LDA Sprite1_X,X
         SBC L00A6
         CLC
         ADC #$08
         CMP #$10
         BCS L8EE1
         LDA Sprite1_Y,X
         CMP #$D8
         BCC L8EE1
         CMP #$F0
         BCS L8EE1
         LDA #$80
         STA L009B
         LDX SRTIMR
         ORA L0400,X
         STA L0400,X
         LDA #$00
         STA L042A,X
         STA L040E,X
         STA L041C,X
         LDX L04BF
L8EE1:   LDY #$05
         STY L04BE
L8EE6:   LDY L04BE
         LDA L009C,Y
         AND #$E0
         BNE L8F39
         LDA L009C,Y
         AND #$0C
         LSR A
         TAY
         SEC
         LDA Sprite1_X,X
         SBC Sprite3_X,Y
         CLC
         ADC #$08
         CMP #$10
         BCS L8F39
         LDA Sprite1_Y,X
         CMP #$C8
         BCC L8F39
         CMP #$E0
         BCS L8F39
         LDY L04BE
         LDA L009C,Y
         AND #$0F
         ORA #$80
         STA L009C,Y
         LDA #$00
         STA HATABS+16,Y
         LDX SRTIMR
         LDA #$80
         ORA L0400,X
         STA L0400,X
         LDA #$00
         STA L042A,X
         STA L040E,X
         STA L041C,X
         LDX L04BF
L8F39:   DEC L04BE
         BPL L8EE6
         JMP L8E67
L8F41:   ASL Sprite2_Prop,X
         ASL Sprite2_Prop,X
         ASL Sprite2_Prop,X
         ASL Sprite2_Prop,X
         ASL Sprite2_Prop,X
         .BYTE $17
         ASL Sprite2_Prop,X
         ASL BUFADR+1,X
         ASL WARMST,X
         ORA (ZRAM_01,X)
         ORA (ZRAM_02+1,X)
         ORA (WARMST,X)
         ORA (ZRAM_02+1,X)
         BRK
         .BYTE $07
         PHP
         PHP
         ORA (ZRAM_02+1,X)
         PHP



; ---------------------------------------------------
;                unknown
; ---------------------------------------------------
L8F61:   LDX SRTIMR             ;
         LDA L0400,X            ;
         ASL A                  ;
         TAY
         LDA LBA1E,Y
         STA ZRAM_01
         LDA LBA1F,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
         STA L04C9
         STA Sprite1_Char,X
         LDA L0400,X
         ASL A
         TAY
L8F8E:   LDA LBA3C,Y
         STA ZRAM_01
         LDA LBA3D,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L04C9
         BEQ L8FAE
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
L8FAE:   STA Sprite1_Prop,X
         LDA L0400,X
         ASL A
         TAY
         LDA LBA5A,Y
         STA ZRAM_01
         LDA LBA5B,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         CLC
         LDA (ZRAM_02+1),Y
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDA L0400,X
         ASL A
         TAY
         LDA LBA78,Y
         STA ZRAM_01
         LDA LBA79,Y
         STA ZRAM_02
         LDY #$00
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
         CLC
         ADC Sprite1_Y,X
         STA Sprite1_Y,X
         LDA #$00
         STA L04C8
L8FFF:   LDA L0438,X
         CMP #$FF
         BNE L9017
         LDA L04C9
         BNE L9016
         LDA #$FF
         STA L0400,X
         LDX SRTIMR
         STA L0400,X
L9016:   RTS
L9017:   INC L04C8
         STX L04BF
         STA L04BD
         LDX SRTIMR
         LDA L0400,X
         ASL A
         TAY
         LDA LBA1E,Y
         STA ZRAM_01
         LDA LBA1F,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L04C9
         BEQ L904B
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
L904B:   LDX L04BD
         STA Sprite1_Char,X
         LDX SRTIMR
         LDA L0400,X
         ASL A
         TAY
         LDA LBA3C,Y
         STA ZRAM_01
         LDA LBA3D,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L04C9
         BEQ L907B
         LDA L040E,X
         TAY
         LDA (ZRAM_02+1),Y
L907B:   LDX L04BD
         STA Sprite1_Prop,X
         LDX SRTIMR
         LDA L0400,X
         ASL A
         TAY
         LDA LBA5A,Y
         STA ZRAM_01
         LDA LBA5B,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         CLC
         LDA (ZRAM_02+1),Y
         LDX L04BF
         ADC Sprite1_X,X
         LDX L04BD
         STA Sprite1_X,X
         LDX SRTIMR
         LDA L0400,X
         ASL A
L90B8:   TAY
         LDA LBA78,Y
         STA ZRAM_01
         LDA LBA79,Y
         STA ZRAM_02
         LDA L04C8
         ASL A
         TAY
         LDA (ZRAM_01),Y
         STA ZRAM_02+1
         INY
         LDA (ZRAM_01),Y
         STA RAMLO
         LDA L040E,X
         TAY
         CLC
         LDA (ZRAM_02+1),Y
         LDX L04BF
         ADC Sprite1_Y,X
         LDX L04BD
         STA Sprite1_Y,X
         JMP L8FFF
L90E5:   LDX SRTIMR
         LDA Sprite1_X,X
         CMP #$F0
         BCS L90F5
         LDA Sprite1_Y,X
         CMP #$E8
         BCS L90F5
         RTS
L90F5:   LDA #$00
         STA Sprite1_Prop,X
         STA L040E,X
         STA L041C,X
         STA L042A,X
         STA L0446,X
L9105:   LDA L0438,X
         CMP #$FF
         BNE L9115
         LDA #$FF
         LDX SRTIMR
         STA L0400,X
         RTS
L9115:   TAX
         LDA #$00
         STA Sprite1_Prop,X
         STA L040E,X
         STA L041C,X
         STA L042A,X
         STA L0446,X
         LDA #$FF
         STA L0400,X
         JMP L9105
L912E:   LDX #$0D
L9130:   TXA
         ASL A
         TAY
         LDA L00B3,Y
         EOR L0462,X
         BEQ L913E
         STA L0470,X
L913E:   LDA L00B3,Y
         STA L0462,X
         DEX
         BPL L9130
         RTS
L9148:   LDA #$00
         STA POKMSK
         LDY #$0D
L914E:   LDA L0400,Y
         CMP #$FF
         BNE L9165
         TXA
         PHA
         LDX POKMSK
         TYA
         STA L047E,X
         INC POKMSK
         PLA
         TAX
         CPX POKMSK
         BEQ L916A
L9165:   DEY
         BPL L914E
         TYA
         TAX
L916A:   RTS
L916B:   LDA L04B8
         BNE L91DA
         LDA L00A9
         BPL L9175
         RTS
L9175:   LDX #$03
         JSR L9148
         CPX #$FF
         BNE L917F
         RTS
L917F:   LDA L047E
         STA L04C0
         LDA L047F
         STA L04C1
         LDA L0480
         STA L04C2
         LDA #$00
         STA L04C3
         STA L04C4
         LDA #$01
         STA L04B8
         LDA #$01
         LDX L04C0
         STA Sprite1_Prop,X
         LDA #$78
         STA Sprite1_X,X
         LDA #$10
         STA Sprite1_Y,X
         LDA #$00
         STA L0400,X
         LDA #$01
         LDX L04C1
         STA Sprite1_Prop,X
         LDA #$78
         STA Sprite1_X,X
         LDA #$20
         STA Sprite1_Y,X
         LDA #$00
         STA L0400,X
         LDA #$01
         LDX L04C2
         STA Sprite1_Prop,X
         LDA #$78
         STA Sprite1_X,X
         LDA #$30
         STA Sprite1_Y,X
         LDA #$00
         STA L0400,X
L91DA:   LDA SysClock
         AND #$0F
         BNE L91E9
         LDA L04C3
         EOR #$01
         STA L04C3
L91E9:   LDA L04C4
         BEQ L9202
         LDA SysClock
         AND #$0F
         BNE L9202
         INC L04C4
         LDA L04C4
         CMP #$10
         BCC L9202
         JMP L92F3
L9202:   LDA L04B8
         LSR A
         TAY
         LDA L95D5,Y
         CLC
         LDX L04C0
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDA L95D5,Y
         CLC
         LDX L04C1
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDA L95D5,Y
         CLC
         LDX L04C2
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDY L04C3
         LDA L931B,Y
         LDX L04C0
         STA Sprite1_Char,X
         LDA L932D,Y
         LDX L04C1
         STA Sprite1_Char,X
         LDA L933F,Y
         LDX L04C2
         STA Sprite1_Char,X
         LDA Sprite1_X,X
         CMP #$03
         BCC L924F
         CMP #$ED
         BCS L9259
         BCC L9261
L924F:   LDA L04B8
         ORA #$02
         STA L04B8
         BNE L9261
L9259:   LDA L04B8
         AND #$FD
         STA L04B8
L9261:   LDA L04C4
         BEQ L9269
         JMP L92F2
L9269:   LDX #$09
L926B:   LDA DDEVIC,X
         BPL L9273
         JMP L92EC
L9273:   ASL A
         TAY
         LDA L4AC8,Y
         STA ZRAM_01
         LDA L4AC9,Y
         STA ZRAM_02
         LSR ZRAM_02
         ROR ZRAM_01
         LSR ZRAM_02
         ROR ZRAM_01
         LDA HATABS+4,X
         ASL A
         TAY
         LDA L4AFE,Y
         STA ZRAM_02+1
         LDA L4AFF,Y
         STA RAMLO
         LDA DDEVIC,X
         TAY
         LDA (ZRAM_02+1),Y
         CLC
         ADC TEMP2,X
         STA ZRAM_02+1
         SEC
         LDA ZRAM_02+1
         LDY L04C0
         SBC Sprite1_X,Y
         SEC
         SBC #$00
         CMP #$08
         BCS L92EC
         SEC
         LDA ZRAM_01
         SBC Sprite1_Y,Y
         CLC
         ADC #$08
         CMP #$38
         BCS L92EC
         LDA #$80
         ORA DDEVIC,X
         STA DDEVIC,X
         LDA #$18
         JSR L51CE              ; Play Sound
         CLC
         LDA L04C3
         ADC #$02
         STA L04C3
         CMP #$10
         BCC L92F2
         LDA #$07
         JSR L53D7
         LDA #$17
         JSR L51CE              ; Play Sound
         DEC L00A9
         LDA #$01
         STA L04C4
         BNE L92F2
L92EC:   DEX
         BMI L92F2
         JMP L926B
L92F2:   RTS
L92F3:   LDX L04C0
         LDA #$00
         STA L04B8
         STA Sprite1_Prop,X
         LDA #$FF
         STA L0400,X
         LDX L04C1
         LDA #$00
         STA Sprite1_Prop,X
         LDA #$FF
         STA L0400,X
         LDX L04C2
         LDA #$00
         STA Sprite1_Prop,X
         LDA #$FF
         STA L0400,X
         RTS
L931B:   LDA (L00A2,X)
         LDA (L00A2,X)
         LDA (L00A2,X)
         LDA (L00A2,X)
         LDA (L00A2,X)
         .BYTE $A7
         TAY
         LDA #$AA
         .BYTE $AB
         LDY LBCB9
L932D:   .BYTE $A3
         LDY L00A3
         LDY L00A3
         LDY L00AD
         LDX LB0AF
         LDA (L00B2),Y
         LDA (L00B2),Y
         LDA (L00B2),Y
         TSX
         LDA LA6A5,X
         .BYTE $B3
         LDY L00B5,X
         LDX DATALN,X
         CLV
         .BYTE $B7
         CLV
         .BYTE $B7
         CLV
         .BYTE $B7
         CLV
         .BYTE $B7
         CLV
         .BYTE $BB
         LDX LB9AD,Y
         .BYTE $04
         BNE L93CA
         LDA L00A7
         BPL L935B
         RTS
L935B:   LDX #$03
         JSR L9148
         CPX #$FF
         BNE L9365
         RTS
L9365:   LDA L047E
         STA L04C2
         LDA L047F
         STA L04C1
         LDA L0480
         STA L04C0
         LDA #$00
         STA L04C3
         STA L04C4
         STA L04C5
         STA L04C6
         STA L04C7
         STA L04C8
         LDA #$01
         STA L04B9
         LDX L04C0
         STA Sprite1_Prop,X
         LDA #$6C
         STA Sprite1_X,X
         LDA #$20
         STA Sprite1_Y,X
         LDA #$00
         STA L0400,X
         LDA #$01
         LDX L04C2
         STA Sprite1_Prop,X
         LDA #$78
         STA Sprite1_X,X
         LDA #$20
         STA Sprite1_Y,X
         LDA #$00
         STA L0400,X
         LDA #$05
         LDX L04C1
         STA Sprite1_Prop,X
         LDA #$84
         STA Sprite1_X,X
         LDA #$20
         STA Sprite1_Y,X
         LDA #$00
         STA L0400,X
L93CA:   LDA L04B9
         BPL L93D2
         JMP L95AA
L93D2:   LDA SysClock
         AND #$0F
         BNE L9406
         LDA L04C3
         EOR #$01
         LDX L04C6
         BNE L93E5
         AND #$01
L93E5:   STA L04C3
         LDA L04C4
         EOR #$01
         LDX L04C7
         BNE L93F4
         AND #$01
L93F4:   STA L04C4
         LDA L04C5
         EOR #$01
         LDX L04C8
         BNE L9403
         AND #$01
L9403:   STA L04C5
L9406:   LDA L04B9
         LSR A
         TAY
         LDA L95D5,Y
         CLC
         LDX L04C0
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDA L95D5,Y
         CLC
         LDX L04C1
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDA L95D5,Y
         CLC
         LDX L04C2
         ADC Sprite1_X,X
         STA Sprite1_X,X
         LDY L04C3
         LDA L95D7,Y
         LDX L04C0
         STA Sprite1_Char,X
         LDY L04C4
         LDA L95DC,Y
         LDX L04C2
         STA Sprite1_Char,X
         LDY L04C5
         LDA L95D7,Y
         LDX L04C1
         STA Sprite1_Char,X
         JSR L95E1
         LDX L04C2
         LDA Sprite1_X,X
         CMP #$08
         BCC L945F
         CMP #$E7
         BCS L9469
         BCC L9471
L945F:   LDA L04B9
         ORA #$02
         STA L04B9
         BNE L9471
L9469:   LDA L04B9
         AND #$FD
         STA L04B9
L9471:   LDX #$09
L9473:   LDA DDEVIC,X
         BPL L947B
         JMP L9555
L947B:   ASL A
         TAY
         LDA L4AC8,Y
         STA ZRAM_01
         LDA L4AC9,Y
         STA ZRAM_02
         LSR ZRAM_02
         ROR ZRAM_01
         LSR ZRAM_02
         ROR ZRAM_01
         LDA HATABS+4,X
         ASL A
         TAY
         LDA L4AFE,Y
         STA ZRAM_02+1
         LDA L4AFF,Y
         STA RAMLO
         LDA DDEVIC,X
         TAY
         LDA (ZRAM_02+1),Y
         CLC
         ADC TEMP2,X
         STA ZRAM_02+1
         SEC
         LDA ZRAM_02+1
         LDY L04C0
         SBC Sprite1_X,Y
         SEC
         SBC #$00
         CMP #$08
         BCS L94E4
         SEC
         LDA ZRAM_01
         SBC Sprite1_Y,Y
         CLC
         ADC #$08
         CMP #$18
         BCS L94E4
         LDA #$18
         JSR L51CE              ; Play Sound
         LDA #$7F
         STA L04C6
         LDA #$02
         ORA L04C3
         STA L04C3
         LDA #$80
         ORA DDEVIC,X
         STA DDEVIC,X
         JMP L9555
L94E4:   SEC
         LDA ZRAM_02+1
         LDY L04C2
         SBC Sprite1_X,Y
         SEC
         SBC #$00
         CMP #$08
         BCS L951E
         SEC
         LDA ZRAM_01
         SBC Sprite1_Y,Y
         CLC
         ADC #$08
         CMP #$18
         BCS L951E
         LDA #$18
         JSR L51CE              ; Play Sound
         LDA #$7F
         STA L04C7
         LDA #$02
         ORA L04C4
         STA L04C4
         LDA #$80
         ORA DDEVIC,X
         STA DDEVIC,X
         JMP L9555
L951E:   SEC
         LDA ZRAM_02+1
         LDY L04C1
         SBC Sprite1_X,Y
         SEC
         SBC #$00
         CMP #$08
         BCS L9555
         SEC
         LDA ZRAM_01
         SBC Sprite1_Y,Y
         CLC
         ADC #$08
         CMP #$18
         BCS L9555
         LDA #$18
         JSR L51CE              ; Play Sound
         LDA #$7F
         STA L04C8
         LDA #$02
         ORA L04C4
         STA L04C4
         LDA #$80
         ORA DDEVIC,X
         STA DDEVIC,X
L9555:   DEX
         BMI L955B
         JMP L9473
L955B:   LDA L04C6
         BEQ L956C
         LDA L04C7
         BEQ L956C
         LDA L04C8
         BEQ L956C
         BNE L9585
L956C:   LDA L04C6
         BEQ L9574
         DEC L04C6
L9574:   LDA L04C7
         BEQ L957C
         DEC L04C7
L957C:   LDA L04C8
         BEQ L9584
         DEC L04C8
L9584:   RTS
L9585:   LDA #$07
         JSR L53D7
         LDA #$17
         JSR L51CE              ; Play Sound
         LDA #$80
         STA L04B9
         DEC L00A7
         LDA #$9C
         LDX L04C0
         STA Sprite1_Char,X
         LDX L04C1
         STA Sprite1_Char,X
         LDA #$9F
         LDX L04C2
         STA Sprite1_Char,X
         RTS
L95AA:   INC L04B9
         BEQ L95B0
         RTS
L95B0:   LDX L04C0
         LDA #$00
         STA Sprite1_Prop,X
         LDA #$FF
         STA L0400,X
         LDX L04C1
         LDA #$00
         STA Sprite1_Prop,X
         LDA #$FF
         STA L0400,X
         LDX L04C2
         LDA #$00
         STA Sprite1_Prop,X
         LDA #$FF
         STA L0400,X
         RTS
L95D5:   INC L9802,X
         STA L9B9A,Y
         .BYTE $9C
L95DC:   STX L0097,X
         STA L9F9E,X
L95E1:   LDA L04BC
         BEQ L95E9
         DEC L04BC
L95E9:   LDX L04C2
         LDA Sprite1_X,X
         CMP #$40
         NOP
         NOP
         NOP
         NOP
         BCC L9614
         CMP #$60
         BCC L95FE
         CMP #$67
         BCC L9614
L95FE:   CMP #$98
         BCC L9606
         CMP #$A0
         BCC L9614
L9606:   CMP #$B8
         BCS L960D
         JMP L969C
L960D:   CMP #$C0
         BCC L9614
         JMP L969C
L9614:   LDA #$A0
         STA Sprite1_Char,X
         LDA L04BC
         BEQ L9620
         JMP L969C
L9620:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE L962C
         JMP L969C
L962C:   LDA #$0A
         JSR L51CE              ; Play Sound
         LDA #$1F
         STA L04BC
         LDX L04C2
         LDA Sprite1_X,X
         CMP BegaXPos1R
         BCC L9653
         SEC
         SBC BegaXPos1R
         LSR A
         LSR A
         LSR A
         LSR A
         TAY
         LDA L9CA7,Y
         LDX L047E
         STA L040E,X
         JMP L9666
L9653:   SEC
         LDA BegaXPos1R
         SBC Sprite1_X,X
         LSR A
         LSR A
         LSR A
         LSR A
         TAY
         LDA L9CB7,Y
         LDX L047E
         STA L040E,X
L9666:   TAY
         TXA
         ASL A
         TAX
         LDA LBD3E,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD3F,Y
         STA L00B4,X
         STA ZRAM_02
         LDX L047E
         LDY #$01
         LDA (ZRAM_01),Y
         STA L0454,X
         LDY L04C2
         LDA Sprite1_X,Y
         SEC
         SBC #$08
         STA Sprite1_X,X
         LDA Sprite1_Y,Y
         STA Sprite1_Y,X
         LDA #$09
         STA L0400,X
         LDA #$FF
         STA L0438,X
L969C:   RTS
L969D:   LDA SysClock
         AND #$01
         BEQ L96A5
         RTS
L96A5:   LDA L048C
         BEQ L96AE
         DEC L048C
         RTS
L96AE:   LDX #$01
         JSR L9148
         CPX #$FF
         BNE L96B8
         RTS
L96B8:   LDA L04A2
         BNE L96CD
         LDA Random             ; Get Random Number
         AND #$7F
         CLC
         ADC #$40
         STA L0497
         LDA #$0B
         JSR L51CE              ; Play Sound
L96CD:   INC L04A2
         LDX L047E
         LDA L0497
         STA Sprite1_X,X
         LDA #$00
         STA Sprite1_Y,X
         LDA #$01
         STA L0400,X
         LDA #$FF
         STA L0438,X
         LDA #$00
         STA L041C,X
         STA L042A
         LDA L0497
         AND #$02
         ASL A
         ASL A
         STA L040E,X
         LDA L0497
         AND #$1F
         ASL A
         TAY
         TXA
         ASL A
         TAX
         LDA LBC7A,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBC7B,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDA L04A2
         CMP #$01
         BCC L972F
         LDY L04BA
         LDA LBE3A,Y
         STA L048C
         LDA #$00
         STA L04A2
L972F:   RTS
L9730:   LDA SysClock
         AND #$01
         BEQ L9738
         RTS
L9738:   LDA L048D
         BEQ L9741
         DEC L048D
         RTS
L9741:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE L974B
         RTS
L974B:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$01
         BNE L975D
         LDA Random             ; Get Random Number
         AND #$0F
         JMP L9765
L975D:   LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
L9765:   TAY
         LDA L97C2,Y
         STA Sprite1_X,X
         LDA L97D2,Y
         STA Sprite1_Y,X
         LDA #$02
         STA L0400,X
         LDA L047F
         STA L0438,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L042A,X
         LDA L97E2,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBCBA,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCBB,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         LDA #$FF
         STA L0438,X
         LDY L04BA
L97BB:   LDA LBE6A,Y
         STA L048D
         RTS
L97C2:   ORA #$20
         .BYTE $33,$43,$43,$54
         .BYTE $62
         JMP (L877C)
         .BYTE $8F
         TXS
         LDA DATAD
         DEX
         .BYTE $DB
L97D2:   EOR FMSZPG+2
         EOR FMSZPG+2
         RTS
         RTS
         RTS
         RTS
         RTS
         RTS
         RTS
         RTS
         BVC L9830
         BVC L9832
L97E2:   BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         .BYTE $02,$02,$02,$02
         .BYTE $02,$02,$02,$02
L97F2:   LDA SysClock
         AND #$01
         BEQ L97FA
         RTS
L97FA:   LDA L048E
         BEQ L9803
         DEC L048E
L9802:   RTS
L9803:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE L980D
         RTS
L980D:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$0F
         TAY
         LDA L9878,Y
         STA Sprite1_X,X
         LDA L9888,Y
         STA Sprite1_Y,X
         LDA #$03
         STA L0400,X
         LDA L047F
         STA L0438,X
         LDA #$00
         STA L041C,X
L9830:   STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA L9898,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBCC2,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCC3,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDY L04BA
         LDA LBE82,Y
         STA L048E
         RTS
L9878:   BVC L98D0
         .BYTE $5C
         RTS
         ROR Sprite2_X
         BVS L98F6
         .BYTE $7C,$80
         STX STARP
         BCC L981C
         .BYTE $9C
         LDY #$40
         RTI
         RTI
         RTI
         RTI
         RTI
         BMI L98C0
         BMI L98C2
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
L9898:   ASL L0E00
         BRK
         .BYTE $0C
         BRK
         .BYTE $02,$02
         BRK
         .BYTE $02,$02
         ASL L0C02
         .BYTE $02,$0C
L98A8:   LDA SysClock
         AND #$01
L98AD:   BEQ L98B0
L98AF:   RTS
L98B0:   LDA L048F
         BEQ L98B9
         DEC L048F
         RTS
L98B9:   LDX #$01
         JSR L9148
         CPX #$FF
L98C0:   BNE L98C3
L98C2:   RTS
L98C3:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$03
         BEQ L98D5
         LDA Random             ; Get Random Number
L98D0:   AND #$07
         JMP L98DF
L98D5:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
L98DF:   TAY
         LDA L992B,Y
         STA Sprite1_X,X
         LDA L9933,Y
         STA Sprite1_Y,X
         LDA #$04
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDA L993B,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBCE2,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCE3,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDY L04BA
         LDA LBE9A,Y
         STA L048F
         RTS
L992B:   BVC L98AD
         BCC L98AF
         BCC L99A1
         BNE L98D3
L9933:   JSR L2020
         JSR L2020
         JSR SpritePic2
         ASL TRAMSZ
         BRK
         .BYTE $02,$04,$02,$04
L9943:   LDA SysClock
         AND #$01
         BEQ L994B
         RTS
L994B:   LDA L0490
         BEQ L9954
         DEC L0490
         RTS
L9954:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE L995E
         RTS
L995E:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$01
         BEQ L9970
         LDA Random             ; Get Random Number
         AND #$07
         JMP L997A
L9970:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
L997A:   TAY
         LDA L99DD,Y
         STA Sprite1_X,X
         LDA L99E5,Y
         STA Sprite1_Y,X
         LDA #$05
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA L047F
         STA L0438,X
         LDA L99ED,Y
L99A1:   TAY
         TXA
         ASL A
         TAX
         LDA LBCF2,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCF3,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDY L04BA
         LDA LBEB2,Y
         STA L0490
         RTS
L99DD:   BVC L9A37
         ADC (Sprite2_X+1,X)
         .BYTE $93
         TYA
         .BYTE $9C
         LDY #$9A
         TXS
         TXS
         TXS
         TXS
         TXS
         TXS
         TXS
L99ED:   BRK
         .BYTE $02,$04,$04
         ASL TRAMSZ
         PHP
         ASL A
L99F5:   LDA SysClock
         AND #$01
         BEQ L99FD
         RTS
L99FD:   LDA L0491
         BEQ L9A06
         DEC L0491
         RTS
L9A06:   LDX #$01
         JSR L9148
         CPX #$FF
         BNE L9A10
         RTS
L9A10:   INC L04A7
         LDX L047E
         LDA Random             ; Get Random Number
         AND #$3F
         CLC
         ADC #$60
         STA Sprite1_X,X
         LDY #$35
         STA Sprite1_Y,X
         LDA #$06
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDA Random             ; Get Random Number
         AND #$02
         TAY
         TXA
         ASL A
         TAX
         LDA LBD0A,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD0B,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDA L04A7
         CMP #$01
         BCC L9A6F
         LDY L04BA
         LDA LBECA,Y
         STA L0491
         LDA #$00
         STA L04A7
L9A6F:   RTS
L9A70:   LDA SysClock
         AND #$01
         BEQ L9A78
         RTS
L9A78:   LDA L0492
         BEQ L9A81
         DEC L0492
         RTS
L9A81:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE L9A8B
         RTS
L9A8B:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$01
         BEQ L9A9D
         LDA Random             ; Get Random Number
         AND #$07
         JMP L9AA7
L9A9D:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
L9AA7:   TAY
         LDA L9B07,Y
         STA Sprite1_X,X
         LDA L9B0F,Y
         STA Sprite1_Y,X
         LDA #$07
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L042A,X
         LDA L047F
         STA L0438,X
         LDA L9B17,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBD0E,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD0F,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDY L04BA
         LDA LBEE2,Y
         STA L0492
         RTS
L9B07:   ORA (ZRAM_01,X)
         AND Buff6
         TXS
         CMP FRE+3
         .BYTE $EF
L9B0F:   BPL L9B51
         JSR L1010
         JSR L1040
L9B17:   BRK
         BRK
         BRK
         BRK
         .BYTE $02,$02,$02,$02


;----------------------------------------------------
;               Attach LUNA or JOE
;----------------------------------------------------
L9B1F:   LDA SysClock             ;
         AND #$01               ;
         BEQ L9B27              ;
         RTS
L9B27:   LDA L0493
         BEQ L9B30
         DEC L0493
         RTS
L9B30:   LDX #$01
         JSR L9148
         CPX #$FF
         BNE L9B3A
         RTS
L9B3A:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$03               ; Random number between 0-3
         BEQ L9B4C              ; 25% of the time we skip ahead
         LDA Random             ; Get Random Number
         AND #$0F
         JMP L9B55
L9B4C:   CLC
         LDA BegaXPos1L
         ADC #$08
L9B51:   LSR A
         LSR A
         LSR A
         LSR A
L9B55:   TAY
         LDA L9BA1,Y
         STA Sprite1_X,X
         LDA L9BB1,Y
         STA Sprite1_Y,X
         LDA #$08
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDA L9BC1,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBD16,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD17,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDY L04BA
L9B9A:   LDA LBEFA,Y
         STA L0493
         RTS
L9BA1:   JSR L382C
         .BYTE $44
         BVC L9C03
         PLA
         .BYTE $74,$80
         STY LA498
         BCS L9B6B
         INY
         .BYTE $DC
L9BB1:   RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
         RTI
L9BC1:   ASL RAMLO
         ASL RAMLO
         .BYTE $02,$02
         BRK
         ASL L0000
         .BYTE $04
         BRK
         BRK
         .BYTE $02,$02
         BRK
         BRK
L9BD1:   LDX #$0D
         STX SRTIMR
L9BD6:   LDX SRTIMR
         LDA L0400,X
         CMP #$02
         BEQ L9BEB
         CMP #$03
         BEQ L9BEB
         CMP #$07
         BEQ L9BEB
         JMP L9C9E
L9BEB:   LDA L04CA,X
         BEQ L9BF3
         JMP L9C85
L9BF3:   LDA Sprite1_Y,X
         CMP #$2A
         BCS L9BFC
         JMP L9C0F
L9BFC:   CMP #$32
         BCC L9C03
         JMP L9C0F
L9C03:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE L9C12
         JMP L9C9E
L9C0F:   JMP L9C9E
L9C12:   LDX SRTIMR
         LDA #$0A
         JSR L51CE              ; Play Sound
         LDA #$3F
         STA L04CA,X
         LDA Sprite1_X,X
         CMP BegaXPos1R
         BCC L9C39
         SEC
         SBC BegaXPos1R
         LSR A
         LSR A
         LSR A
         LSR A
         TAY
         LDA L9CA7,Y
         LDX L047E
         STA L040E,X
         JMP L9C4C
L9C39:   SEC
         LDA BegaXPos1R
         SBC Sprite1_X,X
         LSR A
         LSR A
         LSR A
         LSR A
         TAY
         LDA L9CB7,Y
         LDX L047E
         STA L040E,X
L9C4C:   TAY
         TXA
         ASL A
         TAX
         LDA LBD3E,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD3F,Y
         STA L00B4,X
         STA ZRAM_02
         LDX L047E
         LDY #$01
         LDA (ZRAM_01),Y
         STA L0454,X
         LDA #$09
         STA L0400,X
         LDA #$FF
         STA L0438,X
         LDY SRTIMR
         CLC
         LDA Sprite1_X,Y
         ADC #$08
         STA Sprite1_X,X
         LDA Sprite1_Y,Y
         STA Sprite1_Y,X
         JMP L9C8B
L9C85:   LDX SRTIMR
         DEC L04CA,X
L9C8B:   LDX SRTIMR
         LDA L0400,X
         CMP #$02
         BEQ L9C9B
         CMP #$03
         BEQ L9C9E
         BNE L9C9E
L9C9B:   JMP L9C9E
L9C9E:   DEC SRTIMR
         BMI L9CA6
         JMP L9BD6
L9CA6:   RTS
L9CA7:   ASL RAMLO
         .BYTE $04,$02,$02,$02
         .BYTE $02,$02,$02,$02
         .BYTE $02,$02
L9CB3:   .BYTE $02,$02,$02,$02
L9CB7:   PHP
         ASL A
         ASL A
         .BYTE $0C,$0C,$0C,$0C
         .BYTE $0C,$0C,$0C,$0C
         .BYTE $0C,$0C,$0C,$0C
         .BYTE $0C
L9CC7:   LDA SysClock
         AND #$03
         BEQ L9CCF
         RTS
L9CCF:   LDA L0496
         BEQ L9CD8
         DEC L0496
         RTS
L9CD8:   LDA L00AB
         BPL L9CDD
         RTS
L9CDD:   LDA L0400
         CMP #$FF
         BNE L9D39
         LDA Random             ; Get Random Number
         AND #$01
         BEQ L9CF3
         LDA Random             ; Get Random Number
         AND #$0F
         JMP L9CFC
L9CF3:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
L9CFC:   STA L04B7
         TAY
         LDA LBD6A,Y
         STA Sprite1_X
         LDA LBD7A,Y
         STA Sprite1_Y
         LDA #$0A
         STA L0400
         LDA LBD8A,Y
         TAY
         LDA LBD56,Y
         STA L00B3
         STA ZRAM_01
         LDA LBD57,Y
         STA L00B4
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         STA L0454
         LDA #$FF
         STA L0438
         LDA #$00
         STA L040E
         STA L041C
         STA L042A
         RTS
L9D39:   LDX #$00
L9D3B:   LDA L0401,X
         CMP #$FF
         BEQ L9D49
         INX
         CPX #$0C
         BNE L9D3B
         BEQ L9D8E
L9D49:   LDY L04B7
         LDA LBD6A,Y
         STA MLTTMP,X
         LDA LBD7A,Y
         STA ZCHAIN,X
         LDA #$0B
         STA L0401,X
         STX SRTIMR
         TXA
         ASL A
         TAX
         LDA LBD8A,Y
         TAY
         LDA LBD56,Y
         STA L00B5,X
         STA ZRAM_01
         LDA LBD57,Y
         STA DATAD,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX SRTIMR
         STA L0455,X
         LDA #$FF
         STA L0439,X
         LDA #$00
         STA L040F,X
         STA L041D,X
         STA L042B,X
         RTS
L9D8E:   LDA L040D
         CMP #$FF
         BEQ L9D96
         RTS
L9D96:   LDY L04B7
         LDA LBD6A,Y
         STA COLAC
         LDA LBD7A,Y
         STA COLCRS+1
         LDA #$0C
         STA L040D
         LDY L04B7
         LDA LBD8A,Y
         TAY
         LDA LBD56,Y
         STA L00CD
         STA ZRAM_01
         LDA LBD57,Y
         STA L00CE
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         STA L0461
         LDA #$FF
         STA L0445
         LDA #$00
         STA L041B
         STA L0429
         STA L0437
         LDY L04BA
         LDA LBF12,Y
         STA L0496
         RTS
L9DDE:   LDA SysClock
         AND #$1F
         BEQ L9DE6
         RTS
L9DE6:   LDX #$01
         JSR L9148
         CPX #$FF
         BNE L9DF0
         RTS
L9DF0:   LDA Random             ; Get Random Number
         AND #$03
         BNE L9DF8
         RTS
L9DF8:   LDA #$15
         JSR L51CE              ; Play Sound
         LDA SysClock
         AND #$80
         BNE L9E0C
         LDA Random             ; Get Random Number
         AND #$07
         JMP L9E16
L9E0C:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
L9E16:   TAY
         LDX L047E
         LDA LBD9A,Y
         STA Sprite1_X,X
         LDA LBDA2,Y
         STA Sprite1_Y,X
         LDA #$01
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         TYA
         ASL A
         TAY
         TXA
         ASL A
         TAX
         LDA LBD1E,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD1F,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         RTS
L9E5B:   LDA SysClock
         AND #$01
         BEQ L9E63
         RTS
L9E63:   LDA L048C
         BEQ L9E6C
         DEC L048C
         RTS
L9E6C:   LDY L04BA
         LDA LBE52,Y
         STA L048C
         LDX #$02
         JSR L9148
         CPX #$FF
         BNE L9E7F
         RTS
L9E7F:   LDA #$15
         JSR L51CE              ; Play Sound
         LDA SysClock
         AND #$80
         BNE L9E93
         LDA Random             ; Get Random Number
         AND #$07
         JMP L9E9D
L9E93:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
L9E9D:   TAY
         LDX L047E
         LDA LBD9A,Y
         STA Sprite1_X,X
         LDA LBDA2,Y
         STA Sprite1_Y,X
         LDA #$0D
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA L047F
         STA L0438,X
         TYA
         ASL A
         TAY
         TXA
         ASL A
         TAX
         LDA LBD1E,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD1F,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$FF
         STA L0438,X
         LDA #$00
         STA L0400,X
         STA L040E,X
         RTS
L9EF3:   LDA SysClock
         AND #$03
         BEQ L9EFB
         RTS
L9EFB:   LDA L048D
         BEQ L9F04
         DEC L048D
         RTS
L9F04:   LDX #$04
         JSR L9148
         CPX #$FF
         BNE L9F0E
         RTS                    ; Return
L9F0E:   LDA L04A3
         BNE L9F23
         LDA Random             ; Get Random Number
         AND #$7F
         CLC
         ADC #$40
         STA L0498
         AND #$01
         STA L04AE
L9F23:   INC L04A3
         LDX L047E
         LDA L0498
         STA Sprite1_X,X
         LDA #$00
         STA Sprite1_Y,X
         LDA #$02
         STA L0400,X
         LDA L047F
         STA L0438,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L042A,X
         LDA L04AE
         ASL A
         TAY
         TXA
         ASL A
         TAX
         LDA LBCBE,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCBF,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         LDA #$FF
         STA L0438,X
         INC L04A3
         LDX L0480
         LDA L0498
         LDA #$00
         SBC L0498
         STA Sprite1_X,X
         LDA #$00
         STA Sprite1_Y,X
         LDA #$02
         STA L0400,X
         LDA L0481
         STA L0438,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L042A,X
         LDA L04AE
         EOR #$01
         ASL A
         TAY
         TXA
         ASL A
         TAX
         LDA LBCBE,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCBF,Y
L9FB9:   STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L0480
         STA L0454,X
         LDX L0481
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         LDA #$FF
         STA L0438,X
         LDA L04A3
         CMP #$02
         BCC L9FEF
         LDY L04BA
         LDA LBE6A,Y
         STA L048D
         LDA #$00
         STA L04A3
L9FEF:   RTS
L9FF0:   LDA SysClock
         AND #$01
         BEQ L9FF8
         RTS
L9FF8:   LDA L0492
         BEQ LA001
         DEC L0492
         RTS
LA001:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE LA00B
         RTS
LA00B:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$01
         BEQ LA01D
         LDA Random             ; Get Random Number
         AND #$07
         JMP LA027
LA01D:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
LA027:   TAY
         LDA LBDAA,Y
         STA Sprite1_X,X
         LDA LBDB2,Y
         STA Sprite1_Y,X
         LDA #$07
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L042A,X
         LDA L047F
         STA L0438,X
         LDA LBDBA,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBD12,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD13,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDY L04BA
         LDA LBEE2,Y
         STA L0492
         RTS
LA087:   LDA SysClock
         AND #$03
         BEQ LA08F
         RTS
LA08F:   LDA L0496
         BEQ LA098
         DEC L0496
         RTS
LA098:   LDA L00AB
         BPL LA09D
         RTS
LA09D:   LDA L0400
         CMP #$FF
         BNE LA0F9
         LDA Random             ; Get Random Number
         AND #$01
         BEQ LA0B3
         LDA Random             ; Get Random Number
         AND #$0F
         JMP LA0BC
LA0B3:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
LA0BC:   STA L04B7
         TAY
         LDA LBDC2,Y
         STA Sprite1_X
         LDA LBDD2,Y
         STA Sprite1_Y
         LDA #$0A
         STA L0400
         LDA LBDE2,Y
         TAY
         LDA LBD4E,Y
         STA L00B3
         STA ZRAM_01
         LDA LBD4F,Y
         STA L00B4
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         STA L0454
         LDA #$FF
         STA L0438
         LDA #$00
         STA L040E
         STA L041C
         STA L042A
         RTS
LA0F9:   LDX #$00
LA0FB:   LDA L0401,X
         CMP #$FF
         BEQ LA109
         INX
         CPX #$0C
         BNE LA0FB
         BEQ LA14E
LA109:   LDY L04B7
         LDA LBDC2,Y
         STA MLTTMP,X
         LDA LBDD2,Y
         STA ZCHAIN,X
         LDA #$0B
         STA L0401,X
         STX SRTIMR
         TXA
         ASL A
         TAX
         LDA LBDE2,Y
         TAY
         LDA LBD4E,Y
         STA L00B5,X
         STA ZRAM_01
         LDA LBD4F,Y
         STA DATAD,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX SRTIMR
         STA L0455,X
         LDA #$FF
         STA L0439,X
         LDA #$00
         STA L040F,X
         STA L041D,X
         STA L042B,X
         RTS
LA14E:   LDA L040D
         CMP #$FF
         BEQ LA156
         RTS
LA156:   LDY L04B7
         LDA LBDC2,Y
         STA COLAC
         LDA LBDD2,Y
         STA COLCRS+1
         LDA #$0C
         STA L040D
         LDY L04B7
         LDA LBDE2,Y
         TAY
         LDA LBD4E,Y
         STA L00CD
         STA ZRAM_01
         LDA LBD4F,Y
         STA L00CE
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         STA L0461
         LDA #$FF
         STA L0445
         LDA #$00
         STA L041B
         STA L0429
         STA L0437
         LDY L04BA
         LDA LBF12,Y
         STA L0496
         RTS
LA19E:   LDA SysClock
         AND #$01
         BEQ LA1A6
         RTS
LA1A6:   LDA L0490
         BEQ LA1AF
         DEC L0490
         RTS
LA1AF:   LDX #$02
         JSR L9148
         CPX #$FF
         BNE LA1B9
         RTS
LA1B9:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$01
         BEQ LA1CB
         LDA Random             ; Get Random Number
         AND #$07
         JMP LA1D5
LA1CB:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
LA1D5:   TAY
         LDA LBDF2,Y
         STA Sprite1_X,X
         LDA LBDFA,Y
         STA Sprite1_Y,X
         LDA #$05
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA L047F
         STA L0438,X
         LDA LBE02,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBCFE,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCFF,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDX L047F
         LDA #$00
         STA L041C,X
         STA L0400,X
         STA L040E,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDY L04BA
         LDA LBEB2,Y
         STA L0490
         RTS
LA238:   LDA SysClock
         AND #$01
         BEQ LA240
         RTS
LA240:   LDA L048F
         BEQ LA249
         DEC L048F
         RTS
LA249:   LDX #$01
         JSR L9148
         CPX #$FF
         BNE LA253
         RTS
LA253:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$03
         BEQ LA265
         LDA Random             ; Get Random Number
         AND #$07
         JMP LA26F
LA265:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
LA26F:   TAY
         LDA LBE0A,Y
         STA Sprite1_X,X
         LDA LBE12,Y
         STA Sprite1_Y,X
         LDA #$04
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDA LBE1A,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBCEA,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBCEB,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDY L04BA
         LDA LBE9A,Y
         STA L048F
         RTS
LA2BB:   LDA SysClock
         AND #$01
         BEQ LA2C3
         RTS
LA2C3:   LDA L048F
         BEQ LA2CC
         DEC L048F
LA2CB:   RTS
LA2CC:   LDX #$01
         JSR L9148
         CPX #$FF
         BNE LA2D6
         RTS
LA2D6:   LDX L047E
         LDA Random             ; Get Random Number
         AND #$03               ;
         BEQ LA2E8              ;
         LDA Random             ; Get Random Number
         AND #$07               ;
         JMP LA2F2              ;
LA2E8:   CLC
         LDA BegaXPos1L
         ADC #$08
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
LA2F2:   TAY
         LDA LBE22,Y
         STA Sprite1_X,X
         LDA LBE2A,Y
         STA Sprite1_Y,X
         LDA #$0E
         STA L0400,X
         LDA #$00
         STA L041C,X
         STA L040E,X
         STA L0446,X
         STA L042A,X
         LDA #$FF
         STA L0438,X
         LDA LBE32,Y
         TAY
         TXA
         ASL A
         TAX
         LDA LBD5E,Y
         STA L00B3,X
         STA ZRAM_01
         LDA LBD5F,Y
         STA L00B4,X
         STA ZRAM_02
         LDY #$01
         LDA (ZRAM_01),Y
         LDX L047E
         STA L0454,X
         LDY L04BA
         LDA LBF2A,Y
         STA L048F
         RTS
LA33E:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$01
         BNE LA376
         LDA L0400,X
         BMI LA377
         LDA L042A,X
         BNE LA373
         LDA #$04
         STA L042A,X
         LDA L040E,X
         AND #$F8
         ORA L041C,X
         STA L040E,X
         INC L041C,X
         LDA L041C,X
         CMP #$08
         BCC LA376
         LDA #$00
         STA L041C,X
LA373:   DEC L042A,X
LA376:   RTS
LA377:   LDA L042A,X
         BEQ LA380
         DEC L042A,X
         RTS
LA380:   LDA #$08
         STA L042A,X
         INC L040E,X
         RTS
LA389:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$02
         BNE LA3AA
         LDA L0400,X
         BPL LA39D
         JMP LA3DF
LA39D:   LDA L042A,X
         BEQ LA3AB
         DEC L042A,X
         LDA L0470,X
         BNE LA3BF
LA3AA:   RTS
LA3AB:   LDA #$01
         STA L042A,X
         INC L041C,X
         LDA L041C,X
         CMP #$04
         BCC LA3BF
         LDA #$00
         STA L041C,X
LA3BF:   TXA
         ASL A
         TAX
         LDA (L00B3,X)
         TAY
         LDA #$00
         LDX LB100,Y
         BMI LA3CE
         ORA #$08
LA3CE:   LDX LB000,Y
         BMI LA3D5
         ORA #$04
LA3D5:   LDX SRTIMR
         ORA L041C,X
         STA L040E,X
         RTS
LA3DF:   LDA L042A,X
         BEQ LA3E8
         DEC L042A,X
         RTS
LA3E8:   LDA #$08
         STA L042A,X
         INC L040E,X
         RTS
LA3F1:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$03
         BNE LA40D
         LDA L0400,X
         BPL LA405
         JMP LA43B
LA405:   LDA L042A,X
         BEQ LA40E
         DEC L042A,X
LA40D:   RTS
LA40E:   LDA #$0E
         STA L042A,X
         LDA L0446,X
         CMP #$FF
         BNE LA425
         LDA L040E,X
         AND #$01
         EOR #$01
         STA L040E,X
         RTS
LA425:   CLC
         ADC #$02
         STA L040E,X
         INC L0446,X
         LDA L0446,X
         CMP #$02
         BCC LA43A
         LDA #$FF
         STA L0446,X
LA43A:   RTS
LA43B:   LDA L042A,X
         BEQ LA444
         DEC L042A,X
         RTS
LA444:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA44D:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$04
         BNE LA46E
         LDA L0400,X
         BPL LA461
         JMP LA4D0
LA461:   LDA L042A,X
         BEQ LA46F
         DEC L042A,X
         LDA L0470,X
         BNE LA485
LA46E:   RTS
LA46F:   LDA #$04
         STA L042A,X
         LDA L0446,X
         CMP #$FF
         BNE LA4BA
         LDA L041C,X
         AND #$01
         EOR #$01
         STA L041C,X
LA485:   LDA L0446,X
         CMP #$FF
         BNE LA4BA
         TXA
         ASL A
         TAX
         LDA (L00B3,X)
         TAY
         LDA #$00
         LDX LB100,Y
         BEQ LA4A2
         ORA #$08
         LDX LB100,Y
         BMI LA4A2
         ORA #$10
LA4A2:   LDX LB000,Y
         BEQ LA4B0
         ORA #$02
         LDX LB000,Y
         BMI LA4B0
         ORA #$04
LA4B0:   LDX SRTIMR
         ORA L041C,X
         STA L040E,X
         RTS
LA4BA:   CLC
         ADC #$20
         STA L040E,X
         INC L0446,X
         LDA L0446,X
         CMP #$06
         BCC LA4CF
         LDA #$FF
         STA L0446,X
LA4CF:   RTS
LA4D0:   LDA L042A,X
         BEQ LA4D9
         DEC L042A,X
         RTS
LA4D9:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA4E2:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$05
         BNE LA4FE
         LDA L0400,X
         BPL LA4F6
         JMP LA53C
LA4F6:   LDA L042A,X
         BEQ LA4FF
         DEC L042A,X
LA4FE:   RTS
LA4FF:   LDA #$02
         STA L042A,X
         LDA L0446,X
         CMP #$FF
         BNE LA521
         LDA L041C,X
         STA L040E,X
         INC L041C,X
         LDA L041C,X
         CMP #$0C
         BCC LA520
         LDA #$00
         STA L041C,X
LA520:   RTS
LA521:   CLC
         ADC #$0C
         STA L040E,X
         INC L041C,X
         LDA L041C,X
         CMP #$07
         BCC LA53B
         LDA #$FF
         STA L0446,X
         LDA #$00
         STA L041C,X
LA53B:   RTS
LA53C:   LDA L042A,X
         BEQ LA545
         DEC L042A,X
         RTS
LA545:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA54E:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$06
         BNE LA56A
         LDA L0400,X
         BPL LA562
         JMP LA581
LA562:   LDA L042A,X
         BEQ LA56B
         DEC L042A,X
LA56A:   RTS
LA56B:   LDA #$04
         STA L042A,X
         LDA L041C,X
         STA L040E,X
         LDA L041C,X
         AND #$01
         EOR #$01
         STA L041C,X
         RTS
LA581:   LDA L042A,X
         BEQ LA58A
         DEC L042A,X
         RTS
LA58A:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA593:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$07
         BNE LA5AF
         LDA L0400,X
         BPL LA5A7
         JMP LA5CB
LA5A7:   LDA L042A,X
         BEQ LA5B0
         DEC L042A,X
LA5AF:   RTS
LA5B0:   LDA #$08
         STA L042A,X
         LDA L041C,X
         STA L040E,X
         INC L041C,X
         LDA L041C,X
         CMP #$03
         BCC LA5CA
         LDA #$00
         STA L041C,X
LA5CA:   RTS
LA5CB:   LDA L042A,X
         BEQ LA5D4
         DEC L042A,X
         RTS
LA5D4:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA5DD:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$08
         BNE LA603
         LDA L0400,X
         BPL LA5F1
         JMP LA628
LA5F1:   LDA L0446,X
         CMP #$FF
         BNE LA604
         LDA Sprite1_Y,X
         LSR A
         LSR A
         LSR A
         LSR A
         AND #$0F
         STA L040E,X
LA603:   RTS
LA604:   LDA L042A,X
         BEQ LA60D
         DEC L042A,X
         RTS
LA60D:   CLC
         ADC #$10
         STA L040E,X
         LDA #$05
         STA L042A,X
         INC L0446,X
         LDA L0446,X
         CMP #$06
         BCC LA627
         LDA #$FF
         STA L0446,X
LA627:   RTS
LA628:   LDA L042A,X
         BEQ LA631
         DEC L042A,X
         RTS
LA631:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA63A:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$09
         BNE LA656
         LDA L0400,X
         BPL LA64E
         JMP LA665
LA64E:   LDA L042A,X
         BEQ LA657
         DEC L042A,X
LA656:   RTS
LA657:   LDA #$0A
         STA L042A,X
LA65C:   LDA L040E,X
         EOR #$01
         STA L040E,X
         RTS
LA665:   LDA L042A,X
         BEQ LA66E
         DEC L042A,X
         RTS
LA66E:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA677:   LDX SRTIMR
         LDA L0400,X
         BPL LA682
         JMP LA6CD
LA682:   LDA L042A,X
         BEQ LA690
         DEC L042A,X
         LDA L0470,X
         BNE LA69F
         RTS
LA690:   LDA #$1F
         STA L042A,X
         LDA L041C,X
         AND #$01
         EOR #$01
         STA L041C,X
LA69F:   TXA
         ASL A
         TAX
         LDA (L00B3,X)
         TAY
LA6A5:   LDA #$00
         LDX LB100,Y
         BEQ LA6B5
         ORA #$08
         LDX LB100,Y
         BMI LA6B5
         ORA #$10
LA6B5:   LDX LB000,Y
         BEQ LA6C3
         ORA #$02
         LDX LB000,Y
         BMI LA6C3
         ORA #$04
LA6C3:   LDX SRTIMR
         ORA L041C,X
         STA L040E,X
         RTS
LA6CD:   LDA L042A
         BEQ LA6D6
         DEC L042A,X
         RTS
LA6D6:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA6DF:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$01
         BNE LA6FF
         LDA L0400,X
         BPL LA6F3
         JMP LA700
LA6F3:   LDA Sprite1_Y,X
         LSR A
         LSR A
         LSR A
         LSR A
         LSR A
         AND #$07
         STA L040E,X
LA6FF:   RTS
LA700:   LDA L042A,X
         BEQ LA709
         DEC L042A,X
         RTS
LA709:   LDA #$02
         STA L042A,X
         INC L040E,X
         RTS
LA712:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$0D
         BNE LA749
         LDA L0400,X
         BPL LA726
         JMP LA74A
LA726:   LDA L042A,X
         BNE LA745
         LDA #$06
         STA L042A,X
         LDA L041C,X
         STA L040E,X
         INC L041C,X
         LDA L041C,X
         CMP #$06
         BCC LA748
         LDA #$00
         STA L041C,X
LA745:   DEC L042A,X
LA748:   RTS
LA749:   RTS
LA74A:   LDA L042A,X
         BEQ LA753
         DEC L042A,X
         RTS
LA753:   LDA #$0F
         STA L042A,X
         INC L040E,X
         RTS
LA75C:   LDX SRTIMR
         LDA L0400,X
         AND #$7F
         CMP #$0E
         BNE LA78F
         LDA L0400,X
         BMI LA790
         LDA L042A,X
         BNE LA78C
         LDA #$06
         STA L042A,X
         LDA L041C,X
         STA L040E,X
         INC L041C,X
         LDA L041C,X
         CMP #$06
         BCC LA78F
         LDA #$00
         STA L041C,X
LA78C:   DEC L042A,X
LA78F:   RTS
LA790:   LDA L042A,X
         BEQ LA799
         DEC L042A,X
         RTS
LA799:   LDA #$08
         STA L042A,X
         INC L040E,X
         RTS
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LA900:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LABBA:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LAD1C:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF


;----------------------------------------------------
;            Girder Dropping Patterns Y
;----------------------------------------------------
LB000    .BYTE $00,$FF,$FF,$00
         .BYTE $01,$01,$01,$00
         .BYTE $FF,$FE,$FE,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $02,$02,$02,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FE,$FD,$FD,$FD
         .BYTE $FD,$FE,$FF,$00
         .BYTE $01,$02,$03,$03
         .BYTE $03,$03,$03,$03
         .BYTE $03,$02,$01,$00
         .BYTE $FF,$FE,$FD,$FD
         .BYTE $FD,$FC,$FC,$FC
         .BYTE $FC,$FC,$FD,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$04,$04,$04
         .BYTE $04,$04,$04,$04
         .BYTE $04,$04,$03,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FD,$FC,$FC,$FC
         .BYTE $FC,$FB,$FB,$FB
         .BYTE $FB,$FB,$FB,$FC
         .BYTE $FD,$FE,$FF,$00
         .BYTE $01,$02,$03,$04
         .BYTE $05,$05,$05,$05
         .BYTE $05,$05,$05,$05
         .BYTE $05,$05,$05,$04
         .BYTE $03,$02,$01,$00
         .BYTE $FF,$FE,$FD,$FC
         .BYTE $FB,$FB,$FB,$FB
         .BYTE $FB,$FA,$FA,$FA
         .BYTE $FA,$FA,$FA,$FA
         .BYTE $FB,$FC,$FD,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$04,$05,$06
         .BYTE $06,$06,$06,$06
         .BYTE $06,$06,$06,$06
         .BYTE $06,$06,$06,$06
         .BYTE $05,$04,$03,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FD,$FC,$FB,$FA
         .BYTE $FA,$FA,$FA,$FA
         .BYTE $FA,$F9,$F9,$F9
         .BYTE $F9,$F9,$F9
LB0AF    .BYTE $F9,$F9,$FA
LB0B2    .BYTE $FB,$FC,$FD,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$04,$05,$06
         .BYTE $07,$07,$07,$07
         .BYTE $07,$07,$07,$30
         .BYTE $07,$07,$07,$07
         .BYTE $07,$07,$30,$06
         .BYTE $05,$04,$03,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FD,$FC,$FB,$FA
         .BYTE $F9,$F9,$F9,$F9
         .BYTE $F9,$F9,$F9,$C8
         .BYTE $F8,$F8,$F8,$F8
         .BYTE $F8,$F8,$F8,$C8
         .BYTE $F9,$FA,$FB,$FC
         .BYTE $FD,$FE,$FF,$00
         .BYTE $01,$02,$03,$04
         .BYTE $05,$06,$30,$C8
         .BYTE $F8,$F8,$F8,$F8
         .BYTE $F8,$F8

;----------------------------------------------------
;            Girder Dropping Patterns X
;----------------------------------------------------
LB100:   .BYTE $00,$00,$01,$01,$01,$00,$FF,$FF,$FF,
         .BYTE $00,$01,$02,$02,$02,$02,$02,$01,$00,
         .BYTE $FF,$FE,$FE,$FE,$FE,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$03,$03,$03
         .BYTE $03,($03),$03,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FD,$FD,$FD,$FD
         .BYTE $FD,$FD,$FD,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$04,$04,$04
         .BYTE $04,$04,$04,$04
         .BYTE $04,$04,$03,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FD,$FC,$FC,$FC
         .BYTE $FC,$FC,$FC,$FC
         .BYTE $FC,$FC,$FD,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$04,$05,$05
         .BYTE $05,$05,$05,$05
         .BYTE $05,$05,$05,$05
         .BYTE $05,$04,$03,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FD,$FC,$FB,$FB
         .BYTE $FB,$FB,$FB,$FB
         .BYTE $FB,$FB,$FB,$FB
         .BYTE $FB,$FC,$FD,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$04,$05,$06
         .BYTE $06,$06,$06,$06
         .BYTE $06,$06,$06,$06
         .BYTE $06,$06,$06,$06
         .BYTE $05,$04,$03,$02
         .BYTE $01,$00,$FF,$FE
         .BYTE $FD,$FC,$FB,$FA
         .BYTE $FA,$FA,$FA,$FA
         .BYTE $FA,$FA,$FA,$FA
         .BYTE $FA,$FA,$FA,$FA
         .BYTE $FB,$FC,$FD,$FE
         .BYTE $FF,$00,$01,$02
         .BYTE $03,$04,$05,$06
         .BYTE $07,$07,$07
LB1B3:   .BYTE $07,$07,$07,$07
         .BYTE $07,$07,$07,$07
         .BYTE $07,$07,$07,$07
         .BYTE $06,$05,$04,$03
         .BYTE $02,$01,$00,$FF
         .BYTE $FE,$FD,$FC,$FB
         .BYTE $FA,$C9,$F9,$F9
         .BYTE $F9,$F9,$F9,$F9
         .BYTE $C9,$F9,$F9,$F9
         .BYTE $F9,$F9,$F9,$F9
         .BYTE $FA,$FB,$FC,$FD
         .BYTE $FE,$FF,$00,$01
         .BYTE $02,$03,$04,$05
         .BYTE $06,$07,$30,$08
         .BYTE $08,$08,$08,$08
         .BYTE $08,$08,$30,$08
         .BYTE $08,$08,$08,$08
         .BYTE $08,$30,$C9,$FA
         .BYTE $FB,$FC,$FD,$FE
         .BYTE $FF

;----------------------------------------------------
;             Level Data Pointers (0-49)  Foto1
;----------------------------------------------------
LB200:   .WORD $B262,$B265,$B268,$B26B,$B26E,$B271
         .WORD $B274,$B277,$B27A,$B27D,$B280,$B283
         .WORD $B286,$B289,$B28C,$B28F,$B292,$B295
         .WORD $B298,$B29B,$B29E,$B2A1,$B2A4,$B2A7
         .WORD $B2AA,$B2AD,$B2B0,$B2B3,$B2B6,$B2B9
         .WORD $B2BC,$B2BF,$B2C2,$B2C5,$B2C8,$B2CB
         .WORD $B2CE,$B2D1,$B2D4,$B2D7,$B2DA,$B2DD
         .WORD $B2E0,$B2E3,$B2E6,$B2E9,$B2EC,$B2EF
         .WORD $B2F2
;----------------------------------------------------
;                Level Data
;----------------------------------------------------
;               Level,Level,Nothing
;----------------------------------------------------
LB262:   .BYTE $01,$00,$00
         .BYTE $02,$02,$00
         .BYTE $03,$00,$00
         .BYTE $04,$04,$00
         .BYTE $05,$05,$00
         .BYTE $06,$00,$00
         .BYTE $07,$00,$00
         .BYTE $08,$08,$08
         .BYTE $09,$09,$00
         .BYTE $10,$10,$00
         .BYTE $11,$00,$00
         .BYTE $12,$00,$00
         .BYTE $13,$12,$00
         .BYTE $14,$14,$00
         .BYTE $15,$00,$00
         .BYTE $16,$16,$00
         .BYTE $17,$00,$00
         .BYTE $18,$00,$00
         .BYTE $19,$00,$00
         .BYTE $20,$00,$00
         .BYTE $21,$00,$00
         .BYTE $22,$00,$00
         .BYTE $23,$00,$00
         .BYTE $24,$00,$00
         .BYTE $25,$00,$00
         .BYTE $26,$00,$00
         .BYTE $27,$00,$00
         .BYTE $28,$00,$00
         .BYTE $29,$00,$00
         .BYTE $30,$00,$00
         .BYTE $31,$00,$00
         .BYTE $32,$00,$00
         .BYTE $33,$00,$00
         .BYTE $34,$00,$00
         .BYTE $35,$00,$00
         .BYTE $36,$00,$00
         .BYTE $37,$00,$00
         .BYTE $38,$00,$00
         .BYTE $39,$00,$00
         .BYTE $40,$00,$00
         .BYTE $41,$00,$00
         .BYTE $42,$00,$00
         .BYTE $43,$00,$00
         .BYTE $44,$00,$00
         .BYTE $45,$00,$00
         .BYTE $46,$00,$00
         .BYTE $47,$00,$00
         .BYTE $48,$00,$00
         .BYTE $49,$00,$00

;----------------------------------------------------
;                Level Data Pointers (0-49) Foto2
;----------------------------------------------------
LB2F5:   .WORD $B357,$B35A,$B35D,$B360,$B363,$B366
         .WORD $B369,$B36C,$B36F,$B372,$B375,$B378
         .WORD $B37B,$B37E,$B381,$B384,$B387,$B38A
         .WORD $B38D,$B390,$B393,$B396,$B399,$B39C
         .WORD $B39F,$B3A2,$B3A5,$B3A8,$B3AB,$B3AE
         .WORD $B3B1,$B3B4,$B3B7,$B3BA,$B3BD,$B3C0
         .WORD $B3C3,$B3C6,$B3C9,$B3CC,$B3CF,$B3D2
         .WORD $B3D5,$B3D8,$B3DB,$B3DE,$B3E1,$B3E4
         .WORD $B3E7
;----------------------------------------------------
;               Level,Routine #, Nothing 
;----------------------------------------------------
LB357:   .BYTE $00,$00,$00
         .BYTE $01,$02,$00
         .BYTE $02,$00,$00
         .BYTE $03,$05,$00
         .BYTE $04,$07,$00
         .BYTE $05,$00,$00
         .BYTE $06,$00,$00
         .BYTE $07,$0B,$00
         .BYTE $08,$0D,$00
         .BYTE $09,$0E,$00
         .BYTE $0A,$00,$00
         .BYTE $0B,$00,$00
         .BYTE $0C,$11,$00
         .BYTE $0D,$13,$00
         .BYTE $0E,$00,$00
         .BYTE $0F,$00,$00
         .BYTE $10,$00,$00
         .BYTE $11,$00,$00
         .BYTE $12,$00,$00
         .BYTE $13,$00,$00
         .BYTE $14,$00,$00
         .BYTE $15,$00,$00
         .BYTE $16,$00,$00
         .BYTE $17,$00,$00
         .BYTE $18,$00,$00
         .BYTE $19,$00,$00
         .BYTE $1A,$00,$00
         .BYTE $1B,$00,$00
         .BYTE $1C,$00,$00
         .BYTE $1D,$00,$00
         .BYTE $1E,$00,$00
         .BYTE $1F,$00,$00
         .BYTE $20,$00,$00
         .BYTE $21,$00,$00
         .BYTE $22,$00,$00
         .BYTE $23,$00,$00
         .BYTE $24,$00,$00
         .BYTE $25,$00,$00
         .BYTE $26,$00,$00
         .BYTE $27,$00,$00
         .BYTE $28,$00,$00
         .BYTE $29,$00,$00
         .BYTE $30,$00,$00
         .BYTE $31,$00,$00
         .BYTE $32,$00,$00
         .BYTE $33,$00,$00
         .BYTE $34,$00,$00
         .BYTE $35,$00,$00
         .BYTE $36,$00,$00

;----------------------------------------------------
;               Data addresses 1     Foto3
;----------------------------------------------------
LB3EA:   .WORD $B44C,$B44F,$B452,$B455,$B458,$B45B
         .WORD $B45E,$B461,$B464,$B467,$B46A,$B46D
         .WORD $B470,$B473,$B476,$B479,$B47C,$B47F
         .WORD $B482,$B485,$B488,$B48B,$B48E,$B491
         .WORD $B494,$B497,$B49A,$B49D,$B4A0,$B4A3
         .WORD $B4A6,$B4A9,$B4AC,$B4AF,$B4B2,$B4B5
         .WORD $B4B8,$B4BB,$B4BE,$B4C1,$B4C4,$B4C7
         .WORD $B4CA,$B4CD,$B4D0,$B4D3,$B4D6,$B4D9
         .WORD $B4DC
;----------------------------------------------------
;               Level,Level,Video Sequence 
;----------------------------------------------------
LB44C:   .BYTE $00,$00,$02   ; 1 Video Sequence 2
         .BYTE $00,$00,$FE   ; 2 NO Video
         .BYTE $00,$00,$03   ; 3 Video Sequence 3
         .BYTE $03,$00,$04
         .BYTE $05,$00,$05
         .BYTE $00,$00,$06
         .BYTE $00,$00,$07
         .BYTE $09,$0A,$08
         .BYTE $0B,$00,$09
         .BYTE $0D,$00,$FE
         .BYTE $00,$00,$0A
         .BYTE $00,$00,$0B
         .BYTE $10,$00,$FE
         .BYTE $12,$13,$0C
         .BYTE $00,$00,$FE
         .BYTE $00,$00,$0D
         .BYTE $00,$00,$0E
         .BYTE $00,$00,$0F
         .BYTE $00,$00,$FE
         .BYTE $00,$00,$10
         .BYTE $00,$00,$11
         .BYTE $00,$00,$FE
         .BYTE $00,$00,$12
         .BYTE $00,$00,$FF
         .BYTE $00,$00,$00
         .BYTE $00,$00,$51
         .BYTE $00,$00,$52
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$53
         .BYTE $00,$00,$00
         .BYTE $00,$00,$54
         .BYTE $00,$00,$55
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$56
         .BYTE $00,$00,$57
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00
;----------------------------------------------------
;               Data addresses 2      Foto4
;----------------------------------------------------
LB4DF:   .WORD $B541,$B544,$B547,$B54A,$B54D,$B550
         .WORD $B553,$B556,$B559,$B55C,$B55F,$B562
         .WORD $B565,$B568,$B56B,$B56E,$B571,$B574
         .WORD $B577,$B57A,$B57D,$B580,$B583,$B586
         .WORD $B589,$B58C,$B58F,$B592,$B595,$B598
         .WORD $B59B,$B59E,$B5A1,$B5A4,$B5A7,$B5AA
         .WORD $B5AD,$B5B0,$B5B3,$B5B6,$B5B9,$B5BC
         .WORD $B5BF,$B5C2,$B5C5,$B5C8,$B5CB,$B5CE
         .WORD $B5D1
;----------------------------------------------------
;               Level,Level,Nothing 
;----------------------------------------------------
LB541:   .BYTE $15,$00,$00
         .BYTE $16,$02,$00
         .BYTE $17,$00,$00
         .BYTE $18,$05,$00
         .BYTE $19,$07,$00
         .BYTE $1A,$00,$00
         .BYTE $1B,$00,$00
         .BYTE $1C,$0B,$00
         .BYTE $1D,$0D,$00
         .BYTE $1E,$0E,$00
         .BYTE $1F,$00,$00
         .BYTE $20,$00,$00
         .BYTE $21,$11,$00
         .BYTE $22,$13,$00
         .BYTE $23,$00,$00
         .BYTE $24,$00,$00
         .BYTE $25,$00,$00
         .BYTE $26,$00,$00
         .BYTE $27,$00,$00
         .BYTE $28,$00,$00
         .BYTE $29,$00,$00
         .BYTE $2A,$00,$00
         .BYTE $2B,$00,$00
         .BYTE $2C,$00,$00
         .BYTE $2D,$00,$00
         .BYTE $1F,$00,$00
         .BYTE $20,$00,$00
         .BYTE $21,$00,$00
         .BYTE $22,$00,$00
         .BYTE $23,$00,$00
         .BYTE $24,$00,$00
         .BYTE $25,$00,$00
         .BYTE $26,$00,$00
         .BYTE $27,$00,$00
         .BYTE $28,$00,$00
         .BYTE $29,$00,$00
         .BYTE $2A,$00,$00
         .BYTE $2B,$00,$00
         .BYTE $2C,$00,$00
         .BYTE $2D,$00,$00
         .BYTE $2E,$00,$00
         .BYTE $2F,$00,$00
         .BYTE $30,$00,$00
         .BYTE $31,$00,$00
         .BYTE $32,$00,$00
         .BYTE $33,$00,$00
         .BYTE $34,$00,$00
         .BYTE $35,$00,$00
         .BYTE $36,$00,$00

LB5D4:   .WORD $B64C,$B64C,$B656,$B67A,$B688,$B6B0
         .WORD $B6DA,$B6DE,$B6EA,$B702,$B726,$B748
         .WORD $B76A,$B78C,$B79C,$B7A4,$B7A4,$B7AE
         .WORD $B7C2,$B7CA,$B7F2,$B81E,$B822,$B82E
         .WORD $B84A,$B86E,$B890,$B892,$B894,$B89E

LB610:   .WORD $B8A6,$B8A6,$B8B0,$B8D4,$B8DD,$B905
         .WORD $B931,$B935,$B941,$B959,$B97D,$B97D
         .WORD $B97D,$B99F,$B9AB

LB62E:   .WORD $B9B3,$B9B3,$B9B5,$B9C9,$B9D2,$B9D4
         .WORD $B9F0,$B9F2,$B9F6,$B9F8,$B97D,$B97D
         .WORD $B97D,$BA0C,$BA16,$B64E

;----------------------------------------------------
;               Level,Level,Nothing 
;----------------------------------------------------         
         .BYTE $26,$25,$24
         .BYTE $27,$23,$22
         .BYTE $21,$20,$5A
         .BYTE $B6,$6A,$B6
         .BYTE $2C,$30,$2E
         .BYTE $30,$2D,$31
         .BYTE $2F,$31,$2C
         .BYTE $30,$2E,$30
         .BYTE $2D,$31,$2F
         .BYTE $31,$2D,$31
         .BYTE $2F,$2F,$2C
         .BYTE $30,$2E,$2E
         .BYTE $2D,$31,$2F
         .BYTE $2F,$2C,$30
         .BYTE $2E,$2E,$7E
         .BYTE $B6,$83,$B6
         .BYTE $3E,$40,$48
         .BYTE $4A,$4C,$3F
         .BYTE $41,$49,$4B
         .BYTE $4D,$8A,$B6
         .BYTE $00,$00,$4E
         .BYTE $4F,$00,$00
         .BYTE $4E,$4F,$50
         .BYTE $51,$52,$53
         .BYTE $00,$00,$52
         .BYTE $53,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $50,$51,$52
         .BYTE $53,$00,$00
         .BYTE $52,$53,$56
         .BYTE $57,$56,$57
         .BYTE $56,$57,$B4
         .BYTE $B6,$C7,$B6
         .BYTE $58,$59,$5A
         .BYTE $5B,$58,$59
         .BYTE $5C,$5C,$5E
         .BYTE $5E,$5C,$5C
         .BYTE $64,$66,$64
         .BYTE $66,$68,$66
         .BYTE $68,$59,$58
         .BYTE $5B,$5A,$59
         .BYTE $58,$5D,$5D
         .BYTE $5F,$5F,$5D
         .BYTE $5D,$65,$67
         .BYTE $65,$67,$69
         .BYTE $67,$69,$DC
         .BYTE $B6,$6A,$6B
         .BYTE $E2,$B6,$E6
         .BYTE $B6,$6F,$70
         .BYTE $71,$70,$6F
         .BYTE $70,$71,$70
         .BYTE $EC,$B6,$8C
         .BYTE $8C,$8C,$8C
         .BYTE $8D,$8D,$8D
         .BYTE $8D,$8E,$8E
         .BYTE $8E,$8E,$8F
         .BYTE $8F,$8F,$8F
         .BYTE $8C,$8C,$8C
         .BYTE $8C,$8C,$8C


         .BYTE $06,$B7,$16,$B7
         .BYTE $84,$86,$80
         .BYTE $82,$7C,$7E
         .BYTE $78,$7A,$78
         .BYTE $7A,$7C,$7E
         .BYTE $80,$82,$84
         .BYTE $86,$85,$87
         .BYTE $81,$83,$7D
         .BYTE $7F,$79,$7B
         .BYTE $79,$7B,$7D
         .BYTE $7F,$81,$83
         .BYTE $85,$87,$28
         .BYTE $B7,$00,$00
         .BYTE $C0,$C1,$00
         .BYTE $00,$C0,$C1
         .BYTE $C2,$C3,$C4
         .BYTE $C5,$00,$00
         .BYTE $C4,$C5,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$C2,$C3
         .BYTE $C4,$C5,$00
         .BYTE $00,$C4,$C5
         .BYTE $4A,$B7,$00
         .BYTE $00,$C6,$C7
         .BYTE $00,$00,$C6
         .BYTE $C7,$C8,$C9
         .BYTE $CA,$CB,$00
         .BYTE $00,$CA,$CB
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$C8
         .BYTE $C9,$CA,$CB
         .BYTE $00,$00,$CA
         .BYTE $CB,$6C,$B7
         .BYTE $00,$00,$CC
         .BYTE $CD,$00,$00
         .BYTE $CC,$CD,$CE
         .BYTE $CF,$D0,$D1
         .BYTE $00,$00,$D0
         .BYTE $D1,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $CE,$CF,$D0
         .BYTE $D1,$00,$00
         .BYTE $D0,$D1,$90
         .BYTE $B7,$96,$B7
         .BYTE $20,$21,$22
         .BYTE $23,$26,$25
         .BYTE $24,$25,$26
         .BYTE $27,$22,$21
         .BYTE $9E,$B7,$D8
         .BYTE $D9,$DA,$DB
         .BYTE $DA,$D9,$A6
         .BYTE $B7,$01,$01
         .BYTE $01,$01,$01
         .BYTE $01,$01,$01
         .BYTE $B2,$B7,$B2
         .BYTE $B7,$01,$01
         .BYTE $01,$01,$03
         .BYTE $03,$03,$03
         .BYTE $05,$05,$05
         .BYTE $05,$07,$07
         .BYTE $07,$07,$C6
         .BYTE $B7,$C6,$B7
         .BYTE $01,$01,$01
         .BYTE $01,$CC,$B7
         .BYTE $00,$00,$03
         .BYTE $03,$00,$00
         .BYTE $01,$01,$01
         .BYTE $01,$03,$03
         .BYTE $00,$00,$01
         .BYTE $01,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $05,$05,$07
         .BYTE $07,$00,$00
         .BYTE $05,$05,$01
         .BYTE $01,$01,$01
         .BYTE $01,$01,$F6
         .BYTE $B7,$0A,$B8
         .BYTE $01,$05,$01
         .BYTE $05,$01,$05
         .BYTE $01,$05,$01
         .BYTE $05,$01,$05
         .BYTE $01,$01,$01
         .BYTE $01,$01,$01
         .BYTE $01,$01,$01
         .BYTE $05,$01,$05
         .BYTE $01,$05,$01
         .BYTE $05,$01,$05
         .BYTE $01,$05,$01
         .BYTE $01,$01,$01
         .BYTE $01,$01,$01
         .BYTE $01,$20,$B8
         .BYTE $01,$01,$26
         .BYTE $B8,$2A,$B8
         .BYTE $01,$01,$01
         .BYTE $01,$05,$05
         .BYTE $05,$05,$30
         .BYTE $B8,$01,$05
         .BYTE $03,$07,$01
         .BYTE $05,$03,$07
         .BYTE $01,$05,$03
         .BYTE $07,$01,$05
         .BYTE $03,$07,$01
         .BYTE $05,$03,$07
         .BYTE $01,$01,$01
         .BYTE $01,$01,$01
         .BYTE $4E,$B8,$5E
         .BYTE $B8,$01,$01
         .BYTE $01,$01,$01
         .BYTE $01,$01,$01
         .BYTE $05,$05,$05
         .BYTE $05,$05,$05
         .BYTE $05,$05,$01
         .BYTE $01,$01,$01
         .BYTE $01,$01,$01
         .BYTE $01,$05,$05
         .BYTE $05,$05,$05
         .BYTE $05,$05,$05
         .BYTE $70,$B8,$00
         .BYTE $00,$01,$01
         .BYTE $00,$00,$03
         .BYTE $03,$01,$01
         .BYTE $01,$01,$00
         .BYTE $00,$03,$03
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$05
         .BYTE $05,$05,$05
         .BYTE $00,$00,$07
         .BYTE $07,$70,$B8
         .BYTE $70,$B8,$98
         .BYTE $B8,$98,$B8
         .BYTE $01,$01,$01
         .BYTE $01,$03,$03
         .BYTE $A0,$B8,$01
         .BYTE $01,$01,$01
         .BYTE $05,$05,$A8
         .BYTE $B8,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $B4,$B8,$C4
         .BYTE $B8,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$FC
         .BYTE $FC,$FC,$FC
         .BYTE $04,$04,$04
         .BYTE $04,$04,$04
         .BYTE $04,$04,$FC
         .BYTE $FC,$FC,$FC
         .BYTE $D8,$B8,$D8
         .BYTE $B8,$00,$00
         .BYTE $00,$00,$00
         .BYTE $DF,$B8,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$09,$B9
         .BYTE $1D,$B9,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $33,$B9,$00
         .BYTE $00,$39,$B9
         .BYTE $3D,$B9,$00
         .BYTE $00,$00,$00
         .BYTE $10,$10,$10
         .BYTE $10,$43,$B9
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$5D,$B9
         .BYTE $6D,$B9,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $F4,$F4,$F8
         .BYTE $F8,$FC,$FC
         .BYTE $00,$00,$00
         .BYTE $00,$04,$04
         .BYTE $08,$08,$0C
         .BYTE $0C,$7F,$B9
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$A3
         .BYTE $B9,$A3,$B9
         .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00,$AD
         .BYTE $B9
LB9AD    .BYTE $00,$00,$00
         .BYTE $00,$00,$00
         .BYTE $A8,$B8,$B4
         .BYTE $B8,$B9,$B9
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$D8,$B8
         .BYTE $CD,$B9,$10
         .BYTE $10,$10,$10
         .BYTE $10,$DF,$B8
         .BYTE $09,$B9,$D8
         .BYTE $B9,$00,$00
         .BYTE $00,$00,$00
         .BYTE $00,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$33,$B9
         .BYTE $39,$B9,$39
         .BYTE $B9,$43,$B9
         .BYTE $5D,$B9,$FC
         .BYTE $B9,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $10,$10,$A8
         .BYTE $B8,$10,$BA
         .BYTE $10,$10,$10
         .BYTE $10,$10,$10
         .BYTE $18,$BA,$00
         .BYTE $00,$00,$00
         .BYTE $00,$00
LBA1E    .BYTE $96
LBA1F    .BYTE $BA,$96,$BA
         .BYTE $9D,$BA,$AB
         .BYTE $BA,$B7,$BA
         .BYTE $BE,$BA,$D0
         .BYTE $BA,$D6,$BA
         .BYTE $E6,$BA,$F1
         .BYTE $BA,$FF,$BA
         .BYTE $0C,$BB,$19
         .BYTE $BB,$26,$BB
         .BYTE $2F,$BB
LBA3C    .BYTE $38
LBA3D    .BYTE $BB,$38,$BB
         .BYTE $43,$BB,$4C
         .BYTE $BB,$54,$BB
         .BYTE $5B,$BB,$6D
         .BYTE $BB,$73,$BB
         .BYTE $83,$BB,$8E
         .BYTE $BB,$97,$BB
         .BYTE $97,$BB,$97
         .BYTE $BB,$97,$BB
         .BYTE $B1,$BB
LBA5A    .BYTE $BA
LBA5B    .BYTE $BB,$BA,$BB
         .BYTE $C5,$BB,$D3
         .BYTE $BB,$DB,$BB
         .BYTE $E2,$BB,$F4
         .BYTE $BB,$FA,$BB
         .BYTE $0A,$BC,$15
         .BYTE $BC,$1E,$BC
         .BYTE $1E,$BC,$1E
         .BYTE $BC,$2B,$BC
         .BYTE $38,$BC
LBA78    .BYTE $41
LBA79    .BYTE $BC,$41
LBA7B    .BYTE $BC,$43
LBA7D    .BYTE $BC,$47,$BC
         .BYTE $4F,$BC,$51
         .BYTE $BC,$5C,$BC
         .BYTE $5E,$BC,$62
         .BYTE $BC,$64,$BC
         .BYTE $1E,$BC,$1E
         .BYTE $BC,$1E,$BC
         .BYTE $6D,$BC,$38
         .BYTE $BC,$98,$BA
         .BYTE $28,$29,$28
         .BYTE $29,$00,$A1
         .BYTE $BA,$A6,$BA
         .BYTE $38,$38,$3A
         .BYTE $3A,$00,$39
         .BYTE $39,$3B,$3B
         .BYTE $00,$AF,$BA
         .BYTE $B3,$BA,$42
         .BYTE $44



LBAB0:   LSR L0000
         .BYTE $43
         EOR FMSZPG+4
         BRK
         LDA L54BA,Y
         EOR ROWCRS,X
         EOR L0000,X
         .BYTE $C2
         TSX
         CMP #$BA
         .BYTE $5A,$5C,$5A,$5C
         RTS
         .BYTE $62
         BRK
         .BYTE $5B
         EOR L5D5B,X
         ADC (LOGCOL,X)
         BRK
         .BYTE $D2
         TSX
         JMP (L6E6D)
         BRK
         .BYTE $DA
         TSX
         CPX #$BA
         .BYTE $72,$74,$72,$74
         ROR L0000,X
         .BYTE $73
         ADC Sprite3_X,X
         ADC DELTAC,X
         BRK
         INX
         TSX
         BCC LBA7B
         BCC LBA7D
         .BYTE $92,$93,$92,$93
         BRK
         SBC STOPLN,X
         .BYTE $FA
         TSX
         DEY
         TXA
         DEY
         TXA
         BRK
         .BYTE $89,$8B,$89,$8B
         BRK
         ORA (STOPLN+1,X)
         .BYTE $D2,$D3,$D2,$D3
         .BYTE $D2,$D3,$D2,$D3
LBB09:   .BYTE $D2,$D3
LBB0B:   BRK
         ASL LD4BB
         CMP RandSeed,X
         CMP RandSeed,X
         CMP RandSeed,X
         CMP RandSeed,X
         CMP L0000,X
         .BYTE $1B,$BB
         DEC RandSeed+3,X
         DEC RandSeed+3,X
         DEC RandSeed+3,X
         DEC RandSeed+3,X
         DEC RandSeed+3,X
         BRK
         ROL A
         .BYTE $BB
         ROL A
         .BYTE $BB
         PLP
         AND #$28
         AND #$00
         AND (STOPLN+1),Y
         INC LFEFF,X
         .BYTE $FF
         INC FPTR2+1,X
         .BYTE $3A,$BB
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         .BYTE $47,$BB,$47,$BB
         ORA (RAMLO+1,X)
         ORA (RAMLO+1,X)
         BRK
         BVC LBB09
         BVC LBB0B
         ORA (ZRAM_01,X)
         ORA (L0000,X)
         LSR STOPLN+1,X
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         .BYTE $5F,$BB
         ROR STOPLN+1
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         .BYTE $6F,$BB
         ORA (ZRAM_01,X)
         ORA (L0000,X)
         .BYTE $77,$BB
         ADC L01BB,X
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (L0000,X)
         STA STOPLN+1
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         .BYTE $92,$BB,$92,$BB
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         STA L01BB,Y
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (L0000,X)
         TAY
         .BYTE $BB
         TAY
         .BYTE $BB
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         .BYTE $B3,$BB
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         ORA (ZRAM_01,X)
         BRK
         LDY STOPLN+1,X
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         CMP #$BB
         DEC STOPLN+1
         BRK
         BRK
         BRK
         BRK
         BPL LBBE0
         BPL LBBE2
         BRK
         .BYTE $D7,$BB,$D7,$BB
         BRK
         BRK
         BRK
         BRK
         CMP STOPLN+1,X
         BRK
         BRK
LBBE0:   BRK
         BRK
LBBE2:   INC STOPLN+1
         SBC STOPLN+1
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BPL LBBEF
LBBEF:   BPL LBBF1
LBBF1:   BPL LBBF3
LBBF3:   BRK
         INC STOPLN+1,X
         BRK
         BRK
         BRK
         BRK
         INC L04BB,X
         LDY L0000,X
         BRK
         BRK
         BRK
         BRK
         BPL LBC16
         BPL LBC18
         BPL LBC0A
LBC0A:   .BYTE $0C
         LDY L0000,X
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
LBC15:   ORA L19BC,Y
LBC18:   LDY L0000,X
         BRK
         BRK
         BRK
LBC1E:   JSR L00BC
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         .BYTE $2F
         LDY LBC2F,X
LBC2F:   BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
LBC38:   .BYTE $3A
         LDY L0000,X
         BRK
         BRK
         BRK
         BRK
         BRK
         LDY LC9BB,X
         .BYTE $BB
         CMP #$BB
LBC47:   .BYTE $D7,$BB,$4B
         LDY L1010,X
         BPL LBC4F
LBC4F:   CMP LE6BB,X
         .BYTE $BB
         EOR L00BC,X
         BRK
         BPL LBC58
LBC58:   BPL LBC5A
LBC5A:   BPL LBC5C
LBC5C:   INC STOPLN+1,X
LBC5E:   INC LFEBB,X
         .BYTE $BB,$0C
         LDY LBC19,X
         PLA
         LDY L1010,X
         BPL LBC7C
         BRK
         LDY L71BB,X
         LDY L1010,X
         BPL LBC85
         BPL LBC87
         BPL LBC89
         BRK
LBC7A:   .BYTE $80
LBC7B:   CMP L00A8
         CMP L00CC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
LBC85:   CMP ROWAC
LBC87:   CMP MEOLFLG
LBC89:   CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
         CMP L00BC
         CMP ROWAC
         CMP MEOLFLG
LBCB9:   CMP RTCLOK
LBCBB:   DEX
         .BYTE $1C
         DEX
LBCBE:   ROR LBCC9,X
         CMP #$DE
LBCC3:   CPY CIX
         CPY TRAMSZ
         CMP ABUFPT+2
LBCC9:   CMP Sprite4_X
         CPY L009E
         CPY STOPLN
         CPY ABUFPT+2
         CMP Sprite4_X
         CPY L009E
         CPY STOPLN
         CPY L00C6
         CPY Sprite4_X
         CPY L009E
         CPY STOPLN
         CPY L00C6
         CPY DiagMessage_HI
LBCE3:   CPY ICAX1Z
         CPY FMSZPG+1
         CPY FKDEF
         CPY TEMP
LBCEB:   .BYTE $C2,$3A,$C2,$44
         CPY FKDEF
         CPY MEOLFLG
LBCF3:   DEX
         LDX LCACA
         DEX
         INC LOADFLG
         .BYTE $02,$CB
         ASL LD2CB,X
LBCFF:   .BYTE $C7,$D4,$C7
         DEC L00C7,X
         CLD
         .BYTE $C7,$DA,$C7,$DA
         .BYTE $C7
LBD0A:   .BYTE $3A
LBD0B:   CMP (FMSZPG+5,X)
         CMP (Sprite4_Prop,X)
LBD0F:   DEX
         ROL LOADFLG,X
LBD12:   LSR LOADFLG
         JMP (L7ACA)
LBD17:   CPY STARP
         CPY L00AE
         CPY L00C6
         CPY Sprite4_X
LBD1F:   CPY Sprite4_X
         CPY Sprite4_X
         CPY Sprite4_X
         CPY STARP
         CPY STARP
LBD29:   CPY STARP
         CPY L00AE
         CPY L00C6
         CPY L00AE
         CPY L00AE
         CPY L00AE
         CPY Mini_LO
         CMP #$7A
         CPY Sprite4_X
         CPY Sprite4_X
         CPY SAVCUR
LBD3F:   CMP (IOCMD,X)
         CMP (PROMPT,X)
         CMP (L00C4,X)
         CMP (L00C4,X)
         CMP (L00C6,X)
         CMP (COLOUR,X)
         CMP (LOADFLG,X)
         CMP (FPTR2,X)
LBD4F:   .BYTE $CB
         CLI
         CPY LCC8E
         BNE LBD22
LBD56:   .BYTE $3A
LBD57:   .BYTE $CB
         ROR LA2CB
         .BYTE $CB,$E2,$CB
LBD5E:   ROL L60C7
         .BYTE $C7
         BVS LBD29
         .BYTE $92
         CMP ICAX5Z
         .BYTE $C7
         RTS
         .BYTE $C7
LBD6A:   BVC LBD8C
         CLI
         BRK
         RTS
         RTI
         BRK
         PLA
         BVS LBD74
LBD74:   LDY #$80
         BRK
         DEY
         CPY #$90
LBD7A:   BVC LBD7C
LBD7C:   BVC LBDEE
         BVC LBD80
LBD80:   BVS LBDD2
         BVC LBDF4
         BRK
         BVC LBDF7
         BVC LBD89
LBD89:   BVC LBD8B
LBD8B:   .BYTE $04
LBD8C:   BRK
         ASL L0000
LBD8F:   .BYTE $04
         ASL ZRAM_02
         BRK
         ASL RAMLO
         .BYTE $02
         ASL ZRAM_02
         .BYTE $04,$02
LBD9A:   PLP
         RTI
         CLI
         PLA
         ROR L9990,X
         LDY #$64
         .BYTE $54,$44
LBDA5:   .BYTE $33,$33
         EOR #$60
         ADC FREQ,X
         .BYTE $47
         LSR L964E
         STX L009C,X
LBDB1:   .BYTE $AB
LBDB2:   EOR Sprite1_X,X
         BVS LBE26
         .BYTE $53,$53,$67
         BVS LBDBB
LBDBB:   BRK
         BRK
         BRK
         .BYTE $02,$02
LBDC0:   .BYTE $02,$02
LBDC2:   JSR L4000          
         LDY #$00
         BCS LBDC9
LBDC9:   CPY #$20
         RTS
         BMI LBDCE
LBDCE:   RTI
         LDY #$00
         CPY #$00
         RTI
         BRK
         BRK
         BVC LBDD8
LBDD8:   RTI
         BRK
         BRK
         BRK
         BRK
         BVC LBDDF
LBDDF:   BRK
         RTI
         BRK
LBDE2:   .BYTE $02
         BRK
         .BYTE $02
         ASL L0000
         ASL L0000
         ASL RAMLO
         .BYTE $02,$04
         BRK
LBDEE:   .BYTE $04,$02
         BRK
         .BYTE $02
LBDF2:   LDA L50A6,X
         STA L8B55,Y
         .BYTE $1A
         BMI LBE4B
         BVC LBE37
         .BYTE $3A,$3A,$3A
         BVC LBE52
LBE02:   .BYTE $02
         BRK
         PHP
         BRK
         ASL WARMST
         ASL RAMLO
LBE0A:   RTI
         BVC LBE6D
         BVS LBD8F
         BCC LBDB1
         BCS LBE33
         JSR L2020
         JSR L2020
         JSR ZRAM_02
         .BYTE $02
         ASL RAMLO
         BRK
         .BYTE $02
         BRK
LBE22:   BMI LBE6C
         PLA
         .BYTE $80
LBE26:   BVS LBDC0
         CLV
         BNE LBE2B
LBE2B:   BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
LBE32:   BRK
LBE33:   .BYTE $04
         BRK
         ASL ZRAM_02
LBE37:   .BYTE $04,$02
         ASL COUNTR+1
         .BYTE $7F,$7F,$7F,$7F
         .BYTE $7F,$7F,$7F,$7F
         .BYTE $7F,$7F,$7F,$7F
         .BYTE $7F,$7F,$7F,$7F
LBE4B:   .BYTE $7F,$7F,$7F,$7F
         .BYTE $7F,$7F,$7F
LBE52:   .BYTE $13,$0C
         ASL L0000
         .BYTE $0B
         ASL ZRAM_02
         BRK
         BRK
         BPL LBE62
         BRK
         BRK
         ORA ZRAM_01
         BRK
LBE62:   .BYTE $3F,$3F,$3F,$3F
         .BYTE $3F,$3F,$3F,$3F
LBE6A:   PLP
         JSR BUFADR
         JSR L0D18
         BRK
         BRK
         BPL LBE7A
         BRK
         BRK
         ORA ZRAM_01
         BRK
LBE7A:   .BYTE $3F,$3F,$3F,$3F
         .BYTE $3F,$3F,$3F,$3F
LBE82:   BPL LBE8A
         .BYTE $03
         BRK
         ORA #$05
         .BYTE $02
         BRK
LBE8A:   BRK
         .BYTE $04,$03
         BRK
         BRK
         .BYTE $02
         ORA (L0000,X)
         .BYTE $17,$17,$17,$17
         .BYTE $17,$17,$17,$17
LBE9A:   BPL LBEA6
         ORA L0000
         ASL A
         PHP
         .BYTE $03
         BRK
         BRK
         .BYTE $03,$02
         BRK
LBEA6:   BRK
         .BYTE $02
         ORA (L0000,X)
         BPL LBEBC
         BPL LBEBE
         BPL LBEC0
         BPL LBEC2
LBEB2:   .BYTE $1A
         BPL LBEBB
         BRK
         .BYTE $12
         ASL ZRAM_02
         BRK
         BRK
LBEBB:   ORA #$05
         BRK
LBEBE:   BRK
         .BYTE $03
LBEC0:   ORA (L0000,X)
LBEC2:   .BYTE $17,$17,$17,$17
         .BYTE $17,$17,$17,$17
LBECA:   .BYTE $3F,$0B,$0B,$0B
LBECE:   .BYTE $0B,$0B,$0B,$0B
         .BYTE $3F,$3F,$3F,$3F
         .BYTE $3F,$3F,$3F,$3F
         .BYTE $3F,$3F,$3F,$3F
         .BYTE $3F,$3F,$3F,$3F
LBEE2:   JSR L090A
         BRK
         ASL A
         ORA #$03
         BRK
         BRK
         ORA #$05
         BRK
         BRK
         .BYTE $03
         ORA (L0000,X)
         JSR L2020
         JSR L2020
         JSR L1020
         ASL A
         ORA L0000
         ASL A
         PHP
         .BYTE $03
         BRK
         BRK
         .BYTE $04,$03
         BRK
         BRK
         .BYTE $02
         ORA (L0000,X)
         BPL LBF1C
         BPL LBF1E
         BPL LBF20
         BPL LBF22
LBF12:   .BYTE $0B
         PHP
         ORA L0000
         PHP
         ORA ZRAM_02+1
         BRK
         BRK
         .BYTE $04
LBF1C:   .BYTE $02
         BRK
LBF1E:   BRK
         .BYTE $02
LBF20:   ORA (L0000,X)
LBF22:   BPL LBF34
         BPL LBF36
         BPL LBF38
         BPL LBF3A
LBF2A:   BPL LBF36
         .BYTE $07
         BRK
         ASL A
         .BYTE $07,$03
         BRK
         BRK
         ORA ZRAM_02
         BRK
LBF36:   BRK
         .BYTE $03
LBF38:   ORA (L0000,X)
LBF3A:   JSR L2020
         JSR L2020
         JSR LDE20

;----------------------------------------------------
;           16 sprite pointers?
;----------------------------------------------------
LBF42:   .WORD LC5DE,LC5EE
         .WORD LC60C,LC5DE
         .WORD LC63C,LC6BE
         .WORD $C664,$C684
         .WORD $C664,$C6BE
         .WORD $C6BE,$C5DE
         .WORD $C5EE,$C664
         .WORD $C60C,$C664


         RTS
         RTS
         LDY #$A0
         LDY #$A0
         RTS
         RTS
         CPX #$E0
         JSR L2020
         JSR LE0E0
         RTS
         RTS
         LDY #$A0
         LDY #$A0
         RTS
         RTS
         CPX #$E0
         JSR L0E20
         CLI
         BPL LBF8E
         .BYTE $12
         PHP
         .BYTE $14
LBF89:   PLP
         ORA ICBLLZ,X
         EOR #$18
LBF8E:   ASL L1058
         PHP
         .BYTE $12
         PHP
         .BYTE $14
         PLP
         ORA ICBLLZ,X
         EOR RAMLO
         .BYTE $42,$04
         EOR (SpritePic2,X)
         .BYTE $14
         CLI
         .BYTE $12
         PHP
         BPL LBFAC
         ASL L0D28
         PLP
         AND L1418,Y
         CLI
LBFAC:   .BYTE $12
         PHP
         BPL LBFB8
         ASL L0D28
         PLP
         AND L4004,X
         .BYTE $04
LBFB8:   EOR (SpritePic2,X)
         ASL L1040
         PHP
         .BYTE $12
         CLC
         .BYTE $14
         JSR L1C15
         EOR #$2C
         ASL L1040
         PHP
         .BYTE $12
         CLC
         .BYTE $14
         JSR L1C15
         EOR RAMLO
         .BYTE $42,$04
         EOR (SpritePic2,X)
         .BYTE $14
         RTI
         .BYTE $12
         PHP
         BPL LBFF4
         ASL L0D20
         .BYTE $1C
         AND L142C,Y
         RTI
         .BYTE $12
         PHP
         BPL LC000
         ASL L0D20
         .BYTE $1C
         AND L4004,X
         .BYTE $04
         EOR (SpritePic2,X)
         ASL L1018
         BPL LC009
         .BYTE $34
         ORA BUFADR+1,X
         EOR #$40
         ASL L1018
         BPL LC013
         .BYTE $34
         ORA BUFADR+1,X
         EOR RAMLO
         .BYTE $42,$04
         EOR (SpritePic2,X)
         .BYTE $14
         CLC
         .BYTE $12
         BPL LC01F
         .BYTE $34
         ORA L3916
LC013:   RTI
         .BYTE $14
         CLC
         .BYTE $12
         BPL LC029
         .BYTE $34
         ORA L3D16
         .BYTE $04
         RTI
LC01F:   .BYTE $04
         EOR (SpritePic2,X)
         .BYTE $13
         BVC LC06A
         JSR L5013
         EOR #$20
         .BYTE $0F
         BVC LC06A
         JSR L500F
         AND L4220,Y
         PHP
         .BYTE $4F,$02,$42
         ASL L024F
         .BYTE $42
         ASL L024F
         .BYTE $42
         BVC LC081
         PHP
         .BYTE $33,$02
         RTI
         ASL SPARE
         RTI
         ASL SPARE
         RTI
         BVC LC063
         BPL LC064
         PHP
         BPL LC05C
         .BYTE $1F
         JSR L040F
         ORA (RAMLO),Y
         .BYTE $13,$04
LC05C:   EOR #$1A
         .BYTE $13
         PHP
         .BYTE $12
         PHP
         ASL L0D10
         .BYTE $14,$1F
         JSR L040F
LC06A:   ORA (RAMLO),Y
         .BYTE $13,$12,$2B
         JSR L1C07
         .BYTE $12
         PHP
         ORA (WARMST),Y
         .BYTE $0F
         RTI
         ASL L0F10
         PHP
         .BYTE $12
         PHP
         .BYTE $2B
         JSR L0413
         ORA (RAMLO),Y
         .BYTE $0F,$04
         AND L0F1A,Y
         PHP
         BPL LC094
         .BYTE $14
         BPL LC0A4
         .BYTE $14,$2B
         JSR L0413
LC094:   ORA (RAMLO),Y
         .BYTE $0F,$12,$1F
         JSR L1C03
         BPL LC0A6
         ORA (WARMST),Y
         .BYTE $13
         RTI
         ASL WARMST,X
LC0A4:   ORA WARMST,X
LC0A6:   .BYTE $14
         PHP
         .BYTE $12
         CLC
         AND WARMST
         BPL LC0BE
         .BYTE $0F
         BPL LC0BE
         .BYTE $14,$0B
LC0B3:   BPL LC0E9
         .BYTE $04
         AND (RAMLO),Y
         .BYTE $4F
         PHP
         EOR L1504
         .BYTE $1C
LC0BE:   .BYTE $13
         PHP
         EOR (DOSINI,X)
         BPL LC0CC
         .BYTE $0F
         PHP
         ORA L0F10
         PHP
         AND L0840,Y
         .BYTE $04
         ASL RAMLO,X
         ORA WARMST,X
         .BYTE $14,$04,$12,$04
         ORA (RAMLO),Y
         BPL LC0E0
         .BYTE $0F
         BPL LC0EB
         ASL DOSINI+1
         PHP
LC0E0:   .BYTE $0C,$04,$0B
         ASL ABUFPT
         .BYTE $02,$34,$02,$32
LC0E9:   .BYTE $02
         AND (ZRAM_02),Y
         BVC LC0F2
         .BYTE $4F,$04,$17
         ASL BUFADR
         ASL RTCLOK+1
         PHP
         ROL ZRAM_02
         BPL LC100
         .BYTE $0F
         PHP
         ORA L3608
         RTI
LC100:   .BYTE $13,$04,$12
         PHP
         ORA (WARMST),Y
         .BYTE $3F
         BPL LC146
         PHP
         RTI
         .BYTE $32
         ORA #$04
         ASL A
         PHP
         .BYTE $0C
         PHP
         .BYTE $0F,$04
         BPL LC11E
         ORA (WARMST),Y
         .BYTE $43
         BPL LC160
         PHP
         .BYTE $42,$32
LC11E:   .BYTE $13
         PHP
         .BYTE $0F
         BPL LC136
         CLC
         .BYTE $0F
         JSR L2013
         .BYTE $0F
         JSR L6013
         .BYTE $0F
         PHP
         .BYTE $13
         BPL LC140
         CLC
         .BYTE $13
         JSR L200F
LC136:   .BYTE $13
         JSR L600F
         .BYTE $17
         BPL LC182
         ASL A
         ROL DOSINI
LC140:   ORA (POKMSK),Y
         BPL LC15C
         .BYTE $0F
         PHP
LC146:   BPL LC188
         .BYTE $0B
         BPL LC188
         ASL A
         BIT DOSINI
         ORA (POKMSK),Y
         .BYTE $12
         CLC
         .BYTE $13
         PHP
         .BYTE $12
         RTI
         .BYTE $12
         BPL LC16A
         PHP
         BPL LC164
LC15C:   .BYTE $43,$0C
         ORA (WARMST),Y
LC160:   BPL LC16A
         .BYTE $43,$0C
LC164:   ORA (FREQ),Y
         BPL LC178
         ORA (WARMST),Y
LC16A:   .BYTE $12
         PHP
         .BYTE $3F,$0C
         ORA (WARMST),Y
         .BYTE $12
         PHP
         .BYTE $3F,$0C
         ORA (FREQ),Y
         .BYTE $17,$04
LC178:   .BYTE $13,$04
         ORA (RAMLO),Y
         .BYTE $0F
         PHP
         ORA (WARMST),Y
         EOR L4902
         .BYTE $04
         EOR ZRAM_02
         ORA (WARMST),Y
LC188:   BPL LC19A
         ORA (WARMST),Y
         EOR L4504
         .BYTE $02
         ORA (DOSINI),Y
         BPL LC1A4
         ORA (WARMST),Y
         EOR L1106
         BVC LC1A6
         .BYTE $04,$0F,$04
         ORA (RAMLO),Y
         .BYTE $13
         PHP
         ORA (WARMST),Y
LC1A4:   AND ZRAM_02,X
LC1A6:   AND L3D04,Y
         .BYTE $02
         ORA (WARMST),Y
         .BYTE $12
         BPL LC1C0
         PHP
         AND RAMLO,X
         AND L1102,X
         .BYTE $0C,$12
         BPL LC1CA
         PHP
         AND TRAMSZ,X
         ORA (TEMP),Y
         PLP
         BEQ LC204
         BEQ LC206
         BEQ LC206
         BEQ LC1EA
         BEQ LC1EC
         BEQ LC1ED
         BEQ LC1F2
         BPL LC1F5
         BPL LC1F8
         PHP
         .BYTE $13
         PHP
         PLP
         PHP
         .BYTE $2B,$04
         AND L3004
         PHP
         .BYTE $1B
         PHP
         SEC
         .BYTE $0C
         AND (WARMST,X)
         PLP
         STY L083F
         EOR WARMST
         AND L4508,X
         PHP
LC1EC:   AND L4508,X
         PHP
         AND L4508,X
         PLP
         .BYTE $43
LC1F5:   .BYTE $04
         AND L4308,X
         PHP
         AND L4308,X
         PHP
         AND L4308,X
         PHP
         AND L242C,X
         PHP
LC206:   LSR WARMST
         ROL L4608,X
         PHP
         BIT DOSVEC
         ROL A
         ASL A
         AND (DOSVEC,X)
         PLP
         AND L0823
         LSR WARMST
         .BYTE $3C
         PHP
         LSR WARMST
         .BYTE $3C
         PHP
         LSR WARMST
         .BYTE $3C
         PHP
         LSR WARMST
         .BYTE $3C
         PHP
         LSR WARMST
         .BYTE $3C
         PLP
         .BYTE $27
         PHP
         .BYTE $3C
         PHP
         .BYTE $44
         PHP
         .BYTE $3C
         PHP
         .BYTE $27
         ASL A
         JSR L290A
         ASL A
         .BYTE $22
         AND L060A
         ORA L1F04,X
         .BYTE $04
         AND (RAMLO,X)
         BIT TRAMSZ
         AND TRAMSZ
         .BYTE $27
         PHP
         ORA (RAMLO),Y
         .BYTE $0F
         PHP
         ORA (RAMLO),Y
         .BYTE $12
         BEQ LC269
         ASL Sprite1_Char
         .BYTE $04,$2B,$04
         AND #$04
         ROL TRAMSZ
         AND TRAMSZ
         .BYTE $23
         PHP
LC25E:   ORA (RAMLO),Y
         .BYTE $13,$12
         ORA (RAMLO),Y
         BPL LC278
         ORA (RAMLO),Y
         .BYTE $12
LC269:   .BYTE $12
         ORA (RAMLO),Y
         BPL LC25E
         EOR L4B02
         .BYTE $04,$3B,$04
         EOR ZRAM_02+1
         PLP
         .BYTE $04
LC278:   ROL WARMST
         AND DOSVEC
         BIT WARMST
         .BYTE $22,$04,$0F
         ASL Sprite3_Cha 
         PHP
         ORA L4D80
         .BYTE $02,$4B,$04,$3B
         .BYTE $04
         EOR ZRAM_02+1
         PLP
         .BYTE $04
         ROL WARMST
         ROL DOSVEC
         BIT WARMST
         .BYTE $22,$04,$0F
         ASL Sprite3_Cha 
         PHP
         .BYTE $0C
         PHP
         .BYTE $0B
         ASL ABUFPT
         .BYTE $04,$1A
         PHP
         ORA L300A,Y
         PHP
         ROL L4D04
         .BYTE $03,$4B,$04,$3B
         .BYTE $04
         EOR ZRAM_02+1
         PLP
         .BYTE $04
         ROL WARMST
         AND DOSVEC
         BIT WARMST
         .BYTE $22,$04,$0F
         ASL Sprite3_Cha 
         PHP
         ORA L3580
         .BYTE $02,$37,$04,$3B
         .BYTE $04
         AND L2203,X
         .BYTE $04
         BIT WARMST
         AND DOSVEC
         ROL WARMST
         PLP
         .BYTE $04,$13
         ASL RTCLOK+2
         PHP
         ORA LOMEM,X
         AND ZRAM_02+1,X
         .BYTE $37,$04,$3B,$04
         AND L2203,X
         .BYTE $04
         BIT WARMST
         AND DOSVEC
         ROL WARMST
         PLP
         .BYTE $04,$13
         ASL RTCLOK+2
         PHP
         ASL WARMST,X
         .BYTE $17
         ASL ICAX5Z
         .BYTE $04
         BMI LC300
         ORA L1A0A,Y
         PHP
         .BYTE $1C,$04
         AND ZRAM_02+1,X
LC300:   .BYTE $37,$04,$3B,$04
         AND L2203,X
         .BYTE $03
         BIT WARMST
         AND DOSVEC
         ROL WARMST
         PLP
         .BYTE $04,$13
         ASL RTCLOK+2
         PHP
         ORA LOMEM,X
         EOR RAMLO
         EOR (RAMLO,X)
         EOR RAMLO
         .BYTE $42
         PHP
         EOR TRAMSZ
         .BYTE $42
         PHP
         EOR RAMLO
         .BYTE $42
         PHP
         EOR ICAX3Z
         EOR (RAMLO,X)
         .BYTE $44
         PHP
         .BYTE $3F,$04,$47,$04
         EOR (TRAMSZ,X)
         EOR RAMLO
         EOR (WARMST,X)
         .BYTE $43,$34,$43,$04
         AND L4502,X
         .BYTE $02,$3F,$04
         EOR RAMLO
         .BYTE $3F,$04
         EOR RAMLO
         .BYTE $3F,$04,$43
         PHP
         AND L4304,X
         BMI LC38C
         .BYTE $04
         EOR (RAMLO,X)
         AND L4004,X
         PHP
         AND L4006,X
         PHP
         AND L4004,X
         PHP
         AND L412C,X
         .BYTE $04
         ROL L4308,X
         .BYTE $04,$3B,$04
         EOR (TRAMSZ,X)
         AND L4104,X
         PHP
         .BYTE $3F,$34,$3F,$04
         EOR ZRAM_02
         AND L4302,X
         .BYTE $04
         AND L4304,X
         .BYTE $04
         AND L4304,X
         .BYTE $04,$3F
         PHP
         EOR RAMLO
         .BYTE $3F
         BMI LC3C8
         CLI
         .BYTE $27,$04
         AND RAMLO
LC38C:   AND #$08
         AND WARMST
         AND #$08
         AND WARMST
         ROL WARMST
         AND #$08
         ROL POKMSK
         PLP
         .BYTE $2B
         ROL RAMLO
         .BYTE $27,$04
         AND RAMLO
         .BYTE $27
         PHP
         BIT RAMLO
         .BYTE $27
         PHP
         BIT RAMLO
         PLP
         .BYTE $04
         AND WARMST
         PLP
         PHP
         AND WARMST
         PLP
         .BYTE $2B
         ROL WARMST
         BIT RAMLO
         .BYTE $27,$04
         BIT RAMLO
         .BYTE $27,$04
         BIT RAMLO
         .BYTE $27,$04
         BIT RAMLO
         .BYTE $27,$04
         BIT RAMLO
LC3C8:   .BYTE $27
         PHP
         BIT RAMLO
         .BYTE $27
         PHP
         BIT ICAX6Z
         .BYTE $23,$04
         AND RAMLO
         AND (WARMST,X)
         AND WARMST
         AND (WARMST,X)
         AND WARMST
         BIT WARMST
         AND (WARMST,X)
         BIT POKMSK
         .BYTE $22,$2B
         BIT RAMLO
         .BYTE $23,$04
         AND RAMLO
         .BYTE $23
         PHP
         ROL RAMLO
         .BYTE $23
         PHP
         ROL RAMLO
         .BYTE $22,$04
         AND WARMST
         .BYTE $22
         PHP
         AND WARMST
         .BYTE $22,$2B
         BIT WARMST
         ROL RAMLO
         .BYTE $23,$04
         ROL RAMLO
         .BYTE $23,$04
         ROL RAMLO
         .BYTE $23,$04
         ROL RAMLO
         .BYTE $23,$04
         ROL RAMLO
         .BYTE $23,$04
         ROL RAMLO
         .BYTE $23
         PHP
         ROL ICAX6Z
         AND BUFCNT
         .BYTE $1B,$02
         ASL L3A04,X
         .BYTE $04,$3C,$04
         ROL L4004,X
         .BYTE $04,$42,$04,$44
         BEQ LC450
         PHP
         ROL Sprite2_Prop
         .BYTE $13
         PHP
         ORA WARMST,X
         .BYTE $17
         PHP
         BMI LC43E
         .BYTE $1A
         PHP
         .BYTE $1C,$04
         ASL L2004,X
         .BYTE $04
LC43E:   .BYTE $22,$04
         BIT WARMST
         AND FCHRFLG
         ROL POKMSK
         BIT WARMST
         .BYTE $0F
         PHP
         ORA L0B08
         PHP
         .BYTE $1A
         PHP
LC450:   BMI LC45A
         ROL L2C04
         .BYTE $04
         ROL A
         .BYTE $04
         PLP
         .BYTE $04
LC45A:   ROL WARMST
LC45C:   BIT WARMST
         .BYTE $23
         BEQ LC486
         PHP
         BIT Sprite2_Prop
         .BYTE $0F
         PHP
         ORA L0B08
         PHP
         .BYTE $1A
         PHP
         BMI LC476
         ROL L2C04
         .BYTE $04
         ROL A
         .BYTE $04
         PLP
         .BYTE $04
LC476:   ROL WARMST
         AND #$F0
         ORA #$09
         CLC
         ORA WARMST
         PHP
         .BYTE $07
         PHP
         ASL WARMST
         ORA RAMLO
LC486:   ORA (RTCLOK),Y
         AND WARMST
         BIT FCHRFLG
         ORA #$09
         ASL A
         ORA ZRAM_02
         PHP
         .BYTE $03
         PHP
         .BYTE $04
         PHP
         ORA RAMLO
         ORA (RTCLOK),Y
         AND WARMST
         ROL FCHRFLG
         ORA #$09
         ASL A
         ORA ABUFPT+2
         .BYTE $03,$1F,$03
         JSR L2503
         .BYTE $04
         AND POKMSK
         BIT FCHRFLG
         ORA #$09
         ASL A
         ORA ZRAM_02
         PHP
         .BYTE $03
         PHP
         .BYTE $04
         PHP
         BIT FCHRFLG
         ORA #$09
         ASL A
         ORA ZRAM_02
         PHP
         .BYTE $03
         PHP
         ORA WARMST
         .BYTE $23
         BEQ LC4F9
         .BYTE $07,$33,$04
         SEC
         .BYTE $04
         AND L4004,X
         .BYTE $04
         AND RAMLO
         ROL WARMST
         PLP
         PHP
         .BYTE $27
         PHP
         ROL POKMSK
         .BYTE $13
         PHP
         .BYTE $12
         SEC
         ORA (SpritePic2,X)
         BIT L2B06
         ASL ICBLHZ
         ASL ICPTHZ
         ASL Sprite1_Prop
         BMI LC4FB
         PHP
LC4EC:   .BYTE $1F
         CLC
         ASL L1D10,X
         BEQ LC4F4
         JSR L061E
         .BYTE $1F
         ASL ICDNOZ
LC4F9:   ASL FactoryTestNo
LC4FB:   ASL Sprite1_Prop
         BMI LC511
         PHP
         .BYTE $2B
         CLC
         BIT L2D10
         BEQ LC54C
         .BYTE $03,$42,$04
         EOR (ZRAM_02+1,X)
         CLC
         PHP
         ORA ZRAM_02,X
         .BYTE $12
LC511:   BPL LC538
         PHP
         BIT WARMST
         AND (WARMST,X)
         BPL LC52A
         ORA (DOSINI),Y
         BPL LC54E
         ORA #$09
         CLC
         ORA WARMST
         PHP
         .BYTE $07
         PHP
         ASL WARMST
         ROL FCHRFLG
LC52A:   BIT POKMSK
         BPL LC536
         ORA (WARMST),Y
         BPL LC53A
         ORA (WARMST),Y
         BPL LC53E
LC536:   ORA (STATUS),Y
LC538:   AND L4003,X
         .BYTE $04
         EOR (ZRAM_02+1,X)
LC53E:   ASL A
         PHP
         ORA L1002
         .BYTE $04
         ORA (RAMLO),Y
         BPL LC54C
         ROL RAMLO
         BIT RAMLO
LC54C:   AND RAMLO
LC54E:   BIT DOSINI
         .BYTE $12
         PHP
         BPL LC55C
         .BYTE $12
         BPL LC568
         BIT IOCB0+ICBAH
         RTI
         .BYTE $04
LC55C:   EOR (ZRAM_02+1,X)
         ASL A
         PHP
         ORA L1002
         PHP
         AND DOSINI
         ROL WARMST
LC568:   AND #$08
         .BYTE $12
         BPL LC57E
         .BYTE $0C,$12
         BMI LC595
         PHP
         PLP
         BPL LC5D3
         PHP
         LSR WARMST
         LSR L4608,X
         PHP
         LSR L4608,X
         BEQ LC5C1
         PHP
         EOR WARMST
         .BYTE $3F,$0C
         LSR WARMST
         .BYTE $23,$0C
         AND #$08
         BIT DOSINI
         .BYTE $43
         PHP
         EOR (ICPTLZ,X)
         BIT WARMST
         ROL POKMSK
         .BYTE $23
         PHP
         .BYTE $27
         PHP
         .BYTE $23,$0C,$13
         PHP
         .BYTE $0F
         PHP
         .BYTE $13
         PHP
         .BYTE $23
         PHP
         ROL WARMST
         ORA ICBAHZ,X
         RTI
         .BYTE $0C,$43,$0C,$3F
         .BYTE $0C,$27
         PHP
         .BYTE $23
         PHP
         .BYTE $43
         BPL LC5C4
         PHP
         .BYTE $13
         PHP
         .BYTE $3F
         PHP
         EOR (DiagTestNo,X)
         ROL WARMST
         .BYTE $22
         BPL LC5E8
LC5C1:   ORA L1823
LC5C4:   .BYTE $27,$0C,$0F
         PHP
         .BYTE $12,$04
         RTI
         PLP
         .BYTE $42
         PHP
         AND L4308,X
         .BYTE $0C,$3C
LC5D3:   PHP
         .BYTE $27,$0C
         AND (WARMST,X)


;----------------------------------------------------
;          16 x 16 sprite info
;----------------------------------------------------
LC5DE:   .BYTE $21,$08,$0F,$08,$1F,$08,$0E,$08,$0F,$08,$11,$08,$13,$2C,$45,$2E

LC5EE:   .BYTE $21,$08,$0F,$08,$1F,$08,$0E,$08,$0F,$08,$11,$08,$13,$24,$11,$04
         .BYTE $0F,$04,$0D,$08,$04,$08,$03,$08,$0F,$08,$24,$08,$41,$2C

LC60C:   .BYTE $21,$08,$0F,$08,$1F,$08,$0F,$08,$11,$08,$13,$24,$11,$04,$0F,$04
         .BYTE $0D,$28,$04,$08,$03,$08,$0F,$08,$24,$08,$41,$2C,$21,$08,$0F,$08
         .BYTE $1F,$08,$0E,$08,$0F,$08,$11,$08,$13,$24,$11,$04,$0F,$04,$0D,$78

LC63C:   .BYTE $21,$08
         .BYTE $0E,$08,$1F,$08,$0F,$08,$21,$08,$0F,$08,$11,$08,$13,$08,$14,$38
         .BYTE $47,$36,$21,$08,$0E,$08,$1F,$08,$0F,$08,$21,$08,$0F,$08,$11,$08
         .BYTE $13,$08,$14,$38,$15,$4C,$21,$08,$0E,$08,$1F,$08,$0F,$08,$21,$08
         .BYTE $0F,$08,$11,$08,$13,$08,$14,$2C,$11,$06,$0D,$08,$04,$08,$03,$08
         .BYTE $0F,$08,$24,$08,$41,$2C,$21,$08,$0E,$08,$1F,$08,$0F,$08,$21,$08
         .BYTE $0F,$08,$11,$08,$13,$08,$14,$2C,$11,$06,$0D,$28,$04,$08,$03,$08
         .BYTE $0F,$08,$24,$08,$41,$2C,$21,$08,$0E,$08,$1F,$08,$0F,$08,$21,$08
         .BYTE $0F,$08,$11,$08,$13,$08,$14,$14,$13,$04,$11,$06,$0E,$10,$0D,$44
         .BYTE $21,$08,$0E,$08,$1F,$08,$0F,$08,$21,$08,$0F,$08,$11,$08,$13,$08
         .BYTE $14,$14,$13,$04
         PHP
         AND (WARMST,X)
         .BYTE $0F
         PHP
         ORA (WARMST),Y
         .BYTE $13
         PHP
         .BYTE $14,$14,$13,$04
         ORA (TRAMSZ),Y
         ASL L0D10
         .BYTE $04,$04
         PHP
         .BYTE $03
         PHP
         .BYTE $0F
         PHP
         BIT WARMST
         EOR (ICAX3Z,X)
         EOR L4904
         .BYTE $04
         EOR RAMLO
         .BYTE $43,$04
         ROL WARMST
         ORA (WARMST),Y
         BIT WARMST
         BPL LC6FA
         .BYTE $0F
         PHP
         ORA L4D80
         .BYTE $04
         EOR #$04
LC6FA:   EOR RAMLO
         .BYTE $43,$04
         ROL WARMST
         ORA (WARMST),Y
         BIT WARMST
         BPL LC70E
         .BYTE $0F
         PHP
         ORA L0E10
         .BYTE $04,$0F,$04
LC70E:   BIT WARMST
         EOR (ICAX3Z,X)
         EOR L4904
         .BYTE $04
         EOR RAMLO
         .BYTE $43,$04
         ROL WARMST
         ORA (WARMST),Y
         BIT WARMST
         BPL LC72A
         .BYTE $0F
         PHP
         ORA L0E30
         .BYTE $04,$0F,$04
LC72A:   BIT WARMST
         EOR (ICAX3Z,X)
         ROL A
         PHP
         JSR L4708
         PHP
         AND L4708,X
         PHP
         AND L4708,X
         PHP
         AND L4708,X
         PHP
         AND L4708,X
         PHP
         AND L4708,X
         PHP
         AND L4708,X
         PHP
         AND L4708,X
         BEQ LC796
         .BYTE $04,$43,$04
         ROL WARMST
         ORA (WARMST),Y
         BIT WARMST
         BPL LC764
         .BYTE $0F
         PHP
         ORA L2A80
         PHP
         JSR L2A08
         JSR L2020
         ROL A
         JSR L2020
         ROL A
         JSR L2020
         ROL A
         JSR L2020
         ROL A
         JSR L2020
         ROL A
         JSR L2020
         ROL A
         JSR L2020
         ROL A
         BEQ LC7C6
         .BYTE $04
         ROL WARMST
         ORA (WARMST),Y
         BIT WARMST
         BPL LC794
         .BYTE $0F
         PHP
         ORA L0E20
         .BYTE $04,$0F,$04
LC794:   BIT WARMST
LC796:   EOR (ICAX3Z,X)
         EOR L4904
         .BYTE $04
         EOR RAMLO
         .BYTE $43,$04
         ROL WARMST
         ORA (WARMST),Y
         BIT WARMST
         BPL LC7B0
         .BYTE $0F
         PHP
         ORA L1D08
         .BYTE $04,$1B
         PHP
LC7B0:   AND (DOSINI),Y
         BVC LC7B8
         .BYTE $4F,$04
         EOR L4904
         .BYTE $04
         EOR RAMLO
         .BYTE $43,$04
         ROL WARMST
         ORA (WARMST),Y
         BIT WARMST
         BPL LC7CE
LC7C6:   .BYTE $0F
         PHP
         ORA L0E40
         .BYTE $04,$0F,$04
LC7CE:   BIT WARMST
         EOR (ICAX3Z,X)
         ROL FPTR2+1
         AND #$FF
         AND (FPTR2+1,X)
         BIT FPTR2+1
         LDA WARMST,Y
         JSR L109B
         BRK
         JSR L083B
         BRK
         ORA ICPTHZ,X
         ORA L0000,X
         ORA ICDNOZ,X
         PHP
         AND #$08
         BPL LC7EF
         .BYTE $2B
         ASL L062C
         AND L2F04
         .BYTE $04
         BMI LC7FE
         ORA L1A04,Y
         .BYTE $04
LC7FE:   .BYTE $1B,$04
         ORA L1E04,X
         .BYTE $04,$1F
         PHP
         JSR L2104
         .BYTE $04,$23,$04
         BIT RAMLO
         AND RAMLO
         ROL RAMLO
         .BYTE $13,$04
         ROL A
         .BYTE $04,$2B
         BEQ LC82A
         PHP
         ROL WARMST
         .BYTE $12
         PHP
         .BYTE $13
         PHP
         ORA FREQ,X
         .BYTE $14,$04,$13,$04
         ROL WARMST
         EOR (ICAX3Z,X)
LC82A:   .BYTE $13,$04
         PHA
         PHP
         .BYTE $13
         PHP
         ORA RAMLO
         .BYTE $0F
         PHP
         JSR L0E0C
         PHP
         ORA RAMLO
         .BYTE $14
         PHP
         PHA
         BPL LC853
         BPL LC846
         .BYTE $04,$04
         PHP
         .BYTE $3A
         BPL LC855
         BPL LC84E
         .BYTE $04,$14
         BPL LC895
         BPL LC863
         PHP
         .BYTE $13
         PHP
         ROL WARMST
         EOR (ICBLLZ,X)
         .BYTE $13,$04
         PHA
         PHP
         .BYTE $13
         PHP
         ORA RAMLO
         .BYTE $0F
         PHP
         JSR L0E0C
LC863:   PHP
         ORA RAMLO
         .BYTE $14
         PHP
         PHA
         BPL LC87F
         BPL LC872
         .BYTE $04,$04
         PHP
         .BYTE $3A
         BPL LC881
         BPL LC87A
         .BYTE $04,$14
         BPL LC8C1
         BPL LC880
         .BYTE $04,$04
         PHP
         ASL L0E10
LC881:   PHP
         .BYTE $04,$04
         BIT WARMST
         EOR (FactoryTestNo,X)
         .BYTE $13,$04
         PHA
         PHP
         .BYTE $13
         PHP
         ORA RAMLO
         .BYTE $0F
         PHP
         JSR L0E0C
LC895:   PHP
         ORA RAMLO
         .BYTE $14
         PHP
         PHA
         BPL LC8B1
         BPL LC8A4
         ORA RAMLO
         PHP
         .BYTE $3A
         BPL LC8B3
         BPL LC8AC
         .BYTE $04,$14
         BPL LC8F3
         BPL LC8B2
         .BYTE $04,$04
         PHP
         ASL L0E10
LC8B3:   PHP
         ORA RAMLO
         .BYTE $13
         PHP
         ORA WARMST,X
         CLC
         PHP
         BVC LC8C6
         ORA L0B06,Y
LC8C1:   .BYTE $04
         ORA L2008,X
         PHP
LC8C6:   .BYTE $0F
         PHP
         ORA (WARMST),Y
         .BYTE $12
         PHP
         .BYTE $13
         BPL LC8E3
         PHP
         .BYTE $12
         PHP
         EOR (ICBLLZ,X)
         .BYTE $0F,$04,$3A
         PHP
         .BYTE $0F
         PHP
         ORA RAMLO
         .BYTE $13
         PHP
         ROL A
         .BYTE $0C,$14
         PHP
         ORA RAMLO
         ASL L3A08
         BPL LC8F7
         BPL LC8F0
         .BYTE $04
         ASL WARMST
         PHA
         BPL LC905
         BPL LC8F8
LC8F3:   .BYTE $04
         ASL L3A10
LC8F7:   BPL LC907
         PHP
         .BYTE $0F
         PHP
         .BYTE $23
         PHP
LC8FE:   EOR (ICBLLZ,X)
         .BYTE $0F,$04,$3A
         PHP
         .BYTE $0F
LC905:   PHP
         ORA RAMLO
         .BYTE $13
         PHP
         ROL A
         .BYTE $0C,$14
         PHP
         ORA RAMLO
         ASL L3A08
         BPL LC923
         BPL LC91C
         .BYTE $04
         ASL WARMST
         PHA
         BPL LC931
         BPL LC924
         .BYTE $04
         ASL L3A10
LC923:   BPL LC92A
         .BYTE $04
         ASL WARMST
         .BYTE $14
         BPL LC93F
         PHP
         ASL RAMLO
         ROL WARMST
         EOR (FactoryTestNo,X)
         .BYTE $0F,$04,$3A
         PHP
         .BYTE $0F
         PHP
         ORA RAMLO
         .BYTE $13
         PHP
         ROL A
         .BYTE $0C,$14
LC93F:   PHP
         ORA RAMLO
         ASL L3A08
         BPL LC955
         BPL LC94E
         .BYTE $04
         ASL WARMST
         PHA
         BPL LC963
         BPL LC956
         .BYTE $04
         ASL L3A10
LC955:   BPL LC95C
         .BYTE $04
         ASL WARMST
         .BYTE $14
         BPL LC971
         PHP
         ORA RAMLO
         .BYTE $0F
         PHP
         ORA L0A08
         PHP
         BIT WARMST
         ORA L1706,Y
         .BYTE $04
         AND L2A08
         PHP
         .BYTE $13
LC971:   PHP
         ORA (WARMST),Y
         BPL LC97E
         .BYTE $0F
         BPL LC987
         PHP
         BPL LC984
         EOR (ICBLLZ,X)
LC97E:   .BYTE $43
         ORA (FMSZPG+4,X)
         ORA (RTCLOK+1,X)
         PHP
LC984:   AND WARMST
         .BYTE $0F
LC987:   PHP
         .BYTE $03
         CLC
         ORA DBYTLO
         CLC
         .BYTE $0B
         PHP
         ORA #$08
         .BYTE $17
         PHP
         ORA DOSINI,X
         .BYTE $07
         BPL LC9AE
         .BYTE $0C,$13
         PHP
         AND WARMST
         .BYTE $0F
         PHP
         JSR L0E10
         PHP
         ORA WARMST
         .BYTE $14
         BPL LC9BE
         PHP
         ASL WARMST
         ORA (RAMLO),Y
LC9AE:   .BYTE $04
         PHP
         ASL L1108
         PHP
         ASL WARMST
         ORA (WARMST),Y
         .BYTE $0F
         PHP
         ORA (ICBLLZ),Y
         .BYTE $3F
         ORA (Sprite3_Cha ,X)
         ORA (APPMHI+1,X)
         PHP
         AND WARMST
         .BYTE $13
         PHP
         .BYTE $07
         CLC
         ORA WARMST,X
         .BYTE $07
         CLC
         .BYTE $17
         PHP
         ORA #$08
         .BYTE $0B
         PHP
         ORA TIMER1
         BPL LC9E4
         .BYTE $0C,$0F
         PHP
         AND WARMST
         .BYTE $13
         PHP
         ROL A
         BPL LC9F5
         PHP
         ORA WARMST
LC9E4:   ASL L0D10
         PHP
         .BYTE $04
         PHP
         ORA (RAMLO),Y
         ASL WARMST
         .BYTE $14
         PHP
         ORA (WARMST),Y
         .BYTE $04
         PHP
         ORA (WARMST),Y
         .BYTE $13
         PHP
         ORA (ICBLLZ),Y
         AND (CMCMD),Y
         BVC LCA02
         ASL RAMLO,X
         .BYTE $43,$04
LCA02:   EOR (RAMLO,X)
         AND RAMLO
         BIT WARMST
         .BYTE $22
         PHP
         .BYTE $23
         PHP
         BIT POKMSK
         .BYTE $0F
         PHP
         BPL LCA4A
         ORA (STATUS,X)
         .BYTE $02
         BPL LCA24
         ORA APPMHI
         .BYTE $04
         BPL LCA1B
         ORA (STATUS,X)
         PHP
         BPL LCA36
         ORA RTCLOK+2
         .BYTE $04
LCA24:   .BYTE $12,$FF,$03
         ORA POKMSK
         ASL ZRAM_02
         BPL LCA3D
         .BYTE $1A,$02
         BPL LCA41
         JSR L1002
         BPL LCA86
LCA36:   .BYTE $07
         ORA RTCLOK
         ASL WARMST,X
         BPL LCA4F
LCA3D:   .BYTE $1A
         PHP
         BPL LCA53
LCA41:   JSR L1008
         .BYTE $12
         BVC LCA5D
         ORA ICCOMT,X
         ORA BOOT
         ORA DOSVEC
         ORA DOSINI
LCA4F:   ORA DOSINI+1
         ASL A
         .BYTE $0F
LCA53:   ASL A
         ORA L0B05
         ASL A
         .BYTE $03
         ASL A
         .BYTE $0F
         ASL A
         ORA L0B0A
         ASL A
         .BYTE $03
         ASL A
         .BYTE $0F
         ASL A
         ORA POKMSK
         .BYTE $13
         BPL LCA7D
         BPL LCAB4
         BVC LCA79
         ORA DOSVEC+1,X
         ORA BOOT
         ORA Sprite2_Prop
         ORA BUFADR+1
         ORA BUFADR
         ASL A
         .BYTE $13
LCA79:   ASL A
         ORA RAMLO+1,X
         .BYTE $17
LCA7D:   ASL A
         .BYTE $07
         ASL A
         .BYTE $13
         ASL A
         ORA DOSVEC,X
         .BYTE $17
         ASL A
LCA86:   .BYTE $07
         ASL A
         .BYTE $13
         ASL A
         ORA POKMSK
         .BYTE $0F
         BPL LCA9D
         BPL LCACA
         BVC LCAA8
         ORA L0716,Y
         CLC
         .BYTE $03
         ORA #$07
         .BYTE $02,$03
         ORA LEF06,X
         .BYTE $0B
         ASL VTIMR1,X
         BPL LCAA6
         ORA CMCMD,X
         BPL LCABD
         .BYTE $3A,$27,$1F
         AND FPTR2+1
         ORA DiagMessage_LO,X
         ASL CMCMD,X
         CLC
         .BYTE $03
LCAB4:   ORA #$07
         .BYTE $02,$03
         ORA LEF06,X
         .BYTE $0B
         ASL VTIMR1,X
         BPL LCAC2
         ORA CMCMD,X
         BPL LCAD8
         BMI LCAEE
         .BYTE $1F
         AND FPTR2+1
LCACA:   ORA DiagMessage_LO,X
         ASL CMCMD,X
         CLC
         .BYTE $03
         ORA #$07
         .BYTE $02,$03
         ORA LEF06,X
         .BYTE $0B
LCAD8:   ASL VTIMR1,X
         BPL LCADE
         ORA CMCMD,X
         BPL LCAF3
         AND ICPTHZ
         .BYTE $1F
         AND FPTR2+1
         ORA L0C19
         .BYTE $07
         ASL A
         .BYTE $03
         ORA #$07
LCAEE:   PHP
         .BYTE $03
         AND LD506
LCAF3:   .BYTE $0B
         BIT L0810
         BPL LCAFA
         ORA ZRAM_02+1,X
         BPL LCB0B
         .BYTE $3A,$23,$1F
         AND FPTR2+1
         ORA L0C19
         .BYTE $07
         ASL A
         .BYTE $03
         ORA #$07
         PHP
LCB0B:   .BYTE $03
         AND LD506
         .BYTE $0B
         BIT L0810
         BPL LCB16
         ORA ZRAM_02+1,X
         BPL LCB28
         BMI LCB3E
         .BYTE $1F
         AND FPTR2+1
         ORA L0C19
         .BYTE $07
         ASL A
         .BYTE $03
         ORA #$07
         PHP
         .BYTE $03
LCB28:   AND LD506
         .BYTE $0B
         BIT L0810
         BPL LCB32
         ORA ZRAM_02+1,X
         BPL LCB45
         AND FactoryTestNo
         .BYTE $1F
         AND FPTR2+1
         ASL WARMST,X
         ORA WARMST,X
LCB3E:   .BYTE $14
         PHP
         .BYTE $12
         PHP
         ORA (WARMST),Y
         BPL LCB4E
         .BYTE $0F
         PHP
         ORA WARMST
         RTI
         SBC (L0000),Y
LCB4E:   ORA L0C04
         PHP
         .BYTE $0B
         PHP
         .BYTE $1A
         PHP
         ORA L1808,Y
         PHP
         ASL WARMST,X
         ORA WARMST,X
         BRK
         RTI
         .BYTE $D3
         BRK
         ORA RAMLO,X
         .BYTE $14
         PHP
         .BYTE $13
         PHP
         .BYTE $12
         PHP
         ORA (Sprite2_Prop),Y
         STA (FCHRFLG),Y
         .BYTE $0C
         PHP
         ORA L0E08
         PHP
         BPL LCB7E
         ORA (WARMST),Y
         .BYTE $12
         PHP
         .BYTE $13
         PHP
         ORA WARMST,X
LCB7E:   BRK
         RTI
         .BYTE $D3
         BRK
         ORA RAMLO,X
         ASL WARMST,X
         .BYTE $17
         PHP
         BMI LCB92
         ORA L0A08,Y
         PHP
         .BYTE $0C
         PHP
         ORA WARMST
         RTI
         SBC (L0000),Y
         ORA L0E04
         PHP
         .BYTE $0F
         PHP
         BPL LCBA6
         ORA (WARMST),Y
         STA (FCHRFLG),Y
         .BYTE $14
         PHP
         .BYTE $13
         PHP
LCBA6:   .BYTE $12
         PHP
         ORA (RAMLO),Y
         BPL LCBB4
         .BYTE $0F
         PHP
         BPL LCBB8
         ORA (RAMLO),Y
         .BYTE $12
         PHP
LCBB4:   .BYTE $13
         PHP
         .BYTE $12
         PHP
LCBB8:   ORA (WARMST),Y
         BPL LCBC4
         .BYTE $0F
         PHP
         BPL LCBC8
         ORA (WARMST),Y
         .BYTE $12
         PHP
LCBC4:   .BYTE $13
         PHP
         .BYTE $12
         PHP
LCBC8:   ORA (WARMST),Y
         BPL LCBD4
         .BYTE $0F
         PHP
         BPL LCBD8
         ORA (WARMST),Y
         .BYTE $12
         PHP
LCBD4:   .BYTE $13
         PHP
         .BYTE $12
         PHP
LCBD8:   ORA (WARMST),Y
         BPL LCBE4
         .BYTE $0F
         PHP
         BPL LCBE8
         ORA (FCHRFLG),Y
         .BYTE $1F
         ROL A
LCBE4:   .BYTE $0C
         PHP
         ASL A
         PHP
LCBE8:   ORA #$08
         CLC
         PHP
         ASL WARMST,X
         ORA WARMST,X
         .BYTE $14
         PHP
         .BYTE $12
         PHP
         ORA (WARMST),Y
         BPL LCC00
         ASL L1F08
         ROL A
         STA FCHRFLG
         AND L3A08,Y
         PHP
         .BYTE $3F
         ASL SOUNDR
         ASL CRITIC
         ASL FMSZPG
         ASL FMSZPG+2
         .BYTE $02
         EOR #$02
         EOR L5002
         .BYTE $04
         AND (RAMLO),Y
         .BYTE $32
         ASL BUFRHI
         ASL BFENHI
         ASL Buff1 
         ASL Buff2 
         .BYTE $02,$3A
         PHP
         .BYTE $3F
         ASL SOUNDR
         ASL CRITIC
         ASL FMSZPG
         ASL FMSZPG+2
         .BYTE $02
         EOR #$02
         EOR L5002
         ASL CHKSUM
         .BYTE $04,$32
         ASL BUFRHI
         ASL BFENHI
         .BYTE $04
         SEC
         ASL Buff2 
         .BYTE $02,$3A
         PHP
         .BYTE $3F
         ASL SOUNDR
         ASL CRITIC
         ASL FMSZPG
         ASL FMSZPG+2
         .BYTE $02
         EOR #$02
         EOR L5002
         ASL CHKSUM
         .BYTE $02,$32
         ASL BUFRHI
         ASL Buff1 
         ASL Buff2 
         BEQ LCC89
         .BYTE $04,$2F,$04
         AND L2C04
         .BYTE $04
         AND L2504
         .BYTE $04
         BIT RAMLO
         .BYTE $23,$04
         AND RAMLO
         ROL RAMLO
         .BYTE $27,$04
         AND RAMLO
         BIT RAMLO
         .BYTE $23,$04
         AND RAMLO
         ROL RAMLO
         .BYTE $27,$04
         AND RAMLO
         BIT RAMLO
         .BYTE $23,$04
         AND RAMLO
         ROL RAMLO
         .BYTE $27,$04
         AND RAMLO
         BIT RAMLO
         .BYTE $23,$04
         AND FCHRFLG
LCC8E:   RTI
         ROL A
         .BYTE $23,$02,$22,$02
         AND (ZRAM_02,X)
         JSR L1F02
         .BYTE $04
         ASL L1D02,X
         .BYTE $02,$1C
         PHP
         .BYTE $1B,$02,$1A,$02
         ORA L3002,Y
         .BYTE $02,$2F,$02
         AND L2C02
         .BYTE $02,$2B,$02
         ROL A
         ORA (ICBLHZ,X)
         ORA (ICBLLZ,X)
         ORA (ICPTHZ,X)
         ORA (ICPTLZ,X)
         ORA (ICBAHZ,X)
         .BYTE $02
         BIT RAMLO
         BIT RAMLO
         .BYTE $23,$02,$22,$02
         AND (ZRAM_02,X)
         JSR L1F02
         .BYTE $02
         ASL L1D02,X
         .BYTE $02,$1C
         BEQ LCD13
         ROL A
         .BYTE $27,$02
         PLP
         .BYTE $02
         AND #$02
         ROL A
         .BYTE $02,$2B,$04
         BIT L2D02
         .BYTE $02
         ROL L2F08
         .BYTE $02
         BMI LCCE8
         ORA L1A02,Y
         .BYTE $02,$1B,$02
         ORA L1E02,X
         .BYTE $02,$1F,$02
         JSR L2101
         ORA (ICCOMZ,X)
         ORA (FactoryTestNo,X)
         ORA (DiagTestNo,X)
         ORA (ICBAHZ,X)
         .BYTE $02
         ROL RAMLO
         ROL RAMLO
         .BYTE $27,$02
         PLP
         .BYTE $02
         AND #$02
         ROL A
         .BYTE $02,$2B,$02
         BIT L2D02
         .BYTE $02
         ROL LCEF0


;----------------------------------------------------
;             SEARCH Frame Pointers 
;----------------------------------------------------
LCD12:   .WORD $CDCE,$CDD3,$CDD8,$CDDD,$CDE2,$CDE7
         .WORD $CDEC,$CDF1,$CDF6,$CDFB,$CE00,$CE05
         .WORD $CE0A,$CE0F,$CE14,$CE19,$CE1E,$CE23
         .WORD $CE28,$CE2D,$CE32,$CE37,$CE3C,$CE41
         .WORD $CE46,$CE4B,$CE50,$CE55,$CE5A

LCD4C:   .WORD $CE5F,$CE64,$CE69,$CE6E,$CE73,$CE78
         .WORD $CE7D,$CE82,$CE87,$CE8C,$CE91,$CE96
         .WORD $CE9B,$CEA0,$CEA5,$CEAA,$CEAF,$CEB4
;----------------------------------------------------
;             REPEAT Frame Pointers 
;----------------------------------------------------
LCD70:   .WORD $CEB9,$CEBE,$CEC3,$CEC8,$CECD,$CED2
         .WORD $CED7,$CEDC,$CEE1,$CEE6,$CEEB,$CEF0
         .WORD $CEF5,$CEFA,$CEFF,$CF04,$CF09,$CF0E
         .WORD $CF13,$CF18,$CF1D,$CF22,$CF27,$CF2C
         .WORD $CF31,$CF36,$CF3B,$CF40,$CF45,$CF4A
         .WORD $CF4F,$CF54,$CF59,$CF5E,$CF63,$CF68
         .WORD $CF6D,$CF72,$CF77,$CF7C,$CF81,$CF86
         .WORD $CF8B,$CF90,$CF95,$CF9A,$CF9F

;----------------------------------------------------
;              SEARCH Frame Data 
;----------------------------------------------------
LCDCE:   .BYTE $30,$35,$33,$36,$35    ; Frame = 05365
         .BYTE $30,$35,$31,$33,$30    ; Frame = 05130
         .BYTE $31,$32,$39,$33,$35    ; Frame = 12935
         .BYTE $31,$31,$32,$39,$38
         .BYTE $32,$30,$35,$39,$34
         .BYTE $32,$33,$30,$33,$34
         .BYTE $32,$35,$39,$35,$37
         .BYTE $32,$39,$30,$35,$36
         .BYTE $33,$32,$30,$35,$36
         .BYTE $33,$33,$33,$33,$32
         .BYTE $33,$39,$34,$32,$30
         .BYTE $34,$31,$39,$31,$37
         .BYTE $32,$32,$32,$31,$34
         .BYTE $31,$38,$33,$38,$30
         .BYTE $32,$32,$33,$37,$32
         .BYTE $32,$36,$32,$32,$37
         .BYTE $33,$39,$39,$30,$37
         .BYTE $34,$32,$31,$33,$34
         .BYTE $31,$35,$31,$38,$35
         .BYTE $34,$32,$34,$35,$34
         .BYTE $34,$33,$35,$34,$32
         .BYTE $30,$36,$37,$39,$38
         .BYTE $31,$33,$33,$38,$35
         .BYTE $31,$36,$33,$35,$38
         .BYTE $30,$38,$35,$39,$38
         .BYTE $32,$31,$33,$31,$34
         .BYTE $32,$33,$39,$33,$36
         .BYTE $32,$37,$32,$35,$36
         .BYTE $32,$39,$33,$35,$36    ;
         .BYTE $33,$32,$33,$31,$36    ;
         .BYTE $33,$35,$34,$34,$33    ;
         .BYTE $33,$38,$33,$38,$30    ;
         .BYTE $34,$31,$30,$31,$37    ;
         .BYTE $30,$35,$38,$39,$38    ;
         .BYTE $31,$30,$33,$39,$38    ;
         .BYTE $31,$37,$32,$35,$38
         .BYTE $31,$39,$36,$39,$34
         .BYTE $32,$33,$39,$33,$36
         .BYTE $32,$34,$38,$33,$36
         .BYTE $33,$30,$32,$35,$36
         .BYTE $33,$34,$35,$34,$33
         .BYTE $34,$31,$30,$31,$37
         .BYTE $30,$39,$34,$39,$38
         .BYTE $30,$37,$36,$39,$38
         .BYTE $31,$38,$37,$39,$34
         .BYTE $34,$33,$39,$30,$31    ; Frame = 43901
         .BYTE $34,$36,$33,$34,$34    ; Frame = 46344
;----------------------------------------------------
;             REPEAT Frame Data 
;----------------------------------------------------
         .BYTE $30,$35,$35,$36,$37    ; Repeat = 05567
         .BYTE $30,$35,$33,$36,$33
         .BYTE $31,$33,$31,$34,$34
         .BYTE $31,$31,$35,$30,$39
         .BYTE $32,$30,$39,$33,$38
         .BYTE $32,$33,$31,$39,$38
         .BYTE $32,$36,$32,$32,$36
         .BYTE $32,$39,$32,$30,$35
         .BYTE $33,$32,$33,$31,$34
         .BYTE $33,$33,$35,$34,$31
         .BYTE $33,$39,$37,$33,$35
         .BYTE $34,$32,$31,$33,$33
         .BYTE $32,$32,$33,$37,$31
         .BYTE $31,$38,$36,$35,$36
         .BYTE $32,$32,$36,$37,$31
         .BYTE $32,$36,$37,$38,$38
         .BYTE $34,$30,$31,$31,$36
         .BYTE $34,$32,$33,$30,$30
         .BYTE $31,$35,$34,$33,$37
         .BYTE $34,$32,$39,$32,$31
         .BYTE $34,$33,$37,$34,$39
         .BYTE $30,$37,$36,$39,$37
         .BYTE $31,$34,$31,$33,$35
         .BYTE $31,$37,$31,$30,$37
         .BYTE $30,$39,$33,$34,$37
         .BYTE $32,$32,$30,$36,$33
         .BYTE $32,$34,$36,$38,$35
         .BYTE $32,$38,$30,$30,$35
         .BYTE $33,$30,$31,$30,$35
         .BYTE $33,$33,$30,$36,$35
         .BYTE $33,$36,$31,$39,$32
         .BYTE $33,$39,$31,$32,$39
         .BYTE $34,$31,$37,$36,$36
         .BYTE $30,$36,$36,$34,$37
         .BYTE $31,$31,$31,$34,$37
         .BYTE $31,$38,$30,$30,$37
         .BYTE $32,$30,$34,$34,$33
         .BYTE $32,$34,$36,$38,$35
         .BYTE $32,$35,$35,$38,$35
         .BYTE $33,$31,$30,$30,$35
         .BYTE $33,$35,$32,$39,$32
         .BYTE $34,$31,$37,$36,$36
         .BYTE $31,$30,$32,$34,$37
         .BYTE $30,$38,$34,$34,$37
         .BYTE $31,$39,$35,$34,$33
         .BYTE $34,$36,$33,$34,$33
         .BYTE $34,$36,$37,$33,$35    ; Frame = 46735


;----------------------------------------------------
;                    unused
;----------------------------------------------------
         .BYTE $FF,$FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF,$FF        
LCFB3:   .BYTE $FF,$FF,$FF
LCFB6:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LCFCA:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LD0BA:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF
LD0CF:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF
LD1B2:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF
LD2CB:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF
LD368:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF
LD4BB:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF
LD506:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LDBDA:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LDE20:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF


; ---------------------------------------------------
;                Interrupt Routines
; ---------------------------------------------------
LE000::  JMP LE068              ; NMI_VECTOR  
LE003::  JMP LE0F8              ; RESET_VECTOR
LE006::  JMP LE009              ; IRQ_VECTOR



; ---------------------------------------------------
;                   IRQ_VECTOR
; ---------------------------------------------------
;             Check for Diagnostic Mode
; ---------------------------------------------------
LE009:   PHA                    ; Save A Register
         TXA                    ; Move X
         PHA                    ; Save X Register
         TYA                    ; Move Y
         PHA                    ; Save Y Register
         LDA L1002              ; Get DIP Bank 2
         AND #$80               ; Check sw8, diagnostic mode
         BEQ LE01D              ; sw8=ON, go Enter Diagnostic Mode
         PLA                    ; Restore Y Register
         TAY                    ; Move Y
         PLA                    ; Restore X Register
         TAX                    ; Move X
         PLA                    ; Restore A Register
         JMP L4006              ; Jump back to main loop
; ---------------------------------------------------
;                Check Diagnostic Mode
; ---------------------------------------------------
LE01D:   LDA ICAX1Z             ;
         BNE LE02E              ; Yes, do diagnostics
         SEI                    ; Enable Interrupts
         LDA L1006              ; reset NMI
         LDA L1007              ; reset NMI
         PLA                    ; Restore Y Register
         TAY                    ; Move Y
         PLA                    ; Restore X Register
         TAX                    ; Move X
         PLA                    ; Restore A Register
         RTI                    ; Return from Interrupt
; ---------------------------------------------------
;              Enter Diagnostic Mode
; ---------------------------------------------------
LE02E:   LDA ICAX1Z             ; Get test number
         CMP #$01               ; Are we at Test #1?
         BNE LE037              ; No, so skip ahead
         JMP LE055              ; Perform Diagnostic Test #1

LE037:   CMP #$02               ; Are we at Test #2?
         BNE LE044              ; No, so skip ahead
         PLA                    ;
         PLA                    ;
         PLA                    ;
         PLA                    ;
         PLA                    ;
         PLA                    ;
         JMP LEBBF              ;
LE044:   CMP #$03               ;
         BNE LE04B              ;
         JMP LF323              ; Perform Diagnostic Test #3 
LE04B:   CMP #$04
         BNE LE052
         JMP LF264
LE052:   JMP LE055
; ---------------------------------------------------
;                Diagnostic Mode
; ---------------------------------------------------
LE055:   PLA
         PLA
         PLA
         PLA
         PLA
         PLA
         SEI
LE05C:   LDA L1006
         BPL LE05C
         JSR LE99F              ; Send CLEAR_ALL if not ACKNOWLEDGED
         INX
         JMP LE96A




; ---------------------------------------------------
;             NMI_VECTOR  (every TBD ms)
; ---------------------------------------------------
;               Diagnostic Check
; ---------------------------------------------------
LE068:   PHA                    ; Save A Register
         TXA                    ; Move X
         PHA                    ; Save X Register
         TYA                    ; Move Y
         PHA                    ; Save Y Register
         LDA L1002              ; Get DIP Bank 2
         AND #$80               ; Check sw8, diagnostic mode
         BEQ LE07C              ; sw8=ON, continue Diagnostic Mode
         PLA                    ; Restore Y Register
         TAY                    ; Move Y
         PLA                    ; Restore X Register
         TAX                    ; Move X
         PLA                    ; Restore A Register
         JMP L4000              ; Jump to Check for Coins
; ---------------------------------------------------
;                 Diagnostic Mode
; ---------------------------------------------------
LE07C:   SEI                    ; Enable Interrupts
         LDA #$00               ; A = No Sound
         STA L1004              ; Write to Sound Chip
         JSR LE3FE              ; Wait for Verticle Blanking
         LDA L1001              ; Get switch inputs
         AND #$C0               ; Check Coin Slots
         BEQ LE0D7              ; Yes coins so skip diagnostics
         LDA #$00               ;
         STA DSTAT              ;
         LDA L1000              ; Get switch inputs
         EOR #$FF               ; Toggle all bits
         AND #$10               ; Check S3 switch
         BNE LE0A8              ; Pressed so don't advance test
; ---------------------------------------------------
;           Cycle to next Diagnostic Test
; ---------------------------------------------------
LE099:   INC DiagTestNo         ; Goto next Diagnostic Test
         LDA #$0B               ; Check for all 11 tests
         CMP DiagTestNo         ; Have we finished all tests?
         BCS LE0A8              ; No, so skip ahead
         LDA #$00               ; Loop back to the first test
         STA DiagTestNo         ; Save test number
         STA L1001
LE0A8:   LDX #$FF
         TXS
         LDA #$1F               ; 31 Blanking Pulses
         STA NumberBlanks       ; Save number of blanking pulses to wait for
         JSR LE3E2              ; Wait for number of Blanking pulses
LE0B2:   LDY #$FF               ; Don't erase screen
         JSR LE467              ; Reset Color RAM
         LDA #$00               ; A = 00
         STA ICAX1Z             ;
         STA L1001              ; Clear 
         STA L1003              ; Clear 
         LDA #$19               ; A = Coin Sound
         STA L1004              ; Write to Sound Chip
; ---------------------------------------------------
;                Call Diagnostic Test
; ---------------------------------------------------
         LDA DiagTestNo         ; Get routine number
         ASL A                  ; 2 bytes per address
         TAX                    ; Index to routine address
         LDA LE0E0,X            ; Get routine address LO
         STA ICBAHZ             ; Setup routine address
         LDA LE0E1,X            ; Get routine address HI
         STA Sprite4_Prop       ; Setup routine address
         JMP (ICBAHZ)           ; Jump to diagnostic routine
LE0D7:   STA L1001              ; reset inputs?
         PLA                    ; Restore Y Register
         TAY                    ; Move Y
         PLA                    ; Restore X Register
         TAX                    ; Move X
         PLA                    ; Restore A Register
         RTI                    ; Return from Interrupt

; ---------------------------------------------------
;           Diagnostic Routine Addresses
; ---------------------------------------------------
LE0E0:   .WORD LE53F   ; #1  Zero Stack Test
         .WORD LE651   ; #3  ROM Checksum Test   
         .WORD LEDAB   ;     Monitor Adjustments (16 tests)
         .WORD LE8D3   ; #4  Character Display Test 
         .WORD LE754   ; #5  Mix Test              
         .WORD LF126   ; #6  DIP Switch Test
         .WORD LF09C   ; #7  Panel switch Test
         .WORD LECFD   ; #8  Sound Test 
         .WORD LF206   ; #9  Line Test
         .WORD LE939   ; #10 LDP Test
         .WORD LEA80   ; #11 LaserDisc Test (upright only)
         .WORD LF2A3   ; 



; ---------------------------------------------------
;                  RESET_VECTOR 
; ---------------------------------------------------
LE0F8:   SEI                    ; Enable Interrupts
         CLD                    ; Clear Decimal Mode
         LDX #$FF               ;
         TXS                    ;
         LDA L1002              ; Get DIP Bank 2
         AND #$80               ; Check Diagnostic sw8
         BEQ LE107              ;
         JMP L4003              ; Next Routine, Start Program
LE107:   LDA #$03
         STA L1006
         LDA #$96
         STA L1006
         LDA #$00
         STA L1000              ; Clear timer2
         STA L1004
         STA L1003              ; Clear timer1
         STA ICAX1Z
         JMP LE121
LE121:   LDY #$00               ; Check ZERO PAGE RAM
         JMP LE467              ; Reset Color RAM
LE126:   JMP LF19C              ; Erase Screen




; ---------------------------------------------------
;       Perform "ZERO PAGE AREA" Diagnostic Test
; ---------------------------------------------------
LE129:   LDX #$00               ; Start at first character
LE12B:   LDA LF413,X            ; Get Diagnostic message text
         STA L3908,X            ; Print message text
         INX                    ; Go to next character
         CPX #$0E               ; Have we printed entire message?
         BNE LE12B              ; No, so loop back until message printed
; ---------------------------------------------------
;   Fill Zero Page RAM with Test Bytes ($0000-$0FFF)
; ---------------------------------------------------
         LDY #$08               ; 8 Test Bytes to use
         LDX #$00               ; Start at beginning of RAM
LE13A:   LDA LE244,Y            ; Get Test Byte from table below
         STA L0000,X            ; Write Test Byte to RAM
         STA L0100,X            ; Write Test Byte to RAM
         STA L0200,X            ; Write Test Byte to RAM
         STA L0300,X            ; Write Test Byte to RAM
         STA L0400,X            ; Write Test Byte to RAM
         STA L0500,X            ; Write Test Byte to RAM
         STA L0600,X            ; Write Test Byte to RAM
         STA L0700,X            ; Write Test Byte to RAM
         STA L0800,X            ; Write Test Byte to RAM
         STA L0900,X            ; Write Test Byte to RAM
         STA L0A00,X            ; Write Test Byte to RAM
         STA L0B00,X            ; Write Test Byte to RAM
         STA L0C00,X            ; Write Test Byte to RAM
         STA L0D00,X            ; Write Test Byte to RAM
         STA L0E00,X            ; Write Test Byte to RAM
         STA L0F00,X            ; Write Test Byte to RAM
         INX                    ; Go to next RAM location
         BNE LE13A              ; Loop back until all of RAM has been written
; ---------------------------------------------------
;          Check RAM location for Test Byte
; ---------------------------------------------------
LE16F:   LDA LE244,Y            ; Get Test Byte
         EOR L0000,X            ; Check RAM for Test Byte
         BEQ LE179              ; Test Byte present so skip ahead
         JMP LE24D              ; Jump ahead and Print "0"
LE179:   LDA LE244,Y            ; Get Test Byte
         EOR L0100,X            ; Check RAM for Test Byte
         BEQ LE184              ; Test Byte present so skip ahead
         JMP LE255              ; Jump ahead and Print "1"
LE184:   LDA LE244,Y            ; Get Test Byte
         EOR L0200,X            ; Check RAM for Test Byte
         BEQ LE18F              ; Test Byte present so skip ahead
         JMP LE25D              ; Jump ahead and Print "2"
LE18F:   LDA LE244,Y            ; Get Test Byte
         EOR L0300,X            ; Check RAM for Test Byte
         BEQ LE19A              ; Test Byte present so skip ahead
         JMP LE265              ; Jump ahead and Print "3"
LE19A:   LDA LE244,Y            ; Get Test Byte
         EOR L0400,X            ; Check RAM for Test Byte
         BEQ LE1A5              ; Test Byte present so skip ahead
         JMP LE26D              ; Jump ahead and Print "4"
LE1A5:   LDA LE244,Y            ; Get Test Byte
         EOR L0500,X            ; Check RAM for Test Byte
         BEQ LE1B0              ; Test Byte present so skip ahead
         JMP LE275              ; Jump ahead and Print "5"
LE1B0:   LDA LE244,Y            ; Get Test Byte
LE1B3:   EOR L0600,X            ; Check RAM for Test Byte
         BEQ LE1BB              ; Test Byte present so skip ahead
         JMP LE27D              ; Jump ahead and Print "6"
LE1BB:   LDA LE244,Y            ; Get Test Byte
         EOR L0700,X            ; Check RAM for Test Byte
         BEQ LE1C6              ; Test Byte present so skip ahead
         JMP LE285              ; Jump ahead and Print "7"
LE1C6:   LDA LE244,Y            ; Get Test Byte
         EOR L0800,X            ; Check RAM for Test Byte
         BEQ LE1D1              ; Test Byte present so skip ahead
         JMP LE28D              ; Jump ahead and Print "8"
LE1D1:   LDA LE244,Y            ; Get Test Byte
         EOR L0900,X            ; Check RAM for Test Byte
         BEQ LE1DC              ; Test Byte present so skip ahead
         JMP LE295              ; Jump ahead and Print "9"
LE1DC:   LDA LE244,Y            ; Get Test Byte
         EOR L0A00,X            ; Check RAM for Test Byte
         BEQ LE1E7              ; Test Byte present so skip ahead
         JMP LE29D              ; Jump ahead and Print "A"
LE1E7:   LDA LE244,Y            ; Get Test Byte
         EOR L0B00,X            ; Check RAM for Test Byte
         BEQ LE1F2              ; Test Byte present so skip ahead
         JMP LE2A5              ; Jump ahead and Print "B"
LE1F2:   LDA LE244,Y            ; Get Test Byte
         EOR L0C00,X            ; Check RAM for Test Byte
         BEQ LE1FD              ; Test Byte present so skip ahead
         JMP LE2AD              ; Jump ahead and Print "C"
LE1FD:   LDA LE244,Y            ; Get Test Byte
         EOR L0D00,X            ; Check RAM for Test Byte
         BEQ LE208              ; Test Byte present so skip ahead
         JMP LE2B5              ; Jump ahead and Print "D"
LE208:   LDA LE244,Y            ; Get Test Byte
         EOR L0E00,X            ; Check RAM for Test Byte
         BEQ LE213              ; Test Byte present so skip ahead
         JMP LE2BD              ; Jump ahead and Print "E"
LE213:   LDA LE244,Y            ; Get Test Byte
         EOR L0F00,X            ; Check RAM for Test Byte
         BEQ LE21E              ; Test Byte present so skip ahead
         JMP LE2C5              ; Jump ahead and Print "F"

LE21E:   INX                    ; Go to next RAM location
         BEQ LE224              ; Jump ahead if we're finished
         JMP LE16F              ; Loop back until all RAM locations verified
LE224:   DEY                    ; Go to next Test Byte in table
         BEQ LE22A              ; All Test Byte used so skip ahead
         JMP LE13A              ; Loop back and repeat with new Test Byte
; ---------------------------------------------------
;          RAM Check finished and passed
; ---------------------------------------------------
LE22A:   LDA #$FF               ; Start at first Diagnostic Test
         STA DiagTestNo         ; Save Diagnostic Test number
         STA L1001              ; reset inputs?
         LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         JSR LE467              ; Reset Color RAM
         JSR LE309              ; Print Diagnostic Program Intro Message
         LDA #$09               ; A = Dong Sound #9
         STA L1004              ; Write to Sound Chip
LE241:   JMP LE241              ; Wait here


; ---------------------------------------------------
;             RAM Check Test Bytes
; ---------------------------------------------------
LE244:   .BYTE $00,$80,$40,$20,$10,$08,$04,$02,$01

   
; ---------------------------------------------------
;                Print "0"
; ---------------------------------------------------
LE24D:   LDA #$01               ; Char = "0"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "1"
; ---------------------------------------------------
LE255:   LDA #$02               ; Char = "1"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "2"
; ---------------------------------------------------
LE25D:   LDA #$03               ; Char = "2"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "3"
; ---------------------------------------------------
LE265:   LDA #$04               ; Char = "3"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "4"
; ---------------------------------------------------
LE26D:   LDA #$05               ; Char = "4"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "5"
; ---------------------------------------------------
LE275:   LDA #$06               ; Char = "5"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "6"
; ---------------------------------------------------
LE27D:   LDA #$07               ; Char = "6"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "7"
; ---------------------------------------------------
LE285:   LDA #$08               ; Char = "7"
         STA L3973              ; Print character
         JMP LE2CD              ; Jump ahead
; ---------------------------------------------------
;                Print "8"
; ---------------------------------------------------
LE28D:   LDA #$09               ; Char = "8"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead
; ---------------------------------------------------
;                Print "9"
; ---------------------------------------------------
LE295:   LDA #$0A               ; Char = "9"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead
; ---------------------------------------------------
;                Print "A"
; ---------------------------------------------------
LE29D:   LDA #$0B               ; Char = "A"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead
; ---------------------------------------------------
;                Print "B"
; ---------------------------------------------------
LE2A5:   LDA #$0C               ; Char = "B"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead
; ---------------------------------------------------
;                Print "C"
; ---------------------------------------------------
LE2AD:   LDA #$0D               ; Char = "C"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead
; ---------------------------------------------------
;                Print "D"
; ---------------------------------------------------
LE2B5:   LDA #$0E               ; Char = "D"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead
; ---------------------------------------------------
;                Print "E"
; ---------------------------------------------------
LE2BD:   LDA #$0F               ; Char = "E"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead
; ---------------------------------------------------
;                Print "F"
; ---------------------------------------------------
LE2C5:   LDA #$10               ; Char = "F"
         STA L3973              ; Print character
         JMP LE2DC              ; Jump ahead

; ---------------------------------------------------
;        Print RAM Socket location "3F" 
; ---------------------------------------------------
LE2CD:   LDA #$04               ; Char = "3"
         STA L396F              ; Print character
         LDA #$10               ; Char = "F"
         STA L3970              ; Print character
         LDY #$00               ; 
         JMP LE2EB              ; Jump ahead
; ---------------------------------------------------
;        Print RAM Socket location "5F" 
; ---------------------------------------------------
LE2DC:   LDA #$06               ; Char = "5"
         STA L398F              ; Print character
         LDA #$10               ; Char = "F"
         STA L3990              ; Print character
         LDY #$00               ;
         JMP LE2EB              ;
; ---------------------------------------------------
;               
; ---------------------------------------------------
LE2EB:   TXA
         AND #$0F
         CLC
         ADC #$01
         STA L3975
         TXA
         LSR A
         LSR A
         LSR A
         LSR A
         CLC
         ADC #$01
         STA L3974
         JMP LE4F9              ; Diagnostic Test Failed, print "ERROR"


;----------------------------------------------------
;
;----------------------------------------------------
LE302:   LDX #$3F               ;
         LDY #$00               ;
         JMP LE3EA              ;


;----------------------------------------------------
;    Print Diagnostic Program Introduction Screen
;----------------------------------------------------
LE309:   LDX #$00               ; Start at first character
LE30B:   LDA LE345,X            ; Get text message
         STA L38C2,X            ; Write Text Line 1
         LDA LE361,X            ; Get text message
         STA L3902,X            ; Write Text Line 2
         LDA LE37D,X            ; Get text message
         STA L3942,X            ; Write Text Line 3
         LDA LE399,X            ; Get text message
         STA L3A22,X            ; Write Text Line 4
         LDA LE3B5,X            ; Get text message
         STA L3A62,X            ; Write Text Line 5
         INX                    ; Go to next character
         CPX #$1C               ; Have all characters been printed?
         BNE LE30B              ; No, loop back until all characters printed
;----------------------------------------------------
;         Check Factory Diagnostic Switch
;----------------------------------------------------
         LDA L1002              ; Get DIP bank 2
         AND #$40               ; Check Sw7,0=User Diagnostics,1=Factory Diagnostics
         BNE LE344              ; Factory Diagnostics, so leave
;----------------------------------------------------
;   Erase "LX2A" and "VER0" from Diagnostic Intro
;----------------------------------------------------
         LDX #$00               ; Starting location
LE337:   LDA #$00               ; Char = SPACE
         STA L38C2,X            ; Erase "LX2A" characters
         STA L38DA,X            ; Erase "VER0" characters
         INX                    ; Next location
         CPX #$04               ; Have all characters been erased?
         BNE LE337              ; No, loop unitl all characters erased
LE344:   RTS                    ; Return

;----------------------------------------------------
;        Diagnostic Program Text Messages
;----------------------------------------------------
LE345:   .BYTE $16,$22,$03,$0B ; "LX2A DIAGNOSTIC PROGRAM VER0"
         .BYTE $00,$0E,$13,$0B
         .BYTE $11,$18,$19,$1D
         .BYTE $1E,$13,$0D,$00
         .BYTE $1A,$1C,$19,$11
         .BYTE $1C,$0B,$17,$00
         .BYTE $20,$0F,$1C,$01

LE361:   .BYTE $00,$0E,$0B,$1E ; " DATE - 20 83 BY S KUMAGAI  "  
         .BYTE $0F,$00,$0A,$00
         .BYTE $03,$01,$00,$09
         .BYTE $04,$00,$0C,$23
         .BYTE $00,$1D,$00,$15
         .BYTE $1F,$17,$0B,$11
         .BYTE $0B,$13,$00,$00

LE37D:   .BYTE $00,$0E,$0B,$1E ; " DATA EAST CORPORATION **** "
         .BYTE $0B,$00,$0F,$0B
         .BYTE $1D,$1E,$00,$0D
         .BYTE $19,$1C,$1A,$19
         .BYTE $1C,$0B,$1E,$13
         .BYTE $19,$18,$00,$68
         .BYTE $69,$6A,$6B,$00

LE399:   .BYTE $00,$1A,$1F,$1D ; " PUSH SERVICE SW TO START   "
         .BYTE $12,$00,$1D,$0F
         .BYTE $1C,$20,$13,$0D
         .BYTE $0F,$00,$1D,$21
         .BYTE $00,$1E,$19,$00
         .BYTE $1D,$1E,$0B,$1C
         .BYTE $1E,$00,$00,$00

LE3B5:   .BYTE $00,$00,$00,$00 ; "                            "
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00
         .BYTE $00,$00,$00,$00


;----------------------------------------------------
;      Print Diagnostic Test Name to Screen
;----------------------------------------------------
LE3D1:   LDY #$00               ; Initialize loop
LE3D3:   LDA (DiagMessage_LO),Y ; Get text characters
         STA L3908,Y            ; Print text to screen
         INY                    ; Increment loop
         CPY #$0E               ; Have all 14 characters been printed?
         BNE LE3D3              ; No, loop until message complete
         RTS                    ; Return


;----------------------------------------------------
;         Wait for 64 Verticle Blanking Pulses   
;----------------------------------------------------
LE3DE:   LDA #$3F               ; Count = 63
         STA NumberBlanks       ; Save Blanking Count
;----------------------------------------------------
;        Wait for number of Blanking pulses 
;----------------------------------------------------
LE3E2:   JSR LE3FE              ; Wait for Verticle Blanking
         DEC NumberBlanks       ; Count blanking pulse
         BNE LE3E2              ; Loop until count is zero
         RTS                    ; Return



;----------------------------------------------------
;               
;----------------------------------------------------
LE3EA:   LDA L1003
         BPL LE3EA
LE3EF:   LDA L1003
         BMI LE3EF
         DEX
         BNE LE3EA
         TYA
         BEQ LE3FB
         RTS
LE3FB:   JMP LE121


;----------------------------------------------------
;            Wait for Verticle Blanking
;----------------------------------------------------
LE3FE:   LDA L1003              ; Get VBlank
         BPL LE3FE              ; Loop until VBlank = 0
LE403:   LDA L1003              ; Get VBlank
         BMI LE403              ; Loop until VBlank = 1
         RTS                    ; Return

;----------------------------------------------------
;           Wait for Horizontal Blanking     
;----------------------------------------------------
LE409:   LDA L1003              ; Get Blanking
         ASL A                  ; Check Horizontal Blanking
         BPL LE409              ; Loop until HBlank = 0
LE40F:   LDA L1003              ; Get Blanking
         ASL A                  ; Check Horizontal Blanking
         BMI LE40F              ; Loop until HBlank = 1
         RTS                    ; Return


;----------------------------------------------------
;               Wait until S1 Pressed
;----------------------------------------------------
LE416:   LDA L1000              ; Get S1 Switch input
         AND #$40               ; Check S1 Switch
         BNE LE416              ; Wait here until S1 pressed
         RTS                    ; S1 pressed so leave

;----------------------------------------------------
;               Wait until S2 Pressed
;----------------------------------------------------
LE41E:   LDA L1000              ; Get S2 Switch input
         AND #$20               ; Check S2 Switch
         BNE LE41E              ; Wait here until S2 pressed
         RTS                    ; S2 pressed so leave

;----------------------------------------------------
;               Wait until S3 Pressed
;----------------------------------------------------
LE426:   LDA L1000              ; Get S3 Switch input
         AND #$10               ; Check S3 Switch
         BNE LE426              ; Wait here until S3 pressed
         RTS                    ; S3 pressed so leave

;----------------------------------------------------
;            Wait until Player1 Pressed
;----------------------------------------------------
LE42E:   LDA L1001              ; Get Switch input
         AND #$20               ; Check Player1 button
         BNE LE42E              ; Wait here until Player1 pressed
         RTS                    ; Player1 pressed so leave

;----------------------------------------------------
;            Wait until Player2 Pressed
;----------------------------------------------------
LE436:   LDA L1001              ; Get Switch input
         AND #$10               ; Check Player2 button
         BNE LE436              ; Wait here until Player2 pressed
         RTS                    ; Player2 pressed so leave

;----------------------------------------------------
;                    unknown
;----------------------------------------------------
LE43E:   LDY #$00               ;
LE440:   TXA                    ;
         STA (SpritePic1),Y     ;
         INY                    ;
         BNE LE440              ;
         INC SpritePic2         ;
         LDA SpritePic2         ;
         CMP ICDNOZ             ;
         BNE LE440              ;
         RTS                    ;


;----------------------------------------------------
;                    unknown
;----------------------------------------------------
LE44F:   LDY #$00               ;
LE451:   TXA                    ;
         STA (SpritePic1),Y     ;
         INC SpritePic1         ;
         BNE LE45A              ;
         INC SpritePic2         ;
LE45A:   LDA SpritePic1
         CMP ICCOMZ
         BNE LE451
         LDA SpritePic2
         CMP ICDNOZ
         BNE LE451
         RTS


; ---------------------------------------------------
;          Reset Color RAM, Erase screen
; ---------------------------------------------------
;           Y = 00, Erase screen
;           Y = FF, Keep screen
; ---------------------------------------------------
;         Copy E47B-E4B3 to 1800-1838
; ---------------------------------------------------
LE467:   LDX #$00               ; Loop = 0
LE469:   LDA LE47B,X            ; Get data
         STA L1800,X            ; Copy data here
         INX                    ; Next byte
         CPX #$38               ; All bytes copied?
         BNE LE469              ; No, so loop until all bytes copied
         TYA                    ; Test Y Flag
         BEQ LE478              ; Yes, so erase screen
         RTS                    ; No, so just return
LE478:   JMP LE126              ; Erase screen




; ---------------------------------------------------
;             unknown
; ---------------------------------------------------
LE47B:   .BYTE $FF
         SBC (L00C7),Y
         CPY #$3F
         SEC
         .BYTE $07
         BRK
         .BYTE $FF
         SED
         .BYTE $FF
         CPY #$D5
         .BYTE $9B
         BCC LE48B
LE48B:   .BYTE $FF
         SED
         .BYTE $FF
         CPY #$D5
         .BYTE $9B
         BCC LE493
LE493:   .BYTE $FF
         SED
         .BYTE $FF
         CPY #$D5
         .BYTE $9B
         BCC LE49B
LE49B:   .BYTE $FF
         SED
         .BYTE $FF
         CPY #$3F
         .BYTE $9B
         EOR #$00
         .BYTE $FF
         SED
         .BYTE $FF
         CPY #$3F
         .BYTE $9B
         EOR #$00
         .BYTE $FF
         SED
         .BYTE $FF
         CPY #$3F
         .BYTE $9B
         EOR #$00
; ---------------------------------------------------
;                     jsr
; ---------------------------------------------------
LE4B3:   LDX #$00               ;
LE4B5:   LDA LE4C1,X            ;
         STA L1800,X            ; Copy data here
         INX
         CPX #$38
         BNE LE4B5
         RTS
LE4C1:   .BYTE $FF
         SED
         .BYTE $C7
         CPY #$3F
         SEC
         .BYTE $07
         BRK
         .BYTE $FF
         SED
         .BYTE $C7
         CPY #$3F
         SEC
         .BYTE $07
         BRK
         .BYTE $FF
         SED
         .BYTE $C7
         CPY #$3F
         SEC
         .BYTE $07
         BRK
         .BYTE $FF
         SED
         .BYTE $C7
         CPY #$3F
         SEC
         .BYTE $07
         BRK
         .BYTE $FF
         SED
         .BYTE $C7
         CPY #$3F
         SEC
         .BYTE $07
         BRK
         .BYTE $FF
         SED
         .BYTE $C7
         CPY #$3F
         SEC
         .BYTE $07
         BRK
         .BYTE $FF
         SED
         .BYTE $C7
         CPY #$3F
         SEC
         .BYTE $07
         BRK


; ---------------------------------------------------
;      Diagnostic Test Failed, print "ERROR"
; ---------------------------------------------------
;         Y = 0, print #
; ---------------------------------------------------
LE4F9:   LDA #$05               ; A = ERROR Sound
         STA L1004              ; Write to Sound Chip
         LDA #$0F               ; Char = "E"
         STA L394D              ; Print character
         LDA #$1C               ; Char = "R"
         STA L394E              ; Print character
         STA L394F              ; Print character
         STA L3951              ; Print character
         LDA #$19               ; Char = "O"
         STA L3950              ; Print character
         TYA                    ; 
         BEQ LE517              ;
         RTS                    ; Return
LE517:   JMP LE302              ;

; ---------------------------------------------------
;          Diagnostic Test Passed, print "OK"
; ---------------------------------------------------
LE51A:   LDA #$09               ; A = PASS Sound
         STA L1004              ; Write to Sound Chip
         LDA #$19               ; Char = "O"
         STA L396F              ; Print character
         LDA #$15               ; Char = "K"
         STA L3970              ; Print character
         RTS                    ; Return

; ---------------------------------------------------
;           
; ---------------------------------------------------
LE52A:   LDA #$02
         STA L1004
         LDA #$0F
         STA L398E
         LDA #$18
         STA L398F
         LDA #$0E
         STA L3990
         RTS



; ---------------------------------------------------
;       Diagnostic Test #1 - Zero Page RAM Test
; ---------------------------------------------------
LE53F:   LDA #$00               ; Clear RAM check results
         STA DOSVEC+1           ; Save RAM check results
         LDX #$00               ; Point to first Test byte to use
         LDY #$00               ; Index = 0 for some reason
         LDA #$00               ; RAM Start = $2000
         STA BOOT               ; Setup RAM Start location (low)
LE54B:   LDA #$20               ; RAM Start = $2000
         STA BOOT+1             ; Setup RAM Start location (high)
; ---------------------------------------------------
;         Write test data to RAM ($2000-$3FFF)
; ---------------------------------------------------
LE54F:   LDA LE643,X            ; Get Test byte from table
         STA (BOOT),Y           ; Write Test Byte into RAM location
         INC BOOT               ; Proceed to next RAM location
         BNE LE54F              ; Loop back until all 256 locations written
         INC BOOT+1             ; Increment RAM high byte pointer
         LDA BOOT+1             ; Check RAM end location
         CMP #$40               ; Have we reached $3FFF?
         BNE LE54F              ; No so continue writing
; ---------------------------------------------------
;       Read test data from RAM ($2000-$3FFF)
; ---------------------------------------------------
         LDA #$20               ; Reset RAM pointer to $2000
         STA BOOT+1             ; Save RAM start pointer
LE564:   LDA LE643,X            ; Get Test Byte from table
         CMP (BOOT),Y           ; Did RAM location contain value?
         BEQ LE56E              ; Yes, so skip ahead
         JSR LE61F              ; Otherwise mark RAM Failure
LE56E:   INC BOOT               ; Check next RAM location
         BNE LE564              ; Loop back until 256 locations have been checked
         INC BOOT+1             ; Increment RAM high byte pointer
         LDA BOOT+1             ; Check RAM end location
         CMP #$40               ; Have we reached $3FFF?
         BNE LE564              ; No so continue checking
         INX                    ; Do next Test byte
         CPX #$0E               ; Have all Test bytes been verified?
         BNE LE54B              ; No, so continue RAM test with new byte
; ---------------------------------------------------
;         Write test data to RAM ($1800-$1837)
; ---------------------------------------------------
         LDX #$00               ; Point to
LE581:   LDY #$00               ; Loop = 0
         LDA LE643,X            ; Get Test byte from table
LE586:   STA L1800,Y            ; Write Test Byte into RAM location
         INY                    ; Go to next RAM location
         CPY #$38               ; Have we written to all RAM locations?
         BNE LE586              ; No, so loop until all locations written
; ---------------------------------------------------
;        Read test data from RAM ($1800-$1837)
; ---------------------------------------------------
         LDY #$00               ; Start at first RAM location
LE590:   LDA LE643,X            ; Get Test Byte from table
         CMP L1800,Y            ; Did RAM location contain value?
         BEQ LE59E              ; Yes, so skip ahead
         LDA #$10               ; Mark Error
         ORA DOSVEC+1           ; Store error in results
         STA DOSVEC+1           ; Save RAM check results
LE59E:   INY                    ; Go to next RAM location
         CPY #$38               ; Have we checked all locations?
         BNE LE590              ; No, loop back until all locations checked
         INX                    ; Do next Test byte 
         CPX #$0E               ; Have all Test bytes been verified?
         BNE LE581              ; No, so continue RAM test with new byte
; ---------------------------------------------------
;          Display RAM Check test results
; ---------------------------------------------------
         LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         LDY #$FF               ; Call for erase screen
         JSR LE467              ; Reset Color RAM
         LDA DOSVEC+1           ; Check test results
         BNE LE5BC              ; Failure found so print location
         JSR LE51A              ; Diagnostic Test Passed, print "OK" 
         JMP LE611              ; Skip to end of test

; ---------------------------------------------------
;           Display RAM Failure Location 
; ---------------------------------------------------
LE5BC:   LDY #$FF               ; FF = reset?
         JSR LE4F9              ; Diagnostic Test Failed, print "ERROR"

         LSR DOSVEC+1           ; RAM 6A Failure?
         BCC LE5CF              ; No, so skip ahead
         LDA #$07               ; Char = "6"
         STA L396F              ; Print character
         LDA #$0B               ; Char = "A"
         STA L3970              ; Print character

LE5CF:   LSR DOSVEC+1           ; RAM 8A Failure?
         BCC LE5DD              ; No, so skip ahead
         LDA #$09               ; Char = "8"
         STA L398F              ; Print character
         LDA #$0B               ; Char = "A"
         STA L3990              ; Print character

LE5DD:   LSR DOSVEC+1           ; RAM 12A Failure?
         BCC LE5F0              ; No, so skip ahead
         LDA #$02               ; Char = "1"
         STA L39AE              ; Print character
         LDA #$03               ; Char = "2"
         STA L39AF              ; Print character
         LDA #$0B               ; Char = "A"
         STA L39B0              ; Print character

LE5F0:   LSR DOSVEC+1           ; RAM 10A Failure?
         BCC LE603              ; No, so skip ahead
         LDA #$02               ; Char = "1"
         STA L39CE              ; Print character
         LDA #$01               ; Char = "0"
         STA L39CF              ; Print character
         LDA #$0B               ; Char = "A"
         STA L39D0              ; Print character

LE603:   LSR DOSVEC+1           ; RAM 5J Failure?
         BCC LE611              ; No, so skip ahead
         LDA #$06               ; Char = "5"
         STA L39EF              ; Print character
         LDA #$14               ; Char = "J"
         STA L39F0              ; Print character

LE611:   LDA #$34               ; LF434 = "RAM TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF434 = "RAM TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen  
LE61C:   JMP LE61C              ; Wait here after diagnostic test


; ---------------------------------------------------
;              Mark RAM as failed 
; ---------------------------------------------------
LE61F:   LDA BOOT+1             ; Check RAM address
         CMP #$28               ; Was RAM ($2000-$27FF) bad?
         BCS LE62A              ; No, so skip ahead
         LDA #$01               ; A = Bad RAM Marker
         JMP LE63E              ; Store failure
LE62A:   CMP #$30               ; Was RAM ($2800-$2FFF) bad?
         BCS LE633              ; No, so skip ahead
         LDA #$02               ; A = Bad RAM Marker
         JMP LE63E              ; Store failure
LE633:   CMP #$38               ; Was RAM ($3000-$37FF) bad?
         BCS LE63C              ; No, so skip ahead
         LDA #$04               ; A = Bad RAM Marker
         JMP LE63E              ; Store failure
LE63C:   LDA #$08               ; A = Bad RAM Marker ($3800-$3FFF)
LE63E:   ORA DOSVEC+1           ; Mark RAM failure in results
         STA DOSVEC+1           ; Store test results
         RTS                    ; Return


; ---------------------------------------------------
;                   RAM Test Data
; ---------------------------------------------------
LE643:   .BYTE  $AA, $55, $FF, $01, $0F, $F0, $66
         .BYTE  $99, $CC, $33, $75, $57, $E7, $00




; ---------------------------------------------------
;         Diagnostic Test #3 - ROM Checksum Test
; ---------------------------------------------------
LE651:   LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         LDA #$26               ; LF426 = "ROM READ TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF426 = "ROM READ TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
; ---------------------------------------------------
;               Setup ROM Checksum Test
; ---------------------------------------------------
         LDY #$00               ; No Failures
         LDX #$00               ; Reset Checksum pointer
         LDA #$00               ; A = 0
         STA ZRAM_02            ; Start at beginning of ROM
         STA TRAMSZ             ; Clear test results
         LDA #$40               ; ROM Start Address = $4000
         STA ZRAM_02+1          ; Save ROM Start Address
         LDA #$60               ; ROM End Address = $6000
         STA RAMLO              ; Save ROM End Address
         LDA #$01               ; Mark first ROM
         STA RAMLO+1            ; First ROM checksum results
         JSR LE716              ; Verify ROM05 $4000-5FFF
         LDA #$80               ; ROM End Address = $8000
         STA RAMLO              ; Save ROM End Address
         JSR LE716              ; Verify ROM04 $6000-$7FFF
         LDA #$A0               ; ROM End Address = $A000
         STA RAMLO              ; Save ROM End Address
         JSR LE716              ; Verify ROM03 $8000-$9FFF
         LDA #$C0               ; ROM End Address = $C000
         STA RAMLO              ; Save ROM End Address
         JSR LE716              ; Verify ROM02 $A000-$BFFF
         LDA #$E0               ; ROM End Address = $E000
         STA RAMLO              ; Save ROM End Address
         JSR LE716              ; Verify ROM01 $C000-$DFFF
         LDA #$F0               ; ROM End Address = $F000
         STA RAMLO              ; Save ROM End Address
         JSR LE716              ; Verify ROM00 $E000-$EFFF
; ---------------------------------------------------
;          Display ROM Checksum Test Results
; ---------------------------------------------------
         LDA TRAMSZ             ; Get Checksum results
         AND #$1F               ; Isolate results
         STA TRAMSZ             ; Save results
         BEQ LE710              ; No problems, so skip ahead
; ---------------------------------------------------
;        Display ROM Failure Location (6F-14F)
; ---------------------------------------------------
         LSR TRAMSZ             ; ROM05 Failure?
         BCC LE6B3              ; No, so skip ahead
         LDA #$07               ; Char = "6"
         STA L3970              ; Print character
         LDA #$10               ; Char = "F"
LE6B0:   STA L3971              ; Print character

LE6B3:   LSR TRAMSZ             ; ROM05 Failure?
         BCC LE6C1              ; No, so skip ahead
         LDA #$09               ; Char = "8"
         STA L3990              ; Print character
         LDA #$10               ; Char = "F"
         STA L3991              ; Print character

LE6C1:   LSR TRAMSZ             ; ROM05 Failure?
         BCC LE6CF              ; No, so skip ahead
         LDA #$0A               ; Char = "9"
         STA L39B0              ; Print character
         LDA #$10               ; Char = "F"
         STA L39B1              ; Print character

LE6CF:   LSR TRAMSZ             ; ROM05 Failure?
         BCC LE6E2              ; No, so skip ahead
         LDA #$02               ; Char = "1"
         STA L39CF              ; Print character
         LDA #$02               ; Char = "1"
         STA L39D0              ; Print character
         LDA #$10               ; Char = "F"
         STA L39D1              ; Print character

LE6E2:   LSR TRAMSZ             ; ROM05 Failure?
         BCC LE6F5              ; No, so skip ahead
         LDA #$02               ; Char = "1"
         STA L39EF              ; Print character
         LDA #$03               ; Char = "2"
         STA L39F0              ; Print character
         LDA #$10               ; Char = "F"
         STA L39F1              ; Print character

LE6F5:   LSR TRAMSZ             ; ROM05 Failure?
         BCC LE708              ; No, so skip ahead
         LDA #$02               ; Char = "1"
         STA L3A0F              ; Print character
         LDA #$05               ; Char = "4"
         STA L3A10              ; Print character
         LDA #$10               ; Char = "F"
         STA L3A11              ; Print character

LE708:   LDY #$FF               ; FF = Issue reset?
         JSR LE4F9              ; Diagnostic Test Failed, print "ERROR"
LE70D:   JMP LE70D              ; Stop here after checksum failure

LE710:   JSR LE51A              ; Diagnostic Test Passed, print "OK"
LE713:   JMP LE713              ; Stop here after checksum test


; ---------------------------------------------------
;             Calculate ROM Checksum
; ---------------------------------------------------
LE716:   LDA #$00               ; Initialize Checksum = $0000
         STA L0000              ; Clear Checksum (low)
         STA ZRAM_01            ; Clear Checksum (high)
LE71C:   LDA (ZRAM_02),Y        ; Get ROM byte
         CLC                    ; Clear carry for addition
         ADC L0000              ; Add byte to checksum
         STA L0000              ; Save checksum (low)
         LDA #$00               ; Get carry from addition
         ADC ZRAM_01            ; Add carry to upper byte
         STA ZRAM_01            ; Save checksum (high)
         INY                    ; Next byte
         BNE LE71C              ; Loop until 256 bytes have been checked
         INC ZRAM_02+1          ; Go to next ROM location
         LDA ZRAM_02+1          ; Check for end of ROM
         CMP RAMLO              ; Have we reached end of ROM?
         BNE LE71C              ; No, so loop back and continue calculation
; ---------------------------------------------------
;              Verify ROM Checksum
; ---------------------------------------------------
         LDA LFFEE,X            ; Get correct checksum value (low)
         CMP ZRAM_01            ; Does the checksum match?
         BEQ LE73E              ; Yes, so check high byte
         JSR LE74D              ; ERROR, Mark ROM as faulty
LE73E:   INX                    ; Point to next checksum byte
         LDA LFFEE,X            ; Get correct Checksum
         CMP L0000              ; Does the Checksum match?
         BEQ LE749              ; Yes so skip ahead
         JSR LE74D              ; ERROR, Mark ROM as faulty
LE749:   ASL RAMLO+1            ; Mark ROM as pass
         INX                    ; Point to next checksum byte
         RTS                    ; Return

; ---------------------------------------------------
;              Mark ROM as failed
; ---------------------------------------------------
LE74D:   LDA RAMLO+1            ; Get which ROM failed
         ORA TRAMSZ             ; Mark failure with a "1"
         STA TRAMSZ             ; Save checksum results
         RTS                    ; Return
                               
                               
                               
                                
; ---------------------------------------------------
;       Diagnostic Test #5 - Mix Test
; ---------------------------------------------------
LE754:   LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         JSR LE4B3              ; color?
         LDA #$6C               ; LF46C = "MIX TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF46C = "MIX TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
         LDA #$00               ;
         STA L1003              ;
         STA DOSINI+1           ;
         JSR LE7AF              ;
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
LE774:   LDA #$FF               ;
         STA DOSINI+1           ;
         LDA #$08               ;
         STA L1003              ;
         JSR LE7AF              ;
LE780:   LDA L1000              ;
         EOR #$FF               ;
         AND #$20               ;
         BEQ LE791              ;
         LDA #$07               ;
         STA L1004              ;
         JMP LE774              ;
LE791:   LDA L1000
         EOR #$FF
         AND #$10
         BEQ LE780
         LDA #$07
         STA L1004
         LDA #$00
         STA DOSINI+1
         STA L1003
         JSR LE7AF
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
         JMP LE780
LE7AF:   LDA #$00
         STA DOSINI
         JSR LE7DE
LE7B6:   JSR LE7F7
         JSR LE82B
LE7BC:   JSR LE859
         JSR LE887              ; Wait for 1 Verticle Blanking Pulse
         JSR LE870              ; Draw
         JSR LE887              ; Wait for 1 Verticle Blanking Pulse
         LDA L3BE3
         CMP #$00
         BNE LE7BC
         INC DOSINI
         LDA DOSINI
         CMP #$04
         BNE LE7B6
         JSR LE7F7
         JSR LE82B
         RTS
LE7DE:   LDX #$01
         LDA #$1E
LE7E2:   STA L2800,X
         STA L2BE0,X
         STA L3800,X
         STA L3BE0,X
         INX
         INX
         INX
         INX
         CPX #$21
         BNE LE7E2
         RTS
LE7F7:   LDA DOSINI
         TAY
         LDX #$00
         LDA LE826,Y
LE7FF:   STA L2800,X
         STA L2BE0,X
         STA L3800,X
         STA L3BE0,X
         INX
         INX
         INX
         INX
         CPX #$20
         BNE LE7FF
         LDA DOSINI+1
         BEQ LE825
         LDA #$00
         STA L281C
         STA L2BFC
         STA L381C
         STA L3BFC
LE825:   RTS
LE826:   ORA (ZRAM_02+1,X)
         ORA CMCMD
         ORA (L00A0,X)
         BRK
         LDX #$02
LE82F:   JSR LE840
         INX
         INY
         JSR LE840
         INX
         INX
         INX
         INY
         CPY #$10
         BNE LE82F
         RTS


LE840:   LDA LE893,Y
         STA L2800,X
         LDA LE8A3,Y
         STA L2BE0,X
         LDA LE8B3,Y
         STA L3800,X
         LDA LE8C3,Y
         STA L3BE0,X
         RTS


LE859:   LDX #$02
LE85B:   INC L2800,X
         INC L2BE0,X
         INC L3800,X
         INC L3BE0,X
         INX
         INX
         INX
         INX
         CPX #$22
         BNE LE85B
         RTS                    ; Return



; ---------------------------------------------------
;           
; ---------------------------------------------------
LE870:   LDX #$03               ;
LE872:   DEC L2800,X            ;
         DEC L2BE0,X            ;
         DEC L3800,X            ;
         DEC L3BE0,X            ;
         INX                    ;
         INX                    ;
         INX                    ;
         INX                    ;
         CPX #$23               ;
         BNE LE872              ;
         RTS                    ; Return


; ---------------------------------------------------
;           Wait for 1 Blanking Pulse        
; ---------------------------------------------------
LE887:   LDA #$01               ; 1 Blanking Pulse
         STA NumberBlanks       ; Save number of blanking pulses to wait for
LE88B:   JSR LE3FE              ; Wait for Blanking pulse
         DEC NumberBlanks       ; Have we counted down?
         BNE LE88B              ; No, loop back and wait more
         RTS                    ; Return


LE893:   RTI
         .BYTE $2F
         RTI
         .BYTE $47
         RTI
         .BYTE $5F
         RTI
         .BYTE $77
         RTI
         .BYTE $97
         RTI
         .BYTE $AF
         RTI
         .BYTE $C7
         RTI
LE8A2:   .BYTE $DF
LE8A3:   CLI
         .BYTE $2F
         CLI
         .BYTE $47
         CLI
         .BYTE $5F
         CLI
         .BYTE $77
         CLI
         .BYTE $97
         CLI
         .BYTE $AF
         CLI
         .BYTE $C7
         CLI
         .BYTE $DF
LE8B3:   BVS LE8E4
         BVS LE8FE
         BVS LE918
         BVS LE932
         BVS LE854
         BVS LE86E
         BVS LE888
         BVS LE8A2
LE8C3:   DEY
         .BYTE $2F
         DEY
         .BYTE $47
         DEY
         .BYTE $5F
         DEY
         .BYTE $77
         DEY
         .BYTE $97
         DEY
         .BYTE $AF
         DEY
         .BYTE $C7
         DEY
         .BYTE $DF


; ---------------------------------------------------
;      Diagnostic Test #4  Character Display Test
; ---------------------------------------------------
LE8D3:   LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Clear Video RAM
; ---------------------------------------------------
;      "A" Group Character ROM  $2000/$2400   
; ---------------------------------------------------
         LDA #$00               ; Setup address $2000 $2400
         STA $001B              ; Save address
         STA $001D              ; Save address
         LDA #$20               ; Setup address $2000 $2400
         STA $001C              ; Save address
         LDA #$24               ; Setup address $2000 $2400
LE8E4:   STA $001E              ; Save address
         JSR LE922              ; Draw Color bars
; ---------------------------------------------------
;         Wait for S2 Switch to be pressed
; ---------------------------------------------------
LE8E9:   LDA L1000              ; Get switch input
         AND #$20               ; Check switch S2
         BNE LE8E9              ; Wait for Switch S2 to be pressed
         LDA #$07               ; A = Dink Sound
         STA L1004              ; Write to Sound Chip
         LDA #$1F               ; 31 Blanking Pulses
         STA NumberBlanks       ; Save number of blanking pulses to wait for 
         JSR LE3E2              ; Wait for number of Blanking pulses
         LDY #$FF               ; Do not check ZERO PAGE RAM
LE8FE:   JSR LF19C              ; Clear Video RAM
; ---------------------------------------------------
;      "B" Group Character ROM  $3000/$3400   
; ---------------------------------------------------
         LDA #$30               ; Setup address $3000
         STA $001C              ; Save address
         LDA #$34               ; Setup address $3400
         STA $001E              ; Save address
         JSR LE922              ; Draw Color Bars
; ---------------------------------------------------
;         Wait for S2 Switch to be pressed
; ---------------------------------------------------
LE90C:   LDA L1000              ; Get switch input
         AND #$20               ; Check switch S2
         BNE LE90C              ; Wait for Switch S2 to be pressed
         LDA #$07               ; A = Dink Sound
         STA L1004              ; Write to Sound Chip
LE918:   LDA #$1F               ; 31 Blanking Pulses
         STA NumberBlanks       ; Save number of blanking pulses to wait for
         JSR LE3E2              ; Wait for number of Blanking pulses
         JMP LE8D3              ; Loop back and show Group "A" again



; ---------------------------------------------------
;                Draw Character Group 
; ---------------------------------------------------
;   $001B-$001C       Address1
;   $001D-$001E       Address2
; ---------------------------------------------------
LE922:   LDY #$00               ; Y = 0
         LDX #$00               ; X = 0
LE926:   TYA                    ; A <= Y
         STA ($001B),Y          ; 0 - 255
         TXA                    ;
         STA ($001D),Y          ; 0 - 3
         INY                    ;
         BNE LE926              ;
         INC $001C              ; 
         INC $001E              ;
         INX                    ; Increment High-byte address
         CPX #$04               ; $3000-3400,$2000-2400?
         BNE LE926              ; Loop and do this 4 times
         RTS                    ; Return




; ---------------------------------------------------
;         Diagnostic Test #10 - LDP Test
; ---------------------------------------------------
LE939:   LDA #$01
         STA ICAX1Z
         LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Clear Video RAM
         LDA #$7A               ; LF47A = "LDP TEST" 
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF47A = "LDP TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
         LDX #$00
LE94F:   LDA #$7B
         STA L2969,X
         INX
         CPX #$06
         BNE LE94F
         LDX #$00
         LDA #$08
         STA L1003
         LDA #$56               ; Command = CLEAR_ALL
         STA SerialByte         ; Save LDPlayer Command Byte
         JSR LEA1B              ; Send Command to LDPlayer
         CLI                    ; Disable interrupts
LE968:   LDX #$00
LE96A:   CPX #$0A
         BCS LE968
; ---------------------------------------------------
;         Wait for S2 Switch to be pressed
; ---------------------------------------------------
LE96E:   LDA L1000              ; Get switch input
         AND #$20               ; Check switch S2
         BNE LE96E              ; Wait for Switch S2 to be pressed
         LDA #$07               ; A = Dink Sound
         STA L1004              ; Write to Sound Chip
         LDA #$1F               ; 31 Blanking Pulses
         STA NumberBlanks       ; Save number of blanking pulses to wait for
         JSR LE3E2              ; Wait for number of Blanking pulses
         JSR LE9B8
         LDA #$50               ; Command = INDEX_ON
         STA SerialByte         ; Save LDPlayer Command Byte
         JSR LEA1B              ; Send Command to LDPlayer
         CLI                    ; Disable interrupts
         LDA LE995,X
         STA L1007
LE992:   JMP LE992              ; Failure - Stop here


; ---------------------------------------------------
;                  ROM Checksum Test
; ---------------------------------------------------
LE995:   .BYTE $3A,$3B,$3C
         AND L4A4F,X
         .BYTE $4B
         JMP L4F4D


; ---------------------------------------------------
;        Send CLEAR_ALL if not ACKNOWLEDGED
; ---------------------------------------------------
LE99F:   LDA L1007              ; Get serial data byte (LDP Status Byte)
         CMP #$0A               ; Was response Acknowledge ($0A)?
         BEQ LE9AD              ; Yes, so simply return
         LDA #$56               ; Command = CLEAR_ALL
         STA SerialByte         ; Save LDPlayer Command Byte
         JSR LEA1B              ; Send Command to LDPlayer
LE9AD:   RTS                    ; Return


LE9AE:   LDA L1000
         EOR #$FF
         AND #$20
         BEQ LE9AE
         RTS
LE9B8:   TXA
         PHA
         TAY
         LDA #$00
         STA FMSZPG+3
         TAX
         TYA
         BEQ LE9CD
LE9C3:   LDA #$06
         CLC
         ADC FMSZPG+3
         STA FMSZPG+3
         DEY
         BNE LE9C3
LE9CD:   LDA FMSZPG+3
         TAY
LE9D0:   LDA LE9DF,Y
         STA L3969,X
         INY
         INX
         CPX #$06
         BNE LE9D0
         PLA
         TAX
         RTS
LE9DF:   BPL LE9E1
LE9E1:   .BYTE $1A
         ASL DOSVEC+1,X
         .BYTE $23
         BPL LE9E7
LE9E7:   BPL LE9F4
         ORA L101E,X
         BRK
         ORA L1916,X
         AND (POKMSK,X)
         BRK
         ORA L0F1E,X
         .BYTE $1A
         ORA L131E,X
         ASL BUFADR+1,X
         BRK
         .BYTE $1C
         BRK
         .BYTE $1A
         ASL DOSVEC+1,X
         .BYTE $23,$1C
         BRK
         BPL LEA12
         ORA L1C1E,X
         BRK
         ORA L1916,X
         AND (ABUFPT,X)
         BRK
         ORA L0F1E,X
         .BYTE $1A
         ORA L131E,X
         ASL BUFADR+1,X
         BRK

; ---------------------------------------------------
;               Send RS-232 Serial Data
; ---------------------------------------------------
LEA1B:   SEI                    ; Enable Interrupts
         LDA L1007              ; Clear serial buffer
         LDA SerialByte         ; Get byte to send
         STA L1007              ; Save byte into serial buffer
LEA24:   LDA L1006              ; Check byte delivery
         BPL LEA24              ; Wait for byte to get sent out
LEA29:   LDA L1007              ; Get Serial Data byte
         STA LDPStatus          ; Save LDP Status Byte
         JSR LE3FE              ; Wait for Verticle Blanking
         LDA L1006
         BMI LEA29
         LDA DSTAT
         BNE LEA3D
         JSR LEA3E              ; Process LDP Status Byte
LEA3D:   RTS                    ; Return

; ---------------------------------------------------
;              Process LDP Status Byte   
; ---------------------------------------------------
LEA3E:   LDA LDPStatus          ; Get LDP Status byte
         CMP #$0A               ; Was response ACKNOWLEDGE ($0A)?
         BNE LEA45              ; No, so skip ahead
         RTS                    ; Command acknowledged, so leave
LEA45:   CMP #$01               ; Was response COMPLETION ($01)?
         BNE LEA4A              ; No, so skip ahead
         RTS                    ; Return
; ---------------------------------------------------
;                  LDP Error Message   
; ---------------------------------------------------
LEA4A:   LDX #$00               ;
LEA4C:   LDA LEA6D,X            ; Get Error Message Text
         STA L39A6,X            ; Print Text
         LDA #$7B               ;
         STA L29A6,X            ;
         INX                    ;
         CPX #$13               ;
         BNE LEA4C              ;
; ---------------------------------------------------
;             Stop Software because of error   
; ---------------------------------------------------
         LDA LDPStatus          ; Get LDP Response Byte
         CMP #$0B               ; $0B = Not Acknowledged
         BNE LEA62              ; Ignore
LEA62:   CMP #$05               ; $05 = Not Target
         BNE LEA66              ; Ignore this
LEA66:   CMP #$02               ; $02 = Error
         BNE LEA6A              ; Ignore this
LEA6A:   JMP LEA6A              ; Error - End here!


; ---------------------------------------------------
;              Error Response Messge RS-232   
; ---------------------------------------------------
LEA6D:   .BYTE $1C,$0F,$0D,$0F,$13,$20,$0F,$1C,$00,$0E,$0B,$1E
         .BYTE $0B,$00,$0F,$1C,$1C,$19,$1C   ; "RECEIVER DATA ERROR"



; ---------------------------------------------------
;      Diagnostic Test #11 - unknown
; ---------------------------------------------------
LEA80:   LDA L1002              ; Get DIP bank 2
         AND #$40               ;
         BNE LEA8A              ;
         JMP LE099              ;
LEA8A:   LDA #$02
         STA ICAX1Z
         LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         LDX #$00
LEA95:   LDA LF4A4,X
         STA L3A28,X
         INX
         CPX #$0B
         BNE LEA95
         LDA #$08
         STA L1003
         LDA #$56               ; Command = CLEAR_ALL
         STA SerialByte         ; Save LDPlayer Command Byte
         JSR LEA1B              ; Send Command to LDPlayer
         CLI                    ; Disable Interrupts
LEAAD:   LDX #$01               ;
LEAAF:   TXA                    ;
         STA L3A66,X            ;
         INX                    ;
         CPX #$11               ;
         BNE LEAAF
         LDX #$7B
         LDA #$00
         STA SpritePic1
         LDA #$2A
         STA SpritePic2
         LDA #$2B
         STA ICDNOZ
         LDA #$E0
         STA ICCOMZ
         JSR LE44F
         LDA #$79               ;
         STA L3A87              ;
         LDX #$01               ;
         JMP LEB63              ;
; ---------------------------------------------------
;        Check for Joystick/Button Input
; ---------------------------------------------------
LEAD7:   LDA #$0D               ; 13 Blanking Pulses
         STA NumberBlanks       ; Save number of blanking pulses to wait for
         JSR LE3E2              ; Wait for number of Blanking pulses
         LDA L1000              ; Get Joystick input
         EOR #$FF               ; Toggle all bits
         AND #$7C               ; Isolate Buttons and Left/Right
         BEQ LEAD7              ; No buttons pressed so loop back
         STA ICAX3Z             ; Save Joystick press
         LDA #$07               ; A = Dink Sound
         STA L1004              ; Write to Sound Chip
         ASL ICAX3Z             ; Check Left joystick press
         ASL ICAX3Z             ; Check Left joystick press
         BCS LEB0D              ; Left pressed so skip ahead
         ASL ICAX3Z             ; Check Right joystick press
         BCS LEB45              ; Right pressed so skip ahead
         ASL ICAX3Z             ; Check FIRE button
         BCS LEB63              ; Jump ahead and handle FIRE button press
         ASL ICAX3Z             ; Check BARRIER button
         BCC LEB03              ; Barrier not press, check TELEPORT
         JMP LEB9B              ; Jump ahead and handle BARRIER button press 
LEB03:   ASL ICAX3Z             ; Check TELEPORT button
         BCC LEB0A              ; TELEPORT not press, so skip ahead
         JMP LEBAD              ; Jump ahead and handle TELEPORT button press
LEB0A:   JMP LEAD7              ; Loop back and monitor joystick input
; ---------------------------------------------------
;               Joystick LEFT Pressed
; ---------------------------------------------------
LEB0D:   LDA Sprite2_Char       ;
         BNE LEB42              ;
         JSR LEB82              ;
         LDA #$00               ;
         STA ICBLHZ
         LDA #$1D
         STA L3ACC
         LDA #$0F
         STA L3ACD
         LDA #$1E
         STA L3ACE
         CLI
         LDA L3AC9
         SEC
         SBC #$01
         ASL A
         ASL A
         ASL A
         ASL A
         STA ICAX5Z
         LDA L3ACA
         SEC
         SBC #$01
         ORA ICAX5Z
         STA L1007
LEB3F:   JMP LEB3F
LEB42:   JMP LEAD7              ; Loop back and monitor joystick input
; ---------------------------------------------------
;               Joystick RIGHT Pressed
; ---------------------------------------------------
LEB45:   LDA Sprite2_Char       ;
         BEQ LEB60              ;
         AND #$02               ;
         BNE LEB58              ;
         TXA                    ;
         STA L3ACA              ;
         LDA #$00               ;
         STA Sprite2_Char       ;
         JMP LEAD7              ; Loop back and monitor joystick input
LEB58:   TXA                    ;
         STA L3AC9
         LDA #$01
         STA Sprite2_Char
LEB60:   JMP LEAD7              ; Loop back and monitor joystick input
; ---------------------------------------------------
;              FIRE Button Pressed
; ---------------------------------------------------
LEB63:   LDA #$78               ;
         STA L3AC9              ;
         STA L3ACA              ;
         LDA #$03               ;
         STA Sprite2_Char       ;
         LDA #$00               ;
         STA L3ACC              ;
         STA L3ACD              ;
         STA L3ACE              ;
         JSR LEB82              ;
         STA ICBLHZ             ;
         JMP LEAD7              ; Loop back and monitor joystick input
LEB82:   LDY #$00               ;
LEB84:   LDA #$00               ;
         STA L3AD0,Y            ;
         STA L3B10,Y            ;
         STA L3B50,Y            ;
         STA L3B90,Y            ;
         STA L3BD0,Y            ;
         INY                    ;
         CPY #$0D
         BNE LEB84
         RTS
; ---------------------------------------------------
;              BARRIER Button Pressed
; ---------------------------------------------------
LEB9B:   CPX #$10               ;
         BCS LEBAA              ;
         LDA #$7B               ;
         STA L3A86,X            ;
         INX                    ;
         LDA #$79
         STA L3A86,X
LEBAA:   JMP LEAD7              ; Loop back and monitor joystick input
; ---------------------------------------------------
;              TELEPORT Button Pressed
; ---------------------------------------------------
LEBAD:   CPX #$01               ;
         BEQ LEBBC              ;
         LDA #$7B               ;
         STA L3A86,X            ;
         DEX                    ;
         LDA #$79
         STA L3A86,X
LEBBC:   JMP LEAD7              ; Loop back and monitor joystick input

; ---------------------------------------------------
;               Diagnostic Test #2
; ---------------------------------------------------
LEBBF:   LDA L1006              ;
         BPL LEBBF              ;
         LDA L1007              ;
         STA ICAX2Z             ;
         AND #$F0               ;
         LSR A
         LSR A
         LSR A
         LSR A
         CLC
         ADC #$01
         STA BFENHI
         LDA ICAX2Z
         AND #$0F
         CLC
         ADC #$01
         STA LTEMP
         LDA ICBLHZ
         BNE LEBFC
         LDA BFENHI
         STA L3AD0
         LDA LTEMP
         STA L3AD1
         LDA #$D3
         STA ICCOMT
         LDA #$3A                 
         STA Sprite2_Prop
         JSR LEC68
LEBF6:   INC ICBLHZ
LEBF8:   CLI
         JMP LEAD7              ; Loop back and monitor joystick input

LEBFC:   CMP #$01
         BNE LEC18
         LDA BFENHI
         STA L3B10
         LDA LTEMP
         STA L3B11
         LDA #$13
         STA ICCOMT
         LDA #$3B
         STA Sprite2_Prop
         JSR LEC68
         JMP LEBF6
LEC18:   CMP #$02
         BNE LEC34
         LDA BFENHI
         STA L3B50
         LDA LTEMP
         STA L3B51
         LDA #$53
         STA ICCOMT
         LDA #$3B
         STA Sprite2_Prop
         JSR LEC68
         JMP LEBF6
LEC34:   CMP #$03
         BNE LEC50
         LDA BFENHI
         STA L3B90
         LDA LTEMP
         STA L3B91
         LDA #$93
         STA ICCOMT
         LDA #$3B
         STA Sprite2_Prop
         JSR LEC68
         JMP LEBF6
LEC50:   LDA BFENHI
         STA L3BD0
         LDA LTEMP
         STA L3BD1
         LDA #$D3
         STA ICCOMT
         LDA #$3B
         STA Sprite2_Prop
         JSR LEC68
         JMP LEBF8
LEC68:   TXA
         PHA
         TYA
         PHA
         LDX #$00
LEC6E:   LDA ICAX2Z
         CMP LEC9F,X
         BEQ LEC7F
         INX
         CPX #$06
         BNE LEC6E
         PLA
         TAY
         PLA
         TAX
         RTS
LEC7F:   TXA
         ASL A
         TAX
         LDA LECA5,X
         STA BUFADR
         LDA LECA6,X
         STA BUFADR+1
         LDY #$00
LEC8E:   LDA (BUFADR),Y
         CMP #$FF
         BEQ LEC9A
         STA (ICCOMT),Y
         INY
         JMP LEC8E
LEC9A:   PLA
         TAY
         PLA
         TAX
         RTS
LEC9F:   ASL A
         .BYTE $0B
         ORA (ZRAM_02,X)
         .BYTE $04
         ORA L00B1
LECA6:   CPX LECB5
         LDA LC4EC,Y
         CPX LECCA
         .BYTE $D2
         CPX L0D0B
         ORA FPTR2+1,X
LECB5:   CLC
        .BYTE $0B
         ORA FPTR2+1,X
         ORA L1719
         .BYTE $1A
         .WORD $0F16,$131E
         .WORD $1819,$0FFF
         .WORD $1C1C,$1C19
         .WORD $1AFF,$111C
         .WORD $0F00,$0E18
         .WORD $18FF,$1E19
         .WORD $1E00,$1C0B
         .WORD $0F11,$FF1E
         .WORD $FF01,$FF02
         .WORD $FF03,$FF04
         .WORD $FF05,$FF06
         .WORD $FF07,$FF08
         .WORD $FF09,$FF0A
         .WORD $1918,$1F1E
         .WORD $0F1D,$0D00
         .WORD $0E19,$FF0F




; ---------------------------------------------------
;     Diagnostic Test #8 - Sound Test (32 sounds)
; ---------------------------------------------------
LECFD:   LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         LDA #$5E               ; LF45E = "SOUND TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF45E = "SOUND TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
         LDX #$00
LED0F:   LDA #$00               ; A = No Sound
         STA L1004              ; Write to Sound Chip
         JSR LE3FE              ; Wait for Verticle Blanking
         LDA LED8A,X            ; A = Sound Level 0-22
         STA L1004              ; Write to Sound Chip
         JSR LED51
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
         INX                    ; Go to next sound
         CPX #$16               ; 22 sounds
         BNE LED0F
         LDX #$00               ; Loop = 0
LED2A:   LDA #$00               ; A = No Sound
         STA L1004              ; Write to Sound Chip
         JSR LE3FE              ; Wait for Verticle Blanking
         LDA LEDA0,X            ; A = Sound Level 0-12
         STA L1004              ; Write to Sound Chip
         JSR LED51              ;
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
         INX                    ; Go to next sound
         CPX #$0B               ; Have we done 12 sounds?
         BNE LED2A              ; No, loop until all sounds played
         JSR LE52A              ;
         LDA #$00               ; A = No Sound
         STA L1004              ; Write to Sound Chip
LED4E:   JMP LED4E              ; Wait here forever



; ---------------------------------------------------
;               sound subroutine
; ---------------------------------------------------
LED51:   STA Sprite1_Prop
         INC Sprite1_Prop
         JSR LED7A
         JSR LED82
         LDA L1004
         CMP Sprite1_Prop
         BEQ LED79
         JSR LE4F9              ; Diagnostic Test Failed, print "ERROR"
         LDA #$1C
         STA L3948
         LDA #$0F
         STA L3949
         LDA #$0B
         STA L394A
         LDA #$0E
         STA L394B
LED79:   RTS
LED7A:   LDA L1005
         AND #$40
         BNE LED7A
         RTS
LED82:   LDA L1005
         AND #$80
         BEQ LED82
         RTS

; ---------------------------------------------------
;                 Diagnostic Data
; ---------------------------------------------------
LED8A:   .BYTE $01,$02,$03,$04
         .BYTE $05,$06,$07,$08
         .BYTE $09,$0A,$0B,$16
         .BYTE $17,$18,$19,$1A
         .BYTE $1B,$1C,$1D,$1E
         .BYTE $1F,$20
; ---------------------------------------------------
;       Panel Switch Test data
; ---------------------------------------------------
LEDA0:   .BYTE $0C,$0D,$0E,$0F
         .BYTE $10,$11,$12,$13
         .BYTE $14,$15,$00





; ---------------------------------------------------
;   Diagnostic Test - Monitor Adjustments (16 tests)
; ---------------------------------------------------
LEDAB:   LDA L1002              ; Get DIP bank 2
         AND #$40               ; Check Sw7,0=User Diagnostics,1=Factory Diagnostics
         BNE LEDB5              ; Factory Diagnostics so continue
         JMP LE099              ; User Diagnostics so leave
; ---------------------------------------------------
;       Factory Diagnostics - Monitor Adjustments
; ---------------------------------------------------
LEDB5:   LDA #$08               ;
         STA RTCLOK             ;
LEDB9:   LDA #$00               ;
         STA FactoryTestNo      ;
LEDBD:   LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         LDA FactoryTestNo      ;
         ASL A                  ;
         TAX                    ;
         LDA LEDED,X            ;
         STA ICPTHZ             ;
         LDA LEDEE,X            ;
         STA ICBLLZ             ;
         JMP (ICPTHZ)           ; Run Monitor Adjustment test
; ---------------------------------------------------
;         Test complete, go to next test
; ---------------------------------------------------
LEDD3:   JSR LEE13              ;
         LDA #$07               ; A = Dink Sound
         STA L1004              ; Write to Sound Chip
         LDA #$1F               ; 31 Blanking Pulses
         STA NumberBlanks       ; Save number of blanking pulses to wait for
         JSR LE3E2              ; Wait for number of Blanking pulses
         INC FactoryTestNo      ; Go to the next test
         LDA #$12               ; Total of 18 tests
         CMP FactoryTestNo      ; Have we completed all 18 tests?
         BCS LEDBD
         JMP LEDB9

; ---------------------------------------------------
;      Monitor Adjustments Routines (16 tests)
; ---------------------------------------------------
LEDED:   .WORD LEE3D     ; 1)  RED BARS
         .WORD LEE47     ; 2)  GREEN BARS
         .WORD LEE51     ; 3)  BLUE BARS
         .WORD LEE5B     ; 4)  WHITE SCREEN
         .WORD LEE78     ; 5)  BLUE BORDER
         .WORD LEE90     ; 6)  R G B BLOCKS
         .WORD LF027     ; 7)  CROSS HATCH
         .WORD LEEC5     ; 8)  CHARACTERS
         .WORD LEF6C     ; 9)  R G B COLOR
         .WORD LEF86     ; 10) R G B COLOR 1/2
         .WORD LF03F     ; 11) BLACK REFERENCE
         .WORD LF04E     ; 12) COLOR BAR
         .WORD LF07A     ; 13) 
         .WORD LF07A     ; 14) 
         .WORD LF07A     ; 15) 
         .WORD LF07A     ; 16) 
         .WORD LF07A     ; 17)
         .WORD LF07A     ; 18)
         .WORD LF07F


; ---------------------------------------------------
;                       unknown
; ---------------------------------------------------
LEE13:   LDA $1000              ; Get switch input
         EOR #$FF               ; Invert all bits
         AND #$10               ; Check for S3 Switch
         BEQ LEE33              ; Not pressed, so skip ahead
; ---------------------------------------------------
;               Switch S3 pressed
; ---------------------------------------------------
         LDA #$1F               ; 31 Blanking Pulses
         STA NumberBlanks       ; Save number of blanking pulses to wait for
         JSR LE3E2              ; Wait for number of Blanking pulses
         LDA RTCLOK             ;
         STA L1003              ;
         BNE LEE2F              ;
         LDA #$00               ;
         JMP LEE31              ;
LEE2F:   LDA #$08               ;
LEE31:   STA RTCLOK             ;
; ---------------------------------------------------
;            Check for Switch S2 press
; ---------------------------------------------------
LEE33:   LDA L1000              ; Get switch input
         EOR #$FF               ; Invert all bits
         AND #$20               ; Check for S2 Switch
         BEQ LEE13              ; Not pressed, so loop back
         RTS                    ; Return
; ---------------------------------------------------
;       Monitor Adjustments Test #1 - RED BARS
; ---------------------------------------------------
LEE3D:   LDX #$00               ; Color = RED (0)
         JSR LEFB1              ; Set color
         LDX #$08               ; 
         JMP LEF71              ;
; ---------------------------------------------------
;      Monitor Adjustments Test #2 - GREEN BARS
; ---------------------------------------------------
LEE47:   LDX #$08               ; Color = GREEN (8)
         JSR LEFB1              ; Set color
         LDX #$08               ;
         JMP LEF71              ;
; ---------------------------------------------------
;      Monitor Adjustments Test #3 - BLUE BARS
; ---------------------------------------------------
LEE51:   LDX #$10               ; Color = BLUE (16)
         JSR LEFB1              ; Set color
         LDX #$08               ;
         JMP LEF71              ;
; ---------------------------------------------------
;     Monitor Adjustments Test #4 - WHITE SCREEN
; ---------------------------------------------------
LEE5B:   LDY #$FF               ; Call for erase screen
         JSR LE467              ; Reset Color RAM
         LDA #$20
         STA SpritePic1
         LDA #$38
         STA SpritePic2
         LDA #$3B
         STA ICDNOZ
         LDA #$E0
         STA ICCOMZ
         LDX #$7F
         JSR LE44F
         JMP LEDD3
; ---------------------------------------------------
;      Monitor Adjustments Test #5 - BLUE BORDER
; ---------------------------------------------------
LEE78:   LDX #$00               ;
LEE7A:   LDA #$7B               ;
         STA L3820,X            ;
         STA L3BC0,X            ;
LEE82:   STA L3000,X            ;
         STA L33E0,X            ;
LEE88:   INX                    ;
         CPX #$20               ; 32? 
         BNE LEE7A
         JMP LEDD3
; ---------------------------------------------------
;    Monitor Adjustments Test #6 - R G B BLOCKS
; ---------------------------------------------------
LEE90:   JSR LE4B3              ; color?
         LDA #$00               ;
         STA SpritePic1          ;
         LDA #$30               ;
         STA SpritePic2             ;
         LDA #$31
         STA ICDNOZ
         LDA #$40
         STA ICCOMZ
         LDX #$79
         JSR LE44F
         LDA #$32
         STA ICDNOZ
         LDA #$A0
         STA ICCOMZ
         LDX #$7A
         JSR LE44F
         LDA #$34
         STA ICDNOZ
         LDA #$00
         STA ICCOMZ
         LDX #$7B
         JSR LE44F
         JMP LEF9A
         JSR LE467              ; Reset Color RAM
         LDY #$00
         LDX #$00
LEECC:   LDA LEEEC,X
         STA L2800,X
         LDA LEF0C,X
         STA L2BE0,X
         LDA LEF2C,X
         STA L3800,X
         LDA LEF4C,X
         STA L3BE0,X
         INX
         CPX #$20
         BNE LEECC
         JMP LEDD3


LEEEC:   ORA (LTEMP,X)
         PLA
         .BYTE $37
         ORA (BufferOut,X)
         PLA
         .BYTE $47
         ORA (Sprite2_Char,X)
         .BYTE $80,$37
         ORA (BFENHI,X)
         .BYTE $80,$47
         ORA (SerialByte,X)
         TYA
         .BYTE $37
         ORA (LDPStatus,X)
         TYA
         .BYTE $47
         BRK
         BRK
LEF06:   BRK
         BRK
         BRK
         BRK
         BRK
         BRK
LEF0C:   ORA (Buff1 ,X)
         PLA
         .BYTE $67
         ORA (Buff2 ,X)
         PLA
         .BYTE $77
         ORA (Buff3 ,X)
         .BYTE $80,$67
         ORA (Sprite3_Cha ,X)
         .BYTE $80,$77
         ORA (Sprite3a_Y,X)
         TYA
         .BYTE $67
         ORA (Sprite3a_Y+1,X)
         TYA
         .BYTE $77
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
LEF2C:   ORA (ICBLLZ,X)
         PLA
         .BYTE $97
         ORA (ICBLHZ,X)
         PLA
         .BYTE $A7
         ORA (ICAX1Z,X)
         SEI
         .BYTE $97
         ORA (ICAX2Z,X)
         SEI
         .BYTE $A7
         ORA (ICAX3Z,X)
         DEY
         .BYTE $97
         ORA (Sprite1_Char,X)
         DEY
         .BYTE $A7
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
LEF4C:   ORA (ICAX5Z,X)
         PLA
         .BYTE $C7
         ORA (ICAX6Z,X)
         PLA
         .BYTE $D7
         ORA (STATUS,X)
         SEI
         .BYTE $C7
         ORA (CHKSUM,X)
         SEI
         .BYTE $D7
         ORA (SerialByte,X)
         DEY
         .BYTE $C7
         ORA (LDPStatus,X)
         DEY
         .BYTE $D7
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
         BRK
LEF6C:   JSR LE4B3              ; color?
         LDX #$00               ;
; ---------------------------------------------------
;             Draw bars - monitor test
; ---------------------------------------------------
LEF71:   LDA #$00               ;
         STA APPMHI             ;
         LDA #$30               ;
         STA APPMHI+1           ;
         LDA #$34               ;
         STA POKMSK             ;
         JSR LEFF1              ;
         JSR LEFDB              ;
         JMP LEDD3              ;
         LDA #$00               ;
         STA APPMHI
         LDA #$30
         STA APPMHI+1
         LDA #$34
         STA POKMSK
         LDX #$00
         JSR LEFF1
         JSR LEFDB
LEF9A:   LDA #$00
         STA SpritePic1
         LDA #$38
         STA SpritePic2
         LDA #$3A
         STA ICDNOZ
         LDX #$00               ;
         JSR LE43E              ; Cycle thru sprites?
         JSR LEFDB
         JMP LEDD3


; ---------------------------------------------------
;             Set color for color bars
; ---------------------------------------------------
;           X = color (0,8,16)
; ---------------------------------------------------
LEFB1:   LDY #$00               ; Loop = 0
LEFB3:   LDA LEFC3,X            ; Get correct color code from table
         STA L1828,Y            ; Save color code $1828-$1830 
         STA L1810,Y            ; Save color code $1810-$1818
         INY                    ; Next color RAM location
         INX                    ; Next color code in table
         CPY #$08               ; All 8 color codes stored?
         BNE LEFB3              ; No, so continue until done
         RTS                    ; Return


; ---------------------------------------------------
;               24 Color Codes
; ---------------------------------------------------
LEFC3:   .BYTE $FF,$FE,$FD,$FC   ; RED
         .BYTE $FB,$FA,$F9,$F8

         .BYTE $FF,$F7,$EF,$E7   ; GREEN
         .BYTE $DF,$D7,$CF,$C7

         .BYTE $FF,$FF,$BF,$BF   ; BLUE
         .BYTE $7F,$7F,$3F,$3F


; ---------------------------------------------------
;               color routine?
; ---------------------------------------------------
LEFDB:   LDX #$00               ;
LEFDD:   LDA #$00               ;
         STA L2800,X            ;
         STA L2BE0,X
         STA L3800,X
         STA L3BE0,X
         INX
         CPX #$20
         BNE LEFDD
         RTS
LEFF1:   LDY #$00
LEFF3:   LDA LF017,X
LEFF6:   STA (APPMHI),Y
         INY
         BEQ LF003
         CPY #$80
         BNE LEFF6
         INX
         JMP LEFF3
LF003:   INX
         INC APPMHI+1
         LDA APPMHI+1
         CMP POKMSK
         BNE LEFF3
         RTS
LF00D:   LDA L1000
         EOR #$FF
         AND #$20
         BEQ LF00D
         RTS

; ---------------------------------------------------
;                  data
; ---------------------------------------------------
LF017:   .BYTE $7F,$7C,$7D,$7A
         .BYTE $7E,$79,$7B,$00
         .BYTE $7F,$7D,$7E,$7B  ; 8=7F
         .BYTE $7C,$7A,$79,$00

; ---------------------------------------------------
;    Monitor Adjustments Test #7 - CROSS HATCH
; ---------------------------------------------------
LF027:   LDA #$38               ;
         STA SpritePic2             ;
         LDA #$20               ;
         STA SpritePic1          ;
         LDA #$3B               ;
         STA ICDNOZ             ;
         LDA #$E0
         STA ICCOMZ
         LDX #$77
         JSR LE44F
         JMP LEDD3
         LDA #$08
         STA L1003
         LDA #$50               ; Command = INDEX_ON
         STA SerialByte         ; Save LDPlayer Command Byte
         JSR LEA1B              ; Send Command to LDPlayer
         JMP LEDD3              ;
         LDA #$03               ;
         STA ICAX1Z             ;
; ---------------------------------------------------
;   SEARCH to CROSS HATCH Screen (frame 43800)
; ---------------------------------------------------
; Note: Sprite RAM is used to store serial data
;       because no sprites are used for this test
; ---------------------------------------------------
         LDA #$56               ; Command = CLEAR_ALL
         STA BufferOut          ; Save command in BufferOut
         LDA #$43               ; Command = SEARCH
         STA Buff1              ; Save command in BufferOut
         LDA #$34               ; Command = "4"
         STA Buff2              ; Save command in BufferOut
         LDA #$33               ; Command = "3"
         STA Buff3              ; Save command in BufferOut
         LDA #$38               ; Command = "8"
         STA Sprite3_Cha              ; Save command in BufferOut
         LDA #$30               ; Command = "0","0"
         STA Buff5              ; Save command in BufferOut
         STA Buff6              ; Save command in BufferOut
         LDA #$40               ; Command = ENTER
         STA Buff7              ; Save command in BufferOut
         LDA #$FF               ; Command = Last Byte Marker
         STA Buff8              ; Save command in BufferOut
LF074:   JSR LF08A              ; Send BufferOut to LDPlayer
         JMP LEDD3              ;
; ---------------------------------------------------
;      Send string of data bytes to LDPlayer
; ---------------------------------------------------
LF07A:   INC Buff5              ;
         JMP LF074
         LDA #$40
         STA Buff2 
         LDA #$FF
         STA Buff3 
         JMP LF074
; ---------------------------------------------------
;  Send string of data bytes to LDPlayer (BufferOut)
; ---------------------------------------------------
LF08A:   LDX #$00               ; Start at first byte
LF08C:   LDA BufferOut,X        ; Get serial data byte
         CMP #$FF               ; Is this the last byte?
         BNE LF093              ; No, so send this byte to the LDPlayer
         RTS                    ; Finished so return
LF093:   STA SerialByte         ; Save LDPlayer Command Byte
         JSR LEA1B              ; Send Command to LDPlayer
         INX                    ; Go to next byte
         JMP LF08C              ; Loop back until all bytes sent




; ---------------------------------------------------
;       Diagnostic Test #7 - Panel switch Test 
; ---------------------------------------------------
LF09C:   LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         LDA #$42               ; LF442 = "PANEL SW TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF442 = "PANEL SW TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
; ---------------------------------------------------
;                  Print "TILT"
; ---------------------------------------------------
         LDA #$1E               ; Char = "T"
         STA L3956              ; Print character
         LDA #$13               ; Char = "I"
         STA L3957              ; Print character
         LDA #$16               ; Char = "L"
         STA L3958              ; Print character
         LDA #$1E               ; Char = "T"
         STA L3959              ; Print character
; ---------------------------------------------------
;            Read Joystick for diagnosis
; ---------------------------------------------------
LF0C0:   LDA L1000              ; Get Joystick data
         EOR #$FF               ; Invert all bits
         STA CMCMD              ; Save joystick byte
         JSR LF11C              ; Get TILE switch status
         STA L2AB2              ; Print TILE switch status
         JSR LF11C              ; Get S1 switch status
         STA L29F2              ; Print S1 switch status
         JSR LF11C              ; Get S2 switch status
         STA L2A4F              ; Print S2 switch status
         JSR LF11C              ; Get S3 switch status
         STA L2A55              ; Print S3 switch status
         JSR LF11C              ; Get RIGHT switch status
         STA L2A5B              ; Print RIGHT switch status
         JSR LF11C              ; Get LEFT switch status
         STA L2A58              ; Print LEFT switch status
         JSR LF11C              ; Get UP switch status
         STA L2A52              ; Print UP switch status
         JSR LF11C              ; Get DOWN switch status
         STA L2954              ; Print DOWN switch status
         JSR LF0FD              ; Check P1 and P2 buttons
         JMP LF0C0              ; Loop continuously checking joystick

; ---------------------------------------------------
;         Check Player1 and Player2 buttons
; ---------------------------------------------------
LF0FD:   LDA L1001              ; Get button inputs
         EOR #$FF               ; Invert all bits
         AND #$30               ; Isolate P1 and P2 buttons
         LSR A                  ; Shift in bit
         LSR A                  ; Shift in bit
         LSR A                  ; Shift in bit
         LSR A                  ; Shift in bit
         STA CMCMD              ; Save input byte
         EOR #$FF               ; Invert all bits
         STA L1000              ; Save input byte
         JSR LF11C              ; Get PLAYER1 switch status
         STA L2AA9              ; Print PLAYER1 switch status
         JSR LF11C              ; Get PLAYER2 switch status
         STA L29E9              ; Print PLAYER2 switch status
         RTS                    ; Return

; ---------------------------------------------------
;          Get graphic to show switch status
; ---------------------------------------------------
LF11C:   LSR CMCMD              ; Shift switch bit in
         BCS LF123              ; Check if switch is pressed
         LDA #$78               ; Pressed graphic?
         RTS                    ; Return
LF123:   LDA #$7F               ; Released graphic?
         RTS                    ; Return




; ---------------------------------------------------
;       Diagnostic Test #6 - DIP Switch Test
; ---------------------------------------------------
LF126:   LDY #$FF               ; Do not check ZERO PAGE RAM
         JSR LF19C              ; Erase Screen
         LDA #$50               ; LF450 = "DIP SW TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF450 = "DIP SW TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
         JSR LF180              ; Print DIP labels on screen for diagnosis
; ---------------------------------------------------
;            Display DIP bank 1 switches    
; ---------------------------------------------------
LF139:   LDX #$00               ; Start with first switch
         LDA L1001              ; Get DIP bank 1
         EOR #$C0               ; Toggle coin inputs
LF140:   LSR A                  ; Check DIP switch
         JSR LF168              ; Print "0" or "1"
         INX                    ; Go to next switch
         CPX #$04               ; Have we checked all 4 DIPs?
         BNE LF140              ; No, so loop back and check all 4 switches
; ---------------------------------------------------
;              Print some graphics?  
; ---------------------------------------------------
         LDA #$76               ; Char = ?
         STA L39B0              ; Print character
         STA L39B1              ; Print character
         STA L39B2              ; Print character
         STA L39B3              ; Print character
; ---------------------------------------------------
;            Display DIP bank 2 switches    
; ---------------------------------------------------
         LDX #$00               ; Start with first switch
         LDA L1002              ; Get DIP bank 2
LF15C:   LSR A                  ; Check DIP switch
         JSR LF174              ; Print "0" or "1"
         INX                    ; Go to next switch
         CPX #$08               ; Have we checked all 8 DIPs?
         BNE LF15C              ; No, so loop back and check all 8 switches
         JMP LF139              ; Loop back and continuously update screen

; ---------------------------------------------------
;        Print 0 or 1 depending on DIP status
; ---------------------------------------------------
LF168:   PHA                    ; Save DIP value
         LDA #$01               ; Print "0"
         BCS LF16F              ; DIP clear so print to screen
         LDA #$02               ; Print "1"
LF16F:   STA L39AC,X            ; Print DIP status to screen row 1
         PLA                    ; Restore DIP value
         RTS                    ; Return

; ---------------------------------------------------
;        Print 0 or 1 depending on DIP status      
; ---------------------------------------------------
LF174:   PHA                    ; Save DIP value
         LDA #$01               ; Print "0"
         BCS LF17B              ; DIP clear so print to screen
         LDA #$02               ; Print "1"
LF17B:   STA L3A0C,X            ; Print DIP status to screen row 2
         PLA                    ; Restore DIP value
         RTS                    ; Return

; ---------------------------------------------------
;      Print DIP labels on screen for diagnosis
; ---------------------------------------------------
LF180:   LDX #$00               ; Screen location
         LDA #$02               ; Char = "1"
         STA L39AA,X            ; Print "1"
         LDA #$03               ; Char = "2"
         STA L3A0A,X            ; Print "2"
         LDY #$02               ; Label starts at "1"
LF18E:   TYA                    ; Char = switch number label
         STA L39CC,X            ; Print DIP bank 1 switch label
         STA L3A2C,X            ; Print DIP bank 2 switch label
         INY                    ; Next label
         INX                    ; Next spot
         CPX #$08               ; All 8 labels printed?
         BNE LF18E              ; Loop until all labels printed
         RTS                    ; Return




; ---------------------------------------------------
;            Clear Video RAM 2000-3FFF
; ---------------------------------------------------
;          Y = 0, Check ZERO PAGE RAM 
; ---------------------------------------------------
LF19C:   LDA #$00               ; Clear = $00
         TAX                    ; Loop = 0
LF19F:   STA L2000,X            ; Clear Video RAM locations
         STA L2100,X            ; Clear Video RAM locations
         STA L2200,X            ; Clear Video RAM locations
         STA L2300,X            ; Clear Video RAM locations
         STA L2400,X            ; Clear Video RAM locations
         STA L2500,X            ; Clear Video RAM locations
         STA L2600,X            ; Clear Video RAM locations
         STA L2700,X            ; Clear Video RAM locations
         STA L2800,X            ; Clear Video RAM locations
         STA L2900,X            ; Clear Video RAM locations
         STA L2A00,X            ; Clear Video RAM locations
         STA L2B00,X            ; Clear Video RAM locations
         STA L2C00,X            ; Clear Video RAM locations
         STA L2D00,X            ; Clear Video RAM locations
         STA L2E00,X            ; Clear Video RAM locations
         STA L2F00,X            ; Clear Video RAM locations
         STA L3100,X            ; Clear Video RAM locations
         STA L3200,X            ; Clear Video RAM locations
         STA L3300,X            ; Clear Video RAM locations
         STA L3400,X            ; Clear Video RAM locations
         STA L3500,X            ; Clear Video RAM locations
         STA L3600,X            ; Clear Video RAM locations
         STA L3700,X            ; Clear Video RAM locations
         STA L3800,X            ; Clear Video RAM locations
         STA L3900,X            ; Clear Video RAM locations
         STA L3A00,X            ; Clear Video RAM locations
         STA L3B00,X            ; Clear Video RAM locations
         STA L3C00,X            ; Clear Video RAM locations
         STA L3D00,X            ; Clear Video RAM locations
         STA L3E00,X            ; Clear Video RAM locations
         STA L3F00,X            ; Clear Video RAM locations
         INX                    ; Increment loop
         BNE LF19F              ; Loop until all locations cleared
         TYA                    ; Should check ZERO PAGE RAM?
         BEQ LF203              ; Yes, so go check ZERO PAGE RAM
         RTS                    ; Return
LF203:   JMP LE129              ; Perform ZERO PAGE RAM Test




; ---------------------------------------------------
;       Diagnostic Test #9 - Line Test
; ---------------------------------------------------
LF206:   LDY #$FF               ; Do not initialize RAM
         JSR LF19C              ; Erase Screen
         LDA #$04               ;
         STA ICAX1Z             ;
         LDA #$C0               ; LF4C0 = "LDP LINE TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF4C0 = "LDP LINE TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
LF21A:   LDA #$2E               ; LF52E = "PUSH BEAM SW TO START"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F5               ; LF52E = "PUSH BEAM SW TO START"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LF293              ; Print Message to screen
LF225:   JSR LE416              ; Wait until S1 Switch pressed (BEAM)
         LDA #$00               ; Char = SPACE
         STA L396F              ; Erase "OK"
         STA L3970              ; Erase "OK"
         LDA #$07               ; A = Special Sound
         STA L1004              ; Write to Sound Chip
         JSR LF287              ;
         LDA #$FF               ;
         STA DSTAT              ;
; ---------------------------------------------------
;   Send CLEAR_ALL to check for LDPlayer response
; ---------------------------------------------------
         LDA #$56               ; Command = CLEAR_ALL
         STA SerialByte         ; Save LDPlayer Command Byte
         JSR LEA1B              ; Send Command Byte to LDPlayer
         LDA LDPStatus          ; Get LDP Status byte
         CMP #$0A               ; Was response Acknowledge ($0A)?
         BNE LF25A              ; Not acknowledged so skip ahead
         LDA #$F8               ; LF4F8 = "SOCKET NOT PLUGED IN"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF4F8 = "SOCKET NOT PLUGED IN"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LF293              ; Print Message to screen
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
         JMP LF225              ; Loop back and recheck connection
; ---------------------------------------------------
;              Test Passed, Play Sound
; ---------------------------------------------------
LF25A:   LDX #$00               ; Reset UART
LF25C:   CLI                    ; Disable interrupts
         TXA                    ; X = byte
         STA L1007              ; Write to UART
LF261:   JMP LF261              ; Finished so end here





; ---------------------------------------------------
;                unknown
; ---------------------------------------------------
LF264:   PLA                    ; Restore Y Register
         TAY                    ; Move Y
         PLA                    ; Restore X Register
         TAX                    ; Move X
         PLA                    ; Restore A Register
         LDA L1007              ; Get serial data byte (LDP Status Byte)
         STA LDPStatus          ; Save LDP Status byte
         CPX LDPStatus          ;
         BEQ LF27B              ;
         JSR LE4F9              ; Diagnostic Test Failed, print "ERROR"
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
         JMP LF206              ;
LF27B:   INX                    ;
         BEQ LF281              ;
         JMP LF25C              ;
LF281:   JSR LE51A              ; Diagnostic Test Passed, print "OK"
         JMP LF21A              ;



; ---------------------------------------------------
;               print ? to screen?
; ---------------------------------------------------
LF287:   LDY #$00               ;
         TYA                    ;
LF28A:   STA L39C6,Y            ;
         INY                    ;
         CPY #$1A               ;
         BNE LF28A              ;
         RTS                    ; Return


; ---------------------------------------------------
;          Print Message to Screen ($FF = EOL)
; ---------------------------------------------------
LF293:   LDY #$00               ; Start at beginning of message
LF295:   LDA (DiagMessage_LO),Y ; Get character of message
         CMP #$FF               ; Is this the end of the message?
         BNE LF29C              ; No so continue
         RTS                    ; Done printing to leave
LF29C:   STA L39C6,Y            ; Print character
         INY                    ; Go to next character
         JMP LF295              ; Loop back and continue until done



; ---------------------------------------------------
;      Diagnostic Test #11 - LaserDisc Test
; ---------------------------------------------------
LF2A3:   LDY #$FF               ; Do not initialize RAM
         JSR LF19C              ; Erase Screen
         LDA #$08               ;
         STA L1003              ;
         LDA #$B2               ; LF4B2 = "DISC TEST"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF4B2 = "DISC TEST"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LE3D1              ; Print Diagnostic Test Name to Screen
; ---------------------------------------------------
;                 Turn on INDEX
; ---------------------------------------------------
         LDA #$50               ; Command = INDEX_ON
         STA SerialByte         ; Save command in BufferOut
         JSR LEA1B              ; Send RS-232 Serial Data
         LDA #$03
         STA ICAX1Z
         LDA #$39
         STA ZCHAIN+1
         LDA #$EB
         STA ZCHAIN
LF2CB:   LDA #$2E               ; LF52E = "PUSH BEAM SW TO START"
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F5               ; LF52E = "PUSH BEAM SW TO START"
         STA DiagMessage_HI     ; Save Text Message address
         JSR LF293              ; Print Message to screen
         JSR LE416              ; Wait until S1 Switch pressed
         LDA #$00
         STA L398E
         STA L398F
         STA L3990
         LDA #$DC               ; LF4DC = ""
         STA DiagMessage_LO     ; Save Text Message address
         LDA #$F4               ; LF4DC = ""
         STA DiagMessage_HI     ; Save Text Message address
         JSR LF293              ; Print Message to screen
; ---------------------------------------------------
;                buffer search?
; ---------------------------------------------------
LF2EF:   LDA #$07               ;
         STA L1004              ;
         LDA #$44               ;
         STA FMSZPG+5           ;
         LDA #$F5               ;
         STA Sprite1_Y          ;
         LDA #$00               ;
         STA FMSZPG+4           ;
; ---------------------------------------------------
;             Fill BufferOut for SEARCH
; ---------------------------------------------------
LF300:   LDY #$00               ; Loop = 0
LF302:   LDA (FMSZPG+5),Y       ; Get outgoing data
         STA Buff2 ,Y           ; Transfer to BufferOut
         INY                    ; Increment loop
         CPY #$05               ; Have all 5 digits been copied?
         BNE LF302              ; No, loop until all digits copied
; ---------------------------------------------------
;                  Setup SEARCH command
; ---------------------------------------------------
         LDA #$56               ; Command = CLEAR_ALL
         STA BufferOut          ; Save command in BufferOut
         LDA #$43               ; Command = SEARCH
         STA Buff1              ; Save command in BufferOut
         LDA #$40               ; Command = ENTER
         STA Buff7              ; Save command in BufferOut
LF318:   LDX #$00
LF31A:   CLI
         LDA BufferOut,X
         STA L1007
LF320:   JMP LF320


; ---------------------------------------------------
;               Diagnostic Test #3
; ---------------------------------------------------
LF323:   PLA
         PLA
         PLA
         PLA
         PLA
         PLA
         LDA L1007              ; Get serial data byte       
         STA LDPStatus          ; Save LDP Status byte
         CMP #$0A               ; Was response Acknowledge ($0A)?
         BNE LF33B
         INX
         CPX #$08
         BNE LF31A
         CLI
LF338:   JMP LF338
LF33B:   CMP #$01
         BNE LF375
LF33F:   LDA L1002              ; Get DIP bank 2
         AND #$01
         BNE LF349
         JSR LE3DE              ; Wait for 64 Verticle Blanking Pulses
LF349:   LDA FMSZPG+5
         CLC
         ADC #$05
         STA FMSZPG+5
         LDA Sprite1_Y
         ADC #$00
         STA Sprite1_Y
         LDA #$F7
         CMP Sprite1_Y
         BEQ LF35F
LF35C:   JMP LF300
LF35F:   LDA #$1A
         CMP FMSZPG+5
         BNE LF35C
         LDA L1002              ; Get DIP bank 2
         AND #$02
         BNE LF36F
         JMP LF2EF
LF36F:   JSR LE52A
         JMP LF2CB
LF375:   CMP #$05
         BEQ LF37C
         JMP LEA4A
LF37C:   LDA FMSZPG+4
         BNE LF385
         INC FMSZPG+4
         JMP LF318
LF385:   CMP #$01
         BNE LF391
         INC FMSZPG+4
         JSR LF3CD
         JMP LF318
LF391:   CMP #$02
         BNE LF3A0
         INC FMSZPG+4
         JSR LF3B2
         JSR LF3B2
         JMP LF318
LF3A0:   LDA #$00
         STA FMSZPG+4
         JSR LF3B2
         JSR LF3E8
         LDY #$FF
         JSR LE4F9              ; Diagnostic Test Failed, print "ERROR"
         JMP LF33F
LF3B2:   TXA
         PHA
         LDX #$06
LF3B6:   INC BufferOut,X
         LDA BufferOut,X
         CMP #$3A
         BEQ LF3C1
LF3BE:   PLA
         TAX
         RTS
LF3C1:   LDA #$30
         STA BufferOut,X
         DEX
         CPX #$01
         BEQ LF3BE
         JMP LF3B6
LF3CD:   TXA
         PHA
         LDX #$06
LF3D1:   DEC BufferOut,X
         LDA BufferOut,X
         CMP #$2F
         BEQ LF3DC
LF3D9:   PLA
         TXA
         RTS
LF3DC:   LDA #$39
         STA BufferOut,X
         DEX
         CPX #$01
         BEQ LF3D9
         JMP LF3D1
LF3E8:   TXA
         PHA
         TYA
         PHA
         LDA ZCHAIN+1
         CMP #$34
         BCS LF40E
         LDY #$00
LF3F4:   LDA Buff2 ,Y
         AND #$0F
         CLC
         ADC #$01
         STA (ZCHAIN),Y
         INY
         CPY #$05
         BNE LF3F4
         LDA ZCHAIN
         CLC
         ADC #$20
         STA ZCHAIN
         BCC LF40E
         INC ZCHAIN+1
LF40E:   PLA
         TAY
         PLA
         TAX
         RTS



; ---------------------------------------------------
;           Diagnostic Text Messages    ASC(x+54)
; ---------------------------------------------------
;    In some cases only 14 characters can be printed
; ---------------------------------------------------
LF413:   .BYTE $24,$0F,$1C,$19,$00,$1A,$0B,$11,$0F,$00,$0B,$1C,$0F,$0B  ;"ZERO PAGE AREA"        
         .BYTE $00,$1E,$0F,$1D,$1E  ; "TEST" (string too long)
 
LF426:   .BYTE $1C,$19,$17,$00,$1C,$0F,$0B,$0E,$00,$1E,$0F,$1D,$1E,$00  ;"ROM READ TEST"

LF434:   .BYTE $1C,$0B,$17,$00,$1E,$0F,$1D,$1E,$00,$00,$00,$00,$00,$00  ;"RAM TEST"               

LF442:   .BYTE $1A,$0B,$18,$0F,$16,$00,$1D,$21,$00,$1E,$0F,$1D,$1E,$00  ;"PANEL SW TEST"

LF450:   .BYTE $0E,$13,$1A,$00,$1D,$21,$00,$1E,$0F,$1D,$1E,$00,$00,$00  ;"DIP SW TEST"

LF45E:   .BYTE $1D,$19,$1F,$18,$0E,$00,$1E,$0F,$1D,$1E,$00,$00,$00,$00  ;"SOUND TEST" 

LF46C:   .BYTE $17,$13,$22,$00,$1E,$0F,$1D,$1E,$00,$00,$00,$00,$00,$00  ;"MIX TEST"

LF47A:   .BYTE $16,$0E,$1A,$00,$1E,$0F,$1D,$1E,$00,$00,$00,$00,$00,$00  ;"LDP TEST"
 
LF488:   .BYTE $16,$0E,$1A,$00,$0D,$19,$16,$19,$1C,$00,$00,$00,$00,$00  ;"LDP COLOR

LF496:   .BYTE $12,$0B,$1C,$0E,$00,$0D,$19,$16,$19,$1C,$00,$00,$00,$00  ;"HARD COLOR"

LF4A4:   .BYTE $16,$0E,$1A,$00,$17,$19,$18,$13,$1E,$19,$1C,$00,$00,$00  ;"LDP MONITOR"

LF4B2:   .BYTE $0E,$13,$1D,$0D,$00,$1E,$0F,$1D,$1E,$00,$00,$00,$00,$00  ;"DISC TEST"

LF4C0:   .BYTE $16,$0E,$1A,$00,$16,$13,$18,$0F,$00,$1E,$0F,$1D,$1E,$00  ;"LDP LINE TEST"

LF4CE:   .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ;"          "
LF4DC:   .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00  ; (blank)
LF4EA:   .BYTE $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FF  ;

LF4F8:   .BYTE $1D,$19,$0D,$15,$0F,$1E,$00,$18,$19,$1E,$00,$1A,$16,$1F  ;"SOCKET NOT PLUGED IN"
         .BYTE $11,$0F,$0E,$00,$13,$18,$FF

LF50D:   .BYTE $1D,$1E,$0B,$1C,$1E,$00,$1A,$1F,$1D,$12,$00,$1D,$12,$19  ;"START PUSH SHOOT SW 1"
         .BYTE $19,$1E,$00,$1D,$21,$00,$02,$FF

LF523:   .BYTE $0F,$22,$0F,$0D,$1F,$1E,$13,$18,$11,$00,$FF              ;"EXECUTING "

LF52E:   .BYTE $1A,$1F,$1D,$12,$00,$0C,$0F,$0B,$17,$00,$1D,$21,$00,$1E  ;"PUSH BEAM SW TO START"
         .BYTE $19,$00,$1D,$1E,$0B,$1C,$1E,$FF




; ---------------------------------------------------
;                unknown data
; ---------------------------------------------------
LF544:   .BYTE $30,$35,$33,$36
         .BYTE $35,$30,$35,$31
         .BYTE $33,$30,$31,$32
         .BYTE $39,$33,$35,$31
         .BYTE $31,$32,$39,$38
         .BYTE $32,$30,$35,$39
         .BYTE $34,$32,$33,$30
         .BYTE $33,$34,$32,$35
         .BYTE $39,$35,$37,$32
         .BYTE $39,$30,$35,$36
         .BYTE $33,$32,$30,$35
         .BYTE $36,$33,$33,$33
         .BYTE $33,$32,$33,$39
         .BYTE $34,$32,$30,$34
         .BYTE $31,$39,$31,$37
         .BYTE $32,$32,$32,$31
         .BYTE $34,$31,$38,$33
         .BYTE $38,$30,$32,$32
         .BYTE $33,$37,$32,$32
         .BYTE $36,$32,$32,$37
         .BYTE $33,$39,$39,$30
         .BYTE $37,$34,$32,$31
         .BYTE $33,$34,$31,$35
         .BYTE $31,$38,$35,$34
         .BYTE $32,$34,$35,$34
         .BYTE $34,$33,$35,$34
         .BYTE $32,$30,$36,$37
         .BYTE $39,$38,$31,$33
         .BYTE $33,$38,$35,$31
         .BYTE $36,$33,$35,$38
         .BYTE $30,$38,$35,$39
         .BYTE $38,$32,$31,$33
         .BYTE $31,$34,$32,$33
         .BYTE $39,$33,$36,$32
         .BYTE $37,$32,$35,$36
         .BYTE $32,$39,$33,$35
         .BYTE $36,$33,$32,$33
         .BYTE $31,$36,$33,$35
         .BYTE $34,$34,$33,$33
         .BYTE $38,$33,$38,$30
         .BYTE $34,$31,$30,$31
         .BYTE $37,$30,$35,$38
         .BYTE $39,$38,$31,$30
         .BYTE $33,$39,$38,$31
         .BYTE $37,$32,$35,$38
         .BYTE $31,$39,$36,$39
         .BYTE $34,$32,$33,$39
         .BYTE $33,$36,$32,$34
         .BYTE $38,$33,$36,$33
         .BYTE $30,$32,$35,$36
         .BYTE $33,$34,$35,$34
         .BYTE $33,$34,$31,$30
         .BYTE $31,$37,$30,$39
         .BYTE $34,$39,$38,$30
         .BYTE $37,$36,$39,$38
         .BYTE $31,$38,$37,$39
         .BYTE $34,$34,$33,$39
         .BYTE $30,$31,$34,$36
         .BYTE $33,$34,$34,$30
         .BYTE $35,$35,$36,$37
         .BYTE $30,$35,$33,$36
         .BYTE $33,$31,$33,$31
         .BYTE $34,$34,$31,$31
         .BYTE $35,$30,$39,$32
         .BYTE $30,$39,$33,$38
         .BYTE $32,$33,$31,$39
         .BYTE $38,$32,$36,$32
         .BYTE $32,$36,$32,$39
         .BYTE $32,$30,$35,$33
         .BYTE $32,$33,$31,$34
         .BYTE $33,$33,$35,$34
         .BYTE $31,$33,$39,$37
         .BYTE $33,$35,$34,$32
         .BYTE $31,$33,$33,$32
         .BYTE $32,$33,$37,$31
         .BYTE $31,$38,$36,$35
         .BYTE $36,$32,$32,$36
         .BYTE $37,$31,$32,$36
         .BYTE $37,$38,$38,$34
         .BYTE $30,$31,$31,$36
         .BYTE $34,$32,$33,$30
         .BYTE $30,$31,$35,$34
         .BYTE $33,$37,$34,$32
         .BYTE $39,$32,$31,$34
         .BYTE $33,$37,$34,$39
         .BYTE $30,$37,$36,$39
         .BYTE $37,$31,$34,$31
         .BYTE $33,$35,$31,$37
         .BYTE $31,$30,$37,$30
         .BYTE $39,$33,$34,$37
         .BYTE $32,$32,$30,$36
         .BYTE $33,$32,$34,$36
         .BYTE $38,$35,$32,$38
         .BYTE $30,$30,$35,$33
         .BYTE $30,$31,$30,$35
         .BYTE $33,$33,$30,$36
         .BYTE $35,$33,$36,$31
         .BYTE $39,$32,$33,$39
         .BYTE $31,$32,$39,$34
         .BYTE $31,$37,$36,$36
         .BYTE $30,$36,$36,$34
         .BYTE $37,$31,$31,$31
         .BYTE $34,$37,$31,$38
         .BYTE $30,$30,$37,$32
         .BYTE $30,$34,$34,$33
         .BYTE $32,$34,$36,$38
         .BYTE $35,$32,$35,$35
         .BYTE $38,$35,$33,$31
         .BYTE $30,$30,$35,$33
         .BYTE $35,$32,$39,$32
         .BYTE $34,$31,$37,$36
         .BYTE $36,$31,$30,$32
         .BYTE $34,$37,$30,$38
         .BYTE $34,$34,$37,$31
         .BYTE $39,$35,$34,$33
         .BYTE $34,$36,$33,$34
         .BYTE $33,$34,$36,$37
         .BYTE $33,$35,$30,$30
         .BYTE $30,$30,$31,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF

LF828:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LF888:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LF8C8:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF
LF967:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LF9F9:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LFAF9:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF
LFBFA:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LFCFC:   .BYTE $FF
LFCFD:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LFDFD:   .BYTE $FF
LFDFE:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LFE10:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF
LFE1D:   .BYTE $FF,$FF
LFE1F:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LFEBB:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LFEFD:   .BYTE $FF
LFEFE:   .BYTE $FF
LFEFF:   .BYTE $FF,$FF,$FF
LFF02:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LFF1C:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF
LFF21:   .BYTE $FF,$FF
LFF23:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF
LFF3A:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF
LFF5F:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LFF6D:   .BYTE $FF,$FF
LFF6F:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
LFF7F:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF
LFF8D:   .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF,$FF,$FF,$FF
         .BYTE $FF


; ---------------------------------------------------
;              ROM Checksums
; ---------------------------------------------------
LFFEE:   .WORD $9D3A,$573A,$9BB9,$DA46
         .WORD $4D7B,$0000

; ---------------------------------------------------
;                Interrupt Vectors
; ---------------------------------------------------
LFFFA:   .WORD LE000        ; NMI_VECTOR     
LFFFC:   .WORD LE003        ; RESET_VECTOR   
LFFFE:   .WORD LE006        ; IRQ_VECTOR      

         .END


;
; System equates
;
; OS EQUATES
; ----------
; 
; Syntax:
; Use '=' for addresses
; and '#' for values
; 
; IO EQUATES
; 
ICHID    = $0000
ICDNO    = $0001
ICCOM    = $0002
ICSTA    = $0003
ICBAL    = $0004
ICBAH    = $0005
ICPTL    = $0006
ICPTH    = $0007
ICBLL    = $0008
ICBLH    = $0009
ICAX1    = $000A
ICAX2    = $000B
ICAX3    = $000C
ICAX4    = $000D
ICAX5    = $000E
ICAX6    = $000F
; 
; DISPLAY LIST EQUATES
; 
ADLI     = $0080
AVB      = $0040
ALMS     = $0040
AVSCR    = $0020
AHSCR    = $0010
AJMP     = $0001
AEMPTY1  = $0000
AEMPTY2  = $0010
AEMPTY3  = $0020
AEMPTY4  = $0030
AEMPTY5  = $0040
AEMPTY6  = $0050
AEMPTY7  = $0060
AEMPTY8  = $0070
; 
; OS VARIABLES FOR XL/XE
; 
; PAGE 0
;
L0000    = $0000 
ZRAM_01  = $0001
ZRAM_02  = $0002
RAMLO    = $0004
TRAMSZ   = $0006
CMCMD    = $0007
WARMST   = $0008
BOOT     = $0009
DOSVEC   = $000A
DOSINI   = $000C
APPMHI   = $000E
POKMSK   = $0010
Sprite1_Prop   = $0011
RTCLOK   = $0012
BUFADR   = $0015
ICCOMT   = $0017
Sprite2_Prop   = $0018
DiagMessage_HI   = $001A
ABUFPT   = $001C
Sprite3_Prop = $001F
SpritePic2   = $0020
ICDNOZ   = $0021
ICCOMZ   = $0022
FactoryTestNo  = $0023
DiagTestNo     = $0024
ICBAHZ   = $0025
Sprite4_Prop   = $0026
ICPTHZ   = $0027
ICBLLZ   = $0028
ICBLHZ   = $0029
ICAX1Z   = $002A
ICAX2Z   = $002B
ICAX3Z   = $002C
Sprite1_Char   = $002D
ICAX5Z   = $002E
ICAX6Z   = $002F
STATUS   = $0030
CHKSUM   = $0031
SerialByte   = $0032
LDPStatus   = $0033
Sprite2_Char   = $0034
BFENHI   = $0035
LTEMP:   = $0036
BufferOut = $0037
Buff1    = $0038
Buff2    = $0039
Buff3    = $003A
Sprite3_Cha    = $003B
Buff5    = $003C
Buff6    = $003D
Buff7    = $003E
Buff8    = $003F
FREQ     = $0040
SOUNDR   = $0041
Sprite4_Char   = $0042
FMSZPG   = $0043
Sprite1_Y = $0049
ZCHAIN   = $004A
DSTAT    = $004C
NumberBlanks   = $004D
DRKMSK   = $004E
COLRSH   = $004F
Sprite2_Y     = $0050
HOLD1    = $0051
LMARG:N   = $0052
RMARGN   = $0053
ROWCRS   = $0054
COLCRS   = $0055
Sprite3_Y   = $0057
Sprite3a_Y  = $0058
OLDROW   = $005A
OLDCOL   = $005B
OLDCHR   = $005D
Sprite4_Y   = $005E
FKDEF    = $0060
PALNTS   = $0062
LOGCO:L  = $0063
ADRESS   = $0064
Sprite1_X = $0065
MLTTMP   = $0066
SAVADR   = $0068
RAMTOP   = $006A
BUFCNT   = $006B
Sprite2_X   = $006C
BITMSK   = $006E
SHFAMT   = $006F
ROWAC    = $0070
COLAC    = $0072
Sprite3_X   = $0073
Sprite3a_X  = $0074
DELTAR   = $0076
DELTAC   = $0077
KEYDEF   = $0079
BegaXPos1L = $007B
BegaXPos1R = $007C
BegaXPos2L = $007D
BegaXPos2R = $007E
BegaXPos3L = $007F
BegaXPos3R = $0080
VNTP     = $0082
VNTD     = $0084
Lunas_swap = $0086
STMTAB   = $0088
STMCUR   = $008A
STARP    = $008C
RUNSTK   = $008E
TOPSTK   = $0090
MEOLFLG  = $0092
POKADR   = $0095
DATAD    = $00B6
DATALN   = $00B7
STOPLN   = $00BA
SAVCUR   = $00BE
IOCMD    = $00C0
IODVC    = $00C1
PROMPT   = $00C2
ERRSAVE  = $00C3
COLOUR   = $00C8
PTABW    = $00C9
LOADF:LG  = $00CA
RandSeed      = $00D4
FRE      = $00DA
FR1      = $00E0
FR2      = $00E6
FRX      = $00EC
EEXP     = $00ED
NSIGN    = $00EE
ESIGN    = $00EF
FCHRFLG  = $00F0
DIGRT    = $00F1
CIX      = $00F2
INBUFF   = $00F3
ZTEMP1   = $00F5
ZTEMP4   = $00F7
ZTEMP3   = $00F9
Mini_LO  = $00FA
Mini_HI  = $00FB
FLPTR    = $00FC
FPTR2    = $00FE
; 
; PAGE 2
; 
VDSLST   = $0200
VPRCED   = $0202
VINTER   = $0204
VBREAK   = $0206
VKEYBD   = $0208
SysClock   = $020A
VSEROR   = $020C
Random   = $020E
VTIMR1   = $0210
VTIMR2   = $0212
VTIMR4   = $0214
VIMIRQ   = $0216
CDTMV1   = $0218
Credits  = $0219
CoinCount   = $021A
CDTMV3   = $021C
NumPlayers   = $021E
CDTMV5   = $0220
Level    = $0222
VVBLKD   = $0224
CDTMA1   = $0226
CDTMA2   = $0228
CDTMF3   = $022A
SRTIMR   = $022B
CDTMF4   = $022C
Score   = $022D
Score1   = $022E
Score2   = $022F
SDLSTL   = $0230
SDLSTH   = $0231
SSKCTL   = $0232
SPARE    = $0233
LPENH:   = $0234
LPENV:   = $0235
BRKKY    = $0236
VPIRQ    = $0238
CDEVIC   = $023A
CCOMND   = $023B
CAUX1    = $023C
CAUX2    = $023D
TMPSIO   = $023E
ERRFLG   = $023F
DFLAGS   = $0240
DBSECT   = $0241
BOOTAD   = $0242
COLDST   = $0244
RECLEN   = $0245
DSKTIM   = $0246
PDVMSK   = $0247
SHPDVS   = $0248
PDMSK    = $0249
RELADR   = $024A
PPTMPA   = $024C
PPTMPX   = $024D
CHSALT   = $026B
Bonus    = $026C
Bonus+1  = $026D
Bonus+2  = $026E
GPRIOR   = $026F
PADDL0   = $0270
PADDL1   = $0271
PADDL2   = $0272
PADDL3   = $0273
PADDL4   = $0274
PADDL5   = $0275
PADDL6   = $0276
PADDL7   = $0277
STICK0   = $0278
STICK1   = $0279
ErrorHEX   = $027A
ErrorNAK   = $027B
STICK4   = $027C
PTRIG1   = $027D
PTRIG2   = $027E
PTRIG3   = $027F
PTRIG4   = $0280
PTRIG5   = $0281
PTRIG6   = $0282
PTRIG7   = $0283
STRIG0   = $0284
STRIG1   = $0285
STRIG2   = $0286
STRIG3   = $0287
HIBYTE   = $0288
WMODE    = $0289
BLIM     = $028A
IMASK    = $028B
JVECK    = $028C
NEWADR   = $028E
TXTROW   = $0290
TXTCOL   = $0291
TINDEX   = $0293
TXTMSC   = $0294
TXTOLD   = $0296
CRETRY   = $029C
HOLD3    = $029D
SUBTMP   = $029E
SUBTMP+1  = $029F
DMASK    = $02A0
TMPLBT   = $02A1
ESCFLG   = $02A2
TABMAP   = $02A3
LOGMA:P   = $02B2
INVFLG   = $02B6
FILFLG   = $02B7
TMPROW   = $02B8
TMPCOL   = $02B9
SCRFLG   = $02BB
HOLD4    = $02BC
DRETRY   = $02BD
SHFLOC   = $02BE
BOTSCR   = $02BF
PCOLR0   = $02C0
PCOLR1   = $02C1
PCOLR2   = $02C2
PCOLR3   = $02C3
COLOR0   = $02C4
COLOR1   = $02C5
COLOR2   = $02C6
COLOR3   = $02C7
COLOR4   = $02C8
RUNADR   = $02C9
HIUSED   = $02CB
ZHIUSE   = $02CD
GBYTEA   = $02CF
LOADA:D   = $02D1
ZLOADA   = $02D3
DSCTLN   = $02D5
ACMISR   = $02D7
KRPDER   = $02D9
KEYREP   = $02DA
NOCLIK   = $02DB
HELPFG   = $02DC
DMASAV   = $02DD
PBPNT    = $02DE
PBUFSZ   = $02DF
RUNAD    = $02E0
INITAD   = $02E2
RAMSIZ   = $02E4
MEMTOP   = $02E5
MEMLO    = $02E7
HNDLOD   = $02E9
DVSTAT   = $02EA
CBAUDL   = $02EE
CBAUDH   = $02EF
CRSINH   = $02F0
KEYDEL   = $02F1
CH1      = $02F2
CHACT    = $02F3
CHBAS    = $02F4
NEWROW   = $02F5
NEWCOL   = $02F6
ROWINC   = $02F8
COLINC   = $02F9
CHAR     = $02FA
ATACHR   = $02FB
CH       = $02FC
FILDAT   = $02FD
DSPFLG   = $02FE
SSFLAG   = $02FF
; 
; PAGE 3
; 
DDEVIC   = $0300
DUNIT    = $0301
DCOMND   = $0302
DSTATS   = $0303
DBUFLO   = $0304
DBUFHI   = $0305
DTIMLO   = $0306
DUNUSE   = $0307
DBYTLO   = $0308
DBYTHI   = $0309
DAUX1    = $030A
DAUX2    = $030B
TIMER1   = $030C
ADDCOR   = $030E
CASFLG   = $030F
TIMER2   = $0310
TEMP1    = $0312
TEMP2    = $0314
TEMP3    = $0315
SAVIO    = $0316
TIMFLG   = $0317
STACKP   = $0318
TSTAT    = $0319
HATABS   = $031A
PUPBT1   = $033D
PUPBT2   = $033E
PUPBT3   = $033F
IOCB0    = $0340
IOCB1    = $0350
IOCB2    = $0360
IOCB3    = $0370
IOCB4    = $0380
IOCB5    = $0390
IOCB6    = $03A0
IOCB7    = $03B0
PRNBUF   = $03C0
SUPERF   = $03E8
CKEY     = $03E9
CASSBT   = $03EA
CARTCK   = $03EB
DERRF    = $03EC
ACMVAR   = $03ED
BASICF   = $03F8
MINTLK   = $03F9
GINTLK   = $03FA
CHLINK   = $03FB
CASBUF   = $03FD
; 
; HARDWARE REGISTERS
; 
; GTIA
; 
HPOSP0   = $D000
HPOSP1   = $D001
HPOSP2   = $D002
HPOSP3   = $D003
HPOSM0   = $D004
HPOSM1   = $D005
HPOSM2   = $D006
HPOSM3   = $D007
SIZEP0   = $D008
SIZEP1   = $D009
SIZEP2   = $D00A
SIZEP3   = $D00B
SIZEM    = $D00C
GRAFP0   = $D00D
GRAFP1   = $D00E
GRAFP2   = $D00F
GRAFP3   = $D010
GRAFM    = $D011
COLPM0   = $D012
COLPM1   = $D013
COLPM2   = $D014
COLPM3   = $D015
COLPF0   = $D016
COLPF1   = $D017
COLPF2   = $D018
COLPF3   = $D019
COLBK    = $D01A
PRIOR    = $D01B
VDELAY   = $D01C
GRACTL   = $D01D
HITCLR   = $D01E
CONSOL   = $D01F
; 
; POKEY
; 
AUDF1    = $D200
AUDC1    = $D201
AUDF2    = $D202
AUDC2    = $D203
AUDF3    = $D204
AUDC3    = $D205
AUDF4    = $D206
AUDC4    = $D207
AUDCTL   = $D208
STIMER   = $D209
SKREST   = $D20A
POTGO    = $D20B
SEROUT   = $D20D
IRQEN    = $D20E
SKCTL    = $D20F
; 
; PIA
; 
PORTA    = $D300
PORTB    = $D301
PACTL    = $D302
PBCTL    = $D303
; 
; ANTIC
; 
DMACLT   = $D400
CHACTL   = $D401
DLISTL   = $D402
DLISTH   = $D403
HSCROL   = $D404
VSCROL   = $D405
PMBASE   = $D407
CHBASE   = $D409
WSYNC    = $D40A
VCOUNT   = $D40B
PENH     = $D40C
PENV     = $D40D
NMIEN    = $D40E
NMIRES   = $D40F
; 
; FLOATING POINT ROUTINES
; 
AFP      = $D800
FASC     = $D8E6
IFP      = $D9AA
FPI      = $D9D2
ZFRO     = $DA44
ZF1      = $DA46
FSUB     = $DA60
FADD     = $DA66
FMUL     = $DADB
FDIV     = $DB28
PLYEVL   = $DD40
FLD0R    = $DD89
FLD0P    = $DD8D
FLD1R    = $DD98
FLD1P    = $DD9C
FSTOR    = $DDA7
FSTOP    = $DDAB
FMOVE    = $DDB6
EXP      = $DDC0
EXP10    = $DDCC
LOG  :   = $DECD
LOG10:   = $DED1
; 
; ROM VECTORS
; 
DSKINV   = $E453
CIOV     = $E456
SIOV     = $E459
SETVBV   = $E45C
SYSVBV   = $E45F
XITVBV   = $E462
SIOINV   = $E465
SENDEV   = $E468
INTINV   = $E46B
CIOINV   = $E46E
SELFSV   = $E471
WARMSV   = $E474
COLDSV   = $E477
RBLOKV   = $E47A
CSOPIV   = $E47D
PUPDIV   = $E480
SELFTSV  = $E483
PENTV    = $E486
PHUNLV   = $E489
PHINIV   = $E48C
GPDVV    = $E48F

